unit AdminCharges;

interface

uses SysUtils, dbtables, bde, classes;

  procedure SaveAllAdminFees(Const nThisBatch: LongInt;var lSuccessful: Boolean);
  procedure PrepareTNdata;
  procedure GetAdminReferences;
  procedure DeleteOldAdminFees;
  procedure AppendFeeRecord;
  procedure ApplyDiscounts(const DiscountID, FeeLostDiscount : string);

implementation

uses
  DataMod, DateFunctions, GenFns, Main, MyMessage, Progress;

var
  nBatch : LongInt;
  sBatch : string;
  qryTN : TQuery;                   // tDataCollection.
  qryTT : TQuery;                   // tTransaction.
  FieldStringList : TStringList;
  ValueStringList : TStringList;
  qryStringList : TStringList;
  sListValues : String;
  sListFields : String;

  frmProgress : TProgressForm;
  nCount : Integer;
  Factor : Real;
  nTotalCount : Integer;
  nBlock : Integer;

  RecordLostFees     : boolean;
  RecordLostDiscount : boolean;

  sAdminFeeRevElement     : String;
  sAdminFeeExpElement     : String;
  sAdminFeeVRTRevElement  : string;
  sAdminFeeVRTExpElement  : string;
  sLostAdminFeeRevElement : String;
  sLostDiscountRevElement : String;
  sInfoMessage            : String;
  rAdminFeeRevRate        : Real;
  rAdminFeeExpRate        : Real;
  rAppliedRate       : Real;
  lRevenue           : boolean;
  lStdRateNotApplied : boolean;

  sFeeRevenueTransaction : String;
  sFeeExpenditureTransaction : String;
  sFeeLostTransaction : String;
  sFeeLostDiscountTransaction : String;
  sDiscountTransaction : String;

  sVRTID : string;

procedure DeleteOldAdminFees;
var
  qryStringList : TStringList;
begin
  GetAdminReferences;
  qryStringList := TStringList.Create;
{
  qryStringList.Add('DELETE FROM tTransaction');
  qryStringList.Add('WHERE ((tTransaction.BatchID='+sBatch+') AND');
  qryStringList.Add(' (tTransaction.ElementID IN ('+
    sAdminFeeRevElement +','+ sAdminFeeExpElement +','+
    sLostAdminFeeRevElement +','+ sLostDiscountRevElement +
    ')))');
}
  qryStringList.Add('DELETE tTransaction');
  qryStringList.Add('FROM (tTransaction INNER JOIN tTransactionType '+
    'ON tTransaction.TransactionTypeID = tTransactionType.ID)');
  qryStringList.Add('LEFT JOIN tTransactionGroup '+
    'ON tTransactionType.TransactionGroupID = tTransactionGroup.ID');
  qryStringList.Add('WHERE tTransaction.BatchID='+sBatch);
  qryStringList.Add('AND tTransactionGroup.Description LIKE ''ADMIN''');
  qryTT.Close;
  qryTT.SQL := qryStringList;
  try
    qryTT.ExecSQL;
  finally
    qryStringList.Free;
    qryTT.Close;
  end;  // try..finally.
end;  //   procedure DeleteOldAdminFees();


procedure GetAdminReferences;
var
  qryElement : TStringList;
  qry : TQuery;
  qryLook : TQuery;
  sDescription : string;
begin
  // Initiate temporary variables.
  qryElement := TStringList.Create;
  qry := TQuery.Create(nil);
  qry.DatabaseName := 'dbPPdata';
  qryLook := TQuery.Create(nil);
  qryLook.DatabaseName := 'dbPPdata';

  // Retrieve references.
  qryStringList.Clear;
  qryStringList.Add('SELECT ParmNumber FROM tParameter');
  qryStringList.Add('WHERE (Parm LIKE ''AdminFeeRevenueElement'')');
  qryLook.SQL := qryStringList;
  qryLook.Open;
  if qryLook.eof then
    begin
    sAdminFeeRevElement := '0';
    end
  else
    begin
      sAdminFeeRevElement :=
        Trim(IntToStr(qryLook.FieldByName('ParmNumber').AsInteger));
    end;  // if qryLook.eof then..else.
  qryStringList.Delete(qryStringList.Count-1);
  qryStringList.Add('WHERE (Parm LIKE ''AdminFeeExpenditureElement'')');
  qryLook.Close;
  qryLook.SQL := qryStringList;
  qryLook.Open;
  if qryLook.eof then
    begin
    sAdminFeeExpElement := '0';
    end
  else
    begin
      sAdminFeeExpElement :=
        Trim(IntToStr(qryLook.FieldByName('ParmNumber').AsInteger));
    end;  // if qryLook.eof then..else.
  qryStringList.Delete(qryStringList.Count-1);
  qryStringList.Add('WHERE (Parm LIKE ''AdminFeeVRTRevenueElement'')');
  qryLook.Close;
  qryLook.SQL := qryStringList;
  qryLook.Open;
  if qryLook.eof then
    begin
    sAdminFeeVRTRevElement := sAdminFeeExpElement;   // VRT Default.
    end
  else
    begin
      sAdminFeeVRTRevElement :=
        Trim(IntToStr(qryLook.FieldByName('ParmNumber').AsInteger));
    end;  // if qryLook.eof then..else.
  qryStringList.Delete(qryStringList.Count-1);
  qryStringList.Add('WHERE (Parm LIKE ''AdminFeeVRTExpenditureElement'')');
  qryLook.Close;
  qryLook.SQL := qryStringList;
  qryLook.Open;
  if qryLook.eof then
    begin
    sAdminFeeVRTExpElement := sAdminFeeExpElement;   // VRT Default.
    end
  else
    begin
      sAdminFeeVRTExpElement :=
        Trim(IntToStr(qryLook.FieldByName('ParmNumber').AsInteger));
    end;  // if qryLook.eof then..else.
  qryStringList.Delete(qryStringList.Count-1);
  qryStringList.Add('WHERE (Parm LIKE ''LostAdminFeeRevenueElement'')');
  qryLook.Close;
  qryLook.SQL := qryStringList;
  qryLook.Open;
  if qryLook.eof then
    begin
    sLostAdminFeeRevElement := '0';
    end
  else
    begin
      sLostAdminFeeRevElement :=
        Trim(IntToStr(qryLook.FieldByName('ParmNumber').AsInteger));
    end;  // if qryLook.eof then..else.
  qryStringList.Delete(qryStringList.Count-1);
  qryStringList.Add('WHERE (Parm LIKE ''LostDiscountRevenueElement'')');
  qryLook.Close;
  qryLook.SQL := qryStringList;
  qryLook.Open;
  if qryLook.eof then
    begin
    sLostDiscountRevElement := '0';
    end
  else
    begin
      sLostDiscountRevElement :=
        Trim(IntToStr(qryLook.FieldByName('ParmNumber').AsInteger));
    end;
  qryStringList.Delete(qryStringList.Count-1);
  qryStringList.Add('WHERE (Parm LIKE ''AdminFeeRevenueRate'')');
  qryLook.Close;
  qryLook.SQL := qryStringList;
  qryLook.Open;
  if qryLook.eof then
    begin
    rAdminFeeRevRate := 0.0000;
    end
  else
    begin
    rAdminFeeRevRate := qryLook['ParmNumber'];
    end;  // if qryLook.eof then..else.
  sDefaultRevAdmRate := FloatToStrF(100*rAdminFeeRevRate,ffFixed,5,2);

  qryStringList.Delete(qryStringList.Count-1);
  qryStringList.Add('WHERE (Parm LIKE ''AdminFeeExpenditureRate'')');
  qryLook.Close;
  qryLook.SQL := qryStringList;
  qryLook.Open;
  if qryLook.eof then
    begin
    rAdminFeeExpRate := 0.0000;
    end
  else
    begin
    rAdminFeeExpRate := qryLook['ParmNumber'];
    end;  // if qryLook.eof then..else.
  sDefaultExpAdmRate := FloatToStrF(100*rAdminFeeExpRate,ffFixed,5,2);

  qryStringList.Clear;
  qryStringList.Add('SELECT id FROM tStatus');
  qryStringList.Add('WHERE (Status LIKE ''Information'')');
  qryLook.SQL := qryStringList;
  qryLook.Open;
  if qryLook.eof then
    begin
    sInfoMessage := '0';
    end
  else
    begin
    sInfoMessage :=
      IntToStr(qryLook.FieldByName('id').AsInteger);
    end;  // if qryLook.eof then..else.

  sFeeRevenueTransaction := '';
  sFeeExpenditureTransaction := '';
  sFeeLostTransaction := '';
  sFeeLostDiscountTransaction := '';
  // Read in the Transaction Types.
  qryLook.Close;
  qryLook.SQL.Clear;
  qryLook.SQL.Add('SELECT id, description FROM tTransactionType ' +
    'WHERE description LIKE ''%Fee%''');
  qryLook.Open;
  while not qryLook.Eof do begin
    sDescription := UpperCase(qryLook.FieldByName('description').AsString);
    if (Pos('REVENUE',sDescription) > 0) then
      sFeeRevenueTransaction := IntToStr(qryLook['id'])
    else if (Pos('EXPENDITURE',sDescription) > 0) then
      sFeeExpenditureTransaction := IntToStr(qryLook['id'])
    else if (Pos('LOST DISCOUNT',sDescription) > 0) then
      sFeeLostDiscountTransaction := IntToStr(qryLook['id'])
    else if (Pos('LOST',sDescription) > 0) then
      sFeeLostTransaction := IntToStr(qryLook['id']);
    qryLook.Next;
  end;  // while not qryLook.Eof do begin.
  if (Length(sFeeRevenueTransaction) = 0) then
    sFeeRevenueTransaction := '9999995';
  if (Length(sFeeExpenditureTransaction) = 0) then
    sFeeExpenditureTransaction := '9999996';
  if (Length(sFeeLostTransaction) = 0) then
    sFeeLostTransaction := '9999997';
  if (Length(sFeeLostDiscountTransaction) = 0) then
    sFeeLostDiscountTransaction := '9999998';;
  qryLook.Close;
  qryLook.SQL.Clear;
  qryLook.SQL.Add('SELECT id FROM tTransactionType ' +
    'WHERE description LIKE ''Discount''');
  qryLook.Open;
  if qryLook.Eof then sDiscountTransaction := '9999999'
  else sDiscountTransaction := IntToStr(qryLook['id']);
  qryLook.Close;
  qryLook.SQL.Clear;
  qryLook.SQL.Add('SELECT id FROM tCustomer ' +
    'WHERE ShipTo LIKE ''VRT''');
  qryLook.Open;
  if qryLook.Eof then sVRTID := '9999990'
  else sVRTID := IntToStr(qryLook['id']);
  qryLook.Close;
  qryLook.SQL.Clear;

  qryLook.Close;
  qryLook.Free;
  qryStringList.Clear;
  qry.Close;
  qry.Free;
  qryElement.Free;
end;  // GetAdminReferences().


procedure PrepareTNdata;
begin
  FieldStringList.Clear;
  ValueStringList.Clear;
  // NOTE: GSTFlag must have a value or the INSERT will fail.
  sListFields := '(BatchID,ServiceID,GSTFlag,';
  sListValues := 'VALUES ('+ sBatch +','+
    IntToStr(qryTN.FieldByName('ServiceID').AsInteger) +',1,';
  if (qryTN['ServiceTypeID']<>Null) then
    begin
    sListFields := sListFields + 'ServiceTypeID,';
    sListValues := sListValues +
      IntToStr(qryTN.FieldByName('ServiceTypeID').AsInteger) +',';
    end;  // ServiceTypeID.
  if (qryTN['CentreID']<>Null) then
    begin
    sListFields := sListFields + 'CentreID,';
    sListValues := sListValues +
      IntToStr(qryTN.FieldByName('CentreID').AsInteger) +',';
    end;  // CentreID.
  if (qryTN['ActivityID']<>Null) then
    begin
    sListFields := sListFields + 'ActivityID,';
    sListValues := sListValues +
      IntToStr(qryTN.FieldByName('ActivityID').AsInteger) +',';
    end;  // ActivityID.
  if (qryTN['SubledgerID']<>Null) then
    begin
    sListFields := sListFields + 'SubledgerID,';
    sListValues := sListValues +
      IntToStr(qryTN.FieldByName('SubledgerID').AsInteger) +',';
    end;  // SubledgerID.
  if (qryTN['LocationID']<>Null) then
    begin
    sListFields := sListFields + 'LocationID,';
    sListValues := sListValues +
      IntToStr(qryTN.FieldByName('LocationID').AsInteger) +',';
    end;  // LocationID.
  if (qryTN['PersonID']<>Null) then
    begin
    sListFields := sListFields + 'PersonID,';
    sListValues := sListValues +
      IntToStr(qryTN.FieldByName('PersonID').AsInteger) +',';
    end;  // PersonID.
  FieldStringList.Add(sListFields);
  ValueStringList.Add(sListValues);
  sListFields := '';
  sListValues := '';
//  if qryTN['RentFrom'] <> Null then begin
  if qryTN['FromDate'] <> Null then begin
    // Assume date is in the correct format.
    sListFields := sListFields+ 'FromDate,';
    sListValues := sListValues+''''+
      StandardDateString(qryTN.FieldByName('FromDate').AsDateTime) +''',';
  end;  // FromDate.
  if qryTN['ToDate'] <> Null then begin
    // Assume date is in the correct format.
    sListFields := sListFields+ 'ToDate,';
    sListValues := sListValues+''''+
      StandardDateString(qryTN.FieldByName('ToDate').AsDateTime) +''',';
  end;  // FromDate.
//  if (qryTN['RentTill'] <> Null) or (qryTN['CallTill'] <> Null) then begin
//    // Assume date is in the correct format.
//    sListFields := sListFields+ 'ToDate,';
//    if (qryTN['RentTill'] = Null) then begin
//      sListValues := sListValues+''''+
//        StandardDateString(qryTN.FieldByName('CallTill').AsDateTime) +''',';
//    end
//    else begin
//      sListValues := sListValues+''''+
//        StandardDateString(qryTN.FieldByName('RentTill').AsDateTime) +''',';
//    end;
//  end;  // ToDate.
  FieldStringList.Add(sListFields);
  ValueStringList.Add(sListValues);
  sListFields := '';
  sListValues := '';
  sListFields := sListFields + 'RevenueFlag,';
  if lRevenue then begin
    sListValues := sListValues + '1,';
    if (qryTN['CustomerID']<>Null) then begin
      sListFields := sListFields + 'CustomerID,';
      sListValues := sListValues +
        IntToStr(qryTN.FieldByName('CustomerID').AsInteger) +',';
    end;  // CustomerID.
  end
  else begin
    sListValues := sListValues + '0,';
    sListFields := sListFields + 'CustomerID,';
    sListValues := sListValues + sVRTID +',';
  end;
  if (qryTN['CustomerCentreID']<>Null) then
    begin
    sListFields := sListFields + 'CustomerCentreID,';
    sListValues := sListValues +
      IntToStr(qryTN.FieldByName('CustomerCentreID').AsInteger) +',';
    end;  // CustomerCentreID.
  if (qryTN['CustomerActivityID']<>Null) then
    begin
    sListFields := sListFields + 'CustomerActivityID,';
    sListValues := sListValues +
      IntToStr(qryTN.FieldByName('CustomerActivityID').AsInteger) +',';
    end;  // CustomerActivityID.
  if (qryTN['CustomerElementID']<>Null) then
    begin
    sListFields := sListFields + 'CustomerElementID,';
    sListValues := sListValues +
      IntToStr(qryTN.FieldByName('CustomerElementID').AsInteger) +',';
    end;  // CustomerElementID.
  if (qryTN['CustomerSubledgerID']<>Null) then
    begin
    sListFields := sListFields + 'CustomerSubledgerID,';
    sListValues := sListValues +
      IntToStr(qryTN.FieldByName('CustomerSubledgerID').AsInteger) +',';
    end;  // CustomerSubledgerID.
  FieldStringList.Add(sListFields);
  ValueStringList.Add(sListValues);
  sListFields := '';
  sListValues := '';
end;  // PrepareTNdata.


procedure ApplyDiscounts(const DiscountID, FeeLostDiscount : string);
// FOR REVENUE ITEMS ONLY - refer SQL statements for filtering.
// Cannot offer a discount on Expenditure transactions.
var
  MessageForm : TMessagesForm;
  sDiscount : string;
  sFeeLostDiscount : string;

  sShortFields : string;
  sShortValues : string;

  curExGST : Currency;
  curIncGST : Currency;
  curGST : Currency;
begin
  // First, advise the user.
  MessageForm := TMessagesForm.Create(Nil);
  MessageForm.Msg.Caption := 'Preparing to apply discounts ...';
  MessageForm.Show;
  frmProgress.ProcessWinMessages(nil);

  sDiscount := DiscountID;
  sFeeLostDiscount := FeeLostDiscount;

  // Now determine how many records to be processed.
  qryStringList.Clear;
  qryStringList.Add('SELECT Count(*) AS TotalCount');
  qryStringList.Add('FROM (SELECT tTransaction.ServiceID, ' +
    'tTransaction.ServiceTypeID, tTransaction.CustomerID,');
  qryStringList.Add('  Sum(tTransaction.AmountExGST) AS TOTAL');
  qryStringList.Add(' FROM tTransaction');
  qryStringList.Add('  WHERE tTransaction.BatchID = ' + IntToStr(nBatch));
  qryStringList.Add('  AND tTransaction.StatusID IS NULL');
  qryStringList.Add('  AND tTransaction.RevenueFlag = 1');
  qryStringList.Add(' GROUP BY tTransaction.ServiceID, ' +
    'tTransaction.ServiceTypeID, tTransaction.CustomerID) AS TN');
  qryStringList.Add(' LEFT JOIN tDiscount ON ((TN.ServiceTypeID = ' +
    'tDiscount.ServiceTypeID) AND (TN.CustomerID = tdiscount.CustomerID) '+
    'AND (tDiscount.Active = 1))');
  qryStringList.Add('WHERE tDiscount.DiscountRate IS NOT NULL ' +
    'AND TN.TOTAL > 0');
  qryTN.Close;
  qryTN.SQL := qryStringList;
  qryTN.Open;
  nCount := qryTN.FieldByName('TotalCount').AsInteger;
  qryTN.Close;

  // Open the query with ALL of the required data.
  // START MAIN QUERY.  *******************************************************
  qryStringList.Clear;
  qryStringList.Add('SELECT TN.*, tDiscount.DiscountRate');
  qryStringList.Add('FROM (SELECT tTransaction.ServiceID, ' +
    'tTransaction.ServiceTypeID, Sum(tTransaction.AmountExGST) AS "TOTAL",');
  qryStringList.Add('  Min(tTransaction.FromDate) AS "FromDate",');
  qryStringList.Add('  Max(tTransaction.ToDate) AS "ToDate",');
//  qryStringList.Add('  Min(CASE tTransactionGroup.Description WHEN ''RENT'' ' +
//    'THEN tTransaction.FromDate END) AS RENTFROM,');
//  qryStringList.Add('  Max(CASE tTransactionGroup.Description WHEN ''RENT'' ' +
//    'THEN tTransaction.ToDate END) AS RENTTILL,');
//  qryStringList.Add('  Max(CASE tTransactionGroup.Description WHEN ''CALL'' ' +
//    'THEN tTransaction.ToDate END) AS CALLTILL,');
  qryStringList.Add('  tTransaction.CentreID, tTransaction.ActivityID, ' +
    'tTransaction.SubledgerID,');
  qryStringList.Add('  tTransaction.LocationID, tTransaction.PersonID, ' +
    'tTransaction.CustomerID,');
  qryStringList.Add('  tTransaction.CustomerCentreID, ' +
    'tTransaction.CustomerActivityID, tTransaction.CustomerElementID,');
  qryStringList.Add('  tTransaction.CustomerSubledgerID');
  qryStringList.Add(' FROM (tTransaction');
  qryStringList.Add('   LEFT JOIN tTransactionType ON ' +
    'tTransaction.TransactionTypeID = tTransactionType.ID)');
  qryStringList.Add('   LEFT JOIN tTransactionGroup ON ' +
    'tTransactionType.TransactionGroupID = tTransactionGroup.ID');
  qryStringList.Add('  WHERE tTransaction.BatchID = ' + IntToStr(nBatch));
  qryStringList.Add('  AND tTransaction.StatusID IS NULL');
  qryStringList.Add('  AND tTransaction.RevenueFlag = 1');
  qryStringList.Add(' GROUP BY tTransaction.ServiceID, ' +
    'tTransaction.ServiceTypeID, tTransaction.CentreID,');
  qryStringList.Add('  tTransaction.ActivityID, tTransaction.SubledgerID,');
  qryStringList.Add('  tTransaction.LocationID, tTransaction.PersonID, ' +
    'tTransaction.CustomerID,');
  qryStringList.Add('  tTransaction.CustomerCentreID, ' +
    'tTransaction.CustomerActivityID, tTransaction.CustomerElementID,');
  qryStringList.Add('  tTransaction.CustomerSubledgerID) AS TN');
  qryStringList.Add(' LEFT JOIN tDiscount ON ((TN.ServiceTypeID = ' +
    'tDiscount.ServiceTypeID) AND (TN.CustomerID = tDiscount.CustomerID) '+
    'AND (tDiscount.Active = 1))');
  qryStringList.Add('WHERE (tDiscount.DiscountRate IS NOT NULL) ' +
    'AND (TN.TOTAL > 0)');
  qryStringList.Add('ORDER BY TN.ServiceID');
  // END MAIN QUERY.  *********************************************************

  frmProgress.Caption := 'Applying administration fees ...';
  frmProgress.Msg.Caption := 'Applying discounts ...';
  qryTN.Close;
  qryTN.SQL := qryStringList;
  qryTN.Open;

  MessageForm.Hide;
  MessageForm.Close;
  MessageForm.Free;
  if (nCount > 0) then begin
    // Open the progress form.
    frmProgress.Show;
    frmProgress.ProcessWinMessages(nil);
    Factor := 100.00 / nCount;
    nCount := 0;

    // PROCESS ALL Grouped RECORDS IN tTransaction FOR THIS BATCH.
    while not qryTN.Eof do begin
      frmProgress.ProcessWinMessages(nil);
      // Flush the string lists used to build the SQL statement.
      qryStringList.Clear;
      FieldStringList.Clear;
      ValueStringList.Clear;

      // **********************************************************************
      frmProgress.ProcessWinMessages(nil);
      PrepareTNdata;            // Bulk of tTransaction data (rest follows).

      // Record that a discount will be given (for Information purposes only).
      sListFields := sListFields + 'StatusID,';
      sListValues := sListValues + sInfoMessage + ',';
      // Define the correct ElementID.
      sListFields := sListFields + 'ElementID,TransactionTypeID,';
      sListValues := sListValues + sLostDiscountRevElement + ',';
      sShortFields := sListFields;
      sShortValues := sListValues;
      sListValues := sListValues + sDiscount + ',';
      sShortValues := sShortValues + sFeeLostDiscount + ',';

      // Calculate the Discount value.
      rAppliedRate := qryTN.FieldByName('DiscountRate').AsFloat - 1.0000;
      curExGST := curFormat(qryTN.FieldByName('TOTAL').AsCurrency *
        rAppliedRate);
      curGST := curFormat(curExGST * 0.1);
      curIncGST := curSum(curExGST, curGST);
      // NOTE LAST ENTRIES OMITS TRAILING COMMA !!
      sListFields := sListFields + 'AmountExGST,GSTAmount,AmountIncGST)';
      sListValues := sListValues + Curr2Str(curExGST) + ',';
      sListValues := sListValues + Curr2Str(curGST) + ',';
      sListValues := sListValues + Curr2Str(curIncGST) + ')';
      // Save the Discount.
      if RecordLostDiscount then AppendFeeRecord;

      // Record the discount.
      sListFields := sShortFields;
      sListValues := sShortValues;
      // Calculate the Discount (thus a negative number).
      // NOTE LAST ENTRIES OMITS TRAILING COMMA !!
      sListFields := sListFields + 'AmountExGST,GSTAmount,AmountIncGST)';
      sListValues := sListValues + Curr2Str(curExGST * -1.0000) + ',';
      sListValues := sListValues + Curr2Str(curGST * -1.0000) + ',';
      sListValues := sListValues + Curr2Str(curIncGST * -1.0000) + ')';
      // Save the Discount.
      AppendFeeRecord;
      sListFields := '';
      sListValues := '';

      qryTN.Next;
      Inc(nCount);
      frmProgress.ProgressBar1.Position := Trunc(nCount * Factor);
      frmProgress.ProcessWinMessages(nil);
    end;  // while not qryTN.Eof do.
  end; // if (nCount > 0) then.
end;  // procedure ApplyDiscounts().


procedure AppendFeeRecord;
var
  nListValues : Integer;
begin
  // Close off the SQL statements.
  qryStringList.Clear;
  qryStringList.Add('INSERT INTO tTransaction');
  for nListValues := 0 to FieldStringList.Count-1 do begin
    qryStringList.Add(FieldStringList[nListValues]);
  end;
  qryStringList.Add(sListFields);
  for nListValues := 0 to ValueStringList.Count-1 do begin
    qryStringList.Add(ValueStringList[nListValues]);
  end;
  qryStringList.Add(sListValues);
  // Update and execute the query.
  qryTT.Close;
  qryTT.SQL := qryStringList;
  qryTT.ExecSQL;
end;  // procedure AppendFeeRecord;


procedure SaveAllAdminFees(Const nThisBatch: LongInt;var lSuccessful: Boolean);
var                                 // processed in a single SELECT statement.
  MessageForm : TMessagesForm;
  lDiscountedServices : boolean;
  AdminFeesDue : boolean;
  curExGST : Currency;
  curIncGST : Currency;
  curGST : Currency;
begin
  nBatch := nThisBatch;
  lDiscountedServices := False;
  sBatch := Trim(IntToStr(nBatch));
  // Create the query that will update the tTransaction table.
  qryTT := TQuery.Create(nil);
  qryTT.DatabaseName := 'dbPPdata';

  // Must gather the data from the tTransaction table - it is best to
  // link in all of the required data now too in the same query.
  qryTN := TQuery.Create(nil);

  // First, check if the lost fees are to be calculated.
  with qryTN do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('SELECT ParmNumber');
    SQL.Add('FROM tParameter');
    SQL.Add('WHERE Parm LIKE ''CalcLostFees''');
    Open;
    // Check if fees lost by charging a rate < standard are to be recorded.
    RecordLostFees := not ((Bof and Eof) or
      VarIsNull(FieldValues['ParmNumber']) or (FieldValues['ParmNumber'] = 0));
    Close;
    SQL.Strings[2] := 'WHERE Parm LIKE ''CalcLostDiscount''';
    Open;
    // Check if fees lost by offering a discount are to be recorded.
    RecordLostDiscount := not ((Bof and Eof) or
      VarIsNull(FieldValues['ParmNumber']) or (FieldValues['ParmNumber'] = 0));
    Close;
    SQL.Clear;
  end;  // qryTN.
  AdminFeesDue := True;
  if RecordLostFees or RecordLostDiscount then begin
    // If lost fees are recorded then check that fees are not in the Revenue items.
    with qryTN do begin
      SQL.Add('SELECT tBatchType.EarlyRevenue');
      SQL.Add('FROM tBatch');
      SQL.Add('LEFT JOIN tBatchType ON tBatch.BatchTypeID = tBatchType.ID');
      SQL.Add('WHERE tBatch.ID = '+IntToStr(nThisBatch));
      Open;
      AdminFeesDue := VarIsNull(FieldValues['EarlyRevenue']) or
        (FieldValues['EarlyRevenue'] = 0);
      Close;
      SQL.Clear;
    end;  // qryTN.
  end;  // determine if AdminFeesDue.
  
  if AdminFeesDue then begin
    // NOTE: As the data read is a summary format, no allowances are made for
    // exceeding the virtual memory capacity of the user's computer, as was done
    // in tDataCollection.
    FieldStringList := TStringList.Create;
    ValueStringList := TStringList.Create;
    qryStringList := TStringList.Create;

    // Open the query with ALL of the required data.
    // START MAIN QUERY.  *******************************************************
    // Note RevenueFlag is cast as an integer to allow it in the Group statement.
    qryStringList.Add('SELECT *');
    qryStringList.Add('FROM (SELECT (TN2.RENT + TN2.CALL + TN2.OTHER) AS TOTAL, TN2.*,'+
      ' tServiceType.AdminFee, tDiscount.DiscountRate,');
    qryStringList.Add(' tDiscount.NonStdRevAdminFee, tDiscount.NonStdExpAdminFee');
    qryStringList.Add('FROM ((SELECT TN1.ServiceID,' +
      ' TN1.ServiceTypeID, TN1.RevFlag,');
    qryStringList.Add('  Sum(CASE TN1.Description WHEN ''RENT'' THEN' +
      ' TN1.AmountExGST ELSE 0 END) AS "RENT",');
    qryStringList.Add('  Sum(CASE TN1.Description WHEN ''CALL'' THEN' +
      ' TN1.AmountExGST ELSE 0 END) AS "CALL",');
    qryStringList.Add('  Sum(CASE TN1.Description WHEN ''OTHER'' THEN' +
      ' TN1.AmountExGST ELSE 0 END) AS "OTHER",');
    qryStringList.Add('  Min(TN1.FromDate) AS "FromDate",');
    qryStringList.Add('  Max(TN1.ToDate) AS "ToDate",');
//    qryStringList.Add('  Min(CASE TN1.Description WHEN ''RENT'' THEN' +
//      ' TN1.FromDate END) AS RENTFROM,');
//    qryStringList.Add('  Max(CASE TN1.Description WHEN ''RENT'' THEN' +
//      ' TN1.ToDate END) AS RENTTILL,');
//    qryStringList.Add('  Max(CASE TN1.Description WHEN ''CALL'' THEN' +
//      ' TN1.ToDate END) AS CALLTILL,');
    qryStringList.Add('  TN1.CentreID, TN1.ActivityID, TN1.SubledgerID,');
    qryStringList.Add('  TN1.LocationID, TN1.PersonID, TN1.CustomerID,');
    qryStringList.Add('  TN1.CustomerCentreID, TN1.CustomerActivityID,' +
      ' TN1.CustomerElementID, TN1.CustomerSubledgerID');
    qryStringList.Add(' FROM (SELECT tTransaction.ServiceID, ' +
      'tTransaction.ServiceTypeID, tTransaction.AmountExGST,');
    qryStringList.Add('   tTransaction.FromDate, tTransaction.ToDate,');
    qryStringList.Add('   tTransaction.CentreID, tTransaction.ActivityID, ' +
      'tTransaction.SubledgerID, tTransaction.LocationID,');
    qryStringList.Add('   tTransaction.PersonID, tTransaction.CustomerID,');
    qryStringList.Add('   tTransaction.CustomerCentreID, ' +
      'tTransaction.CustomerActivityID,  tTransaction.CustomerElementID,');
    qryStringList.Add('   tTransaction.CustomerSubledgerID, ' +
      '(CASE tTransaction.RevenueFlag WHEN 1 THEN 1 ELSE 0 END) AS RevFlag,');
    qryStringList.Add('   tTransactionGroup.Description');
    qryStringList.Add('  FROM tTransaction LEFT JOIN ');
    qryStringList.Add('   (tTransactionType LEFT JOIN tTransactionGroup');
    qryStringList.Add('    ON tTransactionType.TransactionGroupID = ' +
      'tTransactionGroup.ID)');
    qryStringList.Add('   ON tTransaction.TransactionTypeID = ' +
      'tTransactionType.ID');
    qryStringList.Add('  WHERE tTransaction.BatchID = '+IntToStr(nBatch)+
      ') AS TN1');
    qryStringList.Add(' GROUP BY TN1.ServiceID, TN1.ServiceTypeID, TN1.RevFlag, '+
      'TN1.CentreID, TN1.ActivityID, TN1.SubledgerID,');
    qryStringList.Add('  TN1.LocationID, TN1.PersonID, TN1.CustomerID,');
    qryStringList.Add('  TN1.CustomerCentreID, TN1.CustomerActivityID,' +
      ' TN1.CustomerElementID, TN1.CustomerSubledgerID) AS TN2');
    qryStringList.Add(' LEFT JOIN tServiceType ON' +
      ' TN2.ServiceTypeID = tServiceType.ID)');
    qryStringList.Add(' LEFT JOIN tDiscount ON' +
      ' ((TN2.ServiceTypeID = tDiscount.ServiceTypeID) AND' +
      ' (TN2.CustomerID = tdiscount.CustomerID) AND'+
      ' (tDiscount.Active = 1))) AS TN3');
    qryStringList.Add('WHERE TOTAL > 0');           // Only charge for +ve totals.
    qryStringList.Add('ORDER BY ServiceID');
    // END MAIN QUERY.  *********************************************************

    MessageForm := TMessagesForm.Create(Nil);
    MessageForm.Msg.Caption := 'Loading the transaction data...';
    MessageForm.Show;
    // Open the progress form.
    frmProgress := TProgressForm.Create(NIL);
    // Display a message to the user.
    frmProgress.Caption := 'Administration...';
    frmProgress.Msg.Caption := 'Determining administration fees...';
    frmProgress.btnCancel.Enabled := False;
    frmProgress.btnCancel.Visible := False;
    frmProgress.ProcessWinMessages(nil);

    qryTN.Close;
    qryTN.SQL := qryStringList;
    qryTN.Open;
    MessageForm.Hide;
    MessageForm.Close;
    MessageForm.Free;
    if qryTN.Eof and qryTN.Bof then begin
      // No data returned.
//      lSuccessful := False;
      lSuccessful := True;
        UpdateBatchStatus(nBatch, MainForm.BATCH_STATUS_ADMINRVW);
    end
    else begin
      // Open the progress form.
      frmProgress.Show;
      frmProgress.ProcessWinMessages(nil);
      Factor := 0.1000;        // Count to 100 every 1000 records.
      nCount := 0;
      nBlock := 0;
      nTotalCount := 0;

//      GetElementReferences;    // includes standard AdminFee rates.
      DeleteOldAdminFees;      // Calls GetAdminReferences().

      // PROCESS ALL Grouped RECORDS IN tTransaction FOR THIS BATCH.
      while not qryTN.Eof do begin
        // Flush the string lists used to build the SQL statement.
        qryStringList.Clear;
        FieldStringList.Clear;
        ValueStringList.Clear;

        // **********************************************************************
        frmProgress.ProcessWinMessages(nil);
        lRevenue := (qryTN['RevFlag'] = 1);
        PrepareTNdata;            // Bulk of tTransaction data (rest at end).

        // Test for bit-field value.
        if VarIsNull(qryTN['AdminFee']) or (qryTN['AdminFee'] = 0) then begin
          // This group of transactions are not charged an handling (admin) fee.
          if RecordLostFees then begin
            // No AdminFee to be charged - record a loss of AdminFee.
            // *****************************************************************
            // *****************************************************************
            // Define the correct ElementID.
            sListFields := sListFields + 'ElementID,';
            sListValues := sListValues + sLostAdminFeeRevElement + ',';
            sListFields := sListFields + 'StatusID,TransactionTypeID,';
            sListValues := sListValues + sInfoMessage + ',' + sFeeLostTransaction + ',';
            // Calculate the Lost fees.
            if lRevenue then begin
              if (qryTN['NonStdRevAdminFee'] = NULL) then begin
                lStdRateNotApplied := False;
                rAppliedRate := rAdminFeeRevRate;
              end
              else begin
                lStdRateNotApplied := True;
                rAppliedRate := qryTN.FieldByName('NonStdRevAdminFee').AsFloat;
              end; // Expenditure.
            end  // if lRevenue then.
            else begin  // Must be Expenditure.
              if (qryTN['NonStdExpAdminFee'] = NULL) then begin
                lStdRateNotApplied := False;
                rAppliedRate := rAdminFeeExpRate;
              end
              else begin
                lStdRateNotApplied := True;
                rAppliedRate := qryTN.FieldByName('NonStdExpAdminFee').AsFloat;
              end; // Expenditure.
            end;  // if lRevenue then..else.
            curExGST := curFormat(qryTN.FieldByName('TOTAL').AsCurrency *
              rAppliedRate);
            curGST := curFormat(curExGST / 10);
            curIncGST := curSum(curExGST, curGST);
            // NOTE LAST ENTRIES OMITS TRAILING COMMA !!
            sListFields := sListFields + 'AmountExGST,GSTAmount,AmountIncGST)';
            sListValues := sListValues + Curr2Str(curExGST) + ',';
            sListValues := sListValues + Curr2Str(curGST) + ',';
            sListValues := sListValues + Curr2Str(curIncGST) + ')';
            // Save the Lost Fee.
            AppendFeeRecord;
            sListFields := '';
            sListValues := '';
          end;  // RecordLostFees.
        end   // if (qryTN.FieldByName('AdminFee').AsInteger = 0) then.
        else begin
          // Yes, AdminFees are to be charged for this transaction group.
          // *****************************************************************
          // *****************************************************************
          // Define the correct ElementID.
          // Calculate the Lost fees.
          if lRevenue then begin
            sListFields := sListFields + 'ElementID,TransactionTypeID,';
            if (qryTN.FieldByName('CustomerID').AsString = sVRTID) then begin
              sListValues := sListValues + sAdminFeeVRTRevElement +',';
            end
            else begin
              sListValues := sListValues + sAdminFeeRevElement +',';
            end;
            sListValues := sListValues + sFeeRevenueTransaction +',';
            if (qryTN['NonStdRevAdminFee'] = NULL) then begin
              lStdRateNotApplied := False;
              rAppliedRate := rAdminFeeRevRate;
            end
            else begin
              lStdRateNotApplied := True;
              rAppliedRate := qryTN.FieldByName('NonStdRevAdminFee').AsFloat;
            end; // Expenditure.
          end  // if lRevenue then.
          else begin  // Must be Expenditure.
            sListFields := sListFields + 'ElementID,TransactionTypeID,';
            if (qryTN.FieldByName('CustomerID').AsString = sVRTID) then begin
              sListValues := sListValues + sAdminFeeVRTExpElement +',';
            end
            else begin
              sListValues := sListValues + sAdminFeeExpElement +',';
            end;
            sListValues := sListValues + sFeeExpenditureTransaction +',';
            if (qryTN['NonStdExpAdminFee'] = NULL) then begin
              lStdRateNotApplied := False;
              rAppliedRate := rAdminFeeExpRate;
            end
            else begin
              lStdRateNotApplied := True;
              rAppliedRate := qryTN.FieldByName('NonStdExpAdminFee').AsFloat;
            end; // Expenditure.
          end;  // if lRevenue then..else.
          curExGST := curFormat(qryTN.FieldByName('TOTAL').AsCurrency *
            rAppliedRate);
          curGST := curFormat(curExGST / 10);
          curIncGST := curSum(curExGST, curGST);
          // NOTE LAST ENTRIES OMITS TRAILING COMMA !!
          sListFields := sListFields + 'AmountExGST,GSTAmount,AmountIncGST)';
          sListValues := sListValues + Curr2Str(curExGST) + ',';
          sListValues := sListValues + Curr2Str(curGST) + ',';
          sListValues := sListValues + Curr2Str(curIncGST) + ')';
          // Save the AdminFee.
          AppendFeeRecord;
          sListFields := '';
          sListValues := '';

          if lStdRateNotApplied and RecordLostFees then begin
            // record the reduction in fees retrieved.
            sListFields := sListFields + 'ElementID,';
            sListValues := sListValues + sLostAdminFeeRevElement + ',';
            sListFields := sListFields + 'StatusID,TransactionTypeID,';
            sListValues := sListValues + sInfoMessage + ','+sFeeLostTransaction+',';
            // Adjust the applied rate accordingly.
            if lRevenue then begin
              rAppliedRate := rAdminFeeRevRate - rAppliedRate;
            end
            else begin
              rAppliedRate := rAdminFeeExpRate - rAppliedRate;
            end;
            curExGST := curFormat(qryTN.FieldByName('TOTAL').AsCurrency *
              rAppliedRate);
            curGST := curFormat(curExGST / 10);
            curIncGST := curSum(curExGST, curGST);
            // NOTE LAST ENTRIES OMITS TRAILING COMMA !!
            sListFields := sListFields + 'AmountExGST,GSTAmount,AmountIncGST)';
            sListValues := sListValues + Curr2Str(curExGST) + ',';
            sListValues := sListValues + Curr2Str(curGST) + ',';
            sListValues := sListValues + Curr2Str(curIncGST) + ')';
            // Save the AdminFee.
            AppendFeeRecord;
            sListFields := '';
            sListValues := '';
          end;  // if lStdRateNotApplied then.
        end;  // (qryTN['AdminFee'] = 0) then..else.

        if not lDiscountedServices then begin
          lDiscountedServices := (qryTN['DiscountRate'] <> NULL);
        end;
        qryTN.Next;

        Inc(nCount);
        Inc(nTotalCount);
        Inc(nBlock);
        if (nBlock=10) then begin
          frmProgress.ProgressBar1.Position := Trunc(nCount * Factor);
          nBlock := 0;
          if ((nCount * Factor)>=100.00) then begin
            // restart the progress bar.
            nCount := 0;
          end;  // if ((nCount * Factor)>100.00) then.
          frmProgress.ProgressBar1.Position := Trunc(nCount * Factor);
          frmProgress.Msg.Caption := 'Processing (transaction: '+
            IntToStr(nTotalCount)+')...';
        end;  // if (nBlock=10) then.
        frmProgress.ProcessWinMessages(nil);
      end;  // while not qryTN.Eof do.
      frmProgress.Hide;
      frmProgress.ProcessWinMessages(nil);
      // Now test if Discounts are to be applied.
      if RecordLostDiscount and lDiscountedServices then begin
        ApplyDiscounts(sDiscountTransaction, sFeeLostDiscountTransaction);
      end;  // if lDiscountedServices then.
      frmProgress.Close;                  // All done, remove progressbar.
      frmProgress.Free;
      qryTT.Close;
      qryTT.Free;
    end;  // if qryTN.Eof and qry.Bof then..else.
  end;  // AdminFeesDue.
  lSuccessful := True;
  UpdateBatchStatus(nBatch, MainForm.BATCH_STATUS_ADMINRVW);
  qryTN.Close;
  qryTN.Free;
  qryStringList.Free;
  ValueStringList.Free;
end;  // procedure SaveAllAdminFees().

end.

