unit CodeFile;

interface

uses
  classes, Dialogs, db, dbTables, FileCtrl, SysUtils;

procedure CodeToFile(const TransactionTable: string; const nBatch: integer);

implementation

uses
  GenFns, Main, MyMessage, Progress;

var
  MainForm : TMainForm;

  
procedure CodeToFile(const TransactionTable: string; const nBatch: integer);
// This procedure creates a CSV file to be imported directly into the
// Web billing program for later distribution to Victrack customers.
var
  SaveDialog : TSaveDialog;
  lContinue : Boolean;
  SundryTextFile : TextFile;
  sTBSdirectory : string;
  Progress : TProgressForm;
  nFactor : Real;
  sFile : String;
  sBatchID : String;
  nline : Integer;
  sShipTo : String;
  sAccountNo : string;
  sQryString : string;
  qryString : TStringList;
  qCode : TQuery;
  qry : TQuery;
  MessagesForm : TMessagesForm;
begin
  sBatchID := IntToStr(nBatch);
  qryString := TStringList.Create;
  qryString.Add('SELECT TN1.RentAmt, TN1.CallAmt, TN1.OtherAmt, '+
    '(TN1.RentAmt + TN1.CallAmt + TN1.OtherAmt) AS "TotalAll", tStatus.Status, '+
    'tCustomer.ShipTo,');
  qryString.Add(' tPerson.Person, tLocation.Description AS "Location",');
  qryString.Add(' tCentre.Description AS "Centre", tActivity.Description AS "Activity", '+
    'tElement.ElementCode AS "Element", tSubledger.Description AS "Subledger",');
  qryString.Add(' tDepartment.Description AS "Department", '+
    'tServiceID.ServiceID AS "ServiceName", tServiceID.Description AS "ServiceDesc",');
  qryString.Add(' tServiceID.InstallationDate, tServiceType.Code');
  qryString.Add('FROM ((((((((((SELECT TXN.ServiceID, TXN.ServiceTypeID, '+
    'TXN.CustomerID, TXN.StatusID, TXN.PersonID,');
  qryString.Add(' TXN.LocationID, TXN.CustomerCentreID, '+
    'TXN.CustomerActivityID, TXN.CustomerElementID,');
  qryString.Add(' TXN.CustomerSubledgerID,');
  qryString.Add(' Sum(CASE tTransactionGroup.Description WHEN ''RENT'' THEN '+
    'TXN.AmountExGST ELSE 0 END) AS "RentAmt",');
  qryString.Add(' Sum(CASE tTransactionGroup.Description WHEN ''CALL'' THEN '+
    'TXN.AmountExGST ELSE 0 END) AS "CallAmt",');
  qryString.Add(' Sum(CASE tTransactionGroup.Description '+
    'WHEN ''OTHER'' THEN TXN.AmountExGST '+
    'WHEN ''ADMIN'' THEN TXN.AmountExGST ELSE 0 END) AS "OtherAmt"');
//  Sum(CASE tTransactionGroup.Description WHEN 'ADMIN' THEN
// tFinalisedTransaction.AmountExGST ELSE 0 END) AS AdminAmt
  qryString.Add('FROM ('+TransactionTable+' TXN');
  qryString.Add('LEFT JOIN tTransactionType '+
    'ON TXN.TransactionTypeID = tTransactionType.ID)');
  qryString.Add('LEFT JOIN tTransactionGroup '+
    'ON tTransactionType.TransactionGroupID = tTransactionGroup.ID');
  qryString.Add('WHERE TXN.BatchID = '+ sBatchID);
  qryString.Add('AND TXN.RevenueFlag = 1');
  qryString.Add('GROUP BY TXN.ServiceID, TXN.ServiceTypeID, '+
    'TXN.CustomerID, TXN.StatusID, TXN.PersonID,');
  qryString.Add(' TXN.LocationID, TXN.CustomerCentreID, '+
    'TXN.CustomerActivityID, TXN.CustomerElementID,');
  qryString.Add(' TXN.CustomerSubledgerID) AS TN1');
  qryString.Add('LEFT JOIN tStatus ON TN1.StatusID = tStatus.ID)');
  qryString.Add('LEFT JOIN tCustomer ON TN1.CustomerID = tCustomer.ID)');
  qryString.Add('LEFT JOIN tPerson ON TN1.PersonID = tPerson.ID)');
  qryString.Add('LEFT JOIN tLocation ON TN1.LocationID = tLocation.ID)');
  qryString.Add('LEFT JOIN (tCentre LEFT JOIN tDepartment '+
    'ON tCentre.DepartmentID = tDepartment.ID) '+
    'ON TN1.CustomerCentreID = tCentre.ID)');
  qryString.Add('LEFT JOIN tActivity ON TN1.CustomerActivityID = tActivity.ID)');
  qryString.Add('LEFT JOIN tElement ON TN1.CustomerElementID = tElement.ID)');
  qryString.Add('LEFT JOIN tSubledger ON TN1.CustomerSubledgerID = tSubledger.ID)');
  qryString.Add('LEFT JOIN tServiceID ON TN1.ServiceID = tServiceID.ID)');
  qryString.Add('LEFT JOIN tServiceType ON TN1.ServiceTypeID = tServiceType.ID');
  qryString.Add('WHERE tStatus.Status IS NULL');
  qryString.Add('OR tStatus.Status NOT LIKE ''%Information%''');
  qryString.Add('ORDER BY tServiceID.ServiceID');

  if (qryString.Count > 0) then begin
    // First, get the account number.
    qry := TQuery.Create(MainForm);
    with qry do begin
      DatabaseName := 'dbPPdata';
      SQL.Add('SELECT tValidAccountNumber.AccountNo');
      SQL.Add('FROM tBatch');
      SQL.Add('LEFT JOIN tValidAccountNumber '+
        'ON tBatch.SupplierAccountID = tValidAccountNumber.ID');
      SQL.Add('WHERE tBatch.ID = '+ sBatchID);
      Open;
      if (Eof) or VarIsNull(FieldValues['AccountNo']) then sAccountNo := ''
      else sAccountNo := VarToStr(FieldValues['AccountNo']);
      Close;
      SQL.Clear;
    end;  // qry.
    MessagesForm := TMessagesForm.Create(nil);
    MessagesForm.Caption := 'Exporting Web information';
    MessagesForm.Msg.Caption := 'Please wait - searching for data';
    MessagesForm.Show;
    MainForm.ProcessWinMessages;
    qCode := TQuery.Create(MainForm);
    qCode.DatabaseName := 'dbPPdata';
    qCode.SQL := qryString;
    qCode.Open;
    if (qCode.Bof and qCode.Eof) then begin
      MessagesForm.Hide;
      MessageDlg('No entries for BatchID '+sBatchID+' found in '+TransactionTable,
        mtWarning, [mbOk], 0);
    end
    else begin
      try
        MessagesForm.Msg.Caption := 'Saving data to file';
        MainForm.ProcessWinMessages;
        // Determine the parameters for the progress bar.
        with qry do begin
          SQL.Add('SELECT ParmText FROM tParameter');
          SQL.Add('WHERE Parm LIKE ''TBSdirectory''');
          Open;
          sTBSdirectory  := VarToStr(FieldValues['ParmText']);
          Close;
          lContinue := (Length(sTBSdirectory)>0) and DirectoryExists(sTBSdirectory);
          SQL.Clear;
          SQL.Add('SELECT COUNT(*) AS TotalAll');
          SQL.Add('FROM tTransaction');
          SQL.Add('WHERE tTransaction.BatchID = '+sBatchID);
          SQL.Add('AND RevenueFlag = 1');
          Open;
          nline := FieldValues['TotalAll'];
          Close;
        end;  // qry.
        nFactor := 100.0 / nline;
        // Reset the record counter.
        nline := 0;

        if lContinue then begin
          sFile := sTBSdirectory + '\' + sBatchID + '.cde';
        end
        else begin
          // Ask the operator if the file is to be saved to the floppy drive.
          SaveDialog := TSaveDialog.Create(nil);
          SaveDialog.Title := 'Save Web Billing Details';
          SaveDialog.FileName := sBatchID;
          SaveDialog.Filter := 'Batch Files (*.cde)|*.cde|All Files (*.*)|*.*';
          SaveDialog.FilterIndex := 1;
          SaveDialog.DefaultExt := 'cde';
          // Use the program directory as the default directory.
          SaveDialog.InitialDir := ExtractFilePath(ParamStr(0));
          SaveDialog.Options := [ofHideReadOnly,ofNoReadOnlyReturn];
          lContinue := SaveDialog.Execute;
          sFile := SaveDialog.FileName;
          SaveDialog.Free;
{
          sTBSdirectory := ExtractFileDir(sFile);
          if lContinue and DirectoryExists(sTBSdirectory) then begin
            // Save the directory name for the TBS file.
            with qry do begin
              Close;
              SQL.Clear;
              SQL.Add('SELECT ParmText FROM tParameter');
              SQL.Add('WHERE Parm LIKE ''TBSdirectory''');
              Open;
              if Bof and Eof then begin
                // Add to the table.
                Close;
                SQL.Clear;
                SQL.Add('INSERT INTO tParameter');
                SQL.Add('(Parm,ParmField,ParmText,ParmDescription,' +
                  'ParmUserDefined)');
                SQL.Add('VALUES (''TBSdirectory'',''ParmText'','''+sTBSdirectory +
                  ''',''The default directory where the TBS reports are saved.'',1)');
                ExecSQL;
              end
              else begin
                // Update the field value.
                Close;
                SQL.Clear;
                SQL.Add('UPDATE tParameter');
                SQL.Add('SET ParmText = '''+ sTBSdirectory +'''');
                SQL.Add('WHERE Parm LIKE ''TBSdirectory''');
                ExecSQL;
              end;
            end;  // qry.
          end;
}
        end;

        if lContinue then begin
          // Open the progress form.
          Progress := TProgressForm.Create(NIL);
          // Disable the Abort button - Printer not active.
          Progress.btnCancel.Enabled := False;
          Progress.btnCancel.Visible := False;
          // Display a message to the user.
          Progress.Msg.Caption := 'Saving Web Billing data for ' + sBatchID +
            ' to file ...';
          Progress.Show;

          qryString := TStringList.Create;
          // Open the required file.
          AssignFile( SundryTextFile, sFile );
          try
            Rewrite( SundryTextFile );
            // OK, now output the data to the file.
            // Start at the beginning.
            with qCode do begin
              First;
              while not Eof do begin
                Progress.ProcessWinMessages(MainForm); // Don't lock the computer.
                // Develop the line of information.
                // Start with the Division details - ShipToCode is preferred.
                sShipTo := Trim(VarToStr(FieldValues['ShipTo']));
                // Save the data.
                sQryString := sShipTo + ',' + sBatchID + ',';
                sQryString := sQryString +
                  FormatCurr('######0.00',FieldByName('RentAmt').AsCurrency)+
                  ','+FormatCurr('######0.00',FieldByName('CallAmt').AsCurrency)+
                  ','+FormatCurr('######0.00',FieldByName('OtherAmt').AsCurrency)+
                  ','+FormatCurr('######0.00',FieldByName('TotalAll').AsCurrency)+
                  ','+VarToStr(FieldValues['Code']);
                sQryString := sQryString +
                  ','+VarToStr(FieldValues['ServiceName']) +
                  ','+VarToStr(FieldValues['Location']) +
                  ','+VarToStr(FieldValues['Person']);
                sQryString := sQryString+','+VarToStr(FieldValues['Centre']);
                if not VarIsNull(FieldValues['Activity']) then
                  sQryString := sQryString+' '+VarToStr(FieldValues['Activity']);
                if not VarIsNull(FieldValues['Element']) then
                  sQryString := sQryString+' '+VarToStr(FieldValues['Element']);
                if not VarIsNull(FieldValues['Subledger']) then
                  sQryString := sQryString+' '+VarToStr(FieldValues['Subledger']);
                sQryString := sQryString +
                  ','+VarToStr(FieldValues['Department']) +
                  ','+sAccountNo +
                  ','+FormatDateTime('dd/mm/yyyy',
                  FieldByName('InstallationDate').AsDateTime) + ',';

                // Only allow blank status for cable services - else, write "Std".
                if VarIsNull(FieldValues['Status']) then begin
                  // Not a cable service, and status is empty.
                  sQryString := sQryString + 'Std';
                end
                else begin
                  sQryString := sQryString + qCode.FieldByName('Status').AsString;
                end;
                sQryString := sQryString+',';

                // Output the information to the file.
                Writeln( SundryTextFile, sQryString );
                // Report progress to the user.
                Inc(nLine);
                Progress.ProgressBar1.Position := Trunc(nline * nFactor);
                // Go to the next line in the result set.
                Next;
              end;    // while (not qCode.Eof) and.
            end;  // qCode.
          finally
            // Done, now close the output file.
            CloseFile( SundryTextFile );
            qryString.Free;
            Progress.Close;
          end;    // try..finally.
        end;    // if lContinue then.
      finally
        qCode.Close;
        qCode.Free;
        qry.Close;
        qry.Free;
      end;  // try..finally.
      MessagesForm.Close;
    end;  // if (qCode.Bof and qCode.Eof) then begin..else.
  end;  // if (qryString.Count > 0) then begin.
end;    // procedure CodeToFile().

end.

