unit DateFunctions;

interface

uses
  SysUtils, dbtables, bde, classes;

  function Year( const dDate: TDateTime ): Word;
  function Month( const dDate: TDateTime ): Word;
  function Day( const dDate: TDateTime ): Word;
  function StandardDateString( const dDate: TDateTime ): string;
  function StandardDateTimeString( const dDate: TDateTime ): string;
  function FormattedDateString( const dDate: TDateTime ): string;
  function EmptyDate( const dDate: TDateTime ): Boolean;
  function DateOnly( const dDate: TDateTime ): TDateTime;
  function StrDMYtoDate( const strDate: string ) : TDatetime;
  function LastMonth( const dDate: TDateTime ) : TDatetime;
  function NextMonth( const dDate: TDateTime ) : TDatetime;
  function NextMultiMonth(
    const dDate: TDateTime;
    const nMonth: Integer ) : TDatetime;
  function FirstDayInMonth( const dDate: TDateTime ) : TDateTime;
  function LastDayInMonth( const dDate: TDateTime ) : TDateTime;
  function fMonthLoad(slMonth: TStringList): TStringList;
  function fShortMonthLoad(slMonth: TStringList): TStringList;
  function fMonth(nMonthno: Integer; slMonth: TStringList): String;
  function fMonthNumber(const sMonth:String;const slMonth:TStringList): String;

implementation

uses
//  Main,
  GenFns;
  
{ Some general functions first to simplify the transition from dBase to Delphi4.}
function Year( const dDate: TDateTime ): Word;
// This function returns the Calendar year of the parameter passed as a date.
var
  wYear, wMnth, wDay : Word;
begin
  DecodeDate( dDate, wYear, wMnth, wDay );
  Result := wYear;
end;    // function Year().


function Month( const dDate: TDateTime ): Word;
// This function returns the Calendar month of the parameter passed as a date.
var
  wYear, wMnth, wDay : Word;
begin
  DecodeDate( dDate, wYear, wMnth, wDay );
  Result := wMnth;
end;    // function Month().


function Day( const dDate: TDateTime ): Word;
// This function returns the Calendar day of the parameter passed as a date.
var
  wYear, wMnth, wDay : Word;
begin
  DecodeDate( dDate, wYear, wMnth, wDay );
  Result := wDay;
end;    // function Day().

function FormattedDateString( const dDate: TDateTime ): string;
// This function returns a date as a string formatted: dd/mm/yyyy.
var
  wYear, wMnth, wDay : Word;
begin
  DecodeDate( dDate, wYear, wMnth, wDay );

  Result := Right('0'+IntToStr(wDay),2) + '/' +
            Right('0'+IntToStr(wMnth),2) + '/' +
            IntToStr(wYear);
end;    // function FormattedDateString().


function StandardDateString( const dDate: TDateTime ): string;
// This function returns a date as a string formatted: yyyymmdd.
var
  wYear, wMnth, wDay : Word;
begin
  DecodeDate( dDate, wYear, wMnth, wDay );
  Result := IntToStr(wYear) +
            Right('0'+IntToStr(wMnth),2) +
            Right('0'+IntToStr(wDay),2);
end;    // function StandardDateString().


function StandardDateTimeString( const dDate: TDateTime ): string;
// This function returns a datetime as a string formatted: yyyymmdd hh:mm:ss.00
var
  wYear, wMnth, wDay : Word;
  sResultSet: string;
begin
  DecodeDate( dDate, wYear, wMnth, wDay );
  sResultSet := IntToStr(wYear) +
                Right('0'+IntToStr(wMnth),2) +
                Right('0'+IntToStr(wDay),2) + ' ' + TimeToStr( dDate );
  Result := sResultSet;
end;    // function StandardDateTimeString().


function EmptyDate( const dDate: TDateTime ): Boolean;
// This function returns true if the date passed is not a date.
var
  wYear, wMnth, wDay : Word;
begin
  if (dDate=null) or (dDate=0) then Result := True
  else begin
    DecodeDate( dDate, wYear, wMnth, wDay );
    Result := ((wDay=0) or (wMnth=0) or (wYear=0));
  end;
end;    // function EmptyDate().


function DateOnly( const dDate: TDateTime ): TDateTime;
// Returns the date only, without the time.
var
  Year  : Word;
  Month : Word;
  Day   : Word;
begin
  DecodeDate( dDate, Year, Month, Day );
  Result := EncodeDate( Year, Month, Day );
end;    // function dateOnly().


function StrDMYtoDate( const strDate: string ) : TDatetime;
// Converts a date string 'dd/mm/yyyy' to a date regardless of System settings.
var
  Year  : Word;
  Month : Word;
  Day   : Word;
  strInputDate : String;
  nDateLen : Integer;
begin
  strInputDate := Trim( strDate );
  nDateLen := Length( strInputDate );
  if ( nDateLen = 10 ) then begin
    // Century included in the date.
    Day  := StrToInt( Copy( strInputDate, 1, 2 ) );
    Month:= StrToInt( Copy( strInputDate, 4, 2 ) );
    Year := StrToInt( Copy( strInputDate, 7, 4 ) );
    Result := EncodeDate( Year, Month, Day );
  end
  else if ( nDateLen = 8 ) then begin
    // Century not included in the date.
    Day  := StrToInt( Copy( strInputDate, 1, 2 ) );
    Month:= StrToInt( Copy( strInputDate, 4, 2 ) );
    Year := StrToInt( Copy( strInputDate, 7, 2 ) );
    Result := EncodeDate( Year, Month, Day );
  end
  else begin
    Result := Null;
  end;
end;    // function StrDMYtoDate().


function LastMonth( const dDate: TDateTime ) : TDatetime;
// Returns a date one month earlier than the date parameter.
var
  Day       : Word;
  Month     : Word;
  Year      : Word;
  NewDate   : TDateTime;
begin
  DecodeDate( dDate, Year, Month, Day );
  Dec( Month );
  if ( Month < 1 ) then begin
    Month := 12;
    Dec( Year );
  end;

  // An error happens if the day for this month is invalid.  Then, select
  // the last day of the month.
  try
    NewDate := EncodeDate( Year, Month, Day );
  except
    on EConvertError do
      begin
      Inc( Month );
      if ( Month > 12 ) then begin
        Month := Month - 12;
        Inc( Year );
      end;    // if ( Month > 12 ) then.
      Day := 1;
      NewDate := EncodeDate( Year, Month, Day );
      NewDate := NewDate - 1;
    end;    // on EConvertError do.
  end;    // try..except.
  Result := NewDate;
end;    // function LastMonth().


function NextMonth( const dDate: TDateTime ) : TDatetime;
// Returns a date one month later than the date parameter.
var
  Day       : Word;
  Month     : Word;
  Year      : Word;
  NewDate   : TDateTime;
begin
  DecodeDate( dDate, Year, Month, Day );
  Inc( Month );
  if ( Month > 12 ) then begin
    Month := 1;
    Inc( Year );
  end;

  // An error happens if the day for this month is invalid.  Then, select
  // the last day of the month.
  try
    NewDate := EncodeDate( Year, Month, Day );
  except
    on EConvertError do
      begin
      Inc( Month );
      if ( Month > 12 ) then begin
        Month := Month - 12;
        Inc( Year );
      end;    // if ( Month > 12 ) then.
      Day := 1;
      NewDate := EncodeDate( Year, Month, Day );
      NewDate := NewDate - 1;
    end;    // on EConvertError do.
  end;    // try..except.
  Result := NewDate;
end;    // function NextMonth().


function NextMultiMonth(
  const dDate: TDateTime;
  const nMonth: Integer ) : TDatetime;
// Returns a date one month later than the date parameter.
var
  Day       : Word;
  Month     : Word;
  Year      : Word;
  NewDate   : TDateTime;
begin
  DecodeDate( dDate, Year, Month, Day );
  Inc( Month, nMonth );
  if ( Month > 12 ) then begin
    Month := Month - 12;
    Inc( Year );
  end;    // if ( Month > 12 ) then.

  // An error happens if the day for this month is invalid.  Then, select
  // the last day of the month.
  try
    NewDate := EncodeDate( Year, Month, Day );
  except
    on EConvertError do begin
      Inc( Month );
      if ( Month > 12 ) then begin
        Month := 1;
        Inc( Year );
      end;    // if ( Month > 12 ) then.
      Day := 1;
      NewDate := EncodeDate( Year, Month, Day );
      NewDate := NewDate - 1;
    end;    // on EConvertError do.
  end;    // try..except.
  Result := NewDate;
end;    // function NextMultiMonth().


function FirstDayInMonth( const dDate: TDateTime ) : TDateTime;
var
  Year  : Word;
  Month : Word;
  Day   : Word;
begin
  DecodeDate( dDate, Year, Month, Day );
  Day := 1;
  Result := EncodeDate( Year, Month, Day );
end;    // function FirstDayInMonth().


function LastDayInMonth( const dDate: TDateTime ) : TDateTime;
var
  Year  : Word;
  Month : Word;
  Day   : Word;
  FirstDayinNextMonth : TDateTime;
begin
  DecodeDate( dDate, Year, Month, Day );
  if ( Month = 12 ) then begin
    Inc( Year );
    Month := 0;
  end;
  Inc( Month );
  Day := 1;
  FirstDayinNextMonth := EncodeDate( Year, Month, Day );

  // Want the previous day, the last day of the chosen month.
  Result := FirstDayinNextMonth - 1;
end;    // function LastDayInMonth().


function fMonthload(slMonth: TStringList): TStringList;
// Load month names into sMonth, in the order they are used in a
// Financial year.
begin
  // Empty the string list.
  slMonth.Clear;
  // Add the Month in order.
  slMonth.Add('July');
  slMonth.Add('August');
  slMonth.Add('September');
  slMonth.Add('October');
  slMonth.Add('November');
  slMonth.Add('December');
  slMonth.Add('January');
  slMonth.Add('February');
  slMonth.Add('March');
  slMonth.Add('April');
  slMonth.Add('May');
  slMonth.Add('June');
  Result := slMonth;
end;    // function fMonthload().


function fShortMonthLoad(slMonth: TStringList): TStringList;
// Load month names into sMonth, in the order they are used in a
// Financial year.
begin
  // Empty the string list.
  slMonth.Clear;
  // Add the Month in order.
  slMonth.Add('JUL');
  slMonth.Add('AUG');
  slMonth.Add('SEP');
  slMonth.Add('OCT');
  slMonth.Add('NOV');
  slMonth.Add('DEC');
  slMonth.Add('JAN');
  slMonth.Add('FEB');
  slMonth.Add('MAR');
  slMonth.Add('APR');
  slMonth.Add('MAY');
  slMonth.Add('JUN');
  Result := slMonth;
end;    // function fShortMonthLoad().


function fMonth(nMonthno: Integer; slMonth: TStringList): String;
// Retrieve the month name from the stringlist slMonth,
// given the number of the month during a calendar year - Jul = 7 etc.
var
  nMonth : Integer;
begin
  nMonth := 0;
  if nMonthno <= 12 then begin
    // Determine the array position - July(month 7)=1, August(month 8)=2, etc.
    if nMonthno < 7 then begin
      nMonth := nMonthno + 6 - 1;      // Offset=1 (0,1,2...).
    end
    else begin
      nMonth := nMonthno - 6 - 1;
    end;
  end;
  // Read the name of the month.
  Result := slMonth.Strings[nMonth];
end;    // function fMonth().

function fMonthNumber(const sMonth:String;const slMonth:TStringList): String;
// Retrieve the month number from the stringlist slMonth,
// given the number of the month during a calendar year - Jul = 7 etc.
// Passed array is FY - July, August, September, etc.
var
  nMonth : Integer;
begin
  nMonth := slMonth.IndexOf(sMonth);
  if nMonth < 0 then
    Result := '00'
  else begin
    // Determine the number of Month - July=7, January=1, etc.
    if nMonth < 6 then begin
      nMonth := nMonth + 7;
    end
    else begin
      nMonth := nMonth - 5;
    end;
    Result := Right(('0'+IntToStr(nMonth)),2); // Adds leading zero if < 10.
  end;
  // Result set above given the name of the month.
end;    // function fMonth().

end.
