unit DubiousTests;

interface

uses SysUtils, dbtables, bde, classes;

  procedure UpdateDubiousTestList(
    const nThisBatch, nThisDubiousType, nThisService,
    nThisCustomer, nThisSupplierServiceCodeID: LongInt);
  // Test 1: For rental variations for the same service type.
  procedure DubiousTest_1(const nThisBatch: LongInt;var lSuccessful: Boolean);
  // Test 2: For Services with Calls less than Rent.
  procedure DubiousTest_2(Const nThisBatch: LongInt;var lSuccessful: Boolean);
  // Test 3: For compliance with agreed Tariff & Rates.
  procedure DubiousTest_3(Const nThisBatch: LongInt;var lSuccessful: Boolean);
  // Test 4: For Services with a Change in Activity.
  procedure DubiousTest_4(Const nThisBatch: LongInt;var lSuccessful: Boolean);
  // Test 5: Test and set results within a Range of Values.
  procedure DubiousTest_5(Const nThisBatch: LongInt;var lSuccessful: Boolean);
  procedure DubiousErrorTest(
    const nThisBatch: LongInt;
    var lSuccessful: Boolean);

implementation

uses
  Main, DataMod, Progress, DateFunctions, GenFns, MyMessage;

var
  qTN : TQuery;                   // tDataCollection.
  qStringList : TStringList;


procedure UpdateDubiousTestList(
  const nThisBatch, nThisDubiousType, nThisService,
  nThisCustomer, nThisSupplierServiceCodeID: LongInt);
var
  sBatch : string;
  sService : string;
  sCustomer : string;
  sDubiousType : string;
  lCustomerOk : boolean;
  sSupplierCode : string;
  lSupplierCodeOk : boolean;
begin
  sBatch := IntToStr(nThisBatch);
  sDubiousType := IntToStr(nThisDubiousType);
  sService := IntToStr(nThisService);
  lCustomerOk := (nThisCustomer > 0);
  if lCustomerOk then
    sCustomer := IntToStr(nThisCustomer)
  else
    sCustomer := 'NULL';
  lSupplierCodeOk := (nThisSupplierServiceCodeID > 0);
  if lCustomerOk then
    sSupplierCode := IntToStr(nThisSupplierServiceCodeID)
  else
    sSupplierCode := 'NULL';
  with qStringList do begin
    Clear;
    Add('SELECT * FROM tDubiousResult');
    Add('WHERE BatchID = '+ sBatch);
    Add('AND DubiousTypeID = '+sDubiousType);
    Add('AND ServiceID = '+sService);
    if lCustomerOk then
      Add('AND CustomerID = '+sCustomer)
    else
      Add('AND CustomerID IS NULL');
    if lSupplierCodeOk then
      Add('AND SupplierServiceCodeID = '+sSupplierCode)
    else
      Add('AND SupplierServiceCodeID IS NULL');
  end;
  qTN.SQL := qStringList;
  qTN.Open;
  if qTN.Eof and qTN.Bof then begin
    // No data returned - must add it.
    with qStringList do begin
      Clear;
      Add('INSERT INTO tDubiousResult');
      Add('(BatchID, ServiceID, CustomerID, SupplierServiceCodeID, '+
        'DubiousTypeID, UserID)');
      Add('VALUES ( '+sBatch +
        ', '+sService+', ' + sCustomer + ', ' + sSupplierCode +
        ', '+sDubiousType+', ' +s_User_ID+ ' )');
    end;
    qTN.Close;
    qTN.SQL := qStringList;
    qTN.ExecSQL;
  end;  // if qryTN.Eof and qryTN.Bof then.
  qTN.Close;
end;  // UpdateDubiousTestList().


procedure DubiousTest_1(Const nThisBatch: LongInt;var lSuccessful: Boolean);
// Test for rental variations for the same service type.
var
  nBatch :LongInt;
  nDubious : LongInt;
  qryTN : TQuery;
  qryStringList : TStringList;
begin
  nBatch := nThisBatch;
  qryStringList := TStringList.Create;
  qryTN := TQuery.Create(nil);
  with qryTN do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('SELECT ID FROM tDubiousType');
    SQL.Add('WHERE Description LIKE ''Rental Variations%''');
    Open;
    if Eof then nDubious := 1
    else nDubious := FieldByName('id').AsInteger;
    Close;
  end;  // qryTN.
  with qryStringList do begin
    Clear;
    // The ROUND() function is used to compare whole cents only.
    Add('SELECT TN2.*');
    Add('FROM (SELECT tTransaction.ServiceID, '+
      'tTransaction.CustomerID, tTransaction.ServiceTypeID,');
    Add(' Sum(CASE tTransactionGroup.Description WHEN ''RENT'' '+
      'THEN tTransaction.AmountExGST ELSE 0 END) AS RENT');
    Add('FROM (tTransaction');
    Add(' LEFT JOIN tTransactionType '+
      'ON tTransaction.TransactionTypeID = tTransactionType.ID)');
    Add(' LEFT JOIN tTransactionGroup '+
      'ON tTransactionType.TransactionGroupID = tTransactionGroup.ID');
    Add('WHERE tTransaction.BatchID = '+IntToStr(nBatch));
    Add('AND tTransaction.RevenueFlag = 1');
    Add('GROUP BY tTransaction.ServiceID, '+
      'tTransaction.CustomerID, tTransaction.ServiceTypeID) AS TN2');
    Add(' LEFT JOIN tServiceType '+
      'ON TN2.ServiceTypeID = tServiceType.ID');
    Add('WHERE tServiceType.ExpectedRent IS NOT NULL');
    Add('AND ROUND(TN2.Rent, 2) <> '+
      'ROUND(tServiceType.ExpectedRent, 2)');
  end;  // qryStringList.
  with qryTN do begin
    SQL := qryStringList;
    Open;
    if Eof and Bof then begin
      // No data returned.
      lSuccessful := True;
    end
    else begin
      lSuccessful := False;
      // Record all service that failed the test.
      while not Eof do begin
        UpdateDubiousTestList(nBatch, nDubious,
          FieldByName('ServiceID').AsInteger,
          FieldByName('CustomerID').AsInteger,
          0); // SupplierServiceCodeID.
        Next;
      end;  // while not qryTN.Eof do begin.
    end;
    Close;
  end;  // qryTN.
  qryTN.Free;
  qryStringList.Free;
end;  // DubiousTest_1.

procedure DubiousTest_2(Const nThisBatch: LongInt;var lSuccessful: Boolean);
// Test for Services with Calls less than Rent.
var
  nBatch : LongInt;
  nDubious : LongInt;
  qryTN : TQuery;
  qryStringList : TStringList;
begin
  nBatch := nThisBatch;
  qryStringList := TStringList.Create;
  qryTN := TQuery.Create(nil);
  with qryTN do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('SELECT ID FROM tDubiousType');
    SQL.Add('WHERE Description LIKE ''Calls less than%''');
    Open;
    if   Eof then nDubious := 2
    else nDubious := FieldByName('id').AsInteger;
    Close;
  end;  // qryTN.
  with qryStringList do begin
    Clear;
    Add('SELECT TN2.*');
    Add('FROM ((SELECT tTransaction.ServiceID, ' +
      'tTransaction.CustomerID, tTransaction.ServiceTypeID,');
    Add(' Sum(CASE tTransactionGroup.Description WHEN ''RENT'' '+
      'THEN tTransaction.AmountIncGST ELSE 0 END) AS RENT,');
    Add(' Sum(CASE tTransactionGroup.Description WHEN ''CALL'' '+
      'THEN tTransaction.AmountIncGST ELSE 0 END) AS CALL');
    Add('FROM (tTransaction');
    Add(' LEFT JOIN tTransactionType '+
      'ON tTransaction.TransactionTypeID = tTransactionType.ID)');
    Add(' LEFT JOIN tTransactionGroup '+
      'ON tTransactionType.TransactionGroupID = tTransactionGroup.ID');
    Add('WHERE tTransaction.BatchID = '+IntToStr(nBatch));
    Add('AND tTransaction.RevenueFlag = 1');
    Add('GROUP BY tTransaction.ServiceID, '+
      'tTransaction.CustomerID, tTransaction.ServiceTypeID) AS TN2');
    Add(' LEFT JOIN tServiceID ON TN2.ServiceID = tServiceID.ID)');
    Add(' LEFT JOIN tServiceType '+
      'ON TN2.ServiceTypeID = tServiceType.ID');
    Add('WHERE TN2.Call <> 0');
    Add('AND TN2.Call < TN2.Rent');
    Add('AND tServiceType.NoCalls = 0');
  end;  // qryStringList.
  with qryTN do begin
    SQL := qryStringList;
    Open;
    if Eof and Bof then begin
      // No data returned.
      lSuccessful := True;
    end
    else begin
      lSuccessful := False;
      // Record all service that failed the test.
      while not Eof do begin
        UpdateDubiousTestList(nBatch, nDubious,
          FieldByName('ServiceID').AsInteger,
          FieldByName('CustomerID').AsInteger,
          0); // SupplierServiceCodeID.
        Next;
      end;  // while not qryTN.Eof do begin.
    end;
    Close;
  end;  // qryTN.
  qryTN.Free;
  qryStringList.Free;
end;  // DubiousTest_2.

procedure DubiousTest_3(Const nThisBatch: LongInt;var lSuccessful: Boolean);
// Test for compliance with agreed Tariff & Rates.
var
  nBatch : LongInt;
  sBatch : string;
  sBatchTypeID : string;
  nValue : integer;
  sVariance : string;
  nDubious : LongInt;
  qryTN : TQuery;
  qryStringList : TStringList;
begin
  nBatch := nThisBatch;
  sBatch := IntToStr(nBatch);
  qryStringList := TStringList.Create;
  qryTN := TQuery.Create(nil);
  with qryTN do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('SELECT ID FROM tDubiousType');
    SQL.Add('WHERE Description LIKE ''Tariff%''');
    Open;
    if Eof then nDubious := 3
    else nDubious := FieldByName('id').AsInteger;
    Close;
    SQL.Clear;
    SQL.Add('SELECT BatchTypeID');
    SQL.Add('FROM tBatch');
    SQL.Add('WHERE ID = '+sBatch);
    Open;
    if Eof then sBatchTypeID := '0'
    else sBatchTypeID := IntToStr(FieldByName('BatchTypeID').AsInteger);
    Close;
    SQL.Clear;

    SQL.Add('SELECT PARMNUMBER AS "TariffVariance"');
    SQL.Add('FROM tParameter');
    SQL.Add('WHERE PARM LIKE ''TariffVariance''');
    Open;
    nValue := Round(FieldByName('TariffVariance').AsFloat);
    if Eof or (nValue < 0) or (nValue > 100) then nValue := 10; // Default=10%.
    Close;
    SQL.Clear;
    Str(((100.0 + nValue) / 100.0) : 7 : 3, sVariance);
  end;  // qryTN.
  with qryStringList do begin
    Clear;
//    Add('SELECT TN.*');      // For testing / evaluation only.
    Add('SELECT DISTINCT TN.ServiceID, TN.CustomerID, '+
      'TN.SupplierServiceCodeID');
    Add('FROM (SELECT TN3.ServiceID, TN3.CustomerID, '+
      'TN3.AmountExGST, TN3.SupplierServiceCodeID,');
    Add(' (CASE WHEN TN3.CalcAmtExGST < TN3.MinCharge '+
      'THEN TN3.MinCharge ELSE TN3.CalcAmtExGST END) AS "CalcAmtExGST"');
    Add('FROM (SELECT TN2.ServiceID, TN2.CustomerID, '+
      'TN2.AmountExGST, TN2.SupplierServiceCodeID,');
    Add(' ((TN2.NumCalls * TN2.FlagFall) + '+
      '(TN2.NumCalls * TN2.FixedCharge) +');
    Add(' (CEILING(TN2.Duration / ChargeInterval) * '+
      'TN2.InitialCharge)) AS "CalcAmtExGST",');
    Add(' TN2.MinCharge');
    Add('FROM (SELECT TN1.*,');
    Add(' (CASE WHEN tAgreementTariffs.FixedCharge IS NULL '+
      'THEN 0 ELSE tAgreementTariffs.FixedCharge END) AS "FixedCharge",');
    Add(' (CASE WHEN tAgreementTariffs.FlagFall IS NULL '+
      'THEN 0 ELSE tAgreementTariffs.FlagFall END) AS "FlagFall",');
    Add(' (CASE WHEN tAgreementTariffs.Stage1Charge IS NULL '+
      'THEN 0 ELSE tAgreementTariffs.Stage1Charge END) AS "InitialCharge",');
    Add(' (CASE WHEN tAgreementTariffs.ChargeIntervalSecs IS NULL THEN 1');
    Add('       WHEN tAgreementTariffs.ChargeIntervalSecs = 0 THEN 1');
    Add('       ELSE tAgreementTariffs.ChargeIntervalSecs END) '+
      'AS "ChargeInterval",');
    Add(' (CASE WHEN tAgreementTariffs.MinCharge IS NULL THEN 0');
    Add('       WHEN tAgreementTariffs.MinCharge < 0 THEN 0');
    Add('       ELSE tAgreementTariffs.MinCharge END) AS "MinCharge"');
    Add('FROM (SELECT ServiceID, CustomerID, Count(*) AS "NumCalls",');
    Add(' Sum(CAST(Duration AS int)) AS "Duration",');
    Add(' Sum(AmountExGST) AS "AmountExGST",');
    Add(' SupplierServiceCodeID');
    Add('FROM tTransaction');
    Add('WHERE BatchID = '+sBatch);
    Add(' AND Duration IS NOT NULL');
    Add('GROUP BY ServiceID, CustomerID, SupplierServiceCodeID) AS TN1');
    Add('LEFT JOIN tAgreementTariffs '+
      'ON TN1.SupplierServiceCodeID = tAgreementTariffs.SupplierServiceCodeID');
    Add('LEFT JOIN tAgreement '+
      'ON tAgreementTariffs.AgreementID = tAgreement.ID');
    Add('WHERE tAgreement.BatchTypeID = '+sBatchTypeID);
    Add('AND tAgreementTariffs.Active = 1');
    Add('AND tAgreement.Active = 1');
    Add('AND ((tAgreementTariffs.FixedCharge IS NOT NULL)');
    Add('  OR (tAgreementTariffs.FlagFall IS NOT NULL)');
    Add('  OR (tAgreementTariffs.Stage1Charge IS NOT NULL))) '+
      'AS TN2) AS TN3) AS TN');
    Add('WHERE TN.CalcAmtExGST > ('+sVariance+' * TN.AmountExGST)');
  end;
  with qryTN do begin
    SQL := qryStringList;
    Open;
    if Eof and Bof then begin
      // No data returned.
      lSuccessful := True;
    end  // if Eof and Bof then begin.
    else begin
      lSuccessful := False;
      // Mark all remaining records that failed.
      while not Eof do begin
        UpdateDubiousTestList(nBatch, nDubious,
          FieldByName('ServiceID').AsInteger,
          FieldByName('CustomerID').AsInteger,
          FieldByName('SupplierServiceCodeID').AsInteger);
        Next;
      end;  // while not qryTN.Eof do begin.
    end;  // if Eof and Bof then begin..else.
    Close;
  end;  // qryTN.
  qryTN.Free;
  qryStringList.Free;
end;  // DubiousTest_3.

procedure DubiousTest_4(Const nThisBatch: LongInt;var lSuccessful: Boolean);
// Test for Services with a Change in Activity.
var
  nBatch :LongInt;
  nDubious : LongInt;
  qryTN : TQuery;
  qryStringList : TStringList;
  sVariance : string;
begin
  nBatch := nThisBatch;
  qryStringList := TStringList.Create;
  qryTN := TQuery.Create(nil);
  with qryTN do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('SELECT ID FROM tDubiousType');
    SQL.Add('WHERE Description LIKE ''Activity%''');
    Open;
    if Eof then nDubious := 4
    else nDubious := FieldByName('id').AsInteger;
    Close;
    SQL.Clear;
    SQL.Add('SELECT ParmNumber FROM tParameter');
    SQL.Add('WHERE Parm LIKE ''ActivityVariancePerCent''');
    Open;
    if Eof then sVariance := '10'
    else sVariance := FloatToStr(FieldByName('ParmNumber').Value);
    Close;
  end;  // qryTN.
  with qryStringList do begin
    Clear;
    Add('SELECT SID.ServiceID, SID.CustomerID,');
    Add(' SID.CurrentRent, SID.PreviousRent, '+
      'SID.RentChange, SID.RentChangePerCent,');
    Add(' SID.CurrentCalls, SID.PreviousCalls, SID.CallChange, '+
      'CallChangePerCent');
    Add('FROM (SELECT TN.ServiceID, TN.CustomerID,'+
      ' tServiceID.CurrentRent, tServiceID.PreviousRent,');
    Add(' (tServiceID.CurrentRent - tServiceID.PreviousRent) '+
      'AS RentChange,');
    Add(' CASE');
    Add('  WHEN tServiceID.PreviousRent > 0 THEN '+
      '(tServiceID.CurrentRent - tServiceID.PreviousRent) / '+
      'tServiceID.PreviousRent * 100');
    Add('  WHEN tServiceID.PreviousRent < 0 THEN '+
      '(tServiceID.PreviousRent - tServiceID.CurrentRent) / '+
      'tServiceID.PreviousRent * 100');
    Add('  WHEN tServiceID.PreviousRent = 0 AND '+
      'tServiceID.CurrentRent > 0 THEN 100.00');
    Add('  WHEN tServiceID.PreviousRent = 0 AND '+
      'tServiceID.CurrentRent < 0 THEN -100.00');
    Add('  ELSE 0.00');
    Add(' END AS RentChangePerCent,');
    Add(' CASE');
    Add('  WHEN tServiceID.PreviousRent > 0 THEN '+
      'tServiceID.PreviousRent');
    Add('  ELSE tServiceID.PreviousRent * -1');
    Add(' END AS CalcRent,');
    Add(' tServiceID.CurrentCalls, tServiceID.PreviousCalls,');
    Add(' (tServiceID.CurrentCalls - tServiceID.PreviousCalls) '+
      'AS CallChange,');
    Add(' CASE');
    Add('  WHEN tServiceID.PreviousCalls > 0 THEN '+
      '(tServiceID.CurrentCalls - tServiceID.PreviousCalls) / '+
      'tServiceID.PreviousCalls * 100');
    Add('  WHEN tServiceID.PreviousCalls < 0 THEN '+
      '(tServiceID.PreviousCalls - tServiceID.CurrentCalls) / '+
      'tServiceID.PreviousCalls * 100');
    Add('  WHEN tServiceID.PreviousCalls = 0 AND '+
      'tServiceID.CurrentCalls > 0 THEN 100');
    Add('  WHEN tServiceID.PreviousCalls = 0 AND '+
      'tServiceID.CurrentCalls < 0 THEN -100');
    Add('  ELSE 0');
    Add(' END AS CallChangePerCent,');
    Add(' CASE');
    Add('  WHEN tServiceID.PreviousCalls > 0 '+
      'THEN tServiceID.PreviousCalls');
    Add('  ELSE tServiceID.PreviousCalls * -1');
    Add(' END AS CalcCalls');
    Add('FROM (SELECT DISTINCT tTransaction.ServiceID, '+
      'tTransaction.CustomerID');
    Add(' FROM tTransaction');
    Add(' WHERE tTransaction.BatchID = ' + IntToStr(nBatch));
    Add(' AND tTransaction.RevenueFlag <> 0) AS TN');
    Add('LEFT JOIN tServiceID ON TN.ServiceID = tServiceID.ID) '+
      'AS SID');
    Add('WHERE SID.RentChangePerCent > '+sVariance);
    Add('OR SID.RentChangePerCent < -'+sVariance);
    Add('OR SID.CallChangePerCent > '+sVariance);
    Add('OR SID.CallChangePerCent < -'+sVariance);
  end;
  with qryTN do begin
    SQL := qryStringList;
    Open;
    if Eof and Bof then begin
      // No data returned.
      lSuccessful := True;
    end
    else begin
      lSuccessful := False;
      // Mark all remaining records that failed.
      while not Eof do begin
        UpdateDubiousTestList(nBatch, nDubious,
          FieldByName('ServiceID').AsInteger,
          FieldByName('CustomerID').AsInteger,
          0); // SupplierServiceCodeID.
        Next;
      end;  // while not qryTN.Eof do begin.
    end;
    Close;
  end;  // qryTN.
  qryTN.Free;
  qryStringList.Free;
end;  // DubiousTest_4.

procedure DubiousTest_5(Const nThisBatch: LongInt;var lSuccessful: Boolean);
// Test and set results within a Range of Values.
var
  nBatch :LongInt;
  nDubious : LongInt;
  qryTN : TQuery;
  qryStringList : TStringList;
begin
  nBatch := nThisBatch;
  qryStringList := TStringList.Create;
  qryTN := TQuery.Create(nil);
  with qryTN do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('SELECT ID FROM tDubiousType');
    SQL.Add('WHERE Description LIKE ''Value Range%''');
    Open;
    if Eof then nDubious := 5
    else nDubious := FieldByName('id').AsInteger;
    Close;
  end;  // qryTN.
  with qryStringList do begin
    Clear;
    Add('SELECT tTransaction.ServiceID, tTransaction.CustomerID, '+
      'tTransaction.AmountIncGST');
    Add('FROM (tTransaction');
    Add('LEFT JOIN tServiceID '+
      'ON tTransaction.ServiceID = tServiceID.ID)');
    Add('LEFT JOIN tCostRange '+
      'ON tTransaction.TransactionTypeID = tCostRange.TransactionTypeID');
    Add('WHERE tTransaction.BatchID = '+ IntToStr(nBatch));
    Add('AND tTransaction.RevenueFlag <> 0');
    Add('AND tCostRange.ValidAccountNumberID = '+
      'tServiceID.SupplierAccountID');
    Add('AND tCostRange.CostRangeFrom IS NOT NULL');
    Add('AND tCostRange.CostRangeTo IS NOT NULL');
    Add('AND (tTransaction.AmountIncGST < tCostRange.CostRangeFrom');
    Add('OR tTransaction.AmountIncGST > tCostRange.CostRangeTo)');
  end;
  with qryTN do begin
    SQL := qryStringList;
    Open;
    if Eof and Bof then begin
      // No data returned.
      lSuccessful := True;
    end
    else begin
      lSuccessful := False;
      // Mark all remaining records that failed.
      while not Eof do begin
        UpdateDubiousTestList(nBatch, nDubious,
          FieldByName('ServiceID').AsInteger,
          FieldByName('CustomerID').AsInteger,
          0); // SupplierServiceCodeID.
        Next;
      end;  // while not Eof do begin.
    end;  // Eof & Bof.
    Close;
  end;  // qryTN.
  qryTN.Free;
  qryStringList.Free;
end;  // DubiousTest_5.


procedure DubiousErrorTest(Const nThisBatch: LongInt;var lSuccessful: Boolean);
var
  DubiousMsg : TMessagesForm;
  nBatch : LongInt;
  sBatch : string;
  qryTN : TQuery;                   // tDataCollection.
  qryBatchType : TQuery;
  qryStringList : TStringList;
  lResultIsGood : boolean;
begin
  DubiousMsg := TMessagesForm.Create(nil);
  DubiousMsg.Caption := 'Dubious Error Checks';
  DubiousMsg.Msg.Caption := 'Clearing previous results ...';
  DubiousMsg.Show;
  DubiousMsg.ProcessWinMessages(nil);

  nBatch := nThisBatch;
  sBatch := Trim(IntToStr(nBatch));
  lSuccessful   := True;
  lResultIsGood := True;
  qStringList := TStringList.Create;
  qryStringList := TStringList.Create;
  qTN := TQuery.Create(nil);
  qTN.DatabaseName := 'dbPPdata';
  qryTN := TQuery.Create(nil);
  qryTN.DatabaseName := 'dbPPdata';
  with qryTN do begin
    SQL.Clear;
    SQL.Add('DELETE');
    SQL.Add('FROM tDubiousResult');
    SQL.Add('WHERE BatchID = '+ sBatch);
    SQL.Add('AND ResolutionID IS NULL');
    ExecSQL;
    SQL.Clear;
  end;  // qryTN.

  qryBatchType := TQuery.Create(nil);
  qryBatchType.DatabaseName := 'dbPPdata';
  with qryStringList do begin
    Clear;
    Add('SELECT tBatchType.*');
    Add('FROM tBatchType INNER JOIN tBatch');
    Add(' ON tBatchType.ID = tBatch.BatchTypeID');
    Add('WHERE (tBatch.ID = ' + sBatch + ')');
  end;
  qryBatchType.SQL := qryStringList;
  qryBatchType.Open;

  if qryBatchType.FieldByName('DubiousCheck01').AsBoolean then begin
    // Test for variations in Rent for the same service type.
    DubiousMsg.Msg.Caption := 'Checking for Rental Variations ...';
    DubiousMsg.ProcessWinMessages(nil);
    DubiousTest_1(nBatch, lResultIsGood);
  end;  // DubiousCheck01.
  if lSuccessful then lSuccessful := lResultIsGood;
  if qryBatchType.FieldByName('DubiousCheck02').AsBoolean then begin
    // Test for Dubious Error 2 - Calls less than Rent.
    DubiousMsg.Msg.Caption := 'Checking for Calls < Rent ...';
    DubiousMsg.ProcessWinMessages(nil);
    DubiousTest_2(nBatch, lResultIsGood);
  end;  // DubiousCheck02.
  if lSuccessful then lSuccessful := lResultIsGood;
  if qryBatchType.FieldByName('DubiousCheck03').AsBoolean then begin
    // Test for Dubious Error 3 - Tariff & Rates Check.
    DubiousMsg.Msg.Caption := 'Checking for Tariff & Rates Compliance ...';
    DubiousMsg.ProcessWinMessages(nil);
    DubiousTest_3(nBatch, lResultIsGood);
  end;  // DubiousCheck03.
  if lSuccessful then lSuccessful := lResultIsGood;
  if qryBatchType.FieldByName('DubiousCheck04').AsBoolean then begin
    // Test for Dubious Error 4 - Activity Change Check.
    DubiousMsg.Msg.Caption := 'Checking for Changes in Activity ...';
    DubiousMsg.ProcessWinMessages(nil);
    DubiousTest_4(nBatch, lResultIsGood);
  end;  // DubiousCheck04.
  if lSuccessful then lSuccessful := lResultIsGood;
  if qryBatchType.FieldByName('DubiousCheck05').AsBoolean then begin
    // Test for Dubious Error 5 - Value Range Check.
    DubiousMsg.Msg.Caption := 'Checking Splits are only between valid Services ...';
    DubiousMsg.ProcessWinMessages(nil);
    DubiousTest_5(nBatch, lResultIsGood);
  end;  // DubiousCheck05.
  if lSuccessful then lSuccessful := lResultIsGood;

  // Final overall check.
  if lSuccessful then begin
    // Update the tBatchStatus table - Dubious Errors check passed.
    UpdateBatchStatus(nBatch, MainForm.BATCH_STATUS_DCHECKOK);
  end
  else begin
    // Update the tBatchStatus table - Dubious errors.
    UpdateBatchStatus(nBatch, MainForm.BATCH_STATUS_DCHECK);
  end;
  qStringList.Free;
  qryStringList.Free;
  qryBatchType.Close;
  qryBatchType.Free;
  qTN.Close;
  qTN.Free;
  qryTN.Close;
  qryTN.Free;
  DubiousMsg.Close;
  DubiousMsg.Free;
end;  // procedure DubiousErrorTest().

end.
