unit OptusFixed;

interface

uses
  Messages, Classes, SysUtils, Graphics, Dialogs,
  Db, DbTables;

  function GetReferences() : boolean;
  procedure GetPhoneData( const ProgramID, nBatch: LongInt; const FName : string;
    var lReturnCode: Boolean);
  procedure OptusFixedInvoice( const ProgramID, nBatch: LongInt; sDirectory: string );

implementation

uses
  CriticalTests, ConstantValues, Math,
//  Main, DataMod, Progress, SaveData, ClrTemp, DateFunctions, FileFunctions,
  Main, DataMod, Progress, DateFunctions, FileFunctions,
  MyMessage, GenFns;

var
  nSupplierID: integer;        // Set in GetReferences().
  sSupplierID: string;         // Set in GetReferences().
  qryOptus : TQuery;         // Set in GetReferences().
  sRentCodeID : string;        // Set in GetReferences().
  sCallCodeID : string;        // Set in GetReferences().
  sOtherCodeID : string;       // Set in GetReferences().
  sDiscountCodeID : string;    // Set in GetReferences().
  sSupplierRentCodeID : string;        // Set in GetReferences().
  sSupplierCallCodeID : string;        // Set in GetReferences().
  sSupplierOtherCodeID : string;       // Set in GetReferences().
  sSupplierDiscountCodeID : string;    // Set in GetReferences().
  fGSTRate : Real;             // GST Rate - read from MainForm.
//  DefaultJob : String;
//  sCreditAccount : string;    // Read from Config-acct for all credits.
//  lSaveRentCredit : boolean;  // Save credits on rentals.
//  lSaveCallCredit : boolean;  // Save credits on calls.
//  lSaveCredit : Boolean;      // True if sCreditAccount is valid.

function GetReferences : boolean;
var
  lSuccessful : boolean;
begin
  // initialise the Static table elements defined as Private for this form.
  fGSTRate := MAINFORM.GST_RATE;
  lSuccessful := True;
  qryOptus := TQuery.Create(MainForm);
  with qryOptus do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('SELECT id');
    SQL.Add('FROM tSupplier');
    SQL.Add('WHERE SupplierName LIKE ''OPTUS%''');
    SQL.Add('AND Active = 1');
    Open;
    if Eof then begin
      lSuccessful := False;
    end
    else begin
      nSupplierID := FieldByName('id').AsInteger;
      sSupplierID := IntToStr(nSupplierID);
      Close;
      SQL.Clear;
      SQL.Add('SELECT Code, CAST(TransactionTypeID AS VARCHAR(20)) '+
        'AS TransactionType, CAST(ID AS VARCHAR(20)) AS SupplierCodeID');
      SQL.Add('FROM tSupplierServiceCode');
      SQL.Add('WHERE SupplierID = '+sSupplierID);
      SQL.Add('AND Active = 1');
      SQL.Add('AND Description LIKE ''RENT''');
      Open;
      if Eof then lSuccessful := False
      else begin
        sRentCodeID := Trim(FieldByName('TransactionType').AsString);
        sSupplierRentCodeID := Trim(FieldByName('SupplierCodeID').AsString);
      end;
      Close;
      SQL.Strings[SQL.Count-1] :=
        'AND Description LIKE ''CALLS''';
      Open;
      if Eof then lSuccessful := False
      else begin
        sCallCodeID := Trim(FieldByName('TransactionType').AsString);
        sSupplierCallCodeID := Trim(FieldByName('SupplierCodeID').AsString);
      end;
      Close;
      SQL.Strings[SQL.Count-1] :=
        'AND Description LIKE ''UNKNOWN%''';
      Open;
      if Eof then lSuccessful := False
      else begin
        sOtherCodeID := FieldByName('TransactionType').AsString;
        sSupplierOtherCodeID := Trim(FieldByName('SupplierCodeID').AsString);
      end;
      Close;
      SQL.Strings[SQL.Count-1] :=
        'AND Description LIKE ''MISC%''';
      Open;
      if Eof then lSuccessful := False
      else begin
        sDiscountCodeID := Trim(FieldByName('TransactionType').AsString);
        sSupplierDiscountCodeID := Trim(FieldByName('SupplierCodeID').AsString);
      end;
      Close;
      if lSuccessful then begin
        qryOptus.SQL.Delete(qryOptus.SQL.Count-1);
        qryOptus.Open;
      end
      else begin
        MessageDlg('Entries for RENT, CALLS, UNKNOWN, MISCELLANEOUS must'#13#10+
          'be added to tSupplierServiceCode for OPTUS before proceeding.',
          mtWarning, [mbOk], 0);
      end;
    end;  // if Eof then begin..else.
  end;
  Result := lSuccessful;
end;  // procedure GetReferences().

procedure GetPhoneData( const ProgramID, nBatch: LongInt; const FName : string;
  var lReturnCode: Boolean );
// Data handled is for calls made from Optus fixed phones (PABX).
// This function scans the input file for relevant data and saves it.
//
// Variable sFieldValueList contains the billing codes processed below:
// 150000  Start of details for a service.
// 361005  Charges summary (all types).
// 361505  Rental credit details.
// 363005  Rental details.
// 353505  Call details.
// 261099  Total invoice value (not processed below).
// 000000  Invoicing date.
const
  RECORD_BYTES_C: Integer = 150;       // Typical minimum bytes per record.
  PROGRESS_COUNTER_C: Integer=100;     // Update Progress every 100 records read
  PROGRESS_MAX_C: Integer = 100;       // Maximum progress is 100%.
  LINE_C: integer = 0;                 // Bill entry number.
  CODE_C: integer = 1;                 // Billing code.
  ABBR_C: integer = 2;                 // Abbreviation.
  DESC1_C: integer = 3;                // Description field (Call date).
  DESC2_C: integer = 4;                // Description field (Call time).
  DESC3_C: integer = 5;                // Description field.
  DESC4_C: integer = 6;                // Description field (Destination).
  DESC5_C: integer = 7;                // Description field (Origin).
  DESC6_C: integer = 8;                // Outgoing phone number.
  CODE2_C: integer = 9;                // Rate description.
  CODE3_C: integer = 10;               // Call duration.
//  // Set 20061222-RJC a/c structure change.
//  COST1_C: integer = 12;               // Call charge field (Code).
//  COST2_C: integer = 13;               // Call charge field (Duration).
//  COST3_C: integer = 14;               // Call charge field (Cost).
//  DESC7_C: integer = 15;               // Peak/Off Peak.
  // Reinstated 20061222-RJC a/c old structure delivered.
  COST1_C: integer = 11;               // Call charge field (Code).
  COST2_C: integer = 12;               // Call charge field (Duration).
  COST3_C: integer = 13;               // Call charge field (Cost).
  DESC7_C: integer = 14;               // Peak/Off Peak.
var
//  COST1_N: integer;                    // Required charge field (Code).
  COST2_N: integer;                    // Required charge field (Duration).
  COST3_N: integer;                    // Required charge field (Cost).
  DESC7_N: integer;                    // Required Peak/Off Peak.
  CALLCODE_N: integer;                 // Which field has the call code for 353501. 
  nFileSize: Integer;                  // Actual file size.
  InputTextFile: TextFile;
  InputTextFileBuffer: array[1..4096] of Char;  // 4KB buffer.
  sCaption : String;                   // Used for modifying Progress caption.
  ReadString: string;                  // Read in from the text file.
  ConvertString: TStringList;          // Used to convert SDF format to strings.
  nProgressCounted : integer;          // Count each set of PROGRESS_COUNTER_C.
  Progress: TProgressForm;
  nCount: Integer;
  nSoFar: Integer;
  nFactor: Real;
  vSubString : variant;                // - ditto, used in the Locate() function.
  sFieldValueList : String;  // Lists those billing codes to be handled here.
  lSaveToTable : boolean;

  sBillCode: String;
  cAbbr: String;
  sServiceID: string;
  SubString: string;
  sBatch : string;
  sListFields : string;
  sListValues : string;
  FieldStringList : TStringList;
  ValueStringList : TStringList;
  qryStringList : TStringList;
  slAddFieldNames : TStringList;         // Calls to AddToTable() only.
  slAddFieldValues : TStringList;        // Calls to AddToTable() only.
  nListValues : integer;
  qryDC : TQuery;
  sLastCallCode : string;
  sLastCallTransactionTypeID : string;
  sLastRentCode : string;
  sLastRentTransactionTypeID : string;
  sLastSupplierCallCodeID : string;
  sLastSupplierRentCodeID : string;
  lFound : boolean;
  nLenSubString : integer;
  curAmountCharged : Currency;
  sCallTill : string;
  sDialledNumber : String;
begin
  lReturnCode := False;
  case ProgramID of
    PIMPORTBILLINGFILE_OPTUS_FIXED_LONG: begin
    // Newer modified Optus Invoice file - some fields are slightly wider.
//                    COST1_N    := COST1_C + 1;
                    COST2_N    := COST2_C + 1;
                    COST3_N    := COST3_C + 1;
                    DESC7_N    := DESC7_C + 1;
                    CALLCODE_N := DESC4_C;
                    end;
  else              begin
    // PIMPORTBILLINGFILE_OPTUS_FIXED_ORIGINAL
    // Original Optus Invoice file structure.
//                    COST1_N    := COST1_C;
                    COST2_N    := COST2_C;
                    COST3_N    := COST3_C;
                    DESC7_N    := DESC7_C;
                    CALLCODE_N := ABBR_C;
                    end;
  end;
  sBatch := IntToStr( nBatch );
  sServiceID := 'NULL,';
  FieldStringList := TStringList.Create;
  ValueStringList := TStringList.Create;
  qryStringList := TStringList.Create;
  slAddFieldNames := TStringList.Create;         // Calls to AddToTable() only.
  slAddFieldValues := TStringList.Create;        // Calls to AddToTable() only.
  qryDC := TQuery.Create(MainForm);
  qryDC.DatabaseName := 'dbPPdata';
  
  sCallTill := '';
  sDialledNumber := '';
  sServiceID := '';

  if GetReferences then begin
    nFileSize := GetFileSize(FName);
    if (nFileSize > 0) then
      nFactor := 100.0 * RECORD_BYTES_C / nFileSize
    else
      nFactor := 100.0;

    // Open the input (text) billing file for reading.
    AssignFile( InputTextFile, Fname );
    // Bigger buffer for faster reads.
    System.SetTextBuf(InputTextFile, InputTextFileBuffer);
    Reset( InputTextFile );

    Progress := TProgressForm.Create(MainForm);
    Progress.Caption := 'Reading The Input Text File';
    Progress.btnCancel.Visible := False;
    // Convert nFileSize to x100kBytes (based on 1.0MB=1,048,576B).
    nFileSize := nFileSize div 104858;
    Str( (nFileSize / 10.0):5:1, sCaption );
    Progress.Msg.Caption := 'Reading ('+Trim(sCaption)+'MB) ...';
    Progress.Show;
    Progress.ProcessWinMessages(MainForm);

    sFieldValueList := ',353505,353501,350505,250500,320405,000000,';
    ConvertString := TStringList.Create; // Used to convert SDF format to strings.
    try
      // Get the first occurence of the search string.
      Readln( InputTextFile, ReadString );

      // To be sure the file isn't empty too...
      if (Length( ReadString ) > 0) or (not Eof( InputTextFile )) then begin
        nProgressCounted := 0;
        // Initiate the data variables.
        nCount := 0;
        // Continue to search through the remaining text of the file
        // for occurrences of the search string.  On each each find,
        // process the resultant string.
        repeat
          Progress.ProcessWinMessages(MainForm);
          ConvertString.CommaText := ReadString;      // Decode the line read.
          sBillCode := Trim(ConvertString.Strings[CODE_C]);
          lSaveToTable := (Pos(sBillCode,sFieldValueList)>0);

          if lSaveToTable then begin
            // Save the strings into the table.
            FieldStringList.Clear;
            ValueStringList.Clear;
            sListFields := '(';
            sListValues := 'VALUES (';
            cAbbr := Trim(UpperCase(ConvertString.Strings[ABBR_C]));
            SubString := Trim(ConvertString.Strings[LINE_C]);
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'SequenceNo,';
              sListValues := sListValues +''''+ SubString +''',';
            end;
            sListFields := sListFields + 'ServiceID,';
            sListValues := sListValues + sServiceID;

            // Process each billing code separately.
            // Calls (USAGE).
            if (sBillCode = '353505') then begin
              if (Length(sLastCallCode) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ sLastCallCode +''',';
              end;
              if (Length(sLastCallTransactionTypeID) > 0) then begin
                sListFields := sListFields + 'TransactionTypeID,';
                sListValues := sListValues + sLastCallTransactionTypeID+',';
              end;
              if (Length(sLastCallTransactionTypeID) > 0) then begin
                sListFields := sListFields + 'SupplierServiceCodeID,';
                sListValues := sListValues + sLastSupplierCallCodeID+',';
              end;
              SubString := Trim(ConvertString.Strings[DESC1_C]);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'TxnDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end;
              SubString := Trim(ConvertString.Strings[DESC2_C]);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'TxnTime,';
                sListValues := sListValues +''''+ Copy(SubString,1,2)+
                  ':'+ Copy(SubString,3,2) +':00'',';
              end;
              SubString := Trim(ConvertString.Strings[DESC5_C]); // DESC4_C RJC 20061221.
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Origin,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;
              SubString := Trim(ConvertString.Strings[DESC4_C]); // DESC5_C RJC 20061221.
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Destination,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;
              SubString := Trim(ConvertString.Strings[COST2_N]);
              if (Length(SubString) > 0) then begin
                SubString := ConvertTimeToSecs(CleanString(SubString));
                sListFields := sListFields + 'Duration,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';
              SubString := CleanString(Trim(ConvertString.Strings[DESC6_C]));
              if (Length(SubString) > 0) then begin
                if (SubString <> sDialledNumber) then begin
                  sDialledNumber := SubString;
                end;
                sListFields := sListFields + 'DialledNumber,';
                sListValues := sListValues +''''+sDialledNumber+''',';
              end;
              SubString := Trim(ConvertString.Strings[DESC7_N]);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,Peak,';
                sListValues := sListValues +''''+ sDialledNumber+': '+
                  CleanString(SubString) +''',';
                if (UpperCase(SubString) = 'PEAK') then
                  sListValues := sListValues +'1,'
                else
                  sListValues := sListValues +'0,';
              end;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';
              SubString := Trim(ConvertString.Strings[COST3_N]);
              curAmountCharged := StrToCurr(SubString);
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + SubString +',';
              sListFields := sListFields + 'GSTFlag,';
              sListValues := sListValues + '''Y'',';
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues +
//                CurrToStrF(curAmountCharged*(1+fGSTRate), ffFixed, 3) +',';
                CurrToStr(curAmountCharged*(1+fGSTRate)) +',';
              sListFields := sListFields + 'GSTAmount,';
              sListValues := sListValues +
//                CurrToStrF(curAmountCharged*fGSTRate, ffFixed, 3) +',';
                CurrToStr(curAmountCharged*fGSTRate) +',';
              sListFields := sListFields + 'ToDate,';
              sListValues := sListValues + sCallTill;
            end  // '353505'.

            // Call Code only.
            else if (sBillCode = '353501') then begin
//              SubString := Trim(ConvertString.Strings[DESC4_C]); // ABBR_C RJC 20061221.
    //          SubString := Trim(ConvertString.Strings[ABBR_C]); // DESC4_C RJC 20070117.
              SubString := Trim(ConvertString.Strings[CALLCODE_N]);
              if (Length(SubString) > 0) then begin
                sLastCallCode := SubString;
                vSubString := SubString;
                with qryOptus do
                  lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                if lFound then begin
                  sLastCallTransactionTypeID := qryOptus['TransactionType'];
                  sLastSupplierCallCodeID := qryOptus['SupplierCodeID'];
                end
                else begin
                  sLastCallTransactionTypeID := sCallCodeID;
                  sLastSupplierCallCodeID := sSupplierCallCodeID;
                  slAddFieldNames.Clear;
                  slAddFieldValues.Clear;
                  slAddFieldNames.Add('Code');
                  slAddFieldValues.Add(''''+sLastCallCode+'''');
                  slAddFieldNames.Add('AmountCategory');
                  slAddFieldValues.Add('''CALLS''');
                  slAddFieldNames.Add('Description');
                  slAddFieldValues.Add('''CALLS''');
                  slAddFieldNames.Add('TransactionTypeID');
                  slAddFieldValues.Add(sLastCallTransactionTypeID);
                  slAddFieldNames.Add('SupplierID');
                  slAddFieldValues.Add(sSupplierID);
                  slAddFieldNames.Add('Active');
                  slAddFieldValues.Add('1');
                  AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                  qryOptus.Close;
                  qryOptus.Open;
                  with qryOptus do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastCallTransactionTypeID := qryOptus['TransactionType'];
                    sLastSupplierCallCodeID := qryOptus['SupplierCodeID'];
                  end
                  else begin
                    sLastCallTransactionTypeID := sCallCodeID;
                    sLastSupplierCallCodeID := sSupplierCallCodeID;
                  end;
                end;  // if (not) lFound.
              end;  // if (Length(SubString) > 0) then begin.
              lSaveToTable := False;
            end  // '353501'.

            // **********  Payment required.  **********
            // Rental (RECUR).
            // note - changed input 20050506 - no rental charged, now a total
            //        for local calls + gst, so omitted from processing (RJC).
            // Reinstated, but Calls omitted from processing-RJC20060218.
            else if (sBillCode = '350505') then begin
              SubString := ';'+Trim(ConvertString.Strings[ABBR_C])+';';
              if (Pos(SubString,';LOCCL;LOC13;NATNL;MOBCL;TOTAL;')=0) then begin
                SubString := Trim(ConvertString.Strings[DESC2_C]);
                curAmountCharged := StrToCurr(SubString);
                if (Round(curAmountCharged*100)<>0) then begin
                  SubString := Trim(ConvertString.Strings[ABBR_C]);
                  if ((Length(SubString) > 0) and
                    (SubString <> sLastRentCode)) then begin
                    sLastRentCode := SubString;
                    vSubString := SubString;
                    with qryOptus do
                      lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                    if lFound then begin
                      sLastRentTransactionTypeID := qryOptus['TransactionType'];
                      sLastSupplierRentCodeID := qryOptus['SupplierCodeID'];
                    end
                    else begin
                      sLastRentTransactionTypeID := sRentCodeID;
                      sLastSupplierRentCodeID := sSupplierRentCodeID;
                      slAddFieldNames.Clear;
                      slAddFieldValues.Clear;
                      slAddFieldNames.Add('Code');
                      slAddFieldValues.Add(''''+sLastRentCode+'''');
                      slAddFieldNames.Add('AmountCategory');
                      slAddFieldValues.Add('''RENT''');
                      slAddFieldNames.Add('Description');
                      slAddFieldValues.Add('''RENT''');
                      slAddFieldNames.Add('TransactionTypeID');
                      slAddFieldValues.Add(sLastRentTransactionTypeID);
                      slAddFieldNames.Add('SupplierID');
                      slAddFieldValues.Add(sSupplierID);
                      slAddFieldNames.Add('Active');
                      slAddFieldValues.Add('1');
                      AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                      qryOptus.Close;
                      qryOptus.Open;
                      with qryOptus do
                        lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                      if lFound then begin
                        sLastRentTransactionTypeID := qryOptus['TransactionType'];
                        sLastSupplierRentCodeID := qryOptus['SupplierCodeID'];
                      end;
                    end;  // if (not) lFound.
                  end;  // if (Length(SubString) > 0) and..
                  sListFields := sListFields + 'TransactionTypeID,'+
                    'SupplierServiceCodeID,';
                  sListValues := sListValues + sLastRentTransactionTypeID+','+
                    sLastSupplierRentCodeID+',';
                  sListFields := sListFields + 'AmountExGST,';
                  sListValues := sListValues +
//                    CurrToStr(curFormat(curAmountCharged)) +',';
                    CurrToStr(curAmountCharged) +',';
                  sListFields := sListFields + 'GSTFlag,';
                  sListValues := sListValues + '''Y'',';
                  sListFields := sListFields + 'AmountIncGST,';
                  sListValues := sListValues +
//                    CurrToStr(curFormat(curAmountCharged*(1+fGSTRate))) +',';
                    CurrToStr(curAmountCharged*(1+fGSTRate)) +',';
                  sListFields := sListFields + 'GSTAmount,';
                  sListValues := sListValues +
//                    CurrToStr(curFormat(curAmountCharged*fGSTRate)) +',';
                    CurrToStr(curAmountCharged*fGSTRate) +',';
                  sListFields := sListFields + 'ToDate,';
                  sListValues := sListValues + sCallTill;
                  FieldStringList.Add(sListFields);
                  ValueStringList.Add(sListValues);
                  sListFields := '';
                  sListValues := '';
                  // Some bills do not include all data - test for it first.
                  if (ConvertString.Count > DESC3_C) then begin
                    SubString := Trim(ConvertString.Strings[DESC1_C])+
                      Trim(ConvertString.Strings[DESC3_C]);
                    nLenSubString := Length(SubString);
                    if (nLenSubString > 0) then begin         // Description added.
                      SubString := Trim(ConvertString.Strings[DESC1_C]);
                      if (Length(SubString) > 0) then begin
                        if (Length(SubString) < nLenSubString) then begin
                          SubString := SubString +' - '+
                            Trim(ConvertString.Strings[DESC3_C]);
                        end;  // if (Length(SubString) < nLenSubString) then begin.
                      end  // if (Length(SubString) > 0) then begin.
                      else begin
                        SubString := Trim(ConvertString.Strings[DESC3_C]);
                      end;  // if (Length(SubString) > 0) then begin..else.
                      sListFields := sListFields + 'RateDescription,';
                      sListValues := sListValues +''''+CleanString(SubString)+''',';
                    end;  // if (nLenSubString > 0) then begin.
                  end;
                end
                else begin
                  lSaveToTable := False;
                end;  // if (Round(StrToCurr(SubString)*100)<>0) then begin.
              end
              else begin
                lSaveToTable := False;
              end;  // if (Round(StrToCurr(SubString)*100)<>0) then begin.
            end  // '350505'.


            // Discount (DTOLL).
            // Calls omitted from processing.
            else if (sBillCode = '320405') then begin
              SubString := Trim(ConvertString.Strings[ABBR_C]);
              if (SubString = 'DTOLL') then begin
                SubString := Trim(ConvertString.Strings[DESC2_C]);
                curAmountCharged := StrToCurr(SubString);
                if (Round(curAmountCharged*100)<>0) then begin
                  // Important.
                  // This routine is only called at the beginning of the file.
                  // Therefore "sServiceID" is unassigned so the SQL statement
                  // will fail unless we add one first. This entry will be
                  // rejected and the operator must assign the correct
                  // ServiceID so the value is unimportant. Use the CallCode.
                  SubString := Trim(ConvertString.Strings[ABBR_C]);
                  sListValues := sListValues + ''''+SubString+''',';
                  // Now, lets get on with the rest of the data.
                  if ((Length(SubString) > 0) and
                    (SubString <> sLastRentCode)) then begin
                    sLastRentCode := SubString;
                    vSubString := SubString;
                    with qryOptus do
                      lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                    if lFound then begin
                      sLastRentTransactionTypeID := qryOptus['TransactionType'];
                      sLastSupplierRentCodeID := qryOptus['SupplierCodeID'];
                    end
                    else begin
                      sLastRentTransactionTypeID := sRentCodeID;
                      sLastSupplierRentCodeID := sSupplierRentCodeID;
                      slAddFieldNames.Clear;
                      slAddFieldValues.Clear;
                      slAddFieldNames.Add('Code');
                      slAddFieldValues.Add(''''+sLastRentCode+'''');
                      slAddFieldNames.Add('AmountCategory');
                      slAddFieldValues.Add('''RENT''');
                      slAddFieldNames.Add('Description');
                      slAddFieldValues.Add('''RENT''');
                      slAddFieldNames.Add('TransactionTypeID');
                      slAddFieldValues.Add(sLastRentTransactionTypeID);
                      slAddFieldNames.Add('SupplierID');
                      slAddFieldValues.Add(sSupplierID);
                      slAddFieldNames.Add('Active');
                      slAddFieldValues.Add('1');
                      AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                      qryOptus.Close;
                      qryOptus.Open;
                      with qryOptus do
                        lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                      if lFound then begin
                        sLastRentTransactionTypeID := qryOptus['TransactionType'];
                        sLastSupplierRentCodeID := qryOptus['SupplierCodeID'];
                      end;
                    end;  // if (not) lFound.
                  end;  // if (Length(SubString) > 0) and..
                  sListFields := sListFields + 'TransactionTypeID,'+
                    'SupplierServiceCodeID,';
                  sListValues := sListValues + sLastRentTransactionTypeID+','+
                    sLastSupplierRentCodeID+',';
                  sListFields := sListFields + 'AmountExGST,';
                  sListValues := sListValues +
//                    CurrToStr(curFormat(curAmountCharged)) +',';
                    CurrToStr(curAmountCharged) +',';
                  sListFields := sListFields + 'GSTFlag,';
                  sListValues := sListValues + '''Y'',';
                  sListFields := sListFields + 'AmountIncGST,';
                  sListValues := sListValues +
//                    CurrToStr(curFormat(curAmountCharged*(1+fGSTRate))) +',';
                    CurrToStr(curAmountCharged*(1+fGSTRate)) +',';
                  sListFields := sListFields + 'GSTAmount,';
                  sListValues := sListValues +
//                    CurrToStr(curFormat(curAmountCharged*fGSTRate)) +',';
                    CurrToStr(curAmountCharged*fGSTRate) +',';
                  sListFields := sListFields + 'ToDate,';
                  sListValues := sListValues + sCallTill;
                  FieldStringList.Add(sListFields);
                  ValueStringList.Add(sListValues);
                  sListFields := '';
                  sListValues := '';
                  // Some bills do not include all data - test for it first.
                  if (ConvertString.Count > DESC3_C) then begin
                    SubString := Trim(ConvertString.Strings[DESC1_C])+
                      Trim(ConvertString.Strings[DESC3_C]);
                    nLenSubString := Length(SubString);
                    if (nLenSubString > 0) then begin         // Description added.
                      SubString := Trim(ConvertString.Strings[DESC1_C]);
                      if (Length(SubString) > 0) then begin
                        if (Length(SubString) < nLenSubString) then begin
                          SubString := SubString +' - '+
                            Trim(ConvertString.Strings[DESC3_C]);
                        end;  // if (Length(SubString) < nLenSubString) then begin.
                      end  // if (Length(SubString) > 0) then begin.
                      else begin
                        SubString := Trim(ConvertString.Strings[DESC3_C]);
                      end;  // if (Length(SubString) > 0) then begin..else.
                      sListFields := sListFields + 'RateDescription,';
                      sListValues := sListValues +''''+CleanString(SubString)+''',';
                    end;  // if (nLenSubString > 0) then begin.
                  end;
                end
                else begin
                  lSaveToTable := False;
                end;  // if (Round(StrToCurr(SubString)*100)<>0) then begin.
              end
              else begin
                lSaveToTable := False;
              end;  // if (Round(StrToCurr(SubString)*100)<>0) then begin.
            end  // '320405'.


            // ServiceID.
            else if (sBillCode = '250500') then begin
              SubString := CleanString(Trim(ConvertString.Strings[DESC3_C]));
              if (Length(SubString) > 0) then begin
                sServiceID := ''''+ SubString +''',';
              end;
              lSaveToTable := False;
            end  // '250500'.

            else if (sBillCode = '000000') then begin
              // General information saved later.
              lSaveToTable := False;
              // First (actually the 2nd line) useful line of file.
              // This appears only once in the file, near the beginning.
              sCallTill := ''''+YMD2DMY(ConvertString.Strings[DESC1_C]) +''',';
            end  // '000000'.

            else begin
              lSaveToTable := False;
            end;
          end;  // if lSaveToTable then.
          if lSaveToTable then begin
            // Now make the SQL statement to save to the database table.
            sListFields := sListFields + 'BatchID)';
            sListValues := sListValues + sBatch +')';
            FieldStringList.Add(sListFields);
            ValueStringList.Add(sListValues);
            qryStringList.Clear;
            qryStringList.Add('INSERT INTO tDataCollection');
            for nListValues := 0 to FieldStringList.Count-1 do begin
              qryStringList.Add(FieldStringList[nListValues]);
            end;
            for nListValues := 0 to ValueStringList.Count-1 do begin
              qryStringList.Add(ValueStringList[nListValues]);
            end;
            // Update and execute the query.
            qryDC.Close;
            qryDC.SQL := qryStringList;
            qryDC.ExecSQL;
          end;  // if lSaveToTable then begin.
          // Keep track of how much work has been done to date.
          Inc(nCount);
          Inc(nProgressCounted);
          if (nProgressCounted > PROGRESS_COUNTER_C) then begin
            // Report progress to the user.
            nSoFar := Round(nCount * nFactor);
            // Do not know how big the file is - make sure don't exceed 100%.
            if (nSoFar > PROGRESS_MAX_C) then
              Progress.ProgressBar1.Position := PROGRESS_MAX_C
            else
              Progress.ProgressBar1.Position := nSoFar;
            nProgressCounted := 0;
          end;  // if (nProgressCounted > PROGRESS_COUNTER_C) then.
          Progress.ProcessWinMessages(MainForm);    // Let windows update all messages.
          Readln( InputTextFile, ReadString ); // Read in the next line of data.
        until Eof( InputTextFile );
        lReturnCode := True;
      end;  // if (Length( ReadString ) > 0) or (not Eof( InputTextFile )) then.
    finally
      // Done with the input file and resources, close them.
      CloseFile( InputTextFile );
      Progress.Close;
      ConvertString.Free;
      FieldStringList.Free;
      ValueStringList.Free;
      qryStringList.Free;
      qryDC.Free;
    end;

    Progress.Close;
  end;  // if GetReferences then begin.
end;  // procedure GetPhoneData().


procedure OptusFixedInvoice( const ProgramID, nBatch: LongInt; sDirectory: string );
// This function prompts the user for the name of the Vodafone supplied
// text file and then calls the VodaMobile function (which reads the
// file into a table and controls its processing).
var
  OpenDlg : TOpenDialog;
  lContinue : Boolean;
  sFile : String;
  slDummy : TStringList;
begin
  // First, we must establish which file is the one to be processed.
  OpenDlg := TOpenDialog.Create(MainForm);
  OpenDlg.Filter := 'Optus Billing file (*.bil)|*.BIL|All files (*.*)|*.*';
  OpenDlg.FilterIndex := 1;
  // Use the program directory as the default directory.
  if (Length(sDirectory) > 0) then OpenDlg.InitialDir := sDirectory
  else OpenDlg.InitialDir := ExtractFilePath(ParamStr(0));
  OpenDlg.Title := 'Select The Optus Input File';
  // Read and save the user's choices before continuing.
  lContinue := OpenDlg.Execute;
  sFile := OpenDlg.FileName;
  OpenDlg.Free;
  if lContinue and (Length(Trim(sFile)) > 0) then begin
    if FileExists(sFile) then begin
      // Remove any data already loaded.
      slDummy := TStringList.Create;
      DeleteFromDC(nBatch, slDummy, lContinue);
      slDummy.Free;
      // Reload the data.
      GetPhoneData(ProgramID, nBatch, sFile, lContinue);
      if lContinue then begin
        UpdateBatchStatus(nBatch, MainForm.BATCH_STATUS_IMPORT);

      // **************************************************************************
      // **************************************************************************
      // Test for critical errors that prevent progress to the next stage.
        lContinue := False;
        CriticalErrorTest( nBatch, lContinue );
        if lContinue then begin
          MessageDlg('Batch '+ IntToStr(nBatch) +
            ': Passed all tests for Critical Errors.', mtInformation,
            [mbOk], 0);
        end
        else begin
          MessageDlg('Batch '+ IntToStr(nBatch) +
            ': Critical errors found.'#13#10#13#10'These must be resolved ' +
            'before this invoice can be progressed.',
            mtWarning, [mbOk], 0);
        end;  // if lContinue then begin.
      end;  // if lContinue then begin.
    end
    else begin
      MessageDlg('That file does not exist.', mtWarning, [mbAbort], 0 );
    end;  // if FileExists(sFile) then begin.
  end;  // if (Length(Trim(sFile)) > 0) then begin.
end;  // procedure OptusFixedInvoice().


end.
