unit PrinterFunctions;

interface

procedure PrinterSetup;

implementation

uses
  Windows, SysUtils, Printers;

procedure PrinterSetup;
var
  //  First 3 - printer access variables used for printer setup.
  ADevice, ADriver, Aport : array [0..255] of char;
  DeviceHandle : THandle;
  DevMode : PDeviceMode;  // A pointer to a TDeviceMode structure.
begin
  // First obtain a handle to the TPrinter's Devicemode structure.
  Printer.GetPrinter(ADevice, ADriver, APort, DeviceHandle);

  // If the DeviceHandle is still 0, then the driver was not loaded. Set
  // the printer index to force the printer driver to load making the
  // handle available.
  if ( DeviceHandle = 0 ) then begin
    Printer.PrinterIndex := Printer.PrinterIndex;
    Printer.GetPrinter(ADevice, ADriver, APort, DeviceHandle);
  end;

  // If the DeviceHandle is still 0, then an error has occurred. Otherwise
  // use GlobalLock() to get the pointer to the TDeviceMode structure.
  if ( DeviceHandle = 0 ) then begin
    Raise Exception.Create('Could Not Initialize Printer TDeviceMode' +
      ' Structure.');
  end
  else begin
    DevMode := GlobalLock(Devicehandle);

    // NOW I can set up the printer!!
    with DevMode^ do begin
      dmFields := dmFields or DM_PAPERSIZE;    // Set paper size to A4.
      dmPaperSize := DMPAPER_A4;
      dmFields := dmFields or DM_ORIENTATION;  // Set landscape mode.
      dmOrientation := DMORIENT_LANDSCAPE;
    end;
  end;

  // Okay, all done with this setup.
  if ( DeviceHandle <> 0 ) then begin
    GlobalLock(Devicehandle);
  end;
end;


end.
 