unit TelstraFixed;
// Revision History.
{
RJC 120325: BillingPhoneInput - Replaced TransactionType with Code for default
                                rent, call, other, discount Telstra codes.
            GetPhoneAccountBills - Added reading code from input, not default.
}

interface
                                
uses
  Messages, SysUtils, Classes, Graphics, Controls, Dialogs,
  db, dbTables;

type
  TServiceDetails = record
    Custno   : string;
    ServCode : string;
    ServiceID: string;
    RentFrom : TDateTime;
    RentTill : TDateTime;
    CallFrom : TDateTime;
    CallTill : TDateTime;
  end;

  TCallHeaderType = record
    SequenceNo      : string[7];
    ServiceID       : string[17];
    TxnDate         : string[8];
    TxnTime         : string[5];
    Origin          : string[20];
    Duration        : string[9];
    CallCode        : string[2];
    RateDescription : string[83];  // Was 32: RJC-080706.
    DialledNumber   : string[17];  // Temporary variable only.
    ServiceNotes    : string[80];
    AmountExGST     : string[13];
    GSTAmount       : string[13];
    AmountIncGST    : string[13];
    FromDate        : string[8];
    ToDate          : string[8];
  end;

//  procedure SaveSummaryCharges(nBatchNumber : LongInt);
  procedure GetDetailFixedOBS( const FName:String;
    D:TDatabase; nBatchNumber:LongInt; var lReturnCode:Boolean);
  procedure BillingInputFixedOBS( stInFiles: Tstrings; nBatch: LongInt );
  procedure TelstraInvoiceFixedOBS( nBatch: LongInt; sDirectory: string;
                           nProgram: LongInt );


implementation

uses
  Math,
  Main, DataMod, GenFns, DateFunctions, FileFunctions,
  ConstantValues, CriticalTests, Progress,
  TelstraFunctions;

var
  // Declare static variables as frequently reset and reused.
  nCount: Integer;                    // Current status for frmProgress.
  nFactor: Real;                      // Weighting for nCount in progress.
  frmProgress: TProgressForm;         // frmProgress report to the operator.
  nSupplierID: integer;              // Set in BillingInput().
  sSupplierID: string;               // Set in BillingInput().
  qryTelstra : TQuery;               // Set in BillingInput().
  sRentCodeID : string;              // Set in BillingInput().
  sCallCodeID : string;              // Set in BillingInput().
  sOtherCodeID : string;             // Set in BillingInput().
  sDiscountCodeID : string;          // Set in BillingInput().
  sSupplierRentCodeID : string;      // Set in BillingInput().
  sSupplierCallCodeID : string;      // Set in BillingInput().
  sSupplierOtherCodeID : string;     // Set in BillingInput().
  sSupplierDiscountCodeID : string;  // Set in BillingInput().
  sAccountNumber : string;           // Set in BillingInput().

  sLastCallCode : string;
  sLastRentCode : string;
  sLastOtherCode: string;
  sLastCallTransactionTypeID : string;
  sLastRentTransactionTypeID : string;
  sLastOtherTransactionTypeID : string;
  sLastDiscountTransactionTypeID : string;
  sLastSupplierCallCodeID : string;
  sLastSupplierRentCodeID : string;
  sLastSupplierOtherCodeID : string;
//  sLastSupplierDiscountCodeID : string;

  lThisIsAPABX : boolean;             // Set in TelstraInvoiceFixedOBS()...
                                      // .. used in SaveSummaryChargesFixedOBS.

procedure GetDetailFixedOBS
          (
           const FName: String;
           D: TDatabase;
           nBatchNumber : LongInt;
           var lReturnCode: Boolean
           );
const
  PROGRESS_MAX_C: Integer = 100;        // Maximum progress is 100%.
  CODE_C:   integer = 0;                // Billing code.
  ACCT_C:   integer = 1;                // AccountNumber.
  TACC_C:   integer = 2;                // Telstra Account.
  INVC_C:   integer = 3;                // Invoice Number.
  LINE_C:   integer = 4;                // Description field (Call time).
  DESC06_C: integer = 5;                // Description field.
  DESC07_C: integer = 6;                // Description field (Destination).
  DESC08_C: integer = 7;                // Description field (Origin).
  DESC09_C: integer = 8;                // Outgoing phone number.
  DESC10_C: integer = 9;                // Description field (Call time).
  DESC11_C: integer = 10;               // Description field.
  DESC12_C: integer = 11;               // Description field (Destination).
  DESC13_C: integer = 12;               // Description field (Origin).
  DESC14_C: integer = 13;               // Outgoing phone number.
  DESC15_C: integer = 14;               // Description field.
  DESC16_C: integer = 15;               // Description field (Destination).
  DESC17_C: integer = 16;               // Description field (Origin).
  DESC18_C: integer = 17;               // Outgoing phone number.
  DESC19_C: integer = 18;               // Description field.
  DESC20_C: integer = 19;               // Description field (Destination).
  DESC21_C: integer = 20;               // Description field (Origin).
  DESC22_C: integer = 21;               // Outgoing phone number.
  DESC23_C: integer = 22;               // Description field.
  DESC24_C: integer = 23;               // Description field (Destination).
  DESC25_C: integer = 24;               // Description field (Origin).
  DESC26_C: integer = 25;               // Outgoing phone number.
  DESC27_C: integer = 26;               // Description field.
  DESC28_C: integer = 27;               // Description field (Destination).
  DESC29_C: integer = 28;               // Description field (Origin).
  DESC30_C: integer = 29;               // Outgoing phone number.
  DESC31_C: integer = 30;               // Outgoing phone number.
  DESC32_C: integer = 31;               // Outgoing phone number.
  DESC33_C: integer = 32;               // Outgoing phone number.
  DESC34_C: integer = 33;               // Outgoing phone number.
var
  nFileSize: Integer;                  // File size in bytes.
  lFirstDF : Boolean;
  lFirstDE : Boolean;
  sPartFileName : string;
  sPartResultSet : string;
  EntryCode : string;
  nBackSlashAt : Integer;
  TelstraFile: TextFile;               // Used for accessing the data.
  CurrentFile: String;                 // Currently open file.
  FileIteration: Integer;              // Current file iteration being processed
  ReadString: string;                  // Line of data read from TextFile.
  SubString: String;                   // A (temp) substring of T.
  vSubString : variant;                // - ditto, used in the Locate() function.
  curValue: Currency;                  // A (temp) currency conversion value.
  nSoFar: Integer;                     // Status - num of records processed.
  dBilling: TDateTime;                 // Billing date.
  dStartBillingPeriod: TDateTime;      // Start of the payment billing period.
  dEndBillingPeriod: TDateTime;        // End of the payment billing period.
  nBatchID: LongInt;
  sBatchID: string;
  LastEntryIs_1 : boolean;
  SaveLastEntry_1 : boolean;           // Set to save when last DC line is LastEntry_1.
  CallHeader : TCallHeaderType;
  sLastDetailCode : string[2];         // To detect a header line AFTER details.

  DBfromDate : string;                 // Section DB - save read dates.
  DBtoDate : string;

  aResultSet : Array of TServiceDetails;
  nResultSet : Integer;
  nMaxResults : Integer;
  nStartSearchAt : Integer;
  lFound : Boolean;
  dMinDate : TDateTime;
  nCommaPosition : integer;
  sListValues : String;
  sListFields : String;
  FieldStringList : TStringList;
  ValueStringList : TStringList;
  qryStringList : TStringList;
  slAddFieldNames : TStringList;         // Calls to AddToTable() only.
  slAddFieldValues : TStringList;        // Calls to AddToTable() only.
  ConvertString : TStringList;
  qryDC : TQuery;
  qryDS : TQuery;
  nListValues : integer;
  nDummy : Real;
  sInputLineCode : string;
  lDataSaved : boolean;
  lContinue : boolean;
  lValueIsReal : boolean;
  nDataCollectionID : LongInt;
  nLengthHeadAccount : integer;

  lNextAccountNumber : boolean;
  sNextAccountNumber : string;
  nNextAccountNumber : integer;

{
procedure SaveSummaryChargesFixedOBS(nBatchNumber : LongInt);
// Save those items in DS not matched with individual call details.
const
  PROGRESS_MAX_C: Integer = 100;        // Maximum progress is 100%.
var
  sBatchID : string;
  qryDCCount  : TQuery;
//  qryDCUpdate : TQuery;
  qryDSCount  : TQuery;
  qryDS       : TQuery;
  n003Entries : integer;
  nDummy      : Real;
  nSoFar      : integer;
  nSavedCalls   : integer;
  nSummaryCalls : integer;
  nDSCalls      : integer;
  slAddFieldNames  : TStringList;
  slAddFieldValues : TStringList;
  sServiceID : string;
  sCallCode  : string;
begin
  sBatchID := IntToStr(nBatchNumber);
  // Set strings for saving DS summary data.
  slAddFieldNames  := TStringList.Create;
  with slAddFieldNames do begin
    Add('SequenceNo');
    Add('ServiceID');
    Add('CallCode');
    Add('TransactionTypeID');
    Add('RateDescription');
    Add('AmountExGST');
    Add('GSTAmount');
    Add('AmountIncGST');
    Add('GSTFlag');
    Add('FromDate');
    Add('ToDate');
    Add('SupplierServiceCodeID');
    Add('BatchID');
  end;  // slAddFieldNames.
  slAddFieldValues := TStringList.Create;
  qryDCCount := TQuery.Create(MainForm);
  qryDCCount.DatabaseName := 'dbPPdata';
  with qryDCCount do begin
    SQL.Add('SELECT COUNT(*) AS NumberOfCalls');
    SQL.Add('FROM tDataCollection');
    SQL.Add('WHERE ServiceID LIKE ''PHONENUMBER''');
    SQL.Add('AND CallCode LIKE ''CALLCODEVALUE''');
    SQL.Add('AND BatchID = '+sBatchID);
  end;  // qryDCCount.
  qryDSCount := TQuery.Create(MainForm);
  qryDSCount.DatabaseName := 'dbPPdata';
  // Count the number of entries to be processed.
  with qryDSCount do begin
    SQL.Add('SELECT COUNT(*) AS NumberDSCalls');
    SQL.Add('FROM tDataCollect_Temp');
    SQL.Add('WHERE BatchID = '+sBatchID);
    if not lThisIsAPABX then
      SQL.Add('AND Var01 LIKE ''003''');
//    SQL.Add('WHERE Var01 LIKE ''003''');
//    SQL.Add('AND BatchID = '+sBatchID);
    Open;
  end;  // qryDSCount.
  n003Entries := qryDSCount.FieldByName('NumberDSCalls').AsInteger;
  if (n003Entries > 0) then begin
    nFactor := 100.0 / n003Entries;
  end
  else begin
    nFactor := 100.0;
  end;
  nCount := 0;
  nSoFar := 0;
  frmProgress.Msg.Caption := 'Processing DS Summary list ...';
  frmProgress.Msg.Refresh;
  frmProgress.ProgressBar1.Position := nSoFar;
  frmProgress.ProcessWinMessages(MainForm);
  // Now reset the qryDSCount SQL statement for processing below.
  with qryDSCount do begin
    Close;
    SQL.Clear;
    SQL.Add('SELECT SUM(NumCalls) AS NumberDSCalls');
    SQL.Add('FROM tDataCollect_Temp');
    SQL.Add('WHERE ServiceID LIKE ''PHONENUMBER''');
    SQL.Add('AND CallCode LIKE ''CALLCODEVALUE''');
    SQL.Add('AND BatchID = '+sBatchID);
  end;  // qryDSCount.
[ start curly brackets
  qryDCUpdate := TQuery.Create(MainForm);
  qryDCUpdate.DatabaseName := 'dbPPdata';
  with qryDCUpdate do begin
    SQL.Add('UPDATE tDataCollect_Temp');
    SQL.Add('SET Active = 1');
    SQL.Add('WHERE ServiceID LIKE ''PHONENUMBER''');
    SQL.Add('AND CallCode LIKE ''CALLCODEVALUE''');
    SQL.Add('AND NumCalls = 1');
    SQL.Add('AND BatchID = '+sBatchID);
  end;  // qryDCUpdate.
] end curly brackets
  // And the temporary table to save potential summary data without call details.
  qryDS := TQuery.Create(MainForm);
  qryDS.DatabaseName := 'dbPPdata';
  // Now open the Summaries and we are ready to see which ones to save.
  with qryDS do begin
    SQL.Add('SELECT *');
    SQL.Add('FROM (SELECT TOP 10000 *');
    SQL.Add('  FROM tDataCollect_Temp');
    SQL.Add('  WHERE BatchID = '+sBatchID);
    if not lThisIsAPABX then
      SQL.Add('  AND Var01 LIKE ''003''');
    SQL.Add('  ORDER BY ServiceID, NumCalls DESC) AS DCT');
    SQL.Add('ORDER BY ServiceID, CallCode');
[ start curly brackets

// This sql puts the order right, starting at biggest NumCalls to smallest:
select BatchID, ServiceID, CallCode, NumCalls, Var01
from (SELECT TOP 10000 BatchID, ServiceID, CallCode, NumCalls, Var01
FROM tDataCollect_Temp
where BatchID = 36789
ORDER BY ServiceID, NumCalls DESC) AS DCT
ORDER BY ServiceID, CallCode
    // Following commented is origina:
    SQL.Add('FROM tDataCollect_Temp');
    SQL.Add('WHERE BatchID = '+sBatchID);
    SQL.Add('AND Var01 LIKE ''003''');
    SQL.Add('ORDER BY ServiceID, CallCode');
] end curly brackets

    Open;
  end;  // qryDS.
  // And here we go.
  while not qryDS.Eof do begin
    // Get the number of details read into the tDataCollection table.
    qryDCCount.Close;
    qryDCCount.SQL.Strings[2] := 'WHERE ServiceID LIKE '''+
      qryDS.FieldByName('ServiceID').AsString+'''';
    qryDCCount.SQL.Strings[3] := 'AND CallCode LIKE '''+
      qryDS.FieldByName('CallCode').AsString+'''';
    qryDCCount.Open;
    if qryDCCount.Eof then
      nSavedCalls := 0
    else
      nSavedCalls := qryDCCount.FieldByName('NumberOfCalls').AsInteger;
    // Get the number of calls in the Summary list of tDataCollext_Temp table.
    qryDSCount.Close;
    qryDSCount.SQL.Strings[2] := 'WHERE ServiceID LIKE '''+
      qryDS.FieldByName('ServiceID').AsString+'''';
    qryDSCount.SQL.Strings[3] := 'AND CallCode LIKE '''+
      qryDS.FieldByName('CallCode').AsString+'''';
    qryDSCount.Open;
    if qryDSCount.Eof then
      nDSCalls := 0
    else
      nDSCalls := qryDSCount.FieldByName('NumberDSCalls').AsInteger;
    // Get the number of calls in this summary item, and ServiceID & CallCode.
    nSummaryCalls := qryDS.FieldByName('NumCalls').AsInteger;
    sServiceID := qryDS.FieldByName('ServiceID').AsString;
    sCallCode := qryDS.FieldByName('CallCode').AsString;
    // Test if there is a mismatch - assume only one entry per CallType.
    if (nSavedCalls < nDSCalls) then begin
      while not qryDS.Eof and
        (sServiceID = qryDS.FieldByName('ServiceID').AsString) and
        (sCallCode  = qryDS.FieldByName('CallCode').AsString) do begin
        if ((nSavedCalls+nSummaryCalls) <= nDSCalls) then begin
          // Save this entry now in the tDataCollection table.
          with slAddFieldValues do begin
            Clear;
            Add(''''+qryDS.FieldByName('SequenceNo').AsString+'''');
            Add(''''+sServiceID+'''');
            Add(''''+sCallCode+'''');
            Add(qryDS.FieldByName('TransactionTypeID').AsString);
            Add(''''+qryDS.FieldByName('RateDescription').AsString+'''');
            Add(qryDS.FieldByName('AmountExGST').AsString);  // String, NOT currency.
            Add(qryDS.FieldByName('GSTAmount').AsString);    // String, NOT currency.
            Add(qryDS.FieldByName('AmountIncGST').AsString); // String, NOT currency.
            Add(''''+qryDS.FieldByName('GSTFlag').AsString+'''');
            Add(''''+qryDS.FieldByName('FromDate').AsString+'''');
            Add(''''+qryDS.FieldByName('ToDate').AsString+'''');
            Add(qryDS.FieldByName('SupplierServiceCodeID').AsString);
            Add(sBatchID);
          end;  // slAddFieldValues.
          AddToTable('tDataCollection',slAddFieldNames,slAddFieldValues);
          // Update the number of calls saved in the tDataCollection table.
          nSavedCalls := nSavedCalls + nSummaryCalls;
[ start curly brackets

        // Mark this DS summary entry for later saving in tDataCollection-set Active=1.
        qryDCUpdate.SQL.Strings[2] := 'WHERE ServiceID LIKE '''+
          qryDS.FieldByName('ServiceID').AsString+'''';
        qryDCUpdate.SQL.Strings[3] := 'AND CallCode LIKE '''+
          qryDS.FieldByName('CallCode').AsString+'''';
        qryDCUpdate.SQL.Strings[4] := 'AND NumCalls = '+IntToStr(nSummaryCalls);
        qryDCUpdate.ExecSQL;
] end curly brackets

        end;  // Save entry in tDataCollection table.
        // Skip to the next entry and reload the number of calls in this Summary.
        qryDs.Next;
        if not qryDS.Eof then
          nSummaryCalls := qryDS.FieldByName('NumCalls').AsInteger;
        Inc(nCount);
      end;  // while not qryDS.eof ...
      if not qryDS.Eof and
        ((sServiceID <> qryDS.FieldByName('ServiceID').AsString) or
         (sCallCode  <> qryDS.FieldByName('CallCode').AsString)) then begin
        // Oops - we have passed out of this ServiceID and CallCode. Go back.
        qryDs.Prior;
        Dec(nCount);
      end;
    end;  // if numbers are a mismatch.
    // Report progress to the user ***********************************
    // Report progress to the user ***********************************
    Inc(nCount);
    nDummy := nFactor * nCount;
    nSoFar := Round( nDummy );
    // Do not know how big the file is - make sure don't exceed 100%.
    if (nSoFar > PROGRESS_MAX_C) then begin
      frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
    end
    else begin
      frmProgress.ProgressBar1.Position := nSoFar;
    end;
    frmProgress.ProcessWinMessages(MainForm);
    qryDS.Next;
  end;  // qryDC.Eof.

  // Done. Now add all Active entries into the tDataCollection table.
  qryDCCount.Close;
//  qryDCUpdate.Close;
  qryDSCount.Close;
  qryDS.Close;
[ start curly brackets
  with qryDCUpdate do begin
    Unprepare;
    SQL.Clear;
    SQL.Add('INSERT INTO tDataCollection');
    SQL.Add('(SequenceNo, ');
    SQL.Add(' ServiceID, ');
    SQL.Add(' CallCode, ');
    SQL.Add(' TransactionTypeID, ');
    SQL.Add(' RateDescription, ');
    SQL.Add(' AmountExGST, ');
    SQL.Add(' GSTAmount, ');
    SQL.Add(' AmountIncGST, ');
    SQL.Add(' GSTFlag, ');
    SQL.Add(' FromDate, ');
    SQL.Add(' ToDate, ');
    SQL.Add(' SupplierServiceCodeID, ');
    SQL.Add(' BatchID)');
    SQL.Add(' SELECT ');
    SQL.Add('   SequenceNo, ');
    SQL.Add('   ServiceID, ');
    SQL.Add('   CallCode, ');
    SQL.Add('   TransactionTypeID, ');
    SQL.Add('   RateDescription, ');
    SQL.Add('   AmountExGST, ');
    SQL.Add('   GSTAmount, ');
    SQL.Add('   AmountIncGST, ');
    SQL.Add('   GSTFlag, ');
    SQL.Add('   FromDate, ');
    SQL.Add('   ToDate, ');
    SQL.Add('   SupplierServiceCodeID, ');
    SQL.Add('   BatchID');
    SQL.Add(' FROM tDataCollect_Temp');
    SQL.Add(' WHERE Active = 1');
    ExecSQL;
  end;  // qryDCUpdate.
] end curly brackets
  slAddFieldNames.Free;
  slAddFieldValues.Free;
  qryDCCount.Free;
  qryDS.Free;
  qryDSCount.Free;
//  qryDCUpdate.Free;
end;  // SaveSummaryChargesFixedOBS.
}


begin
  lNextAccountNumber := False;
  sNextAccountNumber := '';
  nNextAccountNumber := 0;

  sLastDetailCode := '  ';
  nMaxResults := 0;
  nBatchID := nBatchNumber;
  sBatchID := IntToStr(nBatchID);
//  lContinue := True;    // False when input file is no longer for this account.
  lDataSaved := False;
  FieldStringList := TStringList.Create;
  ValueStringList := TStringList.Create;
  qryStringList := TStringList.Create;
  slAddFieldNames := TStringList.Create;      // Calls to AddToTable() only.
  slAddFieldValues := TStringList.Create;     // Calls to AddToTable() only.

  // Establish the query to be used to save the data read.
  qryDC := TQuery.Create(MainForm);
  qryDC.DatabaseName := 'dbPPdata';
  // And the temporary table to save potential summary data without call details.
  qryDS := TQuery.Create(MainForm);
  qryDS.DatabaseName := 'dbPPdata';
  qryDS.SQL.Add('DELETE');                    // Flush any old data left behind.
  qryDS.SQL.Add('FROM tDataCollect_Temp');
  qryDS.ExecSQL;
  
  // Make sure dates are defined.
  qryDC.SQL.Add('SELECT tBatch.BillingPeriod');
  qryDC.SQL.Add('FROM tBatch');
  qryDC.SQL.Add('WHERE tBatch.ID = '+sBatchID);
  qryDC.Open;
  SubString := qryDC.FieldByName('BillingPeriod').AsString;
  dStartBillingPeriod := StrDMYtoDate('01/'+Copy(SubString,6,2)+'/'+
    Copy(SubString,1,4));
  dEndBillingPeriod := LastDayInMonth(dStartBillingPeriod);
  qryDC.Close;
  qryDC.SQL.Clear;

//  LastEntryIs_1 := False;
//  SaveLastEntry_1 := False;
  lFirstDF := True;
  lReturnCode := False;
  CurrentFile := Fname;
  FileIteration := 0;               // Original file ext'n, then 001,002, etc.
  // Set the earliest a valid date is likely to be.
  dMinDate := EncodeDate(1980,1,1);
  // Set a dummy date for initialisation.
//  dBilling := dMinDate;
  if FileExists(CurrentFile) then begin
    // Read the file size to determine the approximate number of records.
    nFileSize := GetFileSize(CurrentFile);
    // Now we can calculate the factor for the progress bar to be displayed.
    // From prev. data: 30,930 lines in a file of 12,135,424 bytes.
    // Estimate the number of data lines in the file.
    if (nFileSize > 0) then begin
      nFactor := 100.0 / ((30930.0 / 12135424.0) * nFileSize);
    end
    else begin
      nFactor := 100.0;
    end;
    nCount := 0;
    sPartFileName := CurrentFile;
    while Pos('\',sPartFileName)>0 do begin
      nBackSlashAt  := Pos('\',sPartFileName);
      sPartFileName := Copy(sPartFileName,
        nBackSlashAt+1, Length(sPartFileName)-nBackSlashAt);
    end;
    frmProgress.Caption := 'Telstra Services OBS Batch: '+ sBatchID;
    frmProgress.Msg.Caption := 'Reading ' + sPartFileName + ' ...';
    frmProgress.ProgressBar1.Position := 0;
    frmProgress.Msg.Refresh;
    ConvertString := TStringList.Create;
    // Open the input (text) billing file for reading.
    AssignFile( TelstraFile, CurrentFile );
    Reset( TelstraFile );
    try
      Readln( TelstraFile, ReadString );
      // Ensure that this is a CSV file format (OBS billing)
      lContinue := (Pos(',',ReadString)>0);
      if not lContinue then begin
        frmProgress.Hide;
        MessageDlg('Incorrect input file type.'+#13#10+
          'File type CSV is required. Try again.',
          mtWarning, [mbOK],0);
      end
      else begin
        // Ensure that this is Not a Mobile file format.
        lContinue := (Copy(ReadString,1,2)<>'RB');
        if not lContinue then begin
          frmProgress.Hide;
          MessageDlg('Incorrect input file type.'+#13#10+
            'This is a Mobile-type file. Try again.',
            mtWarning, [mbOK],0);
        end;
      end;
      if not lContinue then begin
        // Account selected is not in this file.
        frmProgress.Hide;
        MessageDlg('The Account specified is not in the file selected.'+#13#10+
          'Please try again.',
          mtWarning, [mbOK],0);
      end
      else begin   // lContinue = True.
        // Convert the line read into the required format and variables.
        // Separate into strings before adding to ConvertString.
        ReadString := StringReplace(ReadString, ',', #13#10,
          [rfReplaceAll]) +'''';
        ConvertString.Text := ReadString;
        // Set the Header test value for evaluation of the string read.
        SubString := 'DH' + sAccountNumber;
        nLengthHeadAccount := Length(SubString);
      lContinue := False;  // Reset then test for correct account.
      // Make sure that we find the valid Account first.
      while not lContinue and
        (not ((Length(Trim(ReadString))=0) and Eof( TelstraFile ))) do begin
        lContinue := ((ConvertString.Strings[CODE_C]+
                       ConvertString.Strings[ACCT_C]) = SubString);
        if not lContinue then begin
          Readln( TelstraFile, ReadString );
          if (not ((Length(Trim(ReadString))=0) and Eof( TelstraFile )))
            then begin
              // Separate into strings before adding to ConvertString.
              ReadString := StringReplace(ReadString, ',', #13#10,
              [rfReplaceAll]) +'''';
              ConvertString.Text := ReadString;
          end;  // not empty or eof.
          // Report progress to the user.
          Inc(nCount);
          nDummy := nFactor * nCount;
          nSoFar := Round( nDummy );
          // Do not know how big the file is - do not exceed 100%.
          if (nSoFar > PROGRESS_MAX_C) then begin
            frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
          end
          else begin
            frmProgress.ProgressBar1.Position := nSoFar;
          end;  // PROGRESS_MAX_C.
        end;  // not lContinue.
      end;  // while <> SubString.
      if lContinue then begin
        // First, extract the billing date (6-char date on the first line).
        // Separate into strings before adding to ConvertString.
        ReadString := StringReplace(ReadString, ',', #13#10, [rfReplaceAll]) +'''';
        ConvertString.Text := ReadString;
        SubString := ConvertString.Strings[DESC10_C];
        dBilling := EncodeDate(StrToInt(Copy(SubString,1,4)),
          StrToInt(Copy(SubString,5,2)),
          StrToInt(Copy(SubString,7,2)));
//        D.StartTransaction;
        // Continue to search through the remaining text of the file
        // for occurrences of the search string.  On each find, process
        // the resultant string.
        //****************************************************************
        //********* START READING BILLS INTO 'tDataCollection' ***********
        //****************************************************************
        try
          // The following assumes that:
          //   DS section precedes the DE section;
          //   Sections are contiguous;
          //   Only the DS and HU (was:DE) sections are of interest here.
          nResultSet := -1;
          repeat
            frmProgress.ProcessWinMessages(MainForm);

            // Read only relevant text into the database.
            sInputLineCode := UpperCase(ConvertString.Strings[CODE_C]);

            // CALL DETAILS *************************************************
            // CALL DETAILS *************************************************
            if (sInputLineCode='DC') then begin
              FieldStringList.Clear;
              ValueStringList.Clear;
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := ConvertString.Strings[LINE_C];
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'SequenceNo,';
                sListValues := sListValues + '''' + SubString +''',';
              end;
              SubString := ConvertString.Strings[DESC07_C];
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+ AccountFormat(SubString) +''',';
              SubString := ConvertString.Strings[DESC19_C];
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'TxnDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end;
              SubString := ConvertString.Strings[DESC21_C];
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'TxnTime,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              SubString := ConvertString.Strings[DESC13_C];
              sListFields := sListFields + 'Destination,';
              sListValues := sListValues +''''+ SubString +''',';
              SubString := ConvertString.Strings[DESC11_C];
              SubString := ConvertTimeToSecs(CleanString(SubString));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Duration,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              SubString := ConvertString.Strings[DESC27_C];
              SubString := CleanString(SubString);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastCallCode <> SubString) then begin
                  sLastCallCode := SubString;
                  vSubString := SubString;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastCallTransactionTypeID :=
                      qryTelstra.FieldByName('TransactionType').AsString;
                    sLastSupplierCallCodeID :=
                      qryTelstra.FieldByName('SupplierCodeID').AsString;
                  end
                  else begin
                    sLastCallTransactionTypeID := sCallCodeID;
                    sLastSupplierCallCodeID := sSupplierCallCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastCallCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastCallTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    slAddFieldNames.Add('Active');
                    slAddFieldValues.Add('1');
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    with qryTelstra do begin
                      Close;
                      Open;
                      Locate('Code',vSubString,[loCaseInsensitive]);
                    end;  // qryTelstra.
                  end;  // if (not) lFound.
                end;  // if (sLastCallCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sLastCallTransactionTypeID+','+
                  sLastSupplierCallCodeID+',';
              end
              else begin
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sCallCodeID+','+
                  sSupplierCallCodeID+',';
              end;  // CallCode.
              SubString := ConvertString.Strings[DESC22_C];
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Peak,';
                if (Pos('N Peak',SubString)>0) then
                  sListValues := sListValues +'1,'
                else
                  sListValues := sListValues +'0,';
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;
              SubString := ConvertString.Strings[DESC13_C];
              sListFields := sListFields + 'DialledNumber,';
              sListValues := sListValues +''''+
              CleanString(SubString) +''',';
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';

              // Check costs are numbers and then save them.
              SubString := Trim(ConvertString.Strings[DESC32_C]);
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + SubString +',';
              SubString := ConvertString.Strings[DESC33_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              curValue := curFormat(StrToFloat(SubString));
              sListFields := sListFields + 'GSTAmount,GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues + SubString + ',''N'',';
              end
              else begin
                sListValues := sListValues + SubString + ',''Y'',';
              end;
              SubString := ConvertString.Strings[DESC34_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + SubString +',';
              SubString := ConvertString.Strings[DESC20_C];
              sListFields := sListFields + 'ToDate,';
              sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              sListFields := sListFields + 'BatchID)';
              sListValues := sListValues + sBatchID +')';

              // Now make the SQL statement to save to the database table.
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              qryStringList.Clear;
              qryStringList.Add('INSERT INTO tDataCollection');
              for nListValues := 0 to FieldStringList.Count-1 do begin
                qryStringList.Add(FieldStringList[nListValues]);
              end;
              for nListValues := 0 to ValueStringList.Count-1 do begin
                qryStringList.Add(ValueStringList[nListValues]);
              end;
              // Update and execute the query.
              qryDC.Close;
              qryDC.SQL := qryStringList;
              qryDC.ExecSQL;
              lDataSaved := True;
            end  // 'DC'.



            // CALL CHARGES IN SUMMARY SECTION ONLY *************************
            // CALL CHARGES IN SUMMARY SECTION ONLY *************************
            else if (
              (sInputLineCode='DS') and
               (Length(ConvertString.Strings[DESC12_C])>0) and not
               (ConvertString.Strings[DESC06_C]='001')
{               (ConvertString.Strings[DESC06_C]='003') and
               (
               (UpperCase(ConvertString.Strings[DESC13_C])='USES') or (
                 (UpperCase(ConvertString.Strings[DESC13_C])='CALLS') and
                 (Pos('TELEPHONE',UpperCase(ConvertString.Strings[DESC12_C]))=0) and
                 (ConvertString.Strings[DESC24_C]='039')
                )
               )
}
              ) then begin
              FieldStringList.Clear;           // 001=Rental - details in DF.
              ValueStringList.Clear;           // 003=Summary only - no details.
              sListFields := '(';              // 004=Total - details given.
              sListValues := 'VALUES (';
              SubString := ConvertString.Strings[LINE_C];
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'SequenceNo,';
                sListValues := sListValues + '''' + SubString +''',';
              end;
              SubString := ConvertString.Strings[DESC07_C];
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+ AccountFormat(SubString) +''',';
              SubString := ConvertString.Strings[DESC24_C];
              SubString := CleanString(SubString);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastCallCode <> SubString) then begin
                  sLastCallCode := SubString;
                  vSubString := SubString;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastCallTransactionTypeID :=
                      qryTelstra.FieldByName('TransactionType').AsString;
                    sLastSupplierCallCodeID :=
                      qryTelstra.FieldByName('SupplierCodeID').AsString;
                  end
                  else begin
                    sLastCallTransactionTypeID := sCallCodeID;
                    sLastSupplierCallCodeID := sSupplierCallCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastCallCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastCallTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    slAddFieldNames.Add('Active');
                    slAddFieldValues.Add('1');
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    with qryTelstra do begin
                      Close;
                      Open;
                      Locate('Code',vSubString,[loCaseInsensitive]);
                    end;  // qryTelstra.
                  end;  // if (not) lFound.
                end;  // if (sLastCallCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sLastCallTransactionTypeID+','+
                  sLastSupplierCallCodeID+',';
              end
              else begin
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sCallCodeID+','+
                  sSupplierCallCodeID+',';
              end;  // CallCode.
              SubString := IntToStr(StrToInt(ConvertString.Strings[DESC14_C]));
              sListFields := sListFields + 'NumCalls,';
              sListValues := sListValues + SubString +',';
              SubString := SubString+' No '+
                ConvertString.Strings[DESC12_C]+' '+
                ConvertString.Strings[DESC13_C];
              sListFields := sListFields + 'RateDescription,';
              sListValues := sListValues +''''+ CleanString(SubString) +''',';
{
              sListFields := sListFields + 'RateDescription, Active,';
              sListValues := sListValues +''''+ CleanString(SubString) +''',';
              if (Pos(' VPN ', SubString) > 0) then
                sListValues := sListValues +'1,'
              else
                sListValues := sListValues +'0,';
}
              SubString := ''''+ConvertString.Strings[DESC06_C]+''',';
              sListFields := sListFields + 'Var01,';
              sListValues := sListValues + SubString;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';

              // NOTE: Save currency as a string in this Temp table qryDS ONLY.
              SubString := ConvertString.Strings[DESC32_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + ''''+SubString +''',';
              SubString := ConvertString.Strings[DESC33_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              curValue := curFormat(StrToFloat(SubString));
              sListFields := sListFields + 'GSTAmount,GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues +''''+SubString+''',''N'',';
              end
              else begin
                sListValues := sListValues +''''+SubString+''',''Y'',';
              end;
              SubString := ConvertString.Strings[DESC34_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues +''''+SubString +''',';
              SubString := Trim(ConvertString.Strings[DESC15_C]);
              if (Length(SubString)>0) then begin
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end;
              SubString := ConvertString.Strings[DESC17_C];
              if (Length(SubString)>0) then begin
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end;
              sListFields := sListFields + 'BatchID)';
              sListValues := sListValues + sBatchID +')';

              // Now make the SQL statement to save to the database table.
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              qryStringList.Clear;
              qryStringList.Add('INSERT INTO tDataCollect_Temp');
              for nListValues := 0 to FieldStringList.Count-1 do begin
                qryStringList.Add(FieldStringList[nListValues]);
              end;
              for nListValues := 0 to ValueStringList.Count-1 do begin
                qryStringList.Add(ValueStringList[nListValues]);
              end;
              // Update and execute the query.
              qryDS.Close;
              qryDS.SQL := qryStringList;
              qryDS.ExecSQL;
              lDataSaved := True;
            end  // 'DS'.



            // LIST OF EQUIPMENT ********************************************
            // LIST OF EQUIPMENT ********************************************
            else if (sInputLineCode='DE') then begin
              // This line has the full ServiceID and all dates-commit to memory.
              SubString := ConvertString.Strings[DESC07_C]; // tServiceID.Service.
              Inc( nResultSet );
               SetLength( aResultSet, nResultSet + 1);
              aResultSet[ nResultSet ].Custno := SubString;
              // Save the billing period dates.
              SubString := ConvertString.Strings[DESC16_C]; // DateFrom.
              if (SubString = '') or (Length(SubString) <> 8) then begin
                dStartBillingPeriod := dBilling;
              end
              else begin
                dStartBillingPeriod := EncodeDate(StrToInt(Copy(SubString,1,4)),
                  StrToInt(Copy(SubString,5,2)),
                  StrToInt(Copy(SubString,7,2)));
              end;  // if (SubString = '') or (Length(SubString) <> 8) then.
              SubString := ConvertString.Strings[DESC17_C]; // DateTo.
              if (SubString = '') or (Length(SubString) <> 8) then begin
                dEndBillingPeriod := dBilling;
              end
              else begin
                dEndBillingPeriod := EncodeDate(StrToInt(Copy(SubString,1,4)),
                  StrToInt(Copy(SubString,5,2)),
                  StrToInt(Copy(SubString,7,2)));
              end;  // if (SubString = '') or (Length(SubString) <> 8) then.
              // Read the code:
              SubString := ConvertString.Strings[DESC25_C]; // Code.
              with aResultSet[ nResultSet ] do begin
                // Rent.
                if (RentFrom<dMinDate) or (dStartBillingPeriod<RentFrom) then
                  RentFrom := dStartBillingPeriod;
                if (RentTill < dMinDate) or (RentTill < dEndBillingPeriod) then
                  RentTill := dEndBillingPeriod;
                // Code.
                if (Length(SubString) > 0) then
                  ServCode := SubString;
              end;  // with aResultSet[ nResultSet ] do.
              lFirstDF := True;
            end  // 'DE'.


            // RENT DETAILS. ***************************************************
            // RENT DETAILS. ***************************************************
            else if (sInputLineCode='DF') then begin
              // This line is a detail rental - save it.
              // but the first thing is to correct the phone numbers.
              SubString := ConvertString.Strings[DESC07_C]; // tServiceID.Service.
              if lFirstDF then begin
                // Return to the start of the results - check the phone numbers.
                lFirstDF := False;
                nMaxResults := Length( aResultSet ) - 1;
                nResultSet := 0;
                sPartResultSet := Trim(Copy(aResultSet[nResultSet].Custno,1,10));
                if (aResultSet[ nResultSet ].Custno <> SubString) and
                  (sPartResultSet <> SubString) then begin
                  lFound := False;
                  for nResultSet := 0 to nMaxResults do begin
                    sPartResultSet :=
                      Trim(Copy(aResultSet[nResultSet].Custno,1,10));
                    if (aResultSet[ nResultSet ].Custno = SubString) or
                      (sPartResultSet = SubString) then begin
                      lFound := True;
                      break;
                    end;
                  end; // for nResultSet := 0 to nMaxResults do.
                  // Check that the array bounds have not been exceeded.
                  if ( nResultSet > nMaxResults ) then begin
                    // Return to the beginning of the array.
                    nResultSet := 0;
                  end;  // if ( nResultSet > nMaxResults ) then.
                end
                else begin
                  lFound := True;
                end;  // if (aResultSet[ nResultSet ].Custno <> SubString) and..
              end  // if lFirstDF then.
              else begin
                lFound := True;
                Inc( nResultSet );
                if (nResultSet > nMaxResults) then begin
                  nResultSet := nMaxResults;
                end;  // if (nResultSet > nMaxResults) then.
                sPartResultSet := Trim(Copy(aResultSet[nResultSet].Custno,1,10));
                if (aResultSet[ nResultSet ].Custno <> SubString) and
                  (sPartResultSet <> SubString) then begin
                  // Must search through the array until it is found.
                  lFound := False;
                  if (nResultSet > nMaxResults) then begin
                    nStartSearchAt := nMaxResults;
                  end
                  else begin
                    nStartSearchAt := nResultSet;
                  end;  // if (nResultSet > nMaxResults) then..else.
                  for nResultSet := 0 to nMaxResults do begin
                    sPartResultSet :=
                      Trim(Copy(aResultSet[nResultSet].Custno,1,10));
                    if (aResultSet[ nResultSet ].Custno <> SubString) and
                      (sPartResultSet <> SubString) then begin
                      lFound := True;
                      break;
                    end;  // if aResultSet[nResultSet].Custno=SubString then.
                  end;  // for nResultSet := 1 to Length( aResultSet ) do.
                  if not lFound then begin
                    nResultSet := nStartSearchAt;
                  end;  // if not lFound then.
                end;  // if (aResultSet[nResultSet].Custno<>SubString) then.
              end;  // if lFirstDF then.
              // Now make the SQL statement to store the data in the table.
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := ConvertString.Strings[LINE_C];
              if (Length(SubString) > 0) then begin
               sListFields := sListFields + 'SequenceNo,';
               sListValues := sListValues +''''+ SubString +''',';
              end;  // SequenceNo.
              SubString := AccountFormat(aResultSet[ nResultSet ].custno);
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+ SubString +''',';
              SubString := CleanString(aResultSet[ nResultSet ].ServCode);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastRentCode <> SubString) then begin
                  sLastRentCode := SubString;
                  vSubString := SubString;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastRentTransactionTypeID := qryTelstra['TransactionType'];
                    sLastSupplierRentCodeID := qryTelstra['SupplierCodeID'];
                  end
                  else begin
                    sLastRentTransactionTypeID := sRentCodeID;
                    sLastSupplierRentCodeID := sSupplierRentCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastRentCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''RENT''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''RENT''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastRentTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    slAddFieldNames.Add('Active');
                    slAddFieldValues.Add('1');
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    with qryTelstra do begin
                      Close;
                      Open;
                      Locate('Code',vSubString,[loCaseInsensitive]);
                    end;  // qryOptus.
                  end;  // if (not) lFound.
                end;  // if (sLastRentCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,'+
                    'SupplierServiceCodeID,';
                sListValues := sListValues + sLastRentTransactionTypeID+','+
                    sLastSupplierRentCodeID+',';
              end
              else begin
                sListFields := sListFields + 'TransactionTypeID,'+
                    'SupplierServiceCodeID,';
                sListValues := sListValues + sRentCodeID+','+
                    sSupplierRentCodeID+',';
              end;  // RentCode.
              SubString := ConvertString.Strings[DESC12_C];
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;  // RateDescription.

              // Get and confirm the amounts are numbers then save them.
              SubString := ConvertString.Strings[DESC15_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + SubString + ',';
              SubString := ConvertString.Strings[DESC16_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              curValue := curFormat(StrToFloat(SubString));
              sListFields := sListFields + 'GSTAmount,';
              sListValues := sListValues + SubString +',';
              sListFields := sListFields + 'GSTFlag,';
              if (curValue = 0.00) then
                sListValues := sListValues + '''N'','
              else
                sListValues := sListValues + '''Y'',';
              SubString := ConvertString.Strings[DESC17_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + SubString +',';
              if lFound then begin
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues +''''+
                  FormattedDateString(aResultSet[ nResultSet ].RentFrom) +''',';
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+
                  FormattedDateString(aResultSet[ nResultSet ].RentTill) +''',';
              end
              else begin
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues +''''+
                  YMD2DMY(ConvertString.Strings[TACC_C]) +''',';
              end;  // if lFound then.
              sListFields := sListFields + 'BatchID)';
              sListValues := sListValues + sBatchID +')';
              // Now save to the database table.
              qryStringList.Clear;
              qryStringList.Add('INSERT INTO tDataCollection');
              qryStringList.Add(sListFields);
              qryStringList.Add(sListValues);
              // Update and execute the query.
              qryDC.Close;
              qryDC.SQL := qryStringList;
              qryDC.ExecSQL;
              lDataSaved := True;
            end  // if (sInputLineCode='DF') then.


            // OTHER DETAILS (miscellaneous).
            // OTHER DETAILS (miscellaneous).
            else if (sInputLineCode='DO') then begin
              // This line is a detailed Other (Miscellaneous) charge - save it.
              // Now make the SQL statement to store the data in the table.
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := ConvertString.Strings[LINE_C];
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'SequenceNo,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+
                AccountFormat(ConvertString.Strings[DESC07_C]) +''',';
              SubString := ConvertString.Strings[DESC13_C];
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;
              SubString := ConvertString.Strings[DESC30_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + SubString +',';
              SubString := ConvertString.Strings[DESC31_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              sListFields := sListFields + 'GSTAmount,';
              sListValues := sListValues + ConvertString.Strings[DESC31_C] +',';
              curValue := curFormat(StrToFloat(SubString));
              sListFields := sListFields + 'GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues + '''N'',';
              end
              else begin
                sListValues := sListValues + '''Y'',';
              end;
              SubString := ConvertString.Strings[DESC32_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + SubString +',';
              sListFields := sListFields + 'FromDate,';
              sListValues := sListValues +''''+
              YMD2DMY(ConvertString.Strings[DESC16_C]) +''',';
              SubString := ConvertString.Strings[DESC17_C];
              if (Length(SubString) = 8) then begin
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end;

              SubString := CleanString(ConvertString.Strings[DESC23_C]);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastOtherCode <> SubString) then begin
                  sLastOtherCode := SubString;
                  vSubString := SubString;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastOtherTransactionTypeID :=
                      qryTelstra.FieldByName('TransactionType').AsString;
                    sLastSupplierOtherCodeID :=
                      qryTelstra.FieldByName('SupplierCodeID').AsString;
                  end
                  else begin
                    sLastOtherTransactionTypeID := sCallCodeID;
                    sLastSupplierOtherCodeID := sSupplierCallCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastOtherCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''OTHER''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''OTHER''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastOtherTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    slAddFieldNames.Add('Active');
                    slAddFieldValues.Add('1');
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    with qryTelstra do begin
                      Close;
                      Open;
                      Locate('Code',vSubString,[loCaseInsensitive]);
                    end;  // qryTelstra.
                  end;  // if (not) lFound.
                end;  // if (sLastOtherCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sLastOtherTransactionTypeID+','+
                  sLastSupplierOtherCodeID+',';
              end
              else if (Length(sOtherCodeID)>0) then begin
                // No codes provided - use defsult codes.
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues + sOtherCodeID +',';
                vSubString := sOtherCodeID;
                with qryTelstra do
                  lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                if lFound then begin
                  sLastOtherTransactionTypeID := qryTelstra['TransactionType'];
                  sLastSupplierOtherCodeID := qryTelstra['SupplierCodeID'];
                  sListFields := sListFields + 'TransactionTypeID,'+
                    'SupplierServiceCodeID,';
                  sListValues := sListValues + sLastOtherTransactionTypeID+','+
                    sLastSupplierOtherCodeID+',';
                end;  // if lFound then begin.
              end;  // OtherCode.
              sListFields := sListFields + 'BatchID)';
              sListValues := sListValues + sBatchID +')';
              // Now save to the database table.
              qryStringList.Clear;
              qryStringList.Add('INSERT INTO tDataCollection');
              qryStringList.Add(sListFields);
              qryStringList.Add(sListValues);
              // Update and execute the query.
              qryDC.Close;
              qryDC.SQL := qryStringList;
              qryDC.ExecSQL;
              lDataSaved := True;
            end  // if (sInputLineCode='DO') then.


            // BILLING ALL (summary).
            // BILLING ALL (summary).
            else if (sInputLineCode='DA') then begin
              // This line may be a detailed discount - save it.
              if (UpperCase(ConvertString.Strings[DESC10_C])='BILL ROUNDING') then begin
                // Now make the SQL statement to store the data in the table.
                sListFields := '(';
                sListValues := 'VALUES (';
                SubString := ConvertString.Strings[LINE_C];
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'SequenceNo,';
                  sListValues := sListValues +''''+ SubString +''',';
                end;
                sListFields := sListFields + 'ServiceID,';
                sListValues := sListValues + '''ROUNDING'',';
                SubString := ConvertString.Strings[DESC10_C];
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'RateDescription,';
                  sListValues := sListValues +''''+ CleanString(SubString) +''',';
                end;
                sListFields := sListFields + 'FromDate,';
                SubString := ConvertString.Strings[DESC13_C];
                if (Length(SubString) > 0) then begin
                  sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
                end
                else begin
                  sListValues := sListValues +''''+
                    FormattedDateString(dStartBillingPeriod) +''',';
                end;
                sListFields := sListFields + 'ToDate,';
                SubString := ConvertString.Strings[DESC14_C];
                if (Length(SubString) > 0) then begin
                  sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
                end
                else begin
                  sListValues := sListValues +''''+
                    FormattedDateString(dEndBillingPeriod) +''',';
                end;
                // Read the invoice amounts, check values then save.
                SubString := ConvertString.Strings[DESC19_C];
                if Length(SubString) = 0 then
                  SubString := '0.0'
                else begin
                  ProveReal(SubString, lValueIsReal);
                  if not lValueIsReal then SubString := '0.0';
                end;
                sListFields := sListFields + 'AmountExGST,';
                sListValues := sListValues + SubString +',';
                SubString := ConvertString.Strings[DESC21_C];
                if Length(SubString) = 0 then
                  SubString := '0.0'
                else begin
                  ProveReal(SubString, lValueIsReal);
                  if not lValueIsReal then SubString := '0.0';
                end;
                sListFields := sListFields + 'AmountIncGST,';
                sListValues := sListValues + SubString +',';
                SubString := ConvertString.Strings[DESC20_C];
                if Length(SubString) = 0 then
                  SubString := '0.0'
                else begin
                  ProveReal(SubString, lValueIsReal);
                  if not lValueIsReal then SubString := '0.0';
                end;
                curValue := curFormat(StrToFloat(SubString));
                sListFields := sListFields + 'GSTAmount,';
                sListValues := sListValues + SubString +',';
                sListFields := sListFields + 'GSTFlag,';
                if (curValue = 0.00) then begin
                  sListValues := sListValues + '''N'',';
                end
                else begin
                  sListValues := sListValues + '''Y'',';
                end;
                if (Length(sOtherCodeID)>0) then begin
                  // Code not provided - use generic ones.
                  sListFields := sListFields + 'CallCode,';
                  sListValues := sListValues + sOtherCodeID +',';
                  vSubString := sOtherCodeID;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastOtherTransactionTypeID := qryTelstra['TransactionType'];
                    sListFields := sListFields + 'TransactionTypeID,';
                    sListValues := sListValues + sLastOtherTransactionTypeID+',';
                  end;  // if lFound then begin.
                end;  // OtherCode.
                sListFields := sListFields + 'BatchID)';
                sListValues := sListValues + sBatchID +')';

                // Now save to the database table.
                qryStringList.Clear;
                qryStringList.Add('INSERT INTO tDataCollection');
                qryStringList.Add(sListFields);
                qryStringList.Add(sListValues);
                // Update and execute the query.
                qryDC.Close;
                qryDC.SQL := qryStringList;
                qryDC.ExecSQL;
                lDataSaved := True;
              end;  // BILL ROUNDING.
            end  // if (sInputLineCode='DA') then.


            // BILLING DETAILS (Discount Billing).
            // BILLING DETAILS (Discount Billing).
            else if (sInputLineCode='DB') then begin
              // This line is a detailed discount - save it only if the
              // total amount is not zero.
              // Read the invoice amounts, check values then save.
              SubString := ConvertString.Strings[DESC18_C];
              if Length(SubString) = 0 then
                SubString := '0.0'
              else begin
                ProveReal(SubString, lValueIsReal);
                if not lValueIsReal then SubString := '0.0';
              end;
              curValue := curFormat(StrToFloat(SubString));
              if (curValue = 0.00) then begin
                SubString := ConvertString.Strings[DESC12_C]; // DateFrom.
                if (Length(DBfromDate) = 0) and (Length(SubString)=8) then
                  DBfromDate := ''''+YMD2DMY(SubString)+''',';
                SubString := ConvertString.Strings[DESC13_C]; // DateTo.
                if (Length(DBtoDate) = 0) and (Length(SubString)=8) then
                  DBtoDate := ''''+YMD2DMY(SubString)+''',';
              end
              else begin
                // Now make the SQL statement to store the data in the table.
                sListFields := '(';
                sListValues := 'VALUES (';
                SubString := ConvertString.Strings[LINE_C];
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'SequenceNo,';
                  sListValues := sListValues +''''+ SubString +''',';
                end;
                sListFields := sListFields + 'ServiceID,';
                sListValues := sListValues +''''+
                  AccountFormat(ConvertString.Strings[DESC07_C]) +''',';
                SubString := ConvertString.Strings[DESC10_C];
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'RateDescription,';
                  sListValues := sListValues +''''+
                    CleanString(SubString) +''',';
                end;
                SubString := ConvertString.Strings[DESC12_C];
                if (Length(SubString) = 8) then begin
                  sListFields := sListFields + 'FromDate,';
                  sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
                  if (Length(DBfromDate) = 0) then
                    DBfromDate := ''''+YMD2DMY(SubString)+''',';
                end
                else if (Length(DBfromDate) > 0) then begin
                  sListFields := sListFields + 'FromDate,';
                  sListValues := sListValues + DBfromDate;
                end;  // FromDate.
                SubString := ConvertString.Strings[DESC13_C];
                if (Length(SubString) = 8) then begin
                  sListFields := sListFields + 'ToDate,';
                  sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
                  if (Length(DBtoDate) = 0) then
                    DBtoDate := ''''+YMD2DMY(SubString)+''',';
                end
                else if (Length(DBtoDate) > 0) then begin
                  sListFields := sListFields + 'ToDate,';
                  sListValues := sListValues + DBtoDate;
                end;  // ToDate.
                // Read the invoice amounts, check values then save.
                SubString := ConvertString.Strings[DESC18_C];
                if Length(SubString) = 0 then
                  SubString := '0.0'
                else begin
                  ProveReal(SubString, lValueIsReal);
                  if not lValueIsReal then SubString := '0.0';
                end;
                sListFields := sListFields + 'AmountExGST,';
                sListValues := sListValues + SubString +',';
                SubString := ConvertString.Strings[DESC25_C];
                if Length(SubString) = 0 then
                  SubString := '0.0'
                else begin
                  ProveReal(SubString, lValueIsReal);
                  if not lValueIsReal then SubString := '0.0';
                end;
                sListFields := sListFields + 'AmountIncGST,';
                sListValues := sListValues + SubString +',';
                SubString := ConvertString.Strings[DESC26_C];
                if Length(SubString) = 0 then
                  SubString := '0.0'
                else begin
                  ProveReal(SubString, lValueIsReal);
                  if not lValueIsReal then SubString := '0.0';
                end;
                sListFields := sListFields + 'GSTAmount,';
                sListValues := sListValues + SubString +',';
                curValue := curFormat(StrToFloat(SubString));
                sListFields := sListFields + 'GSTFlag,';
                if (curValue = 0.00) then begin
                  sListValues := sListValues + '''N'',';
                end
                else begin
                  sListValues := sListValues + '''Y'',';
                end;  // curValue = 0.00.
                if (Length(sOtherCodeID)>0) then begin
                  // No code provided - use the generic ones.
                  sListFields := sListFields + 'CallCode,';
                  sListValues := sListValues + sDiscountCodeID +',';
                  vSubString := sDiscountCodeID;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastDiscountTransactionTypeID := qryTelstra['TransactionType'];
                    sListFields := sListFields + 'TransactionTypeID,';
                    sListValues := sListValues + sLastDiscountTransactionTypeID+',';
                  end;  // lFound.
                end;  // OtherCode.

                sListFields := sListFields + 'BatchID)';
                sListValues := sListValues + sBatchID +')';

                // Now save to the database table ****************************
                // Now save to the database table ****************************
                qryStringList.Clear;
                qryStringList.Add('INSERT INTO tDataCollection');
                qryStringList.Add(sListFields);
                qryStringList.Add(sListValues);
                // Update and execute the query.
                qryDC.Close;
                qryDC.SQL := qryStringList;
                qryDC.ExecSQL;
                lDataSaved := True;
              end;  // if (curValue <> 0.00) then..else.
            end;  // if (sInputLineCode='DB') then.


            // Report progress to the user ***********************************
            // Report progress to the user ***********************************
            Inc(nCount);
            nDummy := nFactor * nCount;
            nSoFar := Round( nDummy );
            // Do not know how big the file is - make sure don't exceed 100%.
            if (nSoFar > PROGRESS_MAX_C) then begin
              frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
            end
            else begin
              frmProgress.ProgressBar1.Position := nSoFar;
            end;
            // Read the next line of text file data.
            Readln( TelstraFile, ReadString);
            if (Length(Trim(ReadString)) > 0) then begin
              // Separate into strings before adding to ConvertString.
              ReadString := StringReplace(ReadString, ',', #13#10, [rfReplaceAll]) +'''';
              ConvertString.Text := ReadString;
              // Make sure that we are still looking at the valid Account.
              lContinue := (ConvertString.strings[ACCT_C] = sAccountNumber);
              while not lContinue and not lNextAccountNumber and
                (not ((Length(Trim(ReadString))=0) and Eof( TelstraFile ))) do begin
                // Wait for 20 lines of the next account before ending processing.
                if (sNextAccountNumber = ConvertString.Strings[ACCT_C]) then begin
                  Inc(nNextAccountNumber);
                  lNextAccountNumber := (nNextAccountNumber = 20);
                end
                else begin
                  sNextAccountNumber := ConvertString.Strings[ACCT_C];
                  nNextAccountNumber := 1;
                end;
                Readln( TelstraFile, ReadString );
                if (not ((Length(Trim(ReadString))=0) and Eof( TelstraFile )))
                  then begin
                  // Separate into strings before adding to ConvertString.
                  ReadString := StringReplace(ReadString, ',', #13#10,
                    [rfReplaceAll]) +'''';
                  ConvertString.Text := ReadString;
                  lContinue := (ConvertString.strings[ACCT_C] = sAccountNumber)
                end;  // not empty or eof.
                // Report progress to the user.
                Inc(nCount);
                nDummy := nFactor * nCount;
                nSoFar := Round( nDummy );
                // Do not know how big the file is - do not exceed 100%.
                if (nSoFar > PROGRESS_MAX_C) then begin
                  frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
                end
                else begin
                  frmProgress.ProgressBar1.Position := nSoFar;
                end;  // PROGRESS_MAX_C.
//                end;  // not lContinue.
              end;  // while not lContinue.
            end;  // not Eof.
          until not lContinue or lNextAccountNumber or
            ((Length(Trim(ReadString))=0) and Eof( TelstraFile ));
//          SaveSummaryChargesFixedOBS(nBatchID);
          SaveTelstraSummaryCharges(nBatchID);
          lReturnCode := lDataSaved;;
        except                           // D.StartTransaction.
          lReturnCode := False;
//          D.Rollback;  // On failure, undo the changes - not useful anyway.
          raise;         // Raise the exception to avoid call to CommitUpdates.
        end;  // D.StartTransaction.
        //****************************************************************
        //*********** END READ BILLS INTO 'tDataCollection' **************
        //****************************************************************
        // Now need to process the last line if CR-LF not at end.
        //  BUT - nothing done !
//        D.Commit;
      end;  // lContinue.
      end;  // not lContinue.
    finally                            // TelstraFile (try).
      // Done with the input file; close it.
      CloseFile( TelstraFile );
      FieldStringList.Free;
      ValueStringList.Free;
      qryStringList.Free;
      qryDC.Close;
      qryDC.Free;
    end;                               // TelstraFile (try).
    // Select the next file if exists - extension 001, 002, etc.
    Inc(FileIteration);
    CurrentFile := Copy(CurrentFile,1,Pos('.',CurrentFile));
    CurrentFile := CurrentFile + Right(('00'+IntToStr(FileIteration)),3);
  end;  // if FileExists(CurrentFile) then begin.
  qryDS.Close;
  qryDS.Free;
end;  // GetDetailFixedOBS.


procedure BillingInputFixedOBS( stInFiles: Tstrings; nBatch: LongInt );
// This program processes the Telstra billing reports supplied on disk
// and places the data into the Temp.dbf database for unverified bill
// information.
//
// The parameter passed contains the names of the input files, which are
// passed on to the ImportFile() procedure.
//
var
  lContinue : Boolean;
  sPartFileName : String;
  nBackSlashAt: Integer;
  nPos : integer;
begin
  // initialise the Static table elements defined as Private for this form.
  lContinue := True;
  qryTelstra := TQuery.Create(MainForm);
  qryTelstra.DatabaseName := 'dbPPdata';
  qryTelstra.SQL.Add('SELECT id');
  qryTelstra.SQL.Add('FROM tSupplier');
  qryTelstra.SQL.Add('WHERE SupplierName LIKE ''TELSTRA%''');
  qryTelstra.Open;
  if qryTelstra.Eof then begin
    MessageDlg('TELSTRA not found in the tSupplier table. Processing stopped.',
      mtWarning, [mbOk], 0);
    lContinue := False;
  end
  else begin
    nSupplierID := qryTelstra['id'];
    sSupplierID := IntToStr(nSupplierID);
    qryTelstra.Close;
    qryTelstra.SQL.Clear;
    with qryTelstra do begin
      SQL.Add('SELECT tValidAccountNumber.AccountNo');
      SQL.Add('FROM tValidAccountNumber');
      SQL.Add('INNER JOIN tBatch ON tValidAccountNumber.BatchTypeID = tBatch.BatchTypeID');
      SQL.Add('WHERE tBatch.ID = '+IntToStr(nBatch));
      Open;
      if Eof then begin
        MessageDlg('BatchType not found in the tValidAccountNumber table. '+
          'Processing stopped.',
          mtWarning, [mbOk], 0);
        lContinue := False;
      end
      else begin
        sAccountNumber := Trim(FieldByName('AccountNo').AsString);
        nPos := Pos(' ',sAccountNumber);
        while (nPos > 0) do begin
          sAccountNumber := Copy(sAccountNumber,1,nPos-1) +
                            Copy(sAccountNumber,nPos+1,Length(sAccountNumber)-nPos);
          nPos := Pos(' ',sAccountNumber);
        end;  // nPos.
      end;  // Eof.
      Close;
    end;  // qryTelstra.
  end;
  if lContinue then begin
    qryTelstra.SQL.Clear;
    qryTelstra.SQL.Add('SELECT Code, CAST(TransactionTypeID AS VARCHAR(20)) '+
      'AS TransactionType, CAST(ID AS VARCHAR(20)) AS SupplierCodeID');
    qryTelstra.SQL.Add('FROM tSupplierServiceCode');
    qryTelstra.SQL.Add('WHERE SupplierID = '+sSupplierID);
    qryTelstra.SQL.Add('AND Active = 1');
    qryTelstra.SQL.Add('AND Description LIKE ''RENT''');
    qryTelstra.Open;
    if qryTelstra.Eof then sRentCodeID := 'NULL'
    else begin
      sRentCodeID := Trim(qryTelstra['Code']);
      sSupplierRentCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Strings[qryTelstra.SQL.Count-1] :=
      'AND Description LIKE ''CALLS''';
    qryTelstra.Open;
    if qryTelstra.Eof then sCallCodeID := 'NULL'
    else begin
      sCallCodeID := Trim(qryTelstra['Code']);
      sSupplierCallCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Strings[qryTelstra.SQL.Count-1] :=
      'AND Description LIKE ''Unknown%''';
    qryTelstra.Open;
    if qryTelstra.Eof then sOtherCodeID := 'NULL'
    else begin
      sOtherCodeID := qryTelstra['Code'];
      sSupplierOtherCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Strings[qryTelstra.SQL.Count-1] :=
      'AND Description LIKE ''MISCELLANEOUS''';
    qryTelstra.Open;
    if qryTelstra.Eof then sDiscountCodeID := 'NULL'
    else begin
      sDiscountCodeID := Trim(qryTelstra['Code']);
      sSupplierDiscountCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Delete(qryTelstra.SQL.Count-1);
    qryTelstra.Open;

    // Initialise the progress bar to display to the operator.
    frmProgress := TProgressForm.Create(MainForm);
    frmProgress.btnCancel.Visible := False;
    frmProgress.Show;

    sPartFileName := stInFiles.Strings[0];
    while Pos('\',sPartFileName)>0 do begin
      nBackSlashAt  := Pos('\',sPartFileName);
      sPartFileName := Copy(sPartFileName,
        nBackSlashAt+1, Length(sPartFileName)-nBackSlashAt);
    end;  // while Pos('\',sPartFileName)>0 do begin.
    frmProgress.Msg.Caption := 'Reading ' + sPartFileName + ' ...';
    frmProgress.ProgressBar1.Position := 0;
    frmProgress.Msg.Refresh;
    // Now move the text data into the Table.
    GetDetailFixedOBS( stInFiles.Strings[0], DM.dbPPdata, nBatch, lContinue);

    frmProgress.Hide;
    frmProgress.Free;
    if lContinue then begin
      UpdateBatchStatus(nBatch, MainForm.BATCH_STATUS_IMPORT);

      // **********************************************************************
      // **********************************************************************
      // Run Test - critical errors that assesses the data for the user.
      lContinue := False;
      CriticalErrorTest( nBatch, lContinue );
      if lContinue then begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Passed all tests for Critical Errors.', mtInformation,
          [mbOk], 0);
      end
      else begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Critical errors found.'#13#10#13#10'These must be resolved ' +
          'before this invoice can be progressed.',
          mtWarning, [mbOk], 0);
      end;
    end;  // if lContinue then begin.
  end;  // if qryTelstra.Eof then begin..else.
end;  // BillingInputFixedOBS.


procedure TelstraInvoiceFixedOBS( nBatch: LongInt; sDirectory: string;
                         nProgram: LongInt );
// This function prompts the user for the name of the Telstra OBS supplied
// CSV text file and then calls the BillingInput function (which reads the
// file into a table and controls its processing).
var
  lContinue : Boolean;
  stFiles : TStringList;
  OpenDlg: TOpenDialog;
begin
  // Create the stringlist which contains the names of selected filenames.
  stFiles := TStringList.Create;
  try
    // Now, we must establish which files are to be processed.
    // Select the source directory.
    OpenDlg := TOpenDialog.Create(MainForm);
    OpenDlg.Filter := 'Telstra OBS Billing file (*.csv)|*.CSV|All files (*.*)|*.*';
    OpenDlg.FilterIndex := 1;
    // Use the program directory as the default directory.
    if (Length(sDirectory) > 0) then OpenDlg.InitialDir := sDirectory
    else OpenDlg.InitialDir := ExtractFilePath(ParamStr(0));
    OpenDlg.Title := 'Select The Telstra Account OBS Data File';
    OpenDlg.Options := [ofFileMustExist, ofNoChangeDir];
    // Read and save the user's choices before continuing.
    lContinue := OpenDlg.Execute;
    stFiles.Text := OpenDlg.Files.Text;
    OpenDlg.Free;
    if lContinue then begin
      lContinue := stFiles.Count > 0;
      // Now call for processing of the input files.
      if lContinue then begin
        lThisIsAPABX := (nProgram = PIMPORTBILLINGFILE_TELSTRA_OBS_PABX);//Global.
        BillingInputFixedOBS(stFiles, nBatch);
      end;  // if lContinue then.
    end;  // if lContinue then.
  finally
    stFiles.Free;
  end;
end;    // TelstraInvoiceFixedOBS.


end.  // TelstraFixed.

