unit TelstraFixedOrigEBS;
// Revision History.
{
RJC 111204: Telstra() split into 2-TelstraPhoneEBS & TelstraMobileEBS to allow
            separate programs for fixed & mobile services with the old EBS
            billing files. This is to allow multiple invoices in hte one file
            and code added to sift through (same as for new OBS system). The
            ImportFile & GetMobileAccountBills routines also deleted.
RJC 090504: Calls - DialledNumber shifted 3 Chars left, and Origin 2 Chars left
                    a/c errors in field for SMS calls.
RJC 120325: BillingPhoneInput - Replaced TransactionType with Code for default
                                rent, call, other, discount Telstra codes.
            GetPhoneAccountBills - Added reading code from file, not default.
}


interface
                                
uses
  Messages, SysUtils, Classes, Graphics, Controls, Dialogs,
  db, dbTables;

type
  TServiceDetails = record
    Custno   : string;
    ServCode : string;
    ServiceID: string;
    RentFrom : TDateTime;
    RentTill : TDateTime;
    CallFrom : TDateTime;
    CallTill : TDateTime;
  end;

  TCallHeaderType = record
    SequenceNo      : string[7];
    ServiceID       : string[17];
    TxnDate         : string[8];
    TxnTime         : string[5];
    Origin          : string[20];
    Duration        : string[9];
    CallCode        : string[2];
    RateDescription : string[83];  // Was 32: RJC-080706.
    DialledNumber   : string[17];  // Temporary variable only.
    ServiceNotes    : string[80];
    AmountExGST     : string[13];
    GSTAmount       : string[13];
    AmountIncGST    : string[13];
    ToDate          : string[8];
  end;

  procedure GetDetailFixedOrigEBS( const FName:String;
    D:TDatabase;
    nBatchNumber:LongInt;
    var lReturnCode:Boolean);
  procedure BillingInputFixedOrigEBS( stInFiles: Tstrings; nBatch: LongInt );
  procedure TelstraInvoiceFixedOrigEBS( nBatch: LongInt; sDirectory: string );


implementation

uses
  Math,
  Main, DataMod, GenFns, DateFunctions, FileFunctions,
  CriticalTests, Progress;

var
  // Declare static variables as frequently reset and reused.
  nCount: Integer;                   // Current status for frmProgress.
  nFactor: Real;                     // Weighting for nCount in progress.
  frmProgress: TProgressForm;        // frmProgress report to the operator.
  nSupplierID: integer;              // Set in BillingInput().
  sSupplierID: string;               // Set in BillingInput().
  qryTelstra : TQuery;               // Set in BillingInput().
  sRentCodeID : string;              // Set in BillingInput().
  sCallCodeID : string;              // Set in BillingInput().
  sOtherCodeID : string;             // Set in BillingInput().
  sDiscountCodeID : string;          // Set in BillingInput().
  sSupplierRentCodeID : string;      // Set in BillingInput().
  sSupplierCallCodeID : string;      // Set in BillingInput().
  sSupplierOtherCodeID : string;     // Set in BillingInput().
  sSupplierDiscountCodeID : string;  // Set in BillingInput().
  sAccountNumber : string;           // Set in BillingInput().

  sLastCallCode : string;
  sLastRentCode : string;
  sLastOtherCode: string;
  sLastCallTransactionTypeID : string;
  sLastRentTransactionTypeID : string;
  sLastOtherTransactionTypeID : string;
  sLastDiscountTransactionTypeID : string;
  sLastSupplierCallCodeID : string;
  sLastSupplierRentCodeID : string;
  sLastSupplierOtherCodeID : string;
//  sLastSupplierDiscountCodeID : string;
  
procedure GetDetailFixedOrigEBS
          (
           const FName: String;
           D: TDatabase;
           nBatchNumber : LongInt;
           var lReturnCode: Boolean
           );
const
  PROGRESS_MAX_C: Integer = 100;       // Maximum progress is 100%.
var
  nFileSize: Integer;                  // File size in bytes.
  lFirstDF : Boolean;
  lFirstDE : Boolean;
  sPartFileName : string;
  sPartResultSet : string;
  EntryCode : string;
  nBackSlashAt : Integer;
  TelstraFile: TextFile;               // Used for accessing the data.
  CurrentFile: String;                 // Currently open file.
  FileIteration: Integer;              // Current file iteration being processed
  T: string;                           // Line of data read from TextFile.
  SubString: String;                   // A (temp) substring of T.
  vSubString : variant;                // - ditto, used in the Locate() function.
  curValue: Currency;                  // A (temp) currency conversion value.
  nSoFar: Integer;                     // Status - num of records processed.
  dBilling: TDateTime;                 // Billing date.
  dStartBillingPeriod: TDateTime;      // Start of the payment billing period.
  dEndBillingPeriod: TDateTime;        // End of the payment billing period.
  nBatch: LongInt;
  LastEntryIs_1 : boolean;
  SaveLastEntry_1 : boolean;           // Set to save when last DC line is LastEntry_1.
  CallHeader : TCallHeaderType;
  sLastDetailCode : string[2];         // To detect a header line AFTER details.

  DBfromDate : string;                 // Section DB - save read dates.
  DBtoDate : string;

  aResultSet : Array of TServiceDetails;
  nResultSet : Integer;
  nMaxResults : Integer;
  nStartSearchAt : Integer;
  lFound : Boolean;
  dMinDate : TDateTime;
  nCommaPosition : integer;
  sListValues : String;
  sListFields : String;
  FieldStringList : TStringList;
  ValueStringList : TStringList;
  qryStringList : TStringList;
  slAddFieldNames : TStringList;         // Calls to AddToTable() only.
  slAddFieldValues : TStringList;        // Calls to AddToTable() only.
  qryDC : TQuery;
  nListValues : integer;
  nDummy : Real;
  sInputLineCode : string;
  lContinue : boolean;
  lDataSaved : boolean;
  nDataCollectionID : LongInt;
begin
  sLastDetailCode := '  ';
  nMaxResults := 0;
  nBatch := nBatchNumber;
//  lContinue := False;
  lDataSaved := False;

  FieldStringList := TStringList.Create;
  ValueStringList := TStringList.Create;
  qryStringList := TStringList.Create;
  slAddFieldNames := TStringList.Create;         // Calls to AddToTable() only.
  slAddFieldValues := TStringList.Create;        // Calls to AddToTable() only.

  // Establish the query to be used to save the data read.
  qryDC := TQuery.Create(MainForm);
  qryDC.DatabaseName := 'dbPPdata';

  // Make sure dates are defined.
  qryDC.SQL.Add('SELECT tBatch.BillingPeriod');
  qryDC.SQL.Add('FROM tBatch');
  qryDC.SQL.Add('WHERE tBatch.ID = '+IntToStr(nBatch));
  qryDC.Open;
  SubString := qryDC.FieldByName('BillingPeriod').AsString;
  dStartBillingPeriod := StrDMYtoDate('01/'+Copy(SubString,6,2)+'/'+
    Copy(SubString,1,4));
  dEndBillingPeriod := LastDayInMonth(dStartBillingPeriod);
  qryDC.Close;
  qryDC.SQL.Clear;

  LastEntryIs_1 := False;
  SaveLastEntry_1 := False;
  lFirstDF := True;
  lReturnCode := False;
  CurrentFile := Fname;
  FileIteration := 0;               // Original file ext'n, then 001,002, etc.
  // Set the earliest a valid date is likely to be.
  dMinDate := EncodeDate(1980,1,1);
  // Set a dummy date for initialisation.
  dBilling := dMinDate;
  if FileExists(CurrentFile) then begin
    // Read the file size to determine the approximate number of records.
    nFileSize := GetFileSize(CurrentFile);
    // Now we can calculate the factor for the progress bar to be displayed.
    // From prev. data: 30,930 lines in a file of 12,135,424 bytes.
    // Estimate the number of data lines in the file.
    if (nFileSize > 0) then begin
      nFactor := 100.0 / ((30930.0 / 12135424.0) * nFileSize);
    end
    else begin
      nFactor := 100.0;
    end;
    nCount := 0;
    sPartFileName := CurrentFile;
    while Pos('\',sPartFileName)>0 do begin
      nBackSlashAt  := Pos('\',sPartFileName);
      sPartFileName := Copy(sPartFileName,
        nBackSlashAt+1, Length(sPartFileName)-nBackSlashAt);
    end;
    frmProgress.Caption := 'Telstra Fixed Services Batch: '+IntToStr(nBatch);
    frmProgress.Msg.Caption := 'Reading ' + sPartFileName + ' ...';
    frmProgress.ProgressBar1.Position := 0;
    frmProgress.Msg.Refresh;
    // Open the input (text) billing file for reading.
    AssignFile( TelstraFile, CurrentFile );
    Reset( TelstraFile );
    try
      Readln( TelstraFile, T );
      // Test for the original EBS file format for a Mobile or Data.
      if (Copy(T,1,8)='HDR:FCAB') then begin
        // This is an account for Fixed services.
        lContinue := True;
      end
      else begin
        lContinue := (Copy(T,1,8)<>'HDR:RACE');
        if not lContinue then begin
          frmProgress.Hide;
          MessageDlg('Incorrect input file type.'+#13#10+
            'This file is for a Mobile Account. Try again.',
            mtWarning, [mbOK],0);
        end
        else begin
          // It may be a new combined file. Continue to test.
          // Ensure that this is not a CSV file format (OBS billing)
          lContinue := (Pos(',',T)=0);
          if not lContinue then begin
            frmProgress.Hide;
            MessageDlg('Incorrect input file type.'+#13#10+
              'File type DAT is required. Try again.', mtWarning, [mbOK],0);
          end
          else begin
            // Ensure that this is a Mobile file format.
            lContinue := (Copy(T,1,2)<>'RB');
            if not lContinue then begin
              frmProgress.Hide;
              MessageDlg('Incorrect input file type.'+#13#10+
                'This file is for a Mobile Account. Try again.',
                mtWarning, [mbOK],0);
            end;
          end;
        end;
      end; // HDR:RACE.

      if lContinue then begin
      // Skip to the Header Line and Determine the billing date only once.
      // Make sure that we find the valid Account first.
      lContinue := False;  // Reset then test for correct account.
      while not lContinue and
        (not ((Length(Trim(T))=0) and Eof( TelstraFile ))) do begin
        lContinue := (Trim(Copy(T,3,10)) = sAccountNumber);
        if not lContinue then begin
          Readln( TelstraFile, T);
          // Report progress to the user.
          Inc(nCount);
          nDummy := nFactor * nCount;
          nSoFar := Round( nDummy );
          // Do not know how big the file is - do not exceed 100%.
          if (nSoFar > PROGRESS_MAX_C) then begin
            frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
          end
          else begin
            frmProgress.ProgressBar1.Position := nSoFar;
          end;  // PROGRESS_MAX_C.
        end;  // not lContinue.
      end;  // while <> SubString.

      // Determine the billing date only once - for the first file only.
      if FileIteration = 0 then begin
        // First, extract the billing date (8-char date on the first valid line).
        SubString := Copy(T,13,8);
        dBilling := EncodeDate(StrToInt(Copy(SubString,1,4)),
          StrToInt(Copy(SubString,5,2)),
          StrToInt(Copy(SubString,7,2)));
      end;
//      D.StartTransaction;
      // Continue to search through the remaining text of the file
      // for occurrences of the search string.  On each find, process
      // the resultant string.
      //****************************************************************
      //********* START READING BILLS INTO 'tDataCollection' ***********
      //****************************************************************
      try
        // The following assumes that:
        //   DS section precedes the DE section;
        //   Sections are contiguous;
        //   Only the DS and HU (was:DE) sections are of interest here.
        nResultSet := -1;
        repeat
          frmProgress.ProcessWinMessages(MainForm);

          // Read only relevant text into the database.
          sInputLineCode := UpperCase(Copy(T,1,2));
          // BILL INFORMATION *********************************************
          if (sInputLineCode='BI') then begin
            // Do nothing - electronic copy of the printed bill, always
            // first in the input file.
          end

          // CALL DETAILS *************************************************
          else if (sInputLineCode='DC') or SaveLastEntry_1 then begin
            EntryCode := Copy(T,441,1);
            if SaveLastEntry_1 or ((EntryCode='3') or
              ((EntryCode='1') and (LastEntryIs_1 or
              (LowerCase(Trim(Copy(T,406,10)))='uses')))) then begin
              // This line is a detail call & not a header/summary - save it.
              LastEntryIs_1 := LastEntryIs_1 and ((EntryCode='1') or SaveLastEntry_1);
              FieldStringList.Clear;
              ValueStringList.Clear;
              sListFields := '(';
              sListValues := 'VALUES (';
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.SequenceNo)
              else
                SubString := Trim(Copy(T,352,7));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'SequenceNo,';
                sListValues := sListValues + '''' + SubString +''',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.ServiceID)
              else
                SubString := Trim(Copy(T,45,17));
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+ AccountFormat(SubString) +''',';
              // Read the code common for each detail and summary - used below
              // when the summary (type "1") is read.
              sLastDetailCode := Copy(T,67,2);                    // Entry code.
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.TxnDate)
              else
                SubString := Copy(T,197,8);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'TxnDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.TxnTime)
              else
                SubString := Copy(T,205,8);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'TxnTime,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.Origin)
              else
                SubString := Trim(Copy(T,147,20));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Origin,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.Duration)
              else
                SubString := Trim(Copy(T,70,9));
              SubString := ConvertTimeToSecs(CleanString(SubString));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Duration,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.CallCode)
              else
                SubString := Trim(Copy(T,268,2));
              SubString := CleanString(SubString);
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastCallCode <> SubString) then begin
                  sLastCallCode := SubString;
                  vSubString := SubString;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then begin
                    sLastCallTransactionTypeID :=
                      qryTelstra.FieldByName('TransactionType').AsString;
                    sLastSupplierCallCodeID :=
                      qryTelstra.FieldByName('SupplierCodeID').AsString;
                  end
                  else begin
                    sLastCallTransactionTypeID := sCallCodeID;
                    sLastSupplierCallCodeID := sSupplierCallCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastCallCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastCallTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    slAddFieldNames.Add('Active');
                    slAddFieldValues.Add('1');
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    with qryTelstra do begin
                      Close;
                      Open;
                      Locate('Code',vSubString,[loCaseInsensitive]);
                    end;  // qryTelstra.
                  end;  // if (not) lFound.
                end;  // if (sLastCallCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sLastCallTransactionTypeID+','+
                  sLastSupplierCallCodeID+',';
              end
              else begin
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sCallCodeID+','+
                  sSupplierCallCodeID+',';
              end;  // CallCode.
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.RateDescription)
              else
                SubString := Trim(Copy(T,213,32));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'Peak,';
                if (Pos('N Peak',SubString)>0) then
                  sListValues := sListValues +'1,'
                else
                  sListValues := sListValues +'0,';
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end
              else if LastEntryIs_1 then begin
                SubString := CleanString(Trim(CallHeader.ServiceNotes));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'RateDescription,';
                  sListValues := sListValues +''''+ SubString +''',';
                end;
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.DialledNumber)
              else
                SubString := Trim(Copy(T,89,17));
              if (Length(SubString) = 0) then begin
                if LastEntryIs_1 then begin
                  // Summary of calls made - assume these are Local/Untimed.
                  // Do nothing - ServicesNotes saved as RateDescription above.
                  // - RJC 070517.
//                  sListFields := sListFields + 'DialledNumber,';
//                  sListValues := sListValues +''''+
//                    CallHeader.ServiceNotes +''',';
                end
                else if (EntryCode='3') then begin
                  // Summary of calls made - assume these are Local/Untimed.
                  sListFields := sListFields + 'DialledNumber,';
                  sListValues := sListValues +''''+
                    CleanString(Trim(Copy(T,375,10))) + ' Local Calls'',';
                end
                else if (EntryCode='1') then begin
                  // Summary of calls made - assume these are Local/Untimed.
                  sListFields := sListFields + 'DialledNumber,';
                  sListValues := sListValues +''''+
                    CleanString(Trim(Copy(T,399,6))) +
                    ' uses of Network Features'',';
                end;
              end
              else begin
                sListFields := sListFields + 'DialledNumber,';
                sListValues := sListValues +''''+
                CleanString(SubString) +''',';
              end;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';

              if LastEntryIs_1 then
                SubString := Trim(CallHeader.AmountExGST)
              else
                SubString := Trim(Copy(T,312,13));
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + SubString +',';
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.GSTAmount)
              else
                SubString := Trim(Copy(T,325,13));
              curValue := curFormat(StrToFloat(SubString));
              sListFields := sListFields + 'GSTAmount,GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues + SubString + ',''N'',';
              end
              else begin
                sListValues := sListValues + SubString + ',''Y'',';
              end;
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.AmountIncGST)
              else
                SubString := Trim(Copy(T,338,13));
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + SubString +',';
              if LastEntryIs_1 then
                SubString := Trim(CallHeader.ToDate)
              else
                SubString := Trim(Copy(T,13,8));
//                SubString := Trim(Copy(T,14,8)); // 20050119 RJC.
              sListFields := sListFields + 'ToDate,';
              sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              sListFields := sListFields + 'BatchID)';
              sListValues := sListValues + IntToStr(nBatch) +')';

              // Now make the SQL statement to save to the database table.
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              qryStringList.Clear;
              qryStringList.Add('INSERT INTO tDataCollection');
              for nListValues := 0 to FieldStringList.Count-1 do begin
                qryStringList.Add(FieldStringList[nListValues]);
              end;
              for nListValues := 0 to ValueStringList.Count-1 do begin
                qryStringList.Add(ValueStringList[nListValues]);
              end;
              // Update and execute the query.
              qryDC.Close;
              qryDC.SQL := qryStringList;
              qryDC.ExecSQL;
              lDataSaved := True;

              if LastEntryIs_1 and not SaveLastEntry_1 then begin
                // Save the values on this summary line in case the summary-only
                // entry is provided.   (Copied from following else-if section.)
                sLastDetailCode := Copy(T,67,2);
                with CallHeader do begin
                  ToDate := Copy(T,13,8);
//                  ToDate := Copy(T,14,8);  // 20060119 RJC.
                  ServiceID := Copy(T,45,17);
                  Duration := Copy(T,70,9);
                  SubString := CleanString(Copy(T,89,17));  // Temporary variable.
                  DialledNumber := SubString;
                  Origin := CleanString(Copy(T,147,20));
                  TxnDate := Copy(T,197,8);
                  TxnTime := Copy(T,205,8);
// RateDescription := CleanString(Copy(T,213,32)); RJC-080706.
                  RateDescription := Trim(CleanString(Copy(T,213,32)) +' '+
                    CleanString(Copy(T,360,50)));
                  CallCode := Copy(T,268,2);
                  AmountExGST := Copy(T,312,13);
                  GSTAmount := Copy(T,325,13);
                  AmountIncGST := Copy(T,338,13);
                  SequenceNo := Copy(T,352,7);
//                  if (Length(Trim(SubString))=0) then  // empty DialledNumber.
                    // Save extra messages.
//                    ServiceNotes  := Trim(Copy(T,388,6)) + ' Local Calls';
                    ServiceNotes  := Trim(Copy(T,353,80));
//                  else
//                    ServiceNotes  := ''; // when empty DialledNumber.
                end;  // with CallHeader do.
              end;  // if LastEntryIs_1. (Copied from section immed. following.)
            end
            else if (EntryCode='1') then begin
              // Save the values on this line in case the summary-only entry
              // is provided (usually only once each invoice).
              //
              // Test to ensure the summary line is not following the details by
              // comparing with the values stored in the previous entry/detail.
              if (sLastDetailCode = Copy(T,67,2)) then begin
                // Summary does follow detail.
                LastEntryIs_1 := False;
              end
              else begin
                // Summary does not follow detail, save it in case no details.
                LastEntryIs_1 := True;
                sLastDetailCode := Copy(T,67,2);
              end;
              with CallHeader do begin
                SequenceNo := Copy(T,352,7);
                ServiceID := Copy(T,45,17);
                TxnDate := Copy(T,197,8);
                TxnTime := Copy(T,205,8);
                Origin := CleanString(Copy(T,147,20));
                Duration := Copy(T,70,9);
                CallCode := Copy(T,268,2);
                RateDescription := CleanString(Copy(T,213,32)) +' '+
                  CleanString(Copy(T,360,50));
                SubString := CleanString(Copy(T,89,17));  // Temporary variable.
                DialledNumber := SubString;
                if (Length(Trim(SubString))=0) then begin // empty DialledNumber.
                  // Save extra messages.
                  ServiceNotes  := Trim(Copy(T,388,6)) + ' Local Calls';
                end
                else begin
                  ServiceNotes  := ''; // when empty DialledNumber.
                end;
                AmountExGST := Copy(T,312,13);
                GSTAmount := Copy(T,325,13);
                AmountIncGST := Copy(T,338,13);
                ToDate := Copy(T,13,8);
//                ToDate := Copy(T,14,8);  // 20060119 RJC.
              end;  // with CallHeader do.
            end  // else if (EntryCode='1') then.

            // Anything else goes here.
            else begin
              // Catch all other instances and reset checking variables.
              LastEntryIs_1 := False;
              sLastDetailCode := Copy(T,65,2);
            end;  // else if (EntryCode='1') then.
          end  // 'DC'.



          // LIST OF EQUIPMENT ********************************************
          else if (sInputLineCode='DE') then begin
            // This line has the full service ID and all dates-commit to memory.
            SubString := Trim(Copy(T,45,17));
            Inc( nResultSet );
            SetLength( aResultSet, nResultSet + 1);
            aResultSet[ nResultSet ].Custno := SubString;
            // Save the billing period dates.
            SubString := Trim(Copy(T,253,8));
            if (SubString = '') or (Length(SubString) <> 8) then begin
              dStartBillingPeriod := dBilling;
            end
            else begin
              dStartBillingPeriod := EncodeDate(StrToInt(Copy(SubString,1,4)),
                StrToInt(Copy(SubString,5,2)),
                StrToInt(Copy(SubString,7,2)));
            end;  // if (SubString = '') or (Length(SubString) <> 8) then.
            SubString := Trim(Copy(T,261,8));
            if (SubString = '') or (Length(SubString) <> 8) then begin
              dEndBillingPeriod := dBilling;
            end
            else begin
              dEndBillingPeriod := EncodeDate(StrToInt(Copy(SubString,1,4)),
                StrToInt(Copy(SubString,5,2)),
                StrToInt(Copy(SubString,7,2)));
            end;  // if (SubString = '') or (Length(SubString) <> 8) then.
            // Read the code:
            SubString := UpperCase(Trim(Copy(T,331,2)));
            with aResultSet[ nResultSet ] do begin
              // Rent.
              if (RentFrom<dMinDate) or (dStartBillingPeriod<RentFrom) then
                RentFrom := dStartBillingPeriod;
              if (RentTill < dMinDate) or (RentTill < dEndBillingPeriod) then
                RentTill := dEndBillingPeriod;
              // Code.
              if (Length(SubString) > 0) then
                ServCode := SubString;
            end;  // with aResultSet[ nResultSet ] do.
            lFirstDF := True;
          end  // 'DE'.


          // RENT DETAILS. ***************************************************
          else if (sInputLineCode='DF') then begin
            // This line is a detail rental - save it.
            // but the first thing is to correct the phone numbers.
            SubString := Trim(Copy(T,45,17));
            if lFirstDF then begin
              // Return to the start of the results - check the phone numbers.
              lFirstDF := False;
              nMaxResults := Length( aResultSet ) - 1;
              nResultSet := 0;
              sPartResultSet := Trim(Copy(aResultSet[nResultSet].Custno,1,10));
              if (aResultSet[ nResultSet ].Custno <> SubString) and
                (sPartResultSet <> SubString) then begin
                lFound := False;
                for nResultSet := 0 to nMaxResults do begin
                  sPartResultSet :=
                    Trim(Copy(aResultSet[nResultSet].Custno,1,10));
                  if (aResultSet[ nResultSet ].Custno = SubString) or
                    (sPartResultSet = SubString) then begin
                    lFound := True;
                    break;
                  end;
                end; // for nResultSet := 0 to nMaxResults do.
                // Check that the array bounds have not been exceeded.
                if ( nResultSet > nMaxResults ) then begin
                  // Return to the beginning of the array.
                  nResultSet := 0;
                end;  // if ( nResultSet > nMaxResults ) then.
              end
              else begin
                lFound := True;
              end;  // if (aResultSet[ nResultSet ].Custno <> SubString) and..
            end  // if lFirstDF then.
            else begin
              lFound := True;
              Inc( nResultSet );
              if (nResultSet > nMaxResults) then begin
                nResultSet := nMaxResults;
              end;  // if (nResultSet > nMaxResults) then.
              sPartResultSet := Trim(Copy(aResultSet[nResultSet].Custno,1,10));
              if (aResultSet[ nResultSet ].Custno <> SubString) and
                (sPartResultSet <> SubString) then begin
                // Must search through the array until it is found.
                lFound := False;
                if (nResultSet > nMaxResults) then begin
                  nStartSearchAt := nMaxResults;
                end
                else begin
                  nStartSearchAt := nResultSet;
                end;  // if (nResultSet > nMaxResults) then..else.
                for nResultSet := 0 to nMaxResults do begin
                  sPartResultSet :=
                    Trim(Copy(aResultSet[nResultSet].Custno,1,10));
                  if (aResultSet[ nResultSet ].Custno <> SubString) and
                    (sPartResultSet <> SubString) then begin
                    lFound := True;
                    break;
                  end;  // if aResultSet[nResultSet].Custno=SubString then.
                end;  // for nResultSet := 1 to Length( aResultSet ) do.
                if not lFound then begin
                  nResultSet := nStartSearchAt;
                end;  // if not lFound then.
              end;  // if (aResultSet[nResultSet].Custno<>SubString) then.
            end;  // if lFirstDF then.
            // Now make the SQL statement to store the data in the table.
            sListFields := '(';
            sListValues := 'VALUES (';
            SubString := Trim(Copy(T,33,9));
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'SequenceNo,';
              sListValues := sListValues +''''+ SubString +''',';
            end;  // SequenceNo.
            SubString := AccountFormat(aResultSet[ nResultSet ].custno);
            sListFields := sListFields + 'ServiceID,';
            sListValues := sListValues +''''+ SubString +''',';
            SubString := CleanString(aResultSet[ nResultSet ].ServCode);
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'CallCode,';
              sListValues := sListValues +''''+ SubString +''',';
              if (sLastRentCode <> SubString) then begin
                sLastRentCode := SubString;
                vSubString := SubString;
                with qryTelstra do
                  lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                if lFound then begin
                  sLastRentTransactionTypeID := qryTelstra['TransactionType'];
                  sLastSupplierRentCodeID := qryTelstra['SupplierCodeID'];
                end
                else begin
                  sLastRentTransactionTypeID := sRentCodeID;
                  sLastSupplierRentCodeID := sSupplierRentCodeID;
                  slAddFieldNames.Clear;
                  slAddFieldValues.Clear;
                  slAddFieldNames.Add('Code');
                  slAddFieldValues.Add(''''+sLastRentCode+'''');
                  slAddFieldNames.Add('AmountCategory');
                  slAddFieldValues.Add('''RENT''');
                  slAddFieldNames.Add('Description');
                  slAddFieldValues.Add('''RENT''');
                  slAddFieldNames.Add('TransactionTypeID');
                  slAddFieldValues.Add(sLastRentTransactionTypeID);
                  slAddFieldNames.Add('SupplierID');
                  slAddFieldValues.Add(sSupplierID);
                  slAddFieldNames.Add('Active');
                  slAddFieldValues.Add('1');
                  AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                  with qryTelstra do begin
                    Close;
                    Open;
                    Locate('Code',vSubString,[loCaseInsensitive]);
                  end;  // qryOptus.
                end;  // if (not) lFound.
              end;  // if (sLastCallCode <> SubString) then begin.
              sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
              sListValues := sListValues + sLastRentTransactionTypeID+','+
                  sLastSupplierRentCodeID+',';
            end
            else begin
              sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
              sListValues := sListValues + sRentCodeID+','+
                  sSupplierRentCodeID+',';
            end;  // RentCode.
            SubString := Trim(Copy(T,98,23));
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'RateDescription,';
              sListValues := sListValues +''''+ CleanString(SubString) +''',';
            end;  // RateDescription.
            sListFields := sListFields + 'AmountExGST,';
            sListValues := sListValues + Copy(T,149,13) + ',';
            curValue := curFormat(StrToFloat(Copy(T,162,13)));
            sListFields := sListFields + 'GSTAmount,';
            sListValues := sListValues + Copy(T,162,13) +',';
            sListFields := sListFields + 'GSTFlag,';
            if (curValue = 0.00) then
              sListValues := sListValues + '''N'','
            else
              sListValues := sListValues + '''Y'',';
            sListFields := sListFields + 'AmountIncGST,';
            sListValues := sListValues + Copy(T,175,13) +',';
            if lFound then begin
              sListFields := sListFields + 'FromDate,';
              sListValues := sListValues +''''+
                FormattedDateString(aResultSet[ nResultSet ].RentFrom) +''',';
              sListFields := sListFields + 'ToDate,';
              sListValues := sListValues +''''+
                FormattedDateString(aResultSet[ nResultSet ].RentTill) +''',';
            end
            else begin
              sListFields := sListFields + 'FromDate,';
              sListValues := sListValues +''''+ YMD2DMY(Copy(T,13,8)) +''',';
// sListValues := sListValues +''''+ YMD2DMY(Copy(T,14,8)) +''',';//20060119 RJC.
            end;  // if lFound then.
            sListFields := sListFields + 'BatchID)';
            sListValues := sListValues + IntToStr(nBatch) +')';
            // Now save to the database table.
            qryStringList.Clear;
            qryStringList.Add('INSERT INTO tDataCollection');
            qryStringList.Add(sListFields);
            qryStringList.Add(sListValues);
            // Update and execute the query.
            qryDC.Close;
            qryDC.SQL := qryStringList;
            qryDC.ExecSQL;
            lDataSaved := True;
          end  // if (sInputLineCode='DF') then.


          // OTHER DETAILS (miscellaneous).
          else if (sInputLineCode='DO') then begin
            // This line is a detailed Other (Miscellaneous) charge - save it.
            // Now make the SQL statement to store the data in the table.
            sListFields := '(';
            sListValues := 'VALUES (';
            SubString := Trim(Copy(T,33,9));
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'SequenceNo,';
              sListValues := sListValues +''''+ SubString +''',';
            end;
            sListFields := sListFields + 'ServiceID,';
            sListValues := sListValues +''''+
              AccountFormat(Trim(Copy(T,45,17))) +''',';
            SubString := Trim(Copy(T,82,70));
            if (Length(Trim(Copy(T,153,15))) > 0) then begin
              if (Length(SubString) > 0) then begin
                SubString := SubString + ' (' + Trim(Copy(T,153,15)) + ')';
              end
              else begin
                SubString := Trim(Copy(T,153,15));
              end;  // if (Length(SubString) > 0) then.
            end;  // if (Length(Trim(Copy(T,153,15))) > 0) then.
            if (Length(Trim(Copy(T,268,90))) > 0) then begin
              if (Length(SubString) > 0) then begin
                SubString := SubString + ' - ';
              end;  // if (Length(SubString) > 0) then.
              SubString := SubString + SingleSpaces(Trim(Copy(T,268,90)));
              SubString := SubString;
            end;  // if (Length(Trim(Copy(T,268,120))) > 0) then.
            if (Length(Trim(Copy(T,358,30))) > 0) then begin
              if (Length(SubString) > 0) then begin
                SubString := SubString + ' - ';
              end;  // if (Length(SubString) > 0) then.
              SubString := SubString + SingleSpaces(Trim(Copy(T,358,30)));
            end;  // if (Length(Trim(Copy(T,268,120))) > 0) then.
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'RateDescription,';
              sListValues := sListValues +''''+ CleanString(SubString) +''',';
            end;
            sListFields := sListFields + 'AmountExGST,';
            sListValues := sListValues + Copy(T,414,13) +',';
            sListFields := sListFields + 'GSTAmount,';
            sListValues := sListValues + Copy(T,427,13) +',';
            curValue := curFormat(StrToFloat(Copy(T,427,13)));
            sListFields := sListFields + 'GSTFlag,';
            if (curValue = 0.00) then begin
              sListValues := sListValues + '''N'',';
            end
            else begin
              sListValues := sListValues + '''Y'',';
            end;
            sListFields := sListFields + 'AmountIncGST,';
            sListValues := sListValues + Copy(T,440,13) +',';
            sListFields := sListFields + 'ToDate,';
            sListValues := sListValues +''''+ YMD2DMY(Copy(T,13,8)) +''',';
// sListValues := sListValues +''''+ YMD2DMY(Copy(T,14,8)) +''',';// 20060119 RJC.
            SubString := Trim(Copy(T,230,2));
            SubString := CleanString(SubString);
            if (Length(SubString) > 0) then begin
              sListFields := sListFields + 'CallCode,';
              sListValues := sListValues +''''+ SubString +''',';
              if (sLastOtherCode <> SubString) then begin
                sLastOtherCode := SubString;
                vSubString := SubString;
                with qryTelstra do
                  lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                if lFound then begin
                  sLastOtherTransactionTypeID :=
                    qryTelstra.FieldByName('TransactionType').AsString;
                  sLastSupplierOtherCodeID :=
                    qryTelstra.FieldByName('SupplierCodeID').AsString;
                end
                else begin
                  sLastOtherTransactionTypeID := sOtherCodeID;
                  sLastSupplierOtherCodeID := sSupplierOtherCodeID;
                  slAddFieldNames.Clear;
                  slAddFieldValues.Clear;
                  slAddFieldNames.Add('Code');
                  slAddFieldValues.Add(''''+sLastOtherCode+'''');
                  slAddFieldNames.Add('AmountCategory');
                  slAddFieldValues.Add('''OTHER''');
                  slAddFieldNames.Add('Description');
                  slAddFieldValues.Add('''OTHER''');
                  slAddFieldNames.Add('TransactionTypeID');
                  slAddFieldValues.Add(sLastOtherTransactionTypeID);
                  slAddFieldNames.Add('SupplierID');
                  slAddFieldValues.Add(sSupplierID);
                  slAddFieldNames.Add('Active');
                  slAddFieldValues.Add('1');
                  AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                  with qryTelstra do begin
                    Close;
                    Open;
                    Locate('Code',vSubString,[loCaseInsensitive]);
                  end;  // qryTelstra.
                end;  // if (not) lFound.
              end;  // if (sLastCallCode <> SubString) then begin.
              sListFields := sListFields + 'TransactionTypeID,'+
                'SupplierServiceCodeID,';
              sListValues := sListValues + sLastOtherTransactionTypeID+','+
                sLastSupplierOtherCodeID+',';
            end
            else if (Length(sOtherCodeID)>0) then begin
              // No codes provided - use defsult codes.
              sListFields := sListFields + 'CallCode,';
              sListValues := sListValues + sOtherCodeID +',';
              vSubString := sOtherCodeID;
              with qryTelstra do
                lFound := Locate('Code',vSubString,[loCaseInsensitive]);
              if lFound then begin
                sLastOtherTransactionTypeID := qryTelstra['TransactionType'];
                sLastSupplierOtherCodeID := qryTelstra['SupplierCodeID'];
                sListFields := sListFields + 'TransactionTypeID,'+
                  'SupplierServiceCodeID,';
                sListValues := sListValues + sLastOtherTransactionTypeID+','+
                  sLastSupplierOtherCodeID+',';
              end;  // if lFound then begin.
            end;  // OtherCode.
            sListFields := sListFields + 'BatchID)';
            sListValues := sListValues + IntToStr(nBatch) +')';
            // Now save to the database table.
            qryStringList.Clear;
            qryStringList.Add('INSERT INTO tDataCollection');
            qryStringList.Add(sListFields);
            qryStringList.Add(sListValues);
            // Update and execute the query.
            qryDC.Close;
            qryDC.SQL := qryStringList;
            qryDC.ExecSQL;
            lDataSaved := True;
          end  // if (sInputLineCode='DO') then.


          // BILLING ALL (summary).
          else if (sInputLineCode='DA') then begin
            // This line may be a detailed discount - save it.
            if (UpperCase(Trim(Copy(T,65,15)))='BILL ROUNDING') then begin
              // Now make the SQL statement to store the data in the table.
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := Trim(Copy(T,33,9));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'SequenceNo,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues + '''ROUNDING'',';
              SubString := Trim(Copy(T,65,80));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ CleanString(SubString) +''',';
              end;
              sListFields := sListFields + 'FromDate,';
              SubString := Trim(Copy(T,159,8));
              if (Length(SubString) > 0) then begin
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end
              else begin
                sListValues := sListValues +''''+
                  FormattedDateString(dStartBillingPeriod) +''',';
              end;
              sListFields := sListFields + 'ToDate,';
              SubString := Trim(Copy(T,167,8));
              if (Length(SubString) > 0) then begin
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
              end
              else begin
                sListValues := sListValues +''''+
                  FormattedDateString(dEndBillingPeriod) +''',';
              end;
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + Copy(T,220,13) +',';
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + Copy(T,233,13) +',';
              curValue := curFormat(StrToFloat(Copy(T,246,13)));
              sListFields := sListFields + 'GSTAmount,';
              sListValues := sListValues + Copy(T,246,13) +',';
              sListFields := sListFields + 'GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues + '''N'',';
              end
              else begin
                sListValues := sListValues + '''Y'',';
              end;
              if (Length(sOtherCodeID)>0) then begin
                // Code not provided - use generic ones.
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues + sOtherCodeID +',';
                vSubString := sOtherCodeID;
                with qryTelstra do
                  lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                if lFound then begin
                  sLastOtherTransactionTypeID := qryTelstra['TransactionType'];
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sLastOtherTransactionTypeID+',';
                end;  // if lFound then begin.
              end;  // OtherCode.
              sListFields := sListFields + 'BatchID)';
              sListValues := sListValues + IntToStr(nBatch) +')';

              // Now save to the database table.
              qryStringList.Clear;
              qryStringList.Add('INSERT INTO tDataCollection');
              qryStringList.Add(sListFields);
              qryStringList.Add(sListValues);
              // Update and execute the query.
              qryDC.Close;
              qryDC.SQL := qryStringList;
              qryDC.ExecSQL;
              lDataSaved := True;
            end;  // BILL ROUNDING.
          end  // if (sInputLineCode='DA') then.


          // BILLING DETAILS (account discounts).
          else if (sInputLineCode='DB') then begin
            // This line is a detailed discount - save it only if the
            // total amount is not zero.
            curValue := curFormat(StrToFloat(Copy(T,300,13)));
            if (curValue = 0.00) then begin
              SubString := Trim(Copy(T,175,8));
              if (Length(DBfromDate) = 0) and (Length(SubString)=8) then
                DBfromDate := ''''+YMD2DMY(SubString)+''',';
              SubString := Trim(Copy(T,183,8));
              if (Length(DBtoDate) = 0) and (Length(SubString)=8) then
                DBtoDate := ''''+YMD2DMY(SubString)+''',';
            end
            else begin
              // Now make the SQL statement to store the data in the table.
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := Trim(Copy(T,33,9));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'SequenceNo,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+
                AccountFormat(Trim(Copy(T,45,17))) +''',';
              SubString := Trim(Copy(T,65,80));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+
                  CleanString(SubString) +''',';
              end;
              SubString := Trim(Copy(T,175,8));
              if (Length(SubString) = 8) then begin
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
                if (Length(DBfromDate) = 0) then
                  DBfromDate := ''''+YMD2DMY(SubString)+''',';
              end
              else if (Length(DBfromDate) > 0) then begin
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues + DBfromDate;
              end;  // FromDate.
              SubString := Trim(Copy(T,183,8));
              if (Length(SubString) = 8) then begin
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+ YMD2DMY(SubString) +''',';
                if (Length(DBtoDate) = 0) then
                  DBtoDate := ''''+YMD2DMY(SubString)+''',';
              end
              else if (Length(DBtoDate) > 0) then begin
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues + DBtoDate;
              end;  // ToDate.
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + Copy(T,223,11) +',';
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + Copy(T,300,13) +',';
              sListFields := sListFields + 'GSTAmount,';
              sListValues := sListValues + Copy(T,313,13) +',';
              curValue := curFormat(StrToFloat(Copy(T,313,13)));
              sListFields := sListFields + 'GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues + '''N'',';
              end
              else begin
                sListValues := sListValues + '''Y'',';
              end;  // if (curValue = 0.00) then.
              if (Length(sOtherCodeID)>0) then begin
                // No code provided - use the generic ones.
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues + sDiscountCodeID +',';
                vSubString := sDiscountCodeID;
                with qryTelstra do
                  lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                if lFound then begin
                  sLastDiscountTransactionTypeID := qryTelstra['TransactionType'];
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sLastDiscountTransactionTypeID+',';
                end;  // lFound.
              end;  // OtherCode.

              sListFields := sListFields + 'BatchID)';
              sListValues := sListValues + IntToStr(nBatch) +')';

              // Now save to the database table.
              qryStringList.Clear;
              qryStringList.Add('INSERT INTO tDataCollection');
              qryStringList.Add(sListFields);
              qryStringList.Add(sListValues);
              // Update and execute the query.
              qryDC.Close;
              qryDC.SQL := qryStringList;
              qryDC.ExecSQL;
              lDataSaved := True;
            end;  // if (curValue <> 0.00) then..else.
          end;  // if (sInputLineCode='DB') then.


          Inc(nCount);
          // Report progress to the user.
          nDummy := nFactor * nCount;
          nSoFar := Round( nDummy );
          // Do not know how big the file is - make sure don't exceed 100%.
          if (nSoFar > PROGRESS_MAX_C) then begin
            frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
          end
          else begin
            frmProgress.ProgressBar1.Position := nSoFar;
          end;
          if SaveLastEntry_1 then begin
            // Reset parameters, then continue with the last read data.
            SaveLastEntry_1 := False;
            LastEntryIs_1   := False;
          end
          else begin
            // Read the next line of text file data.
            Readln( TelstraFile, T );


            if (Length(Trim(T)) > 0) then begin
              lContinue := (Trim(Copy(T,3,10)) = sAccountNumber);
              while not lContinue and
                (not ((Length(Trim(T))=0) and Eof( TelstraFile ))) do begin
                lContinue := (Trim(Copy(T,3,10)) = sAccountNumber);
                if not lContinue then begin
                  Readln( TelstraFile, T);
                  // Report progress to the user.
                  Inc(nCount);
                  nDummy := nFactor * nCount;
                  nSoFar := Round( nDummy );
                  // Do not know how big the file is - do not exceed 100%.
                  if (nSoFar > PROGRESS_MAX_C) then begin
                    frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
                  end
                  else begin
                    frmProgress.ProgressBar1.Position := nSoFar;
                  end;
                end;  // not lContinue.
              end;  // while not AccountNumber.
            end;  // not Eof.


            // Test if the last entry for 'DC' must be saved.
            if LastEntryIs_1 and (Length(T) > 0) and not Eof(TelstraFile) then begin
              if (UpperCase(Copy(T,1,2)) = 'DC') then begin
                SubString := Trim(CleanString(Copy(T,213,32)) +' '+
                    CleanString(Copy(T,360,50)));
                if ((Pos('Local ',SubString)>0) and (Pos('(NDD1)',SubString)=0)) then begin
                  SubString := CallHeader.RateDescription;
                  // Reset LastEntryIs_1 if consecutive headings for Local calls.
                  LastEntryIs_1 :=
                    ((Pos('Local ',SubString)=0) or (Pos('(NDD1)',SubString)>0));
                end;
              end;
            end;  // LastEntryIs_1...
            // Test if the last entry for 'DC' must be saved.
            SaveLastEntry_1 := LastEntryIs_1 and (Length(T) > 0) and
              not Eof(TelstraFile) and (UpperCase(Copy(T,1,2)) <> 'DC');
          end;
        until (T='') and Eof( TelstraFile );
        lReturnCode := lDataSaved;;
      except                           // D.StartTransaction.
        lReturnCode := False;
//        D.Rollback;  // On failure, undo the changes - not useful anyway.
        raise;         // Raise the exception to avoid call to CommitUpdates.
      end;  // D.StartTransaction.

      end;  // lContinue.

      //****************************************************************
      //*********** END READ BILLS INTO 'tDataCollection' **************
      //****************************************************************
      // Now need to process the last line if CR-LF not at end.
      //  BUT - nothing done !
//      D.Commit;
    finally                            // TelstraFile (try).
      // Done with the input file; close it.
      CloseFile( TelstraFile );
      FieldStringList.Free;
      ValueStringList.Free;
      qryStringList.Free;
      qryDC.Close;
      qryDC.Free;
    end;                               // TelstraFile (try).
    // Select the next file if exists - extension 001, 002, etc.
    Inc(FileIteration);
    CurrentFile := Copy(CurrentFile,1,Pos('.',CurrentFile));
    CurrentFile := CurrentFile + Right(('00'+IntToStr(FileIteration)),3);
  end;  // if FileExists(CurrentFile) then begin.
end;  // GetPhoneAccountBills.


procedure BillingInputFixedOrigEBS( stInFiles: Tstrings; nBatch: LongInt );
// This program processes the Telstra billing reports supplied on disk
// and places the data into the tDataCollection table for unverified bill
// information.
//
// The parameter passed contains the names of the input files, which are
// passed on to the GetDetailFixedOrigEBS() procedure.
//
var
  lContinue : Boolean;
  sPartFileName : String;
  nBackSlashAt: Integer;
  nPos : integer;
begin
  // initialise the Static table elements defined as Private for this form.
  qryTelstra := TQuery.Create(MainForm);
  qryTelstra.DatabaseName := 'dbPPdata';
  qryTelstra.SQL.Add('SELECT id');
  qryTelstra.SQL.Add('FROM tSupplier');
  qryTelstra.SQL.Add('WHERE SupplierName LIKE ''TELSTRA%''');
  qryTelstra.Open;
  if qryTelstra.Eof then begin
    MessageDlg('TELSTRA not found in the tSupplier table. Processing stopped.',
      mtWarning, [mbOk], 0);
  end
  else begin
    nSupplierID := qryTelstra['id'];
    sSupplierID := IntToStr(nSupplierID);

    with qryTelstra do begin
      Close;
      SQL.Clear;
      SQL.Add('SELECT tValidAccountNumber.AccountNo');
      SQL.Add('FROM tValidAccountNumber');
      SQL.Add('INNER JOIN tBatch ON tValidAccountNumber.BatchTypeID = tBatch.BatchTypeID');
      SQL.Add('WHERE tBatch.ID = '+IntToStr(nBatch));
      Open;
      if Eof then begin
        MessageDlg('BatchType not found in the tValidAccountNumber table. '+
          'Processing stopped.',
          mtWarning, [mbOk], 0);
        lContinue := False;
      end
      else begin
        sAccountNumber := Trim(FieldByName('AccountNo').AsString);
        nPos := Pos(' ',sAccountNumber);
        while (nPos > 0) do begin
          sAccountNumber := Copy(sAccountNumber,1,nPos-1) +
                            Copy(sAccountNumber,nPos+1,Length(sAccountNumber)-nPos);
          nPos := Pos(' ',sAccountNumber);
        end;  // nPos.
        lContinue := True;
      end;  // Eof.
      Close;
    end;  // qryTelstra.
  end;
  if lContinue then begin
    with qryTelstra do begin
      Close;
      SQL.Clear;
      SQL.Add('SELECT Code, CAST(TransactionTypeID AS VARCHAR(20)) '+
        'AS TransactionType, CAST(ID AS VARCHAR(20)) AS SupplierCodeID');
      SQL.Add('FROM tSupplierServiceCode');
      SQL.Add('WHERE SupplierID = '+sSupplierID);
      SQL.Add('AND Active = 1');
      SQL.Add('AND Description LIKE ''RENT''');
      Open;
    end;  // qryTelstra.
    if qryTelstra.Eof then sRentCodeID := 'NULL'
    else begin
      sRentCodeID := Trim(qryTelstra['Code']);
      sSupplierRentCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Strings[qryTelstra.SQL.Count-1] :=
      'AND Description LIKE ''CALLS''';
    qryTelstra.Open;
    if qryTelstra.Eof then sCallCodeID := 'NULL'
    else begin
      sCallCodeID := Trim(qryTelstra['Code']);
      sSupplierCallCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Strings[qryTelstra.SQL.Count-1] :=
      'AND Description LIKE ''Unknown%''';
    qryTelstra.Open;
    if qryTelstra.Eof then sOtherCodeID := 'NULL'
    else begin
      sOtherCodeID := qryTelstra['Code'];
      sSupplierOtherCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Strings[qryTelstra.SQL.Count-1] :=
      'AND Description LIKE ''MISCELLANEOUS''';
    qryTelstra.Open;
    if qryTelstra.Eof then sDiscountCodeID := 'NULL'
    else begin
      sDiscountCodeID := Trim(qryTelstra['Code']);
      sSupplierDiscountCodeID := Trim(qryTelstra['SupplierCodeID']);
    end;
    qryTelstra.Close;
    qryTelstra.SQL.Delete(qryTelstra.SQL.Count-1);
    qryTelstra.Open;

    // Initialise the progress bar to display to the operator.
    frmProgress := TProgressForm.Create(MainForm);
    frmProgress.btnCancel.Visible := False;
    frmProgress.Show;

    sPartFileName := stInFiles.Strings[0];
    while Pos('\',sPartFileName)>0 do begin
      nBackSlashAt  := Pos('\',sPartFileName);
      sPartFileName := Copy(sPartFileName,
        nBackSlashAt+1, Length(sPartFileName)-nBackSlashAt);
    end;  // while Pos('\',sPartFileName)>0 do begin.
    frmProgress.Msg.Caption := 'Reading ' + sPartFileName + ' ...';
    frmProgress.ProgressBar1.Position := 0;
    frmProgress.Msg.Refresh;
    // Now move the text data into the Table.
//
// MainForm.StartTime;
//
    GetDetailFixedOrigEBS(stInFiles.Strings[0], DM.dbPPdata, nBatch, lContinue);
//
// MainForm.EndTime;
//

    frmProgress.Hide;
    frmProgress.Free;
    if lContinue then begin
      UpdateBatchStatus(nBatch, MainForm.BATCH_STATUS_IMPORT);

      // **********************************************************************
      // **********************************************************************
      // Test for critical errors that prevent progress to the next stage.
      lContinue := False;
      CriticalErrorTest( nBatch, lContinue );
      if lContinue then begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Passed all tests for Critical Errors.', mtInformation,
          [mbOk], 0);
      end
      else begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Critical errors found.'#13#10#13#10'These must be resolved ' +
          'before this invoice can be progressed.',
          mtWarning, [mbOk], 0);
      end;
    end;  // lContinue.
  end;  // lContinue.
end;  // BillingInputFixedOrigEBS.


procedure TelstraInvoiceFixedOrigEBS( nBatch: LongInt; sDirectory: string );
// This function prompts the user for the name of the Telstra supplied
// text file and then calls the BillingInput function (which reads the
// file into a table and controls its processing).
var
  lContinue : Boolean;
  stFiles : TStringList;
  OpenDlg: TOpenDialog;
begin
  // Create the stringlist which contains the names of selected filenames.
  stFiles := TStringList.Create;
  try
    // Now, we must establish which files are to be processed.
    // Select the source directory.
    OpenDlg := TOpenDialog.Create(MainForm);
    OpenDlg.Filter := 'Telstra Billing file (*.dat)|*.DAT|All files (*.*)|*.*';
    OpenDlg.FilterIndex := 1;
    // Use the program directory as the default directory.
    if (Length(sDirectory) > 0) then OpenDlg.InitialDir := sDirectory
    else OpenDlg.InitialDir := ExtractFilePath(ParamStr(0));
    OpenDlg.Title := 'Select The Telstra Account Data File';
    OpenDlg.Options := [ofFileMustExist, ofNoChangeDir];
    // Read and save the user's choices before continuing.
    lContinue := OpenDlg.Execute;
    stFiles.Text := OpenDlg.Files.Text;
    OpenDlg.Free;
    if lContinue then begin
      lContinue := stFiles.Count > 0;
      // Now call for processing of the input files.
      if lContinue then begin
        BillingInputFixedOrigEBS(stFiles, nBatch);
      end;  // if lContinue then.
    end;  // if lContinue then.
  finally
    stFiles.Free;
  end;
end;    // TelstraInvoiceFixedOrigEBS.


end.
