unit TelstraMobileDataEBS;

interface
                                
uses
  Messages, SysUtils, Classes, Graphics, Controls, Dialogs,
  db, dbTables;

type
  TServiceDetails = record
    Custno   : string;
    ServCode : string;
    ServiceID: string;
    RentFrom : TDateTime;
    RentTill : TDateTime;
    CallFrom : TDateTime;
    CallTill : TDateTime;
  end;

  TCallHeaderType = record
    SequenceNo      : string[7];
    ServiceID       : string[17];
    TxnDate         : string[8];
    TxnTime         : string[5];
    Origin          : string[20];
    Duration        : string[9];
    CallCode        : string[2];
    RateDescription : string[32];
    DialledNumber   : string[17];  // Temporary variable only.
    ServiceNotes    : string[22];
    AmountExGST     : string[13];
    GSTAmount       : string[13];
    AmountIncGST    : string[13];
    ToDate          : string[8];
  end;

  procedure GetDetailDataEBS(
    const FName: string;
    D:TDatabase;
    nBatchNumber : LongInt;
    var lReturnCode:Boolean);
{
  procedure ApplyDataRatesEBS(const nBatch: integer;
    LumpedCosts, IncludedRate: boolean);
}
  procedure BillingInputDataEBS( const LumpedCosts, IncludedRate: boolean;
    const stInFiles: Tstrings; nBatch: LongInt );
  procedure TelstraInvoiceDataEBS( const LumpedCosts, IncludedRate: boolean;
    const nBatch: LongInt; sDirectory: string );


implementation

uses
  CriticalTests,
  DataMod, DateFunctions,
  GenFns, FileFunctions,
  Main, Math, MyMessage,
  Progress,
  TelstraFunctions;

var
  // Declare static variables as frequently reset and reused.
  nCount: Integer;             // Current status for frmProgress.
  nFactor: Real;               // Weighting for nCount in progress.
  frmProgress: TProgressForm;  // frmProgress report to the operator.
  sAccountNumber: string;      // Set in BillingInput().
  nSupplierID: integer;        // Set in BillingInput().
  sSupplierID: string;         // Set in BillingInput().
  qryTelstra : TQuery;         // Set in BillingInput().
  sRentCodeID : string;        // Set in BillingInput().
  sCallCodeID : string;        // Set in BillingInput().
  sOtherCodeID : string;       // Set in BillingInput().
  sDiscountCodeID : string;    // Set in BillingInput().
  sSupplierRentCodeID : string;      // Set in BillingInput().
  sSupplierCallCodeID : string;      // Set in BillingInput().
  sSupplierOtherCodeID : string;     // Set in BillingInput().
  sSupplierDiscountCodeID : string;  // Set in BillingInput().

procedure GetDetailDataEBS
          (
//          const FName, sSupplierAccount, sSupplierInvoice: String;
          const FName: string;
          D: TDatabase;
          nBatchNumber : LongInt;
          var lReturnCode: Boolean
          );
const
  PROGRESS_MAX_C: Integer = 100; // Maximum progress is 100%.
var
  nFileSize: Integer;                  // File size in bytes.
  sPartFileName : string;
  sPartResultSet : string;
  EntryCode : string;
  nBackSlashAt : Integer;
  TelstraFile: TextFile;               // Used for accessing the data.
  CurrentFile: String;                 // Currently open file.
  FileIteration: Integer;              // Current file iteration being processed
  T: string;                           // Line of data read from TextFile.
  SubString: String;                   // A (temp) substring of T.
  vSubString : variant;                // - ditto, used in the Locate() function.
  curValue: Currency;                  // A (temp) currency conversion value.
  nSoFar: Integer;                     // Status - num of records processed.
  dBilling: TDateTime;                 // Billing date.
//  dStartBillingPeriod: TDateTime;      // Start of the payment billing period.
//  dEndBillingPeriod: TDateTime;        // End of the payment billing period.
  sBatch : string;
  LastEntryIs_1 : boolean;
  CallHeader : TCallHeaderType;
  sLastDetailCode : string[2];         // To detect a header line AFTER details.

  DBfromDate : string;                 // Section DB - save read dates.
  DBtoDate : string;

  aResultSet : Array of TServiceDetails;
  nResultSet : Integer;
  nStartSearchAt : Integer;
  lFound : Boolean;
//  dMinDate : TDateTime;
  nCommaPosition : integer;
  sListValues : String;
  sListFields : String;
  FieldStringList : TStringList;
  ValueStringList : TStringList;
  qryStringList : TStringList;
  slAddFieldNames : TStringList;         // Calls to AddToTable() only.
  slAddFieldValues : TStringList;        // Calls to AddToTable() only.
  qryDC : TQuery;
  nListValues : integer;
  nDummy : Real;
  sInputLineCode : string;
  lSaveToTable : boolean;
  sRateDescription : string;
  sLastService : string;
  sLastServiceID : string;
  lDataSaved : boolean;
  lContinue : boolean;

  nDataCollectionID : LongInt;
  sLastCallCode : string;
  sLastTransactionTypeID : string;
  lNextAccountNumber : boolean;
  sNextAccountNumber : string;
  nNextAccountNumber : integer;
begin
  lNextAccountNumber := False;
  sNextAccountNumber := '';
  nNextAccountNumber := 0;

  sLastDetailCode := '  ';
  sBatch := IntToStr(nBatchNumber);
  lDataSaved := False;

  FieldStringList := TStringList.Create;
  ValueStringList := TStringList.Create;
  qryStringList := TStringList.Create;
  slAddFieldNames := TStringList.Create;         // Calls to AddToTable() only.
  slAddFieldValues := TStringList.Create;        // Calls to AddToTable() only.

  // Establish the query to be used to save the data read.
  qryDC := TQuery.Create(MainForm);
  qryDC.DatabaseName := 'dbPPdata';

  lReturnCode := False;
  CurrentFile := Fname;
  FileIteration := 0;            // Original file extension, then 001,002, etc.
//  dMinDate := EncodeDate(1980,1,1);
  while FileExists(CurrentFile) do begin
    // Read the file size to determine the approximate number of records.
    nFileSize := GetFileSize(CurrentFile);
    // Now we can calculate the factor for the progress bar to be displayed.
    // From prev. data: 8876 lines in a file of 1,585,557 bytes.
    // Estimate the number of data lines in the file.
    if (nFileSize > 0) then begin
      nFactor := 100.0 / 8876.0 * (1585557.0/nFileSize);
    end
    else begin
      nFactor := 100.0;
    end;
    nCount := 0;
    sPartFileName := CurrentFile;
    while Pos('\',sPartFileName)>0 do begin
      nBackSlashAt  := Pos('\',sPartFileName);
      sPartFileName := Copy(sPartFileName,
        nBackSlashAt+1, Length(sPartFileName)-nBackSlashAt);
    end;
    frmProgress.Caption := 'Telstra Mobile Data Services Batch: '+sBatch;
    frmProgress.Msg.Caption := 'Reading ' + sPartFileName + ' ...';
    frmProgress.ProgressBar1.Position := 0;
    frmProgress.Msg.Refresh;

    // Open the input (text) billing file for reading.
    AssignFile( TelstraFile, CurrentFile );
    Reset( TelstraFile );
    try
      Readln( TelstraFile, T );
      // Test for the original EBS file format for a Mobile or Data.
      if (Copy(T,1,8)='HDR:RACE') then begin
        // This is an account for Mobile services.
        lContinue := True;
      end
      else begin
        lContinue := (Copy(T,1,8)<>'HDR:FCAB');
        if not lContinue then begin
          frmProgress.Hide;
          MessageDlg('Incorrect input file type.'+#13#10+
            'This is a Fixed Calls type file. Try again.', mtWarning, [mbOK],0);
        end
        else begin
          // It may be the old file or a new combined file. Continue to test.
          // Ensure that this is not a CSV file format (OBS billing)
          lContinue := (Pos(',',T)=0);
          if not lContinue then begin
            frmProgress.Hide;
            MessageDlg('Incorrect input file type.'+#13#10+
              'File type DAT is required. Try again.', mtWarning, [mbOK],0);
          end
          else begin
            // Ensure that this is a Mobile file format.
            lContinue := (Copy(T,1,2)='RB');
            if not lContinue then begin
              frmProgress.Hide;
              MessageDlg('Incorrect input file type.'+#13#10+
              'This is NOT a Mobile-type file. Try again.',mtWarning, [mbOK],0);
            end;
          end;
        end;
      end; // HDR:RACE.
      if not lContinue then begin
        // Account selected is not in this file.
        frmProgress.Hide;
        MessageDlg('The Account specified is not in the file selected.'+#13#10+
          'Please try again.',
          mtWarning, [mbOK],0);
      end
      else begin   // lContinue = True.
      lContinue := False;  // Reset then test for correct account.
      // Skip to the Header Line and Determine the billing date only once.
      // Make sure that we find the valid Account first.
      while not lContinue and
        (not Eof( TelstraFile )) do begin
//        (not ((Length(Trim(T))=0) and Eof( TelstraFile ))) do begin
        lContinue := (Trim(Copy(T,37,10)) = sAccountNumber);
        if not lContinue then begin
          Readln( TelstraFile, T);
          // Report progress to the user.
          Inc(nCount);
          nDummy := nFactor * nCount;
          nSoFar := Round( nDummy );
          // Do not know how big the file is - do not exceed 100%.
          if (nSoFar > PROGRESS_MAX_C) then begin
            frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
          end
          else begin
            frmProgress.ProgressBar1.Position := nSoFar;
          end;  // PROGRESS_MAX_C.
        end;  // not lContinue.
      end;  // while <> SubString.
      if lContinue then begin


      // Calculate the billing date only once.
//      D.StartTransaction;
      // Continue to search through the remaining text of the file
      // for occurrences of the search string.  On each find, process
      // the resultant string.
      //****************************************************************
      //********* START READING BILLS **********************************
      //****************************************************************
      try
        repeat
          frmProgress.ProcessWinMessages(MainForm);
          lSaveToTable := True;
          // Read only relevant text into the database.
          // If T is shorter than indicated, Copy() returns an empty string.
          sInputLineCode := UpperCase(Copy(T, 82, 2));


          // CALL DETAILS.
          if (sInputLineCode='2D') then begin
            sInputLineCode := UpperCase(Trim(Copy(T, 82, 5)));
            if (Pos(' ',sInputLineCode) > 0) then begin
              lSaveToTable := False;
            end
            else if (Length(Trim(Copy(T, 87, 5))) = 0) then begin
              // Column heading, read the RateDescription and skip on.
              if (Copy(sInputLineCode,5,1) = 'D') then begin
                // Heading - Read the transaction type description.
                sRateDescription := CleanString(Trim(Copy(T,92,30)));
              end;  // ..and ignore any totals etc.
              lSaveToTable := False;
            end
            else if (Trim(Copy(T, 87, 5)) = 'SEQ') then begin
              // Column heading, skip on.
              lSaveToTable := False;
            end
            else begin
              // This line is a detail item & not a header/summary - save it.
              if (sInputLineCode = '2D10D') and
                (Pos(' DATA ', sRateDescription)>0) then begin
                FieldStringList.Clear;
                ValueStringList.Clear;
                sListFields := '(';
                sListValues := 'VALUES (';
                SubString := Trim(Copy(T,87,5));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'SequenceNo,';
                  sListValues := sListValues + '''' + SubString +''',';
                end;
                SubString := Trim(Copy(T,61,17));
                if not (SubString = sLastService) then begin
                  sLastService := SubString;
                  sLastServiceID := AccountFormat(sLastService);
                end;
                sListFields := sListFields + 'ServiceID,';
                sListValues := sListValues +''''+ sLastServiceID +''',';
                // Read the code common for each detail and summary - used below
                // when the summary (type "1") is read.
                SubString := Copy(T,93,8);
                if (Length(SubString) > 0) then begin
                  // Only 2-digit year, add 2000.
                  SubString := Copy(SubString,1,6) +'20'+ Copy(SubString,7,2);
                  sListFields := sListFields + 'TxnDate,';
                  sListValues := sListValues +''''+ SubString +''',';
                end;
                SubString := Copy(T,102,5);
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'TxnTime,';
                  sListValues := sListValues +''''+ SubString +':00'',';
                end;
                SubString := Trim(Copy(T,108,13));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'Origin,';
                  sListValues := sListValues +''''+ CleanString(SubString) +''',';
                end;
                SubString := Trim(Copy(T,137,8));
                if (Length(SubString) > 0) then begin
                  SubString := ConvertTimeToSecs(CleanString(SubString));
                  sListFields := sListFields + 'Duration,';
                  sListValues := sListValues +''''+ SubString +''',';
                end;
                SubString := CleanString(Copy(sInputLineCode,1,4));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'CallCode,';
                  sListValues := sListValues +''''+ SubString +''',';
                  if (sLastCallCode <> SubString) then begin
                    sLastCallCode := SubString;
                    vSubString := SubString;
                    with qryTelstra do
                      lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                    if lFound then
                      sLastTransactionTypeID := qryTelstra['TransactionType']
                    else begin
                      sLastTransactionTypeID := sCallCodeID;
                      slAddFieldNames.Clear;
                      slAddFieldValues.Clear;
                      slAddFieldNames.Add('Code');
                      slAddFieldValues.Add(''''+sLastCallCode+'''');
                      slAddFieldNames.Add('AmountCategory');
                      slAddFieldValues.Add('''CALLS''');
                      slAddFieldNames.Add('Description');
                      slAddFieldValues.Add('''CALLS''');
                      slAddFieldNames.Add('TransactionTypeID');
                      slAddFieldValues.Add(sLastTransactionTypeID);
                      slAddFieldNames.Add('SupplierID');
                      slAddFieldValues.Add(sSupplierID);
                      AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                      qryTelstra.Close;
                      qryTelstra.Open;
                    end;  // if (not) lFound.
                  end;  // if (sLastCallCode <> SubString) then begin.
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sLastTransactionTypeID+',';
                end
                else begin
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sCallCodeID+',';
                end;  // CallCode.
                SubString := Trim(Copy(T,134,2));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'Peak,';
                  if (Right(SubString,2) = ' P') then
                    sListValues := sListValues +'1,'
                  else
                    sListValues := sListValues +'0,';
                end;
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ sRateDescription +''',';
                SubString := Trim(Copy(T,121,13));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'DialledNumber,';
                  sListValues := sListValues +''''+
                  CleanString(SubString) +''',';
                end;
                FieldStringList.Add(sListFields);
                ValueStringList.Add(sListValues);
                sListFields := '';
                sListValues := '';

                SubString := Trim(Copy(T,146,7));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'Var01,';
                  sListValues := sListValues +''''+ SubString +''',';
                end;
                SubString := Trim(Copy(T,166,11));
                sListFields := sListFields + 'RevenueFlag,AmountExGST,';
                sListValues := sListValues + '0,'+ SubString +',';
                SubString := Trim(Copy(T,178,11));
                curValue := curFormat(StrToFloat(SubString));
                sListFields := sListFields + 'GSTAmount,GSTFlag,';
                if (curValue = 0.00) then begin
                  sListValues := sListValues + SubString + ',''N'',';
                end
                else begin
                  sListValues := sListValues + SubString + ',''Y'',';
                end;
                SubString := Trim(Copy(T,189,11));
                sListFields := sListFields + 'AmountIncGST,';
                sListValues := sListValues + SubString +',';
                if (Length(DBfromDate) = 0) or (Length(DBtoDate) = 0) then begin
                  SubString := '20'+ Trim(Copy(T,55,6));
                  DBfromDate :=
                    FormattedDateString(LastMonth(StrDMYtoDate(YMD2DMY(SubString))));
                  DBtoDate :=
                  FormattedDateString(StrDMYtoDate(YMD2DMY(SubString))-1);
                end;
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues +''''+ DBfromDate +''',';
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+ DBtoDate +''',';
              end  // 2D10D.

              // Test for call diversions (2D13D) and SMS Enhanced (2D17).
              else if (sInputLineCode = '2D13D') or
                (sInputLineCode = '2D17D') then begin
                FieldStringList.Clear;
                ValueStringList.Clear;
                sListFields := '(';
                sListValues := 'VALUES (';
                SubString := Trim(Copy(T,87,5));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'SequenceNo,';
                  sListValues := sListValues + '''' + SubString +''',';
                end;
                SubString := Trim(Copy(T,61,17));
                if not (SubString = sLastService) then begin
                  sLastService := SubString;
                  sLastServiceID := AccountFormat(sLastService);
                end;
                sListFields := sListFields + 'ServiceID,';
                sListValues := sListValues +''''+ sLastServiceID +''',';
                // Read the code common for each detail and summary - used below
                // when the summary (type "1") is read.
                SubString := Copy(T,93,8);
                if (Length(SubString) > 0) then begin
                  // Only 2-digit year, add 2000.
                  SubString := Copy(SubString,1,6) +'20'+ Copy(SubString,7,2);
                  sListFields := sListFields + 'TxnDate,';
                  sListValues := sListValues +''''+ SubString +''',';
                end;
                SubString := Copy(T,103,5);
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'TxnTime,';
                  sListValues := sListValues +''''+ SubString +':00'',';
                end;
                SubString := Trim(Copy(T,110,13));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'Destination,';
                  sListValues := sListValues +''''+ CleanString(SubString) +''',';
                end;
                if (sInputLineCode = '2D13D') then begin
                  // First causes an error for 2D17D, other 3 not relevant - RJC070717.
                  SubString := Trim(Copy(T,146,6));
                  if (Length(SubString) > 0) then begin
                    SubString := ConvertTimeToSecs(CleanString(SubString));
                    sListFields := sListFields + 'Duration,';
                    sListValues := sListValues +''''+ SubString +''',';
                  end;
                  SubString := Trim(Copy(T,132,2));
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'Peak,';
                    if (Right(SubString,2) = ' P') then
                      sListValues := sListValues +'1,'
                    else
                      sListValues := sListValues +'0,';
                  end;
                  if (sInputLineCode = '2D17D') then begin
                    sRateDescription := CleanString(Trim(Copy(T,110,30)));
                  end;
                  sListFields := sListFields + 'RateDescription,';
                  sListValues := sListValues +''''+ sRateDescription +''',';
                  SubString := Trim(Copy(T,121,13));
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'DialledNumber,';
                    sListValues := sListValues +''''+
                    CleanString(SubString) +''',';
                  end;
                end;
                SubString := CleanString(Copy(sInputLineCode,1,4));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'CallCode,';
                  sListValues := sListValues +''''+ SubString +''',';
                  if (sLastCallCode <> SubString) then begin
                    sLastCallCode := SubString;
                    vSubString := SubString;
                    with qryTelstra do
                      lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                    if lFound then
                      sLastTransactionTypeID := qryTelstra['TransactionType']
                    else begin
                      sLastTransactionTypeID := sCallCodeID;
                      slAddFieldNames.Clear;
                      slAddFieldValues.Clear;
                      slAddFieldNames.Add('Code');
                      slAddFieldValues.Add(''''+sLastCallCode+'''');
                      slAddFieldNames.Add('AmountCategory');
                      slAddFieldValues.Add('''CALLS''');
                      slAddFieldNames.Add('Description');
                      slAddFieldValues.Add('''CALLS''');
                      slAddFieldNames.Add('TransactionTypeID');
                      slAddFieldValues.Add(sLastTransactionTypeID);
                      slAddFieldNames.Add('SupplierID');
                      slAddFieldValues.Add(sSupplierID);
                      AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                      qryTelstra.Close;
                      qryTelstra.Open;
                    end;  // if (not) lFound.
                  end;  // if (sLastCallCode <> SubString) then begin.
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sLastTransactionTypeID+',';
                end
                else begin
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sCallCodeID+',';
                end;  // CallCode.
                FieldStringList.Add(sListFields);
                ValueStringList.Add(sListValues);
                sListFields := '';
                sListValues := '';

                // Do not save to RevenueFlag - left NULL allows separation of
                // the data for creating revenue transactions. RevenueFlag is
                // set at teh end of the importing process.
                SubString := Trim(Copy(T,166,11));
                sListFields := sListFields + 'AmountExGST,';
                sListValues := sListValues + SubString +',';
                SubString := Trim(Copy(T,178,11));
                curValue := curFormat(StrToFloat(SubString));
                sListFields := sListFields + 'GSTAmount,GSTFlag,';
                if (curValue = 0.00) then begin
                  sListValues := sListValues + SubString + ',''N'',';
                end
                else begin
                  sListValues := sListValues + SubString + ',''Y'',';
                end;
                SubString := Trim(Copy(T,189,11));
                sListFields := sListFields + 'AmountIncGST,';
                sListValues := sListValues + SubString +',';
                if (Length(DBfromDate) = 0) or (Length(DBtoDate) = 0) then begin
                  SubString := '20'+ Trim(Copy(T,55,6));
                  DBfromDate :=
                    FormattedDateString(LastMonth(StrDMYtoDate(YMD2DMY(SubString))));
                  DBtoDate :=
                  FormattedDateString(StrDMYtoDate(YMD2DMY(SubString))-1);
                end;
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues +''''+ DBfromDate +''',';
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+ DBtoDate +''',';
              end  // 2D13D.
              else if (Pos(sInputLineCode,'@2D11D@@2D12D@') > 0) then begin
                lSaveToTable := False;
              end
              else begin
                lSaveToTable := False;
              end;  //
            end;  // ..sInputLineCode contents.
          end  // if (sInputLineCode='2D') then.


          else if (sInputLineCode='2S') or (sInputLineCode='1A') or
            (sInputLineCode='2A') then begin
            // This line is a summary subtotal - check it for Discounts.
            sInputLineCode := UpperCase(Trim(Copy(T, 82, 5)));
            SubString := UpperCase(Trim(Copy(T, 92, 30)));
            if ((sInputLineCode = '2S10V') and
              ((Pos('DISCOUNT', SubString) > 0) or
              (Pos('REBATE', SubString) > 0))) or
              (sInputLineCode = '1A02V') or ((sInputLineCode = '2A02V') and
              (Pos('ROUND', SubString)>0)) then begin
              // This is a discount item.
              FieldStringList.Clear;
              ValueStringList.Clear;
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := Trim(Copy(T,61,17));
              if not (SubString = sLastService) then begin
                sLastService := SubString;
                sLastServiceID := AccountFormat(sLastService);
              end;
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+ sLastServiceID +''',';
              SubString := CleanString(Copy(sInputLineCode,1,4));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastCallCode <> SubString) then begin
                  sLastCallCode := SubString;
                  vSubString := SubString;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then
                    sLastTransactionTypeID := qryTelstra['TransactionType']
                  else begin
                    sLastTransactionTypeID := sDiscountCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastCallCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''OTHER''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''OTHER''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    qryTelstra.Close;
                    qryTelstra.Open;
                  end;  // if (not) lFound.
                end;  // if (sLastCallCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,';
                sListValues := sListValues + sLastTransactionTypeID+',';
              end
              else begin
                sListFields := sListFields + 'TransactionTypeID,';
                sListValues := sListValues + sDiscountCodeID+',';
              end;  // CallCode.
              sRateDescription := CleanString(Trim(Copy(T,92,30)));
              SubString := sRateDescription;
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';

              SubString := Trim(Copy(T,166,11));
              if (Length(SubString)=0) and
                (Length(Trim(Copy(T,178,11)+Copy(T,189,11)))=0) then begin
                lSaveToTable := False;               // No discount - ignore.
              end
              else begin
                if (Length(SubString) = 0) then SubString := '0';
                sListFields := sListFields + 'AmountExGST,';
                sListValues := sListValues + SubString +',';
                SubString := Trim(Copy(T,178,11));
                curValue := curFormat(StrToFloat(SubString));
                if (Length(SubString) = 0) then SubString := '0';
                sListFields := sListFields + 'GSTAmount,GSTFlag,';
                if (curValue = 0.00) then begin
                  sListValues := sListValues + SubString + ',''N'',';
                end
                else begin
                  sListValues := sListValues + SubString + ',''Y'',';
                end;
                SubString := Trim(Copy(T,189,11));
                if (Length(SubString) = 0) then SubString := '0';
                sListFields := sListFields + 'AmountIncGST,';
                sListValues := sListValues + SubString +',';
                if (Length(DBfromDate) = 0) or (Length(DBtoDate) = 0) then begin
                  SubString := '20'+ Trim(Copy(T,55,6));
                  DBfromDate :=
                    FormattedDateString(LastMonth(StrDMYtoDate(YMD2DMY(SubString))));
                  DBtoDate :=
                  FormattedDateString(StrDMYtoDate(YMD2DMY(SubString))-1);
                end;
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues +''''+ DBfromDate +''',';
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+ DBtoDate +''',';
              end;  // if (Length(SubString)=0) then begin..else.
            end  // if (sInputLineCode = '2S10V') then begin.

            else if (sInputLineCode = '2S02D') or
                (sInputLineCode = '2S02V') or
                ((sInputLineCode = '2S10V') and
                 (Pos('DATA', UpperCase(Trim(Copy(T, 92, 30)))) = 0)) or
                 (sInputLineCode = '2S11V') then begin
//               or (sInputLineCode = '2S13V') then begin  // Removed a/c SmartBus 070523.
              // This is a discount or rounding item, or a Call item, provided
              // the description is not like 'PACKET DATA SESSION' for CAMS bill.
              FieldStringList.Clear;
              ValueStringList.Clear;
              sListFields := '(';
              sListValues := 'VALUES (';
              SubString := Trim(Copy(T,61,17));
              if not (SubString = sLastService) then begin
                sLastService := SubString;
                sLastServiceID := AccountFormat(sLastService);
              end;
              sListFields := sListFields + 'ServiceID,';
              sListValues := sListValues +''''+ sLastServiceID +''',';
              SubString := CleanString(Copy(sInputLineCode,1,4));
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'CallCode,';
                sListValues := sListValues +''''+ SubString +''',';
                if (sLastCallCode <> SubString) then begin
                  sLastCallCode := SubString;
                  vSubString := SubString;
                  with qryTelstra do
                    lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                  if lFound then
                    sLastTransactionTypeID := qryTelstra['TransactionType']
                  else begin
                    sLastTransactionTypeID := sCallCodeID;
                    slAddFieldNames.Clear;
                    slAddFieldValues.Clear;
                    slAddFieldNames.Add('Code');
                    slAddFieldValues.Add(''''+sLastCallCode+'''');
                    slAddFieldNames.Add('AmountCategory');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('Description');
                    slAddFieldValues.Add('''CALLS''');
                    slAddFieldNames.Add('TransactionTypeID');
                    slAddFieldValues.Add(sLastTransactionTypeID);
                    slAddFieldNames.Add('SupplierID');
                    slAddFieldValues.Add(sSupplierID);
                    AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                    qryTelstra.Close;
                    qryTelstra.Open;
                  end;  // if (not) lFound.
                end;  // if (sLastCallCode <> SubString) then begin.
                sListFields := sListFields + 'TransactionTypeID,';
                sListValues := sListValues + sLastTransactionTypeID+',';
              end
              else begin
                sListFields := sListFields + 'TransactionTypeID,';
                sListValues := sListValues + sCallCodeID+',';
              end;  // CallCode.
              sRateDescription := CleanString(Trim(Copy(T,92,30)));
              SubString := sRateDescription;
              if (Length(SubString) > 0) then begin
                sListFields := sListFields + 'RateDescription,';
                sListValues := sListValues +''''+ SubString +''',';
              end;
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              sListFields := '';
              sListValues := '';
              // Do not save to RevenueFlag - left NULL allows separation of
              // the data for creating revenue transactions. RevenueFlag is
              // set at the end of the importing process.
              SubString := Trim(Copy(T,166,11));
              if (Length(SubString) = 0) then SubString := '0';
              sListFields := sListFields + 'AmountExGST,';
              sListValues := sListValues + SubString +',';
              SubString := Trim(Copy(T,178,11));
              if (Length(SubString) = 0) then SubString := '0';
              curValue := curFormat(StrToFloat(SubString));
              sListFields := sListFields + 'GSTAmount,GSTFlag,';
              if (curValue = 0.00) then begin
                sListValues := sListValues + SubString + ',''N'',';
              end
              else begin
                sListValues := sListValues + SubString + ',''Y'',';
              end;
              SubString := Trim(Copy(T,189,11));
              if (Length(SubString) = 0) then SubString := '0';
              sListFields := sListFields + 'AmountIncGST,';
              sListValues := sListValues + SubString +',';
              if (Length(DBfromDate) = 0) or (Length(DBtoDate) = 0) then begin
                SubString := '20'+ Trim(Copy(T,55,6));
                DBfromDate :=
                  FormattedDateString(LastMonth(StrDMYtoDate(YMD2DMY(SubString))));
                DBtoDate :=
                  FormattedDateString(StrDMYtoDate(YMD2DMY(SubString))-1);
              end;
              sListFields := sListFields + 'FromDate,';
              sListValues := sListValues +''''+ DBfromDate +''',';
              sListFields := sListFields + 'ToDate,';
              sListValues := sListValues +''''+ DBtoDate +''',';
            end  // 2so2d or 2so2v.

            else begin
              lSaveToTable := False;
            end;  // ..else begin.
          end  // ..else if (sInputLineCode='2S') then begin.

          else begin
            lSaveToTable := False;
          end;  // ..else begin.
          if lSaveToTable then begin
            // Now make the SQL statement to save to the database table.
            sListFields := sListFields + 'BatchID)';
            sListValues := sListValues + sBatch +')';
            FieldStringList.Add(sListFields);
            ValueStringList.Add(sListValues);
            qryStringList.Clear;
            qryStringList.Add('INSERT INTO tDataCollection');
            for nListValues := 0 to FieldStringList.Count-1 do begin
              qryStringList.Add(FieldStringList[nListValues]);
            end;
            for nListValues := 0 to ValueStringList.Count-1 do begin
              qryStringList.Add(ValueStringList[nListValues]);
            end;
            // Update and execute the query.
            qryDC.Close;
            qryDC.SQL := qryStringList;
            qryDC.ExecSQL;
            lDataSaved := True;
          end;  // if lSaveToTable then begin.
          Inc(nCount);
          // Report progress to the user.
          nSoFar := Round(nCount * nFactor);
          // Do not know how big the file is - make sure don't exceed 100%.
          if (nSoFar > PROGRESS_MAX_C) then begin
            frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
          end
          else begin
            frmProgress.ProgressBar1.Position := nSoFar;
          end;
          // Read the next line of text file data.
          Readln( TelstraFile, T );

          // Ensure this next entry is in the selected account. 
          if (Length(Trim(T)) > 0) then begin
            lContinue := (Trim(Copy(T,37,10)) = sAccountNumber);
            while not lContinue and not lNextAccountNumber and
              (not ((Length(Trim(T))=0) and Eof( TelstraFile ))) do begin
              SubString := Trim(Copy(T,37,10));
              lContinue := (SubString = sAccountNumber);
              if not lContinue then begin
                // Wait for 20 lines of the next account before ending processing.
                if (sNextAccountNumber = SubString) then begin
                  Inc(nNextAccountNumber);
                  lNextAccountNumber := (nNextAccountNumber = 20);
                end
                else begin
                  sNextAccountNumber := SubString;
                  nNextAccountNumber := 1;
                end;
                Readln( TelstraFile, T);
                // Report progress to the user.
                Inc(nCount);
                nDummy := nFactor * nCount;
                nSoFar := Round( nDummy );
                // Do not know how big the file is - do not exceed 100%.
                if (nSoFar > PROGRESS_MAX_C) then begin
                  frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
                end
                else begin
                  frmProgress.ProgressBar1.Position := nSoFar;
                end;
              end;  // not lContinue.
            end;  // while not AccountNumber.
          end;  // not Eof.

        until not lContinue or lNextAccountNumber or Eof( TelstraFile );
        lReturnCode := lDataSaved;;
      except    // D.StartTransaction.
        lReturnCode := False;
//        D.Rollback;       // On failure, undo the changes - not useful anyway.
        raise;            // Raise the exception to avoid call to CommitUpdates.
      end;    // D.StartTransaction.

      end;  // lContinue.
      end;  // not lContinue.

      // Now need to process the last line if CR-LF not at end.
      //  BUT - nothing done !
//      D.Commit;
      //****************************************************************
      //*********** END READING BILLS **********************************
      //****************************************************************
    finally    // TelstraFile (try).
      // Done with the input file; close it.
      CloseFile( TelstraFile );
    end;    // TelstraFile (try).
    // Select the next file if exists - extension 001, 002, etc.
    Inc(FileIteration);
    CurrentFile := Copy(CurrentFile,1,Pos('.',CurrentFile));
    CurrentFile := CurrentFile + Right(('00'+IntToStr(FileIteration)),3);
  end;  // if FileExists(CurrentFile) then begin.
end;  // procedure GetDetailDataEBS.

{
procedure ApplyDataRatesEBS(const nBatch: integer;
  LumpedCosts, IncludedRate: boolean);
var
  MessageForm : TMessagesForm;
  sBatchID : string;
  sServiceTypeID : string;
  sSystemID : string;
  sFromDate : string;
  sToDate : string;
  curAmountExGST : currency;
  curGSTAmount : currency;
  curExcessAmtExGST : currency;
  curTotalGSTAmount : currency;
  sNewServiceID : string;
  nIncludedUse : integer;
  sIncludedUse : string;
  sTransactionTypeID : string;
  nTransactionTypeID : integer;
  slDC : TStringList;
  qryDC : TQuery;
  sVRTID : string;
  nkByte : integer;
  skByte : string;
  nExcessKByte : integer;
  nGByte : real;
  sGByte : string;
  nUsage : real;
  sGSTRate : string;
  sRateCost : string;
  sServiceType : string;
  fPartCost : real;
//  nSequenceNoLength : integer;
//  nSequenceNoLast : Integer;
  nServiceCount : integer;
begin
  sBatchID := IntToStr(nBatch);
  MessageForm := TMessagesForm.Create(nil);
  MessageForm.Caption := 'Importing Batch:  '+sBatchID;
  MessageForm.Msg.Caption := 'Please wait. Creating revenue transactions...';
  slDC := TStringList.Create;
  qryDC := TQuery.Create(nil);
  qryDC.DatabaseName := 'dbPPdata';
  Str(MAINFORM.GST_RATE:6:4, sGSTRate);

  sVRTID := '0';                                   // Define VRT CustomerID.
  with slDC do begin
    Add('SELECT ID');
    Add('FROM tCustomer');
    Add('WHERE ShipTo LIKE ''VRT''');
    Add('AND Active = 1');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  if not qryDC.Eof then
    sVRTID := IntToStr(qryDC['id']);
  qryDC.Close;
  qryDC.SQL.Clear;

  // Read invoice data usage.                      // Determine total data used.
  with slDC do begin
    Clear;
    Add('SELECT Sum(CAST(Var01 AS integer)) AS kByte');
    Add('FROM tDataCollection');
    Add('WHERE BatchID = '+sBatchID);
    Add('AND RevenueFlag = 0');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  nkByte := qryDC.FieldByName('kByte').AsInteger;
  qryDC.Close;
  Str(nkByte:11,skByte);
  nGByte := nkByte / 1024.0 / 1024.0;
  Str(nGByte:11:4,sGByte);

  // Get the ServiceTypeID linked to this invoice.
  with slDC do begin
    Clear;
    Add('SELECT tServiceType.ID AS "ServiceTypeID", tSystem.ID AS "SystemID"');
    Add('FROM tServiceType');
    Add('INNER JOIN tSystem ON tServiceType.SystemID = tSystem.ID');
    Add('INNER JOIN tBatchType ON tSystem.BatchTypeID = tBatchType.ID');
    Add('INNER JOIN tBatch ON tBatchType.ID = tBatch.BatchTypeID');
    Add('WHERE tBatch.ID = '+sBatchID);
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  sServiceTypeID := qryDC.FieldByName('ServiceTypeID').AsString;
  sSystemID := qryDC.FieldByName('SystemID').AsString;
  qryDC.Close;
  if (Length(sServiceTypeID) = 0) then sServiceTypeID := 'NULL';
  if (Length(sSystemID) = 0) then sSystemID := 'NULL';

  // Now read the transaction type to be used.
  with slDC do begin
    Clear;
    Add('SELECT tTransactionType.ID');
    Add('FROM tTransactionType');
    Add('WHERE tTransactionType.Description LIKE ''%Data Usage%''');
    Add('AND tTransactionType.Description LIKE ''%Aggregate%''');
    Add('AND tTransactionType.SystemID = '+sSystemID);
    Add('AND tTransactionType.Active = 1');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  sTransactionTypeID:= qryDC.FieldByName('ID').AsString;
  qryDC.Close;
  if (Length(sTransactionTypeID) = 0) then sTransactionTypeID := 'NULL';

  // Now get the data usage included in the fixed cost.
  with slDC do begin
    Clear;
    Add('SELECT CAST(Min(tRate.Length) AS int) AS "Length"');
    Add('FROM tRate');
    Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
    Add('AND tRate.Active = 1');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  nIncludedUse := qryDC.FieldByName('Length').AsInteger;
  sIncludedUse := IntToStr(nIncludedUse);
  qryDC.Close;

  // Look up the appropriate rate from tRate and convert to monthly rates.
  with slDC do begin
    Clear;
    Add('SELECT tRate.Cost');
    Add('FROM tRate');
    Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
    Add('AND tRate.Length = '+sIncludedUse);
    Add('AND tRate.Active = 1');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  curAmountExGST := qryDC.FieldByName('Cost').AsCurrency / 12;
  Str(qryDC.FieldByName('Cost').AsCurrency:11:4,sRateCost);
  qryDC.Close;
  curGSTAmount := curAmountExGST * MAINFORM.GST_RATE;
  curTotalGSTAmount := curGSTAmount;

  // Try to get the dates from the tDataCollection table.
  with slDC do begin
    Clear;
    Add('SELECT DISTINCT tDataCollection.FromDate, tDataCollection.ToDate');
    Add('FROM tDataCollection');
    Add('WHERE tDataCollection.BatchID = '+sBatchID);
    Add('AND tDataCollection.FromDate IS NOT NULL');
    Add('AND tDataCollection.ToDate IS NOT NULL');
  end;
  qryDC.SQL.Text := slDC.Text;
  qryDC.Open;
  if not (qryDC.Bof and qryDC.Eof) then begin
    sFromDate := ''''+ qryDC.FieldByName('FromDate').AsString +''', ';
    sToDate := ''''+ qryDC.FieldByName('ToDate').AsString +''', ';
  end;
  qryDC.Close;
  // If unsuccessful, use the dates in the tBatch table.
  if (Length(sFromDate) = 0) or (Length(sToDate) = 0) then begin
    // Read the dates from the tBatch table.
    with slDC do begin
      Clear;
      Add('SELECT tBatch.RentStartDate, tBatch.RentEndDate');
      Add('FROM tBatch');
      Add('WHERE tBatch.ID = '+sBatchID);
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    if VarIsNull(qryDC['RentStartDate']) then begin
      sFromDate := 'NULL';
    end
    else begin
      sFromDate := ''''+
      FormattedDateString(qryDC.FieldByName('RentStartDate').AsDateTime)+''', ';
    end;
    if VarIsNull(qryDC['RentEndDate']) then begin
      sToDate := 'NULL';
    end
    else begin
      sToDate := ''''+
        FormattedDateString(qryDC.FieldByName('RentEndDate').AsDateTime)+''', ';
    end;
    qryDC.Close;
  end;  // FromDate or EndDate unknown.

  // Create the Revenue transactions in the tDataCollection table.
  if LumpedCosts then begin                   // add test for "IncludedRate".
    // For lumped sums, all transactions have AmtExGST=0.
    // Charges are lumped into 1 or 2 new transactions. Also,
    // any ServiceID not on the invoice is to be added from the tServiceID
    // table.
    // This applies to SmartBus invoices.

    // First, read the ServiceID to hold the base charges.
    with slDC do begin
      Clear;
      Add('SELECT tParameter.ParmText');
      Add('FROM tParameter');
      Add('WHERE tParameter.Parm LIKE ''BaseDataServiceID''');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    sNewServiceID := qryDC.FieldByName('ParmText').AsString;
    qryDC.Close;
    if (Length(sNewServiceID) = 0) then begin
      MessageDlg('No entry found for Parm=BaseDataServiceID in'+#13#10+
      'the tParameter table.  Data is incomplete.',
        mtError, [mbOk], 0);
    end
    else begin
      // Ready to add the basic (minimum) charge.
      with slDC do begin
        Clear;
        Add('INSERT INTO tDataCollection');
        Add('(BatchID, ServiceID, FromDate, ToDate, TransactionTypeID, '+
            'AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag)');
        Add('VALUES('+ sBatchID+', ');
        Add(''''+sNewServiceID+''', ');
        Add(sFromDate);
        Add(sToDate);
        Add(sTransactionTypeID+', ');
        Add(CurrToStr(curAmountExGST)+', ');
        Add(CurrToStr(curGSTAmount)+', ');
        Add(CurrToStr(curAmountExGST + curGSTAmount)+', ');
        Add('''Y'', ');
        Add('1)');
      end;
      qryDC.SQL.Text := slDC.Text;
      qryDC.ExecSQL;

      // Now test to see if the total usage exceeds the basic allowance.
      // First, see if the data usage exceeds the included data. Values are in
      // kiloBytes (kB).
      if (nkByte > nIncludedUse) then begin
[ Open curly bracket...
        curExcessAmtExGST := 0;
] Close curly bracket...
        nExcessKByte := nkByte - nIncludedUse;
        // Read the transaction type to be used.
        with slDC do begin
          Clear;
          Add('SELECT tTransactionType.ID');
          Add('FROM tTransactionType');
          Add('WHERE tTransactionType.Description LIKE ''%Data Usage%''');
          Add('AND tTransactionType.Description LIKE ''%Excess%''');
          Add('AND tTransactionType.SystemID = '+sSystemID);
          Add('AND tTransactionType.Active = 1');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.Open;
        sTransactionTypeID:= qryDC.FieldByName('ID').AsString;
        qryDC.Close;
        if (Length(sTransactionTypeID) = 0) then sTransactionTypeID := 'NULL';
        // And determine the appropriate rate (PartCost) for excess usage.
        with slDC do begin
          Clear;
          Add('SELECT tRate.PartCost');
          Add('FROM tRate');
          Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
          Add('AND tRate.Length =');
          Add(' (SELECT Max(tRate.Length)');
          Add(' FROM tRate');
          Add(' WHERE tRate.ServiceTypeID = '+sServiceTypeID);
          Add(' AND tRate.Length <= '+IntToStr(nkByte));
          Add(' AND tRate.Active = 1)');
          Add('AND tRate.Active = 1');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.Open;
        if (qryDC.Bof and qryDC.Eof) then begin
          curExcessAmtExGST := 0;
        end
        else begin
          // Round up the usage to the nearest MegaByte of data (rounded up).
          nUsage := Trunc((nkByte - nIncludedUse) / 1024.0) + 1;
          curExcessAmtExGST := nUsage * qryDC.FieldByName('PartCost').AsCurrency;
        end;
        qryDC.Close;
        if (curExcessAmtExGST > 0) then begin
          curGSTAmount := curExcessAmtExGST * MAINFORM.GST_RATE;
          curTotalGSTAmount := curTotalGSTAmount + curGSTAmount;
          // First, read the ServiceID to hold the Excess charges.
          with slDC do begin
            Clear;
            Add('SELECT ParmText');
            Add('FROM tParameter');
            Add('WHERE tParameter.Parm LIKE ''ExcessDataServiceID''');
          end;
          qryDC.SQL.Text := slDC.Text;
          qryDC.Open;
          sNewServiceID := qryDC.FieldByName('ParmText').AsString;
          qryDC.Close;
          if (Length(sNewServiceID) = 0) then begin
            MessageDlg('No entry found for Parm=ExcessDataServiceID in'+#13#10+
              'the tParameter table.  Data is incomplete.',
               mtError, [mbOk], 0);
          end
          else begin
            // Ready to add the Excess charges above the included allowance.
            with slDC do begin
              Clear;
              Add('INSERT INTO tDataCollection');
              Add('(BatchID, ServiceID, FromDate, ToDate, TransactionTypeID, '+
                  'AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag)');
              Add('VALUES('+ sBatchID+', ');
              Add(''''+sNewServiceID+''', ');
              Add(sFromDate);
              Add(sToDate);
              Add(sTransactionTypeID+', ');
              Add(CurrToStr(curExcessAmtExGST)+', ');
              Add(CurrToStr(curGSTAmount)+', ');
              Add(CurrToStr(curExcessAmtExGST + curGSTAmount)+', ');
              Add('''Y'', ');
              Add('1)');
            end;
            qryDC.SQL.Text := slDC.Text;
            qryDC.ExecSQL;
          end;  // empty NewServiceID.
        end;  // curAmountExGST > 0.
      end
      else begin
        curExcessAmtExGST := 0;
        nExcessKByte := 0;
      end;  // nkByte > nIncludedUse.
      // Set RevenueFlag=1 for all of the data transactions (with AmountExGST=0).
      // Any other RevenueFlag=NULL will be reset to "0" later.
      with slDC do begin
        Clear;
        Add('UPDATE tDataCollection');
        Add('SET RevenueFlag = 1');
        Add('WHERE BatchID = '+ sBatchID);
        Add('AND AmountExGST = 0');
        Add('AND AmountIncGST = 0');
      end;
      qryDC.SQL.Text := slDC.Text;
      qryDC.ExecSQL;
      // Get the TransactionTypeID from the tDataCollection table where
      // no charges have been calculated.
      with slDC do begin
        Clear;
        Add('SELECT DISTINCT tDataCollection.TransactionTypeID');
        Add('FROM tDataCollection');
        Add('WHERE tDataCollection.BatchID = '+ sBatchID);
        Add('AND tDataCollection.AmountExGST = 0');
        Add('AND tDataCollection.AmountIncGST = 0');
      end;
      qryDC.SQL.Text := slDC.Text;
      qryDC.Open;
      nTransactionTypeID := qryDC.FieldByName('TransactionTypeID').AsInteger;
      qryDC.Close;
      // Add those services that did not register any usage on the network.
      AddMissingServiceID(nBatch, nTransactionTypeID);

      // Make sure that all entries for this BatchID do not have
      // RevenueFlag=Null so that revenue Transactions will not be created when
      // transferring to the tTransaction table later using SaveToTransaction().
      with slDC do begin
        Clear;
        Add('UPDATE tDataCollection');
        Add('SET RevenueFlag = 0');
        Add('WHERE RevenueFlag IS NULL');
        Add('AND BatchID = '+sBatchID);
      end;
      qryDC.SQL.Text := slDC.Text;
      qryDC.ExecSQL;

      // Update the values stored in the tBatch table.
      with slDC do begin
        Clear;
        Add('UPDATE tBatch');
        Add('SET TotalRevenue = '+CurrToStr(curAmountExGST)+',');
//        Add(' PaymentsReceived = '+ IntToStr(nStdkByte)+',');
        Add(' InvoiceNo = '''+ sServiceType +''',');
        Add(' OrderNumber = '''+ IntToStr(nExcesskByte) +''',');
        Add(' RequisitionNumber = '''+ IntToStr(nIncludedUse) +''',');
        Add(' OpeningBalance = '+ CurrToStr(curExcessAmtExGST) +',');
        Add(' TotalCredits = '+ sRateCost +',');
//        Add(' TotalPayable = '+ CurrToStr(curAmountExGST + curExcessAmtExGST) +',');
        Add(' GSTAmount = '+ CurrToStr(curTotalGSTAmount));
        Add('WHERE tBatch.ID = '+ sBatchID);
      end;
      qryDC.SQL.Text := slDC.Text;
      qryDC.ExecSQL;
    end;  // empty NewServiceID.
  end  // LumpedSum.
  else if IncludedRate then begin
    // A fixed fee for each ServiceID plus excess data for the total account.
    // To start, create the revenue transactions with the fixed monthly fee
    // for each service. Set All item charges to zero - included in rental.
    with slDC do begin
      Clear;
      Add('INSERT INTO tDataCollection');
      Add('(SequenceNo, ServiceID, BatchID,');
      Add(' TxnDate, TxnTime, Origin,');
      Add(' Duration, CallCode, RateDescription,');
      Add(' FromDate, ToDate, TransactionTypeID,');
      Add(' AmountExGST, GSTAmount, GSTFlag, AmountIncGST,');
      Add(' RevenueFlag, Var01, Var04)');
      Add('SELECT tDataCollection.SequenceNo, tDataCollection.ServiceID, '+
          'tDataCollection.BatchID,');
      Add(' tDataCollection.TxnDate, tDataCollection.TxnTime, '+
          'tDataCollection.Origin,');
      Add(' tDataCollection.Duration, tDataCollection.CallCode, '+
          'tDataCollection.RateDescription,');
      Add(' tDataCollection.FromDate, tDataCollection.ToDate, '+
          'tDataCollection.TransactionTypeID,');
      // Set all recorded amounts to zero.
      Add(' (0) AS "AmountExGST", (0) AS "GSTAmount", tDataCollection.GSTFlag,');
      Add(' (0) AS "AmountIncGST", (1) AS "RevenueFlag",'+
      ' tDataCollection.Var01, ('+skByte+') AS "Var04"');
[ Open curly bracket...
      Add(' tDataCollection.AmountExGST, tDataCollection.GSTAmount, '+
          'tDataCollection.GSTFlag,');
      Add(' tDataCollection.AmountIncGST, (1) AS "RevenueFlag",'+
          ' tDataCollection.Var01, ('+sGByte+') AS "Var04"');
] Close curly bracket...
      Add('FROM tDataCollection');
      Add('WHERE tDataCollection.BatchID = '+sBatchID);
      Add('AND tDataCollection.RevenueFlag = 0');
[ Open curly bracket...
      Add('INSERT INTO tDataCollection');
      Add('(SequenceNo, ServiceID, BatchID,');
      Add(' TxnDate, TxnTime, Origin,');
      Add(' Duration, CallCode, RateDescription,');
      Add(' FromDate, ToDate, TransactionTypeID,');
      Add(' AmountExGST, GSTAmount, GSTFlag, AmountIncGST,');
      Add(' RevenueFlag, Var01, Var02, Var03, Var04)');
      Add('SELECT tDataCollection.SequenceNo, tDataCollection.ServiceID, '+
          'tDataCollection.BatchID,');
      Add(' tDataCollection.TxnDate, tDataCollection.TxnTime, '+
          'tDataCollection.Origin,');
      Add(' tDataCollection.Duration, tDataCollection.CallCode, '+
          'tDataCollection.RateDescription,');
      Add(' tDataCollection.FromDate, tDataCollection.ToDate, '+
          'tDataCollection.TransactionTypeID,');
      Add(' DC3.AmountExGST, DC3.GSTAmount, DC3.GSTFlag,');
      Add(' CAST((DC3.AmountExGST + DC3.GSTAmount) AS money) AS '+
          '"AmountIncGST", (1) AS "RevenueFlag",');
      Add(' tDataCollection.Var01, CONVERT(varchar,DC3.Cost,2) AS "Var02",');
      Add(' CONVERT(varchar,DC3.AmountExGST,2) AS "Var03", ('+
          sGByte+') AS "Var04"');
      Add('FROM (SELECT DC2.*,');
      Add(' CAST((CASE WHEN DC2.GSTFlag LIKE ''Y'' THEN (DC2.AmountExGST * '+
          sGSTRate+') ELSE 0 END) AS money) AS "GSTAmount"');
      Add('FROM (SELECT tDataCollection.ID AS "DataCollectionID", '+
          'tServiceID.ServiceTypeID, tGroup.CustomerID, tRate.Cost,');
      Add('  (tRate.Cost / 12) AS "AmountExGST", '+
          '(CASE WHEN tGroup.CustomerID = '+sVRTID+
          ' THEN ''N'' ELSE ''Y'' END) AS "GSTFlag"');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID '+
          'ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('LEFT JOIN tCentre ON tServiceID.CustomerCentreID = tCentre.ID');
      Add('LEFT JOIN tDepartment ON tCentre.DepartmentID = tDepartment.ID');
      Add('LEFT JOIN tGroup ON tDepartment.GroupID = tGroup.ID');
      Add('LEFT JOIN tRate '+
          'ON ((tServiceID.ServiceTypeID = tRate.ServiceTypeID)'+
          ' AND (tRate.Active = 1))');
      Add('WHERE tDataCollection.BatchID = '+sBatchID);
      Add('AND tDataCollection.RevenueFlag = 0) AS DC2) AS DC3');
      Add('LEFT JOIN tDataCollection '+
        'ON DC3.DataCollectionID = tDataCollection.ID');
] Close curly bracket...
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;

    // Fix the date fields first.
    sFromDate := Copy(sFromDate, 1, Length(sFromDate) - 2);
    sToDate := Copy(sToDate, 1, Length(sToDate) - 2);
    // Read the transaction type to be used.
    with slDC do begin
      Clear;
      Add('SELECT tTransactionType.ID');
      Add('FROM tTransactionType');
//      Add('WHERE tTransactionType.Description LIKE ''%Data Usage%''');
      Add('AND tTransactionType.Description LIKE ''%Excess%''');
      Add('AND tTransactionType.SystemID = '+sSystemID);
      Add('AND tTransactionType.Active = 1');
    end;
[ Open curly bracket...
// Not used - refer to read in common area above in this procedure.
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    sTransactionTypeID := qryDC.FieldByName('ID').AsString;
    qryDC.Close;
] Close curly bracket...
    // Count how many mobile services are being invoiced.
    with slDC do begin
      Clear;
      Add('SELECT COUNT(DISTINCT tDataCollection.ServiceID) AS "TotalCount"');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID '+
          'ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      Add('AND tServiceID.ServiceTypeID = '+sServiceTypeID);
//      Add('AND tDataCollection.RevenueFlag = 0');
//      Add('AND tServiceID.ServiceID IS NOT NULL');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    nServiceCount := qryDC.FieldByName('TotalCount').AsInteger;
    qryDC.Close;

    // Get the rental & data usage included in the fixed cost for each mobile.
    with slDC do begin
      Clear;
      Add('SELECT CAST(tRate.Length AS int) AS "Length", tRate.PartCost,'+
          ' tRate.Cost');
      Add('FROM tRate');
      Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
      Add('AND tRate.Active = 1');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    nIncludedUse := qryDC.FieldByName('Length').AsInteger * nServiceCount;
    sIncludedUse := IntToStr(nIncludedUse);
    fPartCost    := qryDC.FieldByName('PartCost').AsFloat;
    curExcessAmtExGST  := qryDC.FieldByName('Cost').AsCurrency / 12;
    qryDC.Close;
    curGSTAmount := curExcessAmtExGST * MAINFORM.GST_RATE;

    // Save the Rentals.
    with slDC do begin
      Clear;
      Add('INSERT INTO tDataCollection');
      Add('(ServiceID, BatchID, FromDate, ToDate, TransactionTypeID,'+
          ' AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag,'+
          ' Var02, Var03, Var04)');
//          ' Var01, Var02, Var03, Var04)');
//      Add('SELECT DISTINCT tDataCollection.ServiceID,');
      Add('SELECT DC.ServiceID,');
      Add('DC.BatchID,');
      Add(sFromDate+' AS "FromDate",');
      Add(sToDate+' AS "ToDate",');
      Add(sTransactionTypeID+' AS "TransactionTypeID", ');
      Add(CurrToStr(curExcessAmtExGST)+' AS "AmountExGST", ');
      Add(CurrToStr(curGSTAmount)+' AS "GSTAmount",');
      Add(CurrToStr(curExcessAmtExGST + curGSTAmount)+' AS "AmountIncGST",');
      Add('''Y'' AS "GSTflag",');
      Add('(1) AS "RevenueFlag",');
//      Add('CAST(DC.kB AS varchar) AS "Var01",');
      Add('('+ CurrToStr(curExcessAmtExGST * 12) +') AS "Var02",');
      Add('('+ CurrToStr(curExcessAmtExGST) +') AS "Var03",');
      Add('('+ skByte +') AS "Var04"');
      Add('FROM (SELECT DISTINCT tDataCollection.ServiceID, tDataCollection.BatchID');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('WHERE tDataCollection.BatchID = '+sBatchID);
      Add('AND tDataCollection.Ratedescription NOT LIKE ''%ROUND%''');
      Add('AND tServiceID.ServiceTypeID = '+sServiceTypeID+') AS DC');
[ Open curly bracket...
      Add('FROM (SELECT DC1.*');
      Add('FROM (SELECT tDataCollection.ServiceID,');
      Add('SUM(CAST(tDataCollection.Var01 AS integer)) AS "kB",');
      Add('tDataCollection.BatchID');
      Add('FROM tDataCollection');
      Add('WHERE tDataCollection.BatchID = '+sBatchID);
//      Add('AND tDataCollection.RevenueFlag = 0');
      Add('AND tDataCollection.Ratedescription NOT LIKE ''%ROUND%''');
      Add('GROUP BY tDataCollection.ServiceID, tDataCollection.BatchID) AS DC1');
      Add('LEFT JOIN tServiceID ON DC1.ServiceID = tServiceID.ServiceID');
      Add('WHERE tServiceID.ServiceTypeID = '+sServiceTypeID+') AS DC');
] Close curly bracket...
      Add('ORDER BY DC.ServiceID');
    end;  
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;

[ Open curly bracket...
    // Now determine if there are any excess usage charges to be claimed.
    // Get the ServiceTypeID linked to this invoice.
    with slDC do begin
      Clear;
      Add('SELECT DISTINCT tServiceID.ServiceTypeID');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID '+
          'ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      Add('AND tDataCollection.Ratedescription NOT LIKE ''%ROUND%''');
      Add('AND tServiceID.ServiceTypeID IS NOT NULL');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    sServiceTypeID := qryDC.FieldByName('ServiceTypeID').AsString;
    qryDC.Close;
] Close curly bracket...
    if (Length(sServiceTypeID) = 0) then sServiceTypeID := 'NULL';

    // Test to see if the total usage exceeds the nett included amount.
    if (nkByte < nIncludedUse) then begin
      // No excess fees are to be applied.
      curExcessAmtExGST := 0;
      nkByte := 0;
    end
    else begin
      // Included data bandwidth exceeded - charge for the excess amount.
      curExcessAmtExGST := (nkByte - nIncludedUse) * fPartCost;
      if (curExcessAmtExGST > 0) then begin
        curGSTAmount := curExcessAmtExGST * MAINFORM.GST_RATE;
        // First, read the ServiceID to hold the Excess charges.
        with slDC do begin
          Clear;
          Add('SELECT ParmText');
          Add('FROM tParameter');
          Add('WHERE tParameter.Parm LIKE ''ExcessDataServiceID''');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.Open;
        sNewServiceID := qryDC.FieldByName('ParmText').AsString;
        qryDC.Close;
        // Now read the transaction type to be used.
        with slDC do begin
          Clear;
          Add('SELECT tTransactionType.ID');
          Add('FROM tTransactionType');
          Add('WHERE tTransactionType.Description LIKE ''%Data Usage%''');
          Add('AND tTransactionType.Description LIKE ''%Excess%''');
          Add('AND tTransactionType.Active = 1');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.Open;
        sTransactionTypeID:= qryDC.FieldByName('ID').AsString;
        qryDC.Close;
        if (Length(sTransactionTypeID) = 0) then sTransactionTypeID := 'NULL';
[ Open curly bracket...
        // Get the number of characters in the sequence number.
        with slDC do begin
          Clear;
          Add('SELECT tDataCollection.SequenceNo');
          Add('FROM tDataCollection');
          Add('WHERE tDataCollection.BatchID = '+sBatchID);
          Add('AND tDataCollection.RevenueFlag = 0');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.Open;
        nSequenceNoLength := Length(qryDC.FieldByName('SequenceNo').AsString);
        qryDC.Close;
        // Get the last entry in sequence number.
        qryDC.SQL.Strings[0] :=
          'SELECT Max(CAST(tDataCollection.SequenceNo AS INT)) AS "SequenceNo"';
        qryDC.Open;
        nSequenceNoLast := qryDC.FieldByName('SequenceNo').AsInteger;
        qryDC.Close;
] Close curly bracket...
        // Ready to add the Excess charges above the included allowance.
[ Open curly bracket...
        // Fix the date fields first.
        sFromDate := Copy(sFromDate, 1, Length(sFromDate) - 2);
        sToDate := Copy(sToDate, 1, Length(sToDate) - 2);
] Close curly bracket...
        with slDC do begin
          Clear;
          Add('INSERT INTO tDataCollection');
          Add('(ServiceID, BatchID, FromDate, ToDate, TransactionTypeID,'+
              ' AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag,'+
              ' Var02)');
          Add('VALUES('''+sNewServiceID+''',');
          Add(sBatchID+',');
          Add(sFromDate+',');
          Add(sToDate+',');
          Add(sTransactionTypeID+',');
          Add(CurrToStr(curExcessAmtExGST)+',');
          Add(CurrToStr(curGSTAmount)+',');
          Add(CurrToStr(curExcessAmtExGST + curGSTAmount)+',');
          Add('''Y'',');
          Add('1, '+CurrToStr(curExcessAmtExGST)+')');
        end;
        qryDC.SQL.Text := slDC.Text;
        qryDC.ExecSQL;
[ Open curly bracket...
        // And now the sequence numbers
...to be continued...
] Close curly bracket...
      end;  // curExcessAmtExGST.
    end;  // Excess data fees.
    // Make sure that all entries for this BatchID do not have
    // RevenueFlag=Null so that revenue Transactions will not be created when
    // transferring to the tTransaction table later using SaveToTransaction().
    with slDC do begin
      Clear;
      Add('UPDATE tDataCollection');
      Add('SET RevenueFlag = 0');
      Add('WHERE RevenueFlag IS NULL');
      Add('AND BatchID = '+sBatchID);
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;

    // Update the values stored in the tBatch table.
    with slDC do begin
      Clear;
      Add('UPDATE tBatch');
      Add('SET TotalRevenue = BatchRate.RevenueAmt,');
      Add(' TotalCredits = '+ FloatToStrF(fPartCost,ffFixed,10,5) +',');
      Add(' OpeningBalance = '+ CurrToStr(curExcessAmtExGST) +',');
      Add(' OrderNumber = '''+ IntToStr(nkByte) +''',');
      Add(' TenderNo = '''+ IntToStr(nServiceCount) +''',');
      Add(' InvoiceNo = '''+ sServiceTypeID +''',');
      Add(' RequisitionNumber = '''+ sIncludedUse +'''');
      Add('FROM (SELECT Sum(tDataCollection.AmountExGST) AS "RevenueAmt"');
      Add('FROM tDataCollection');
      Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      Add('AND tDataCollection.RevenueFlag = 1) AS BatchRate');
      Add('WHERE tBatch.ID = '+ sBatchID);
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;
  end  // IncludedRate.
  else begin
    // The usage for each ServiceID is calculated and the transactions updated.
    with slDC do begin
      Clear;
      Add('INSERT INTO tDataCollection');
      Add('(SequenceNo, ServiceID, BatchID,');
      Add(' TxnDate, TxnTime, Origin,');
      Add(' Duration, CallCode, RateDescription,');
      Add(' FromDate, ToDate, TransactionTypeID,');
      Add(' AmountExGST, GSTAmount, GSTFlag, AmountIncGST,');
      Add(' RevenueFlag, Var01, Var02, Var03, Var04)');
      Add('SELECT tDataCollection.SequenceNo, tDataCollection.ServiceID, '+
          'tDataCollection.BatchID,');
      Add(' tDataCollection.TxnDate, tDataCollection.TxnTime, '+
          'tDataCollection.Origin,');
      Add(' tDataCollection.Duration, tDataCollection.CallCode, '+
          'tDataCollection.RateDescription,');
      Add(' tDataCollection.FromDate, tDataCollection.ToDate, '+
          'tDataCollection.TransactionTypeID,');
      Add(' DC3.AmountExGST, DC3.GSTAmount, DC3.GSTFlag,');
      Add(' CAST((DC3.AmountExGST + DC3.GSTAmount) AS money) AS '+
          '"AmountIncGST", (1) AS "RevenueFlag",');
      Add(' tDataCollection.Var01, CONVERT(varchar,DC3.Cost,2) AS "Var02",');
      Add(' CONVERT(varchar,DC3.AmountExGST,2) AS "Var03", ('+
          sGByte+') AS "Var04"');
      Add('FROM (SELECT DC2.*,');
      Add(' CAST((CASE WHEN DC2.GSTFlag LIKE ''Y'' THEN (DC2.AmountExGST * '+
          sGSTRate+') ELSE 0 END) AS money) AS "GSTAmount"');
      Add('FROM (SELECT DC1.DataCollectionID, DC1.Cost,');
      Add(' (CAST(tDataCollection.Var01 AS Integer)*Cost) AS "AmountExGST",');
      Add(' (CASE WHEN DC1.CustomerID = '+sVRTID+
          ' THEN ''N'' ELSE ''Y'' END) AS "GSTFlag"');
      // Include Incremental values in determining the final charge rate (cost).
      Add('FROM (SELECT DCFullRate.DataCollectionID, '+
          'DCFullRate.ServiceTypeID, DCFullRate.CustomerID,');
      Add(' (DCFullRate.Cost + (CASE WHEN '+sGByte+' > DCFullRate.Usage THEN');
      Add(' (DCFullRate.PartCost * ('+sGByte+' - DCFullRate.Usage)) '+
          'ELSE 0 END)) AS "Cost"');
      // Use Min() rate a/c price is reduced for more usage.
      Add('FROM (SELECT DCRate.*, tRate.Length AS "Usage",');
      Add(' (CASE WHEN tRate.PartCost IS NULL THEN 0 ELSE tRate.PartCost END'+
          ') AS "PartCost"');
      Add('FROM (SELECT DC0.*, Min(tRate.Cost) AS "Cost"');
      Add('FROM (SELECT tDataCollection.ID AS "DataCollectionID",');
      Add(' tServiceID.ServiceTypeID, tGroup.CustomerID');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID '+
          'ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('LEFT JOIN tCentre ON tServiceID.CustomerCentreID = tCentre.ID');
      Add('LEFT JOIN tDepartment ON tCentre.DepartmentID = tDepartment.ID');
      Add('LEFT JOIN tGroup ON tDepartment.GroupID = tGroup.ID');
      Add('WHERE tDataCollection.BatchID = '+sBatchID);
      Add('AND tDataCollection.RevenueFlag = 0) AS DC0');
      Add('LEFT JOIN tRate '+
          'ON ((DC0.ServiceTypeID = tRate.ServiceTypeID) AND (tRate.Length <= '+
          sGByte+') AND (tRate.Active = 1))');
      Add('GROUP BY DC0.DataCollectionID, DC0.ServiceTypeID, DC0.CustomerID'+
          ') AS DCRate');
      Add('LEFT JOIN tRate ON ((DCRate.ServiceTypeID = tRate.ServiceTypeID)');
      Add(' AND (tRate.Cost = DCRate.Cost) AND (tRate.Active = 1))');
      Add('WHERE tRate.Cost = DCRate.Cost) AS DCFullRate) AS DC1');
      Add('LEFT JOIN tDataCollection '+
          'ON DC1.DataCollectionID = tDataCollection.ID) AS DC2) AS DC3');
      Add('LEFT JOIN tDataCollection '+
          'ON DC3.DataCollectionID = tDataCollection.ID');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;

    // Read a value to include in the update of the tBatch table.
    with slDC do begin
      Clear;
      Add('SELECT CAST(CAST(DC1.Cost AS real) AS varchar) AS "RateCost",');
      Add(' CAST(ServiceTypeID AS varchar) AS "ServiceType"');
      Add('FROM (SELECT Min(tRate.Cost) AS "Cost", DC0.ServiceTypeID');
      Add('FROM (SELECT tServiceID.ServiceTypeID');
      Add('FROM tDataCollection');
      Add('LEFT JOIN tServiceID ON tDataCollection.ServiceID = tServiceID.ServiceID');
      Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      Add('AND tDataCollection.RevenueFlag = 0) AS DC0');
      Add('LEFT JOIN tRate ON ((DC0.ServiceTypeID = tRate.ServiceTypeID)');
      Add(' AND (tRate.Active = 1)');
      Add(' AND (tRate.Length <= '+ sGByte+'))');
      Add('GROUP BY DC0.ServiceTypeID) AS DC1');
      Add('WHERE DC1.Cost <> 0');
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.Open;
    sRateCost := qryDC.FieldByName('RateCost').AsString;
    sServiceType := qryDC.FieldByName('ServiceType').AsString;
    if (Length(sRateCost) = 0) then sRateCost := '0';  // Avoids invalid "Set =".
    qryDC.Close;

    // Make sure that all entries for this BatchID do not have
    // RevenueFlag=Null so that extra revenue Transactions will not be created
    // when transferring to tTransaction later using SaveToTransaction().
    with slDC do begin
      Clear;
      Add('UPDATE tDataCollection');
      Add('SET RevenueFlag = 0');
      Add('WHERE RevenueFlag IS NULL');
      Add('AND BatchID = '+sBatchID);
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;

    // Update some more of the values stored in the tBatch table.
    with slDC do begin
      Clear;
      Add('UPDATE tBatch');
      Add('SET TotalRevenue = BatchRate.RevenueAmt,');
      Add(' TotalCredits = '+ sRateCost +',');
      Add(' OrderNumber = '+ sGByte +',');
      Add(' InvoiceNo = '''+ sServiceType +''',');
      Add(' RequisitionNumber = '+ IntToStr(nkByte));
      Add('FROM (SELECT Sum(tDataCollection.AmountExGST) AS "RevenueAmt"');
      Add('FROM tDataCollection');
      Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      Add('AND tDataCollection.RevenueFlag = 1) AS BatchRate');
      Add('WHERE tBatch.ID = '+ sBatchID);
    end;
    qryDC.SQL.Text := slDC.Text;
    qryDC.ExecSQL;
  end;  // LumpedCosts.

  // Done. Do some housework.
  MessageForm.Close;
  MessageForm.Free;
  slDC.Free;
  qryDC.Free;
end;  // ApplyDataRatesEBS.
}


procedure BillingInputDataEBS( const LumpedCosts, IncludedRate: boolean;
  const stInFiles: Tstrings; nBatch: LongInt );
// This program processes the Telstra billing reports supplied on disk
// and places the data into the tDataCollection table for verification.
//
// The parameter passed contains the names of the input files, which are
// passed on to the ImportFile() procedure.
//
var
  lContinue : Boolean;
  sPartFileName : String;
  nBackSlashAt: Integer;
  nPos : integer;
  EmptyList : TStringList;          // Used only when replacing DC transactions.
begin
  // initialise the Static table elements defined as Private for this form.
  qryTelstra := TQuery.Create(MainForm);
  with qryTelstra do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('SELECT id');
    SQL.Add('FROM tSupplier');
    SQL.Add('WHERE SupplierName LIKE ''TELSTRA%''');
    Open;
  end;
  if qryTelstra.Eof then begin
    MessageDlg('TELSTRA not found in the tSupplier table. Processing stopped.',
      mtWarning, [mbOk], 0);
  end
  else begin
    nSupplierID := qryTelstra['id'];
    sSupplierID := IntToStr(nSupplierID);

    with qryTelstra do begin
      Close;
      SQL.Clear;
      SQL.Add('SELECT tValidAccountNumber.AccountNo');
      SQL.Add('FROM tValidAccountNumber');
      SQL.Add('INNER JOIN tBatch ON tValidAccountNumber.BatchTypeID = tBatch.BatchTypeID');
      SQL.Add('WHERE tBatch.ID = '+IntToStr(nBatch));
      Open;
      if Eof then begin
        MessageDlg('BatchType not found in the tValidAccountNumber table. '+
          'Processing stopped.',
          mtWarning, [mbOk], 0);
        lContinue := False;
      end
      else begin
        sAccountNumber := Trim(FieldByName('AccountNo').AsString);
        nPos := Pos(' ',sAccountNumber);
        while (nPos > 0) do begin
          sAccountNumber := Copy(sAccountNumber,1,nPos-1) +
                            Copy(sAccountNumber,nPos+1,Length(sAccountNumber)-nPos);
          nPos := Pos(' ',sAccountNumber);
        end;  // nPos.
        lContinue := True;
      end;  // Eof.
      Close;
    end;  // qryTelstra.
  end;
  if lContinue then begin
    with qryTelstra do begin
      Close;
      SQL.Clear;
      SQL.Add('SELECT Code, CAST(TransactionTypeID AS VARCHAR(20)) '+
        'AS TransactionType, CAST(ID AS VARCHAR(20)) AS SupplierCodeID');
      SQL.Add('FROM tSupplierServiceCode');
      SQL.Add('WHERE SupplierID = '+sSupplierID);
      SQL.Add('AND Description LIKE ''RENT''');
      Open;
      if Eof then sRentCodeID := 'NULL'
      else begin
        sRentCodeID := Trim(FieldByName('Code').AsString);
        sSupplierRentCodeID := Trim(FieldByName('SupplierCodeID').AsString);
      end;
      Close;
      SQL.Strings[qryTelstra.SQL.Count-1] := 'AND Description LIKE ''CALLS''';
      Open;
      if Eof then sCallCodeID := 'NULL'
      else begin
        sCallCodeID := Trim(FieldByName('Code').AsString);
        sSupplierCallCodeID := Trim(FieldByName('SupplierCodeID').AsString);
      end;
      Close;
      SQL.Strings[qryTelstra.SQL.Count-1] := 'AND Description LIKE ''Unknown%''';
      Open;
      if Eof then sOtherCodeID := 'NULL'
      else begin
        sOtherCodeID := FieldByName('Code').AsString;
        sSupplierOtherCodeID := Trim(FieldByName('SupplierCodeID').AsString);
      end;
      Close;
      SQL.Strings[qryTelstra.SQL.Count-1] :='AND Description LIKE ''MISCELLANEOUS''';
      Open;
      if qryTelstra.Eof then sDiscountCodeID := 'NULL'
      else begin
        sDiscountCodeID := Trim(FieldByName('Code').AsString);
        sSupplierDiscountCodeID := Trim(FieldByName('SupplierCodeID').AsString);
      end;
      Close;
      SQL.Delete(SQL.Count-1);
      Open;
    end;  // with qryTelstra.

    // Initialise the progress bar to display to the operator.
    frmProgress := TProgressForm.Create(MainForm);
    frmProgress.btnCancel.Visible := False;
    frmProgress.Show;

    sPartFileName := stInFiles.Strings[0];
    while Pos('\',sPartFileName)>0 do begin
      nBackSlashAt  := Pos('\',sPartFileName);
      sPartFileName := Copy(sPartFileName,
        nBackSlashAt+1, Length(sPartFileName)-nBackSlashAt);
    end;  // while Pos('\',sPartFileName)>0 do begin.
    frmProgress.Msg.Caption := 'Reading ' + sPartFileName + ' ...';
    frmProgress.ProgressBar1.Position := 0;
    frmProgress.Msg.Refresh;

    // Flush out any data in case this is a reload.
    EmptyList := TStringList.Create;       // Removes all entries for the batch.
    lContinue := True;
    DeleteFromDC( nBatch, EmptyList, lContinue);
    EmptyList.Free;

    // Now move the text data into the Table.
    if lContinue then begin
{      ImportDataFile( stInFiles.Strings[0], DM.dbPPdata, nBatch, lContinue);
}
      GetDetailDataEBS( stInFiles.Strings[0],
          DM.dbPPdata, nBatch, lContinue );
    end;

    frmProgress.Hide;
    frmProgress.Free;
    if lContinue then begin
      ApplyTelstraDataRates(nBatch, LumpedCosts, IncludedRate);
      if not LumpedCosts then UpdateBatchTotals(nBatch);
      UpdateBatchStatus(nBatch, MainForm.BATCH_STATUS_IMPORT);

      // **********************************************************************
      // **********************************************************************
      // Test for critical errors that prevent progress to the next stage.
      lContinue := False;
      CriticalErrorTest( nBatch, lContinue );
      if lContinue then begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Passed all tests for Critical Errors.', mtInformation,
          [mbOk], 0);
      end
      else begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Critical errors found.'#13#10#13#10'These must be resolved ' +
          'before this invoice can be progressed.',
          mtWarning, [mbOk], 0);
      end;
    end;  // if lContinue then begin.
  end;  // if qryTelstra.Eof then begin..else.
end;    // BillingInputDataEBS.


procedure TelstraInvoiceDataEBS( const LumpedCosts, IncludedRate: boolean;
  const nBatch: LongInt; sDirectory: string );
// This function prompts the user for the name of the Telstra supplied
// text file and then calls the BillingInput function (which reads the
// file into a table for further processing).
var
  lContinue : Boolean;
  stFiles : TStringList;
  OpenDlg: TOpenDialog;
  EmptyList : TStringList;
begin
  // Create the stringlist which contains the names of selected filenames.
  stFiles := TStringList.Create;
  try
    // Now, we must establish which files are to be processed.
    // Select the source directory.
    OpenDlg := TOpenDialog.Create(MainForm);
    OpenDlg.Filter := 'Telstra Billing file (*.dat)|*.DAT|All files (*.*)|*.*';
    OpenDlg.FilterIndex := 1;
    // Use the program directory as the default directory.
    if (Length(sDirectory) > 0) then OpenDlg.InitialDir := sDirectory
    else OpenDlg.InitialDir := ExtractFilePath(ParamStr(0));
    OpenDlg.Title := 'Select The Telstra Data Account File';
    OpenDlg.Options := [ofFileMustExist, ofNoChangeDir];
    // Read and save the user's choices before continuing.
    lContinue := OpenDlg.Execute;
    stFiles.Text := OpenDlg.Files.Text;
    OpenDlg.Free;
    if lContinue then begin
      lContinue := stFiles.Count > 0;
      // Now call for processing of the input files.
      if lContinue then begin
        // Empty stringlist => Remove all entries for BatchID.
        EmptyList := TStringList.Create;
        DeleteFromDC( nBatch, EmptyList, lContinue);
        EmptyList.Free;
        BillingInputDataEBS(LumpedCosts, IncludedRate, stFiles, nBatch);
      end;  // if lContinue then.
    end;  // if lContinue then.
  finally
    stFiles.Free;
  end;
end;    // procedure TelstraInvoiceDataEBS.


end.
