unit TelstraMobileDataOBS;

interface
                                
uses
  Messages, SysUtils, Classes, Graphics, Controls, Dialogs,
  db, dbTables;

type
  TServiceDetails = record
    Custno   : string;
    ServCode : string;
    ServiceID: string;
    RentFrom : TDateTime;
    RentTill : TDateTime;
    CallFrom : TDateTime;
    CallTill : TDateTime;
  end;

  TCallHeaderType = record
    SequenceNo      : string[7];
    ServiceID       : string[17];
    TxnDate         : string[8];
    TxnTime         : string[5];
    Origin          : string[20];
    Duration        : string[9];
    CallCode        : string[2];
    RateDescription : string[32];
    DialledNumber   : string[17];  // Temporary variable only.
    ServiceNotes    : string[22];
    AmountExGST     : string[13];
    GSTAmount       : string[13];
    AmountIncGST    : string[13];
    ToDate          : string[8];
  end;

  procedure TelstraData( const LumpedCosts, IncludedRate: boolean;
    const nBatch: LongInt; sDirectory: string );
//  procedure GetMobileData(const FName, sSupplierAccount, sSupplierInvoice:String;
  procedure GetMobileData(const FName:String;
    D:TDatabase;
    nBatchNumber : LongInt;
    var lReturnCode:Boolean);
{
  procedure ImportDataFile( const FName: String; D: TDatabase; nBatch: LongInt;
    var lReturnCode: Boolean);
}
  procedure BillingInputData( const LumpedCosts, IncludedRate: boolean;
    const stInFiles: Tstrings; nBatch: LongInt );
  procedure ApplyDataRates(const nBatch: integer;
    LumpedCosts, IncludedRate: boolean);


implementation

uses
  CriticalTests,
  DataMod, DateFunctions,
  GenFns, FileFunctions,
  Main, Math, MyMessage,
  Progress;

var
  // Declare static variables as frequently reset and reused.
  nCount: Integer;             // Current status for frmProgress.
  nFactor: Real;               // Weighting for nCount in progress.
  frmProgress: TProgressForm;  // frmProgress report to the operator.
  nSupplierID: integer;        // Set in BillingInput().
  sSupplierID: string;         // Set in BillingInput().
  qryTelstra : TQuery;         // Set in BillingInput().
  sRentCodeID : string;        // Set in BillingInput().
  sCallCodeID : string;        // Set in BillingInput().
  sOtherCodeID : string;       // Set in BillingInput().
  sDiscountCodeID : string;    // Set in BillingInput().
  sAccountNumber : string;     // Set in BillingInput().

procedure GetMobileData
          (
//          const FName, sSupplierAccount, sSupplierInvoice: String;
          const FName: String;
          D: TDatabase;
          nBatchNumber : LongInt;
          var lReturnCode: Boolean
          );
const
  PROGRESS_MAX_C: Integer = 100; // Maximum progress is 100%.
  CODE_C:   integer = 0;                // Billing code.
  TACC_C:   integer = 1;                // To Account.
  ACCTO_C:  integer = 2;                // Telstra Acct sent to (at VicTrack).
  ACCT_C:   integer = 3;                // AccountNumber.
  DESC05_C :integer = 4;                // Invoice Date.
  DESC06_C: integer = 5;                // Description field (ServiceID).
  DESC07_C: integer = 6;                // Description field (Code 1).
  DESC08_C: integer = 7;                // Description field (Code 2).
  DESC09_C: integer = 8;                // Description field (Code 3).
  DESC10_C: integer = 9;                // Description field (Code 4).
  DESC11_C: integer = 10;               // Description field (Call Detail=T).
  DESC12_C: integer = 11;               // Description field (AmountExGST).
  DESC13_C: integer = 12;               // Description field.
  DESC14_C: integer = 13;               // Description field (AmountGST).
  DESC15_C: integer = 14;               // Description field (AmountIncGST).
  DESC16_C: integer = 15;               // Description field.
  DESC17_C: integer = 16;               // Description field.
  DESC18_C: integer = 17;               // Description field.
  DESC19_C: integer = 18;               // Description field.
  DESC20_C: integer = 19;               // Description field.
  DESC21_C: integer = 20;               // Description field.
  DESC22_C: integer = 21;               // Description field.
  DESC23_C: integer = 22;               // Description field.
  DESC24_C: integer = 23;               // Description field.
  DESC25_C: integer = 24;               // Description field.
  DESC26_C: integer = 25;               // Description field.
  DESC27_C: integer = 26;               // Description field.
  DESC28_C: integer = 27;               // Description field.
  DESC29_C: integer = 28;               // Description field.
  DESC30_C: integer = 29;               // Description field.
  DESC31_C: integer = 30;               // Description field.
  DESC32_C: integer = 31;               // Description field.
  DESC33_C: integer = 32;               // Description field.
  DESC34_C: integer = 33;               // Description field.
var
  nFileSize: Integer;                  // File size in bytes.
  sPartFileName : string;
  sPartResultSet : string;
  EntryCode : string;
  nBackSlashAt : Integer;
  TelstraFile: TextFile;               // Used for accessing the data.
  CurrentFile: String;                 // Currently open file.
  FileIteration: Integer;              // Current file iteration being processed
  T: string;                           // Line of data read from ReadString.
  ReadString: string;                  // Line of data read from TextFile.
  SubString: String;                   // A (temp) substring of T.
  vSubString : variant;                // - ditto, used in the Locate() function.
  curValue: Currency;                  // A (temp) currency conversion value.
  nSoFar: Integer;                     // Status - num of records processed.
  dBilling: TDateTime;                 // Billing date.
  sBatch : string;
  LastEntryIs_1 : boolean;
  CallHeader : TCallHeaderType;
  sLastDetailCode : string[2];         // To detect a header line AFTER details.

  DBfromDate : string;                 // Section DB - save read dates.
  DBtoDate : string;

  aResultSet : Array of TServiceDetails;
  nResultSet : Integer;
  nStartSearchAt : Integer;
  lFound : Boolean;
  nCommaPosition : integer;
  sListValues : String;
  sListFields : String;
  FieldStringList : TStringList;
  ValueStringList : TStringList;
  qryStringList : TStringList;
  slAddFieldNames : TStringList;         // Calls to AddToTable() only.
  slAddFieldValues : TStringList;        // Calls to AddToTable() only.
  ConvertString : TStringList;
  qryDC : TQuery;
  nListValues : integer;
  nDummy : Real;
  sInputLineCode : string;
  lSaveToTable : boolean;
  lDataSaved : boolean;
  lContinue : boolean;
  sRateDescription : string;
  sLastService : string;
  sLastServiceID : string;

  nDataCollectionID : LongInt;
  sLastCallCode : string;
  sLastTransactionTypeID : string;
begin
//  lContinue := False;
  lDataSaved := False;
  sLastDetailCode := '  ';
  sBatch := IntToStr(nBatchNumber);

  FieldStringList := TStringList.Create;
  ValueStringList := TStringList.Create;
  qryStringList := TStringList.Create;
  slAddFieldNames := TStringList.Create;         // Calls to AddToTable() only.
  slAddFieldValues := TStringList.Create;        // Calls to AddToTable() only.
  ConvertString := TStringList.Create;

  // Establish the query to be used to save the data read.
  qryDC := TQuery.Create(MainForm);
  qryDC.DatabaseName := 'dbPPdata';

  lReturnCode := False;
  CurrentFile := Fname;
  FileIteration := 0;            // Original file extension, then 001,002, etc.
//  dMinDate := EncodeDate(1980,1,1);
  while FileExists(CurrentFile) do begin
    // Read the file size to determine the approximate number of records.
    nFileSize := GetFileSize(CurrentFile);
    // Now we can calculate the factor for the progress bar to be displayed.
    // From prev. data: 8876 lines in a file of 1,585,557 bytes.
    // Estimate the number of data lines in the file.
    if (nFileSize > 0) then begin
      nFactor := 100.0 / 8876.0 * (1585557.0/nFileSize);
    end
    else begin
      nFactor := 100.0;
    end;
    nCount := 0;
    sPartFileName := CurrentFile;
    nBackSlashAt  := Pos('\',sPartFileName);
    while (nBackSlashAt > 0) do begin
      sPartFileName := Copy(sPartFileName,
        nBackSlashAt+1, Length(sPartFileName)-nBackSlashAt);
      nBackSlashAt  := Pos('\',sPartFileName);
    end;
    frmProgress.Caption := 'Telstra Mobile Data Batch: '+sBatch;
    frmProgress.Msg.Caption := 'Reading ' + sPartFileName + ' ...';
    frmProgress.ProgressBar1.Position := 0;
    frmProgress.Msg.Refresh;

    // Open the input (text) billing file for reading.
    AssignFile( TelstraFile, CurrentFile );
    Reset( TelstraFile );
    try
      Readln( TelstraFile, ReadString );
      // Ensure that this is a CSV file format (OBS billing)
      lContinue := (Pos(',',ReadString)>0);
      if not lContinue then begin
        frmProgress.Hide;
        MessageDlg('Incorrect input file type.'+#13#10+
          'File type CSV is required. Try again.',
          mtWarning, [mbOK],0);
      end
      else begin
        // Ensure that this is a Mobile file format.
        lContinue := (Copy(ReadString,1,2)='RB');
        if not lContinue then begin
          frmProgress.Hide;
          MessageDlg('Incorrect input file type.'+#13#10+
            'This is NOT a Mobile-type file. Try again.',
            mtWarning, [mbOK],0);
        end;
      end;
      if lContinue then begin
        // Separate into strings before adding to ConvertString.
        ReadString := StringReplace(ReadString, ',', #13#10, [rfReplaceAll]) +'''';
        ConvertString.Text := ReadString;
        // Skip to the Header Line and Determine the billing date only once.
        SubString := 'RB' + sAccountNumber;
//      end;  // lContinue.
      lContinue := False;  // Reset then test for correct account.
      // Make sure that we find the valid Account first.
      while not lContinue and
        (not ((Length(Trim(ReadString))=0) and Eof( TelstraFile ))) do begin
        lContinue := ((ConvertString.Strings[CODE_C]+
                       ConvertString.Strings[ACCT_C]) = SubString);
        if not lContinue then begin
          Readln( TelstraFile, ReadString );
          if (not ((Length(Trim(ReadString))=0) and Eof( TelstraFile )))
            then begin
              // Separate into strings before adding to ConvertString.
              ReadString := StringReplace(ReadString, ',', #13#10,
              [rfReplaceAll]) +'''';
              ConvertString.Text := ReadString;
          end;  // not empty or eof.
          // Report progress to the user.
          Inc(nCount);
          nDummy := nFactor * nCount;
          nSoFar := Round( nDummy );
          // Do not know how big the file is - do not exceed 100%.
          if (nSoFar > PROGRESS_MAX_C) then begin
            frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
          end
          else begin
            frmProgress.ProgressBar1.Position := nSoFar;
          end;  // PROGRESS_MAX_C.
        end;  // not lContinue.
      end;  // while <> SubString.
//      if not ((Trim(ReadString)='') and Eof( TelstraFile )) then begin
      if lContinue then begin
        // Calculate the billing date only once.
//      D.StartTransaction;
        // Continue to search through the remaining text of the file
        // for occurrences of the search string.  On each find, process
        // the resultant string.
        //****************************************************************
        //********* START READING BILLS **********************************
        //****************************************************************
        try
          repeat
            frmProgress.ProcessWinMessages(MainForm);
            lSaveToTable := True;
            // Read only relevant text into the database.

            // Some lines have fewer fields, so avoid program failure.
            if (ConvertString.Count >= DESC11_C) then
              T := ConvertString.Strings[DESC11_C];
            sInputLineCode := UpperCase(Trim(ConvertString.Strings[DESC07_C])+
                              Trim(ConvertString.Strings[DESC08_C]));


              // CALL DETAILS.
              if (sInputLineCode='2D') then begin
                sInputLineCode := sInputLineCode +
                  UpperCase(ConvertString.Strings[DESC09_C]+
                            ConvertString.Strings[DESC10_C]);
                SubString :=UpperCase(Trim(Copy(T,1,5)));            // SEQ.
                if (Length(sInputLineCode) < 5) then begin
                  lSaveToTable := False;
                end
                else if (Length(Trim(SubString)) = 0) then begin
                  // Column heading, skip on after checking if RateDescription.
                  if (Copy(sInputLineCode,5,1) = 'D') then begin
                    // Heading - Read the transaction type description.
                    sRateDescription := CleanString(Trim(Copy(T,6,30)));
                  end;  // ..and ignore any totals etc.
                  lSaveToTable := False;
                end
                else if (Trim(SubString) = 'SEQ') then begin
                  // Column heading, skip on.
                  lSaveToTable := False;
                end
                else begin
                // This line is a detail item & not a header/summary - save it.

                // **************************************************************

                if (sInputLineCode = '2D10D') and
                  (Pos(' DATA ', sRateDescription)>0) then begin
                  FieldStringList.Clear;
                  ValueStringList.Clear;
                  sListFields := '(';
                  sListValues := 'VALUES (';
                  SubString := Trim(Copy(T,1,5));
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'SequenceNo,';
                    sListValues := sListValues + '''' + SubString +''',';
                  end;
                  SubString := ConvertString.Strings[DESC06_C];
                  if not (SubString = sLastService) then begin
                    sLastService := SubString;
                    sLastServiceID := AccountFormat(sLastService);
                  end;
                  sListFields := sListFields + 'ServiceID,';
                  sListValues := sListValues +''''+ sLastServiceID +''',';
                  // Read the code common for each detail.
                  SubString := Copy(T,7,8);
                  if (Length(SubString) > 0) then begin
                    // Only 2-digit year, add 2000.
                    SubString := Copy(SubString,1,6) +'20'+ Copy(SubString,7,2);
                    sListFields := sListFields + 'TxnDate,';
                    sListValues := sListValues +''''+ SubString +''',';
                  end;
                  SubString := Copy(T,17,5);
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'TxnTime,';
                    sListValues := sListValues +''''+ SubString +':00'',';
                  end;
                  SubString := Trim(Copy(T,24,13));
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'Origin,';
                    sListValues := sListValues +''''+ CleanString(SubString) +''',';
                  end;
                  SubString := Trim(Copy(T,59,8));
                  if (Length(SubString) > 0) then begin
                    SubString := ConvertTimeToSecs(CleanString(SubString));
                    sListFields := sListFields + 'Duration,';
                    sListValues := sListValues +''''+ SubString +''',';
                  end;
                  SubString := CleanString(Copy(sInputLineCode,1,4));
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'CallCode,';
                    sListValues := sListValues +''''+ SubString +''',';
                    if (sLastCallCode <> SubString) then begin
                      sLastCallCode := SubString;
                      vSubString := SubString;
                      with qryTelstra do
                        lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                      if lFound then
                        sLastTransactionTypeID := qryTelstra['TransactionType']
                      else begin
                        sLastTransactionTypeID := sCallCodeID;
                        slAddFieldNames.Clear;
                        slAddFieldValues.Clear;
                        slAddFieldNames.Add('Code');
                        slAddFieldValues.Add(''''+sLastCallCode+'''');
                        slAddFieldNames.Add('AmountCategory');
                        slAddFieldValues.Add('''CALLS''');
                        slAddFieldNames.Add('Description');
                        slAddFieldValues.Add('''CALLS''');
                        slAddFieldNames.Add('TransactionTypeID');
                        slAddFieldValues.Add(sLastTransactionTypeID);
                        slAddFieldNames.Add('SupplierID');
                        slAddFieldValues.Add(sSupplierID);
                        slAddFieldNames.Add('Active');
                        slAddFieldValues.Add('1');
                        AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                        qryTelstra.Close;
                        qryTelstra.Open;
                      end;  // if (not) lFound.
                    end;  // if (sLastCallCode <> SubString) then begin.
                    sListFields := sListFields + 'TransactionTypeID,';
                    sListValues := sListValues + sLastTransactionTypeID+',';
                  end
                  else begin
                    sListFields := sListFields + 'TransactionTypeID,';
                    sListValues := sListValues + sCallCodeID+',';
                  end;  // CallCode.
                  SubString := Trim(Copy(T,55,4));
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'Peak,';
                    if (Right(SubString,1) = 'P') then
                      sListValues := sListValues +'1,'
                    else
                      sListValues := sListValues +'0,';
                  end;
                  sListFields := sListFields + 'RateDescription,';
                  sListValues := sListValues +''''+ sRateDescription +''',';
                  SubString := Trim(Copy(T,38,17));
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'DialledNumber,';
                    sListValues := sListValues +''''+
                    CleanString(SubString) +''',';
                  end;
                  FieldStringList.Add(sListFields);
                  ValueStringList.Add(sListValues);
                  sListFields := '';
                  sListValues := '';

                  SubString := Trim(Copy(T,60,7));
                  if (Length(SubString) > 0) then begin           // Usage (kB).
                    sListFields := sListFields + 'Var01,';
                    sListValues := sListValues +''''+ SubString +''',';
                  end;                                            // AmountExGST.
                  SubString := Trim(ConvertString.Strings[DESC12_C]);
                  sListFields := sListFields + 'RevenueFlag,AmountExGST,';
                  sListValues := sListValues + '0,'+ SubString +',';
                  SubString := Trim(ConvertString.Strings[DESC14_C]);
                  curValue := curFormat(StrToFloat(SubString));   // GSTAmount.
                  sListFields := sListFields + 'GSTAmount,GSTFlag,';
                  if (curValue = 0.00) then begin
                    sListValues := sListValues + SubString + ',''N'',';
                  end
                  else begin
                    sListValues := sListValues + SubString + ',''Y'',';
                  end;
                  SubString := Trim(ConvertString.Strings[DESC15_C]);
                  sListFields := sListFields + 'AmountIncGST,';   // AmountIncGST.
                  sListValues := sListValues + SubString +',';
                  if (Length(DBfromDate) = 0) or (Length(DBtoDate) = 0) then begin
                    SubString := '20'+ ConvertString.Strings[DESC05_C];
                    DBfromDate :=
                      FormattedDateString(LastMonth(StrDMYtoDate(YMD2DMY(SubString))));
                    DBtoDate :=
                    FormattedDateString(StrDMYtoDate(YMD2DMY(SubString))-1);
                  end;
                  sListFields := sListFields + 'FromDate,';
                  sListValues := sListValues +''''+ DBfromDate +''',';
                  sListFields := sListFields + 'ToDate,';
                  sListValues := sListValues +''''+ DBtoDate +''',';
                end  // 2D10D.

                // **********************************************************

                // Test for call diversions (2D13D) and SMS Enhanced (2D17).
                else if (sInputLineCode = '2D13D') or
                  (sInputLineCode = '2D17D') then begin
                  FieldStringList.Clear;
                  ValueStringList.Clear;
                  sListFields := '(';
                  sListValues := 'VALUES (';
                  SubString := Trim(Copy(T,1,5));
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'SequenceNo,';
                    sListValues := sListValues + '''' + SubString +''',';
                  end;
                  SubString := ConvertString.Strings[DESC06_C];    // ServiceID.
                  if not (SubString = sLastService) then begin
                    sLastService := SubString;
                    sLastServiceID := AccountFormat(sLastService);
                  end;
                  sListFields := sListFields + 'ServiceID,';
                  sListValues := sListValues +''''+ sLastServiceID +''',';
                  // Read the code common for each detail.
                  SubString := Copy(T,7,8);
                  if (Length(SubString) > 0) then begin
                    // Only 2-digit year, add 2000.
                    SubString := Copy(SubString,1,6) +'20'+ Copy(SubString,7,2);
                    sListFields := sListFields + 'TxnDate,';
                    sListValues := sListValues +''''+ SubString +''',';
                  end;
                  SubString := Copy(T,17,5);
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'TxnTime,';
                    sListValues := sListValues +''''+ SubString +':00'',';
                  end;
                  SubString := Trim(Copy(T,24,13));
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'Destination,';
                    sListValues := sListValues +''''+ CleanString(SubString) +''',';
                  end;
                  if (sInputLineCode = '2D13D') then begin
                    // First causes an error for 2D17D, other 3 not relevant - RJC070717.
                    SubString := Trim(Copy(T,59,6));
                    if (Length(SubString) > 0) then begin
                      SubString := ConvertTimeToSecs(CleanString(SubString));
                      sListFields := sListFields + 'Duration,';
                      sListValues := sListValues +''''+ SubString +''',';
                    end;
                    SubString := Trim(Copy(T,55,4));
                    if (Length(SubString) > 0) then begin
                      sListFields := sListFields + 'Peak,';
                      if (Right(SubString,2) = ' P') then
                        sListValues := sListValues +'1,'
                      else
                        sListValues := sListValues +'0,';
                    end;
                    if (sInputLineCode = '2D17D') then begin
                      sRateDescription := CleanString(Trim(Copy(T,24,30)));
                    end;
                    sListFields := sListFields + 'RateDescription,';
                    sListValues := sListValues +''''+ sRateDescription +''',';
                    SubString := Trim(Copy(T,112,13));
                    if (Length(SubString) > 0) then begin
                      sListFields := sListFields + 'DialledNumber,';
                      sListValues := sListValues +''''+
                      CleanString(SubString) +''',';
                    end;
                  end;
                  SubString := CleanString(Copy(sInputLineCode,1,4));
                  if (Length(SubString) > 0) then begin
                    sListFields := sListFields + 'CallCode,';
                    sListValues := sListValues +''''+ SubString +''',';
                    if (sLastCallCode <> SubString) then begin
                      sLastCallCode := SubString;
                      vSubString := SubString;
                      with qryTelstra do
                        lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                      if lFound then
                        sLastTransactionTypeID := qryTelstra['TransactionType']
                      else begin
                        sLastTransactionTypeID := sCallCodeID;
                        slAddFieldNames.Clear;
                        slAddFieldValues.Clear;
                        slAddFieldNames.Add('Code');
                        slAddFieldValues.Add(''''+sLastCallCode+'''');
                        slAddFieldNames.Add('AmountCategory');
                        slAddFieldValues.Add('''CALLS''');
                        slAddFieldNames.Add('Description');
                        slAddFieldValues.Add('''CALLS''');
                        slAddFieldNames.Add('TransactionTypeID');
                        slAddFieldValues.Add(sLastTransactionTypeID);
                        slAddFieldNames.Add('SupplierID');
                        slAddFieldValues.Add(sSupplierID);
                        slAddFieldNames.Add('Active');
                        slAddFieldValues.Add('1');
                        AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                        qryTelstra.Close;
                        qryTelstra.Open;
                      end;  // if (not) lFound.
                    end;  // if (sLastCallCode <> SubString) then begin.
                    sListFields := sListFields + 'TransactionTypeID,';
                    sListValues := sListValues + sLastTransactionTypeID+',';
                  end
                  else begin
                    sListFields := sListFields + 'TransactionTypeID,';
                    sListValues := sListValues + sCallCodeID+',';
                  end;  // CallCode.
                  FieldStringList.Add(sListFields);
                  ValueStringList.Add(sListValues);
                  sListFields := '';
                  sListValues := '';

                  // Do not save to RevenueFlag - left NULL allows separation of
                  // the data for creating revenue transactions. RevenueFlag is
                  // set at teh end of the importing process.
                  SubString := Trim(ConvertString.Strings[DESC12_C]);
                  sListFields := sListFields + 'AmountExGST,';
                  sListValues := sListValues + SubString +',';
                  SubString := Trim(ConvertString.Strings[DESC14_C]);
                  curValue := curFormat(StrToFloat(SubString));
                  sListFields := sListFields + 'GSTAmount,GSTFlag,';
                  if (curValue = 0.00) then begin
                    sListValues := sListValues + SubString + ',''N'',';
                  end
                  else begin
                    sListValues := sListValues + SubString + ',''Y'',';
                  end;
                  SubString := Trim(ConvertString.Strings[DESC15_C]);
                  sListFields := sListFields + 'AmountIncGST,';
                  sListValues := sListValues + SubString +',';
                  if (Length(DBfromDate) = 0) or (Length(DBtoDate) = 0) then begin
                    SubString := '20'+ ConvertString.Strings[DESC06_C];
                    DBfromDate :=
                      FormattedDateString(LastMonth(StrDMYtoDate(YMD2DMY(SubString))));
                    DBtoDate :=
                    FormattedDateString(StrDMYtoDate(YMD2DMY(SubString))-1);
                  end;
                  sListFields := sListFields + 'FromDate,';
                  sListValues := sListValues +''''+ DBfromDate +''',';
                  sListFields := sListFields + 'ToDate,';
                  sListValues := sListValues +''''+ DBtoDate +''',';
                end  // 2D13D.
                else if (Pos(sInputLineCode,'@2D11D@@2D12D@') > 0) then begin
                  lSaveToTable := False;
                end
                else begin
                  lSaveToTable := False;
                end;  //
              end;  // 2D13D or 2D17D.
            end  // if (sInputLineCode='2D') then.

            // *****************************************************************
          
            else if (sInputLineCode='2S') or (sInputLineCode='1A') or
              (sInputLineCode='2A') then begin
              // This line is a summary subtotal - check it for Discounts.
              sInputLineCode := sInputLineCode +
                UpperCase(ConvertString.Strings[DESC09_C]+
                          ConvertString.Strings[DESC10_C]);
              SubString := UpperCase(Trim(ConvertString.Strings[DESC11_C]));
              if ((sInputLineCode = '2S10V') and
                ((Pos('DISCOUNT', SubString) > 0) or
                (Pos('REBATE', SubString) > 0))) or
                (((sInputLineCode = '1A02V') or ((sInputLineCode = '2A02V')) and
                (Pos('ROUND', SubString)>0))) then begin
                // This is a discount item.
                FieldStringList.Clear;
                ValueStringList.Clear;
                sListFields := '(';
                sListValues := 'VALUES (';
                SubString := Trim(ConvertString.Strings[DESC06_C]); // ServiceID.
                if not (SubString = sLastService) then begin
                  sLastService := SubString;
                  sLastServiceID := AccountFormat(sLastService);
                end;
                sListFields := sListFields + 'ServiceID,';
                sListValues := sListValues +''''+ sLastServiceID +''',';
                SubString := CleanString(Copy(sInputLineCode,1,4));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'CallCode,';
                  sListValues := sListValues +''''+ SubString +''',';
                  if (sLastCallCode <> SubString) then begin
                    sLastCallCode := SubString;
                    vSubString := SubString;
                    with qryTelstra do
                      lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                    if lFound then
                      sLastTransactionTypeID := qryTelstra['TransactionType']
                    else begin
                      sLastTransactionTypeID := sDiscountCodeID;
                      slAddFieldNames.Clear;
                      slAddFieldValues.Clear;
                      slAddFieldNames.Add('Code');
                      slAddFieldValues.Add(''''+sLastCallCode+'''');
                      slAddFieldNames.Add('AmountCategory');
                      slAddFieldValues.Add('''OTHER''');
                      slAddFieldNames.Add('Description');
                      slAddFieldValues.Add('''OTHER''');
                      slAddFieldNames.Add('TransactionTypeID');
                      slAddFieldValues.Add(sLastTransactionTypeID);
                      slAddFieldNames.Add('SupplierID');
                      slAddFieldValues.Add(sSupplierID);
                      slAddFieldNames.Add('Active');
                      slAddFieldValues.Add('1');
                      AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                      qryTelstra.Close;
                        qryTelstra.Open;
                    end;  // if (not) lFound.
                  end;  // if (sLastCallCode <> SubString) then begin.
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sLastTransactionTypeID+',';
                end
                else begin
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sDiscountCodeID+',';
                end;  // CallCode.
                sRateDescription := CleanString(Trim(Copy(T,6,30)));
                SubString := sRateDescription;
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'RateDescription,';
                  sListValues := sListValues +''''+ SubString +''',';
                end;
                FieldStringList.Add(sListFields);
                ValueStringList.Add(sListValues);
                sListFields := '';
                sListValues := '';

                SubString := Trim(ConvertString.Strings[DESC12_C]);// AmountExGST.
                if (Length(SubString)=0) and
                   (Length(Trim(ConvertString.Strings[DESC14_C]+
                                ConvertString.Strings[DESC12_C]))=0) then begin
                  lSaveToTable := False;               // No discount - ignore.
                end
                else begin
                  if (Length(SubString) = 0) then SubString := '0';
                  sListFields := sListFields + 'AmountExGST,';
                  sListValues := sListValues + SubString +',';
                  SubString := Trim(ConvertString.Strings[DESC14_C]);// GSTAmount.
                  curValue := curFormat(StrToFloat(SubString));
                  if (Length(SubString) = 0) then SubString := '0';
                  sListFields := sListFields + 'GSTAmount,GSTFlag,';
                  if (curValue = 0.00) then begin
                    sListValues := sListValues + SubString + ',''N'',';
                  end
                  else begin
                    sListValues := sListValues + SubString + ',''Y'',';
                  end;
                  SubString := Trim(ConvertString.Strings[DESC15_C]);// AmountIncGST.
                  if (Length(SubString) = 0) then SubString := '0';
                  sListFields := sListFields + 'AmountIncGST,';
                  sListValues := sListValues + SubString +',';
                  if (Length(DBfromDate) = 0) or (Length(DBtoDate) = 0) then begin
                    SubString := '20'+ ConvertString.Strings[DESC05_C];
                    DBfromDate :=
                      FormattedDateString(LastMonth(StrDMYtoDate(YMD2DMY(SubString))));
                    DBtoDate :=
                    FormattedDateString(StrDMYtoDate(YMD2DMY(SubString))-1);
                  end;
                  sListFields := sListFields + 'FromDate,';
                  sListValues := sListValues +''''+ DBfromDate +''',';
                  sListFields := sListFields + 'ToDate,';
                  sListValues := sListValues +''''+ DBtoDate +''',';
                end;  // if (Length(SubString)=0) then begin..else.
              end  // if (sInputLineCode = '2S10V') then begin.

              // ***************************************************************

              else if (sInputLineCode = '2S02D') or
                (sInputLineCode = '2S02V') or
               ((sInputLineCode = '2S10V') and
                (Pos('DATA', UpperCase(Trim(Copy(T,6,37)))) = 0)) or
                (sInputLineCode = '2S11V') then begin
                // This is a discount or rounding item, or a Call item, provided
                // the description is not like 'PACKET DATA SESSION' for CAMS bill.
                FieldStringList.Clear;
                ValueStringList.Clear;
                sListFields := '(';
                sListValues := 'VALUES (';
                SubString := Trim(ConvertString.Strings[DESC06_C]); // ServiceID.
                if not (SubString = sLastService) then begin
                  sLastService := SubString;
                  sLastServiceID := AccountFormat(sLastService);
                end;
                sListFields := sListFields + 'ServiceID,';
                sListValues := sListValues +''''+ sLastServiceID +''',';
                SubString := CleanString(Copy(sInputLineCode,1,4));
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'CallCode,';
                  sListValues := sListValues +''''+ SubString +''',';
                  if (sLastCallCode <> SubString) then begin
                    sLastCallCode := SubString;
                    vSubString := SubString;
                    with qryTelstra do
                      lFound := Locate('Code',vSubString,[loCaseInsensitive]);
                    if lFound then
                      sLastTransactionTypeID := qryTelstra['TransactionType']
                    else begin
                      sLastTransactionTypeID := sCallCodeID;
                      slAddFieldNames.Clear;
                      slAddFieldValues.Clear;
                      slAddFieldNames.Add('Code');
                      slAddFieldValues.Add(''''+sLastCallCode+'''');
                      slAddFieldNames.Add('AmountCategory');
                      slAddFieldValues.Add('''CALLS''');
                      slAddFieldNames.Add('Description');
                      slAddFieldValues.Add('''CALLS''');
                      slAddFieldNames.Add('TransactionTypeID');
                      slAddFieldValues.Add(sLastTransactionTypeID);
                      slAddFieldNames.Add('SupplierID');
                      slAddFieldValues.Add(sSupplierID);
                      slAddFieldNames.Add('Active');
                      slAddFieldValues.Add('1');
                      AddToTable('tSupplierServiceCode',slAddFieldNames,slAddFieldValues);
                      qryTelstra.Close;
                      qryTelstra.Open;
                    end;  // if (not) lFound.
                  end;  // if (sLastCallCode <> SubString) then begin.
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sLastTransactionTypeID+',';
                end
                else begin
                  sListFields := sListFields + 'TransactionTypeID,';
                  sListValues := sListValues + sCallCodeID+',';
                end;  // CallCode.
                sRateDescription := CleanString(Trim(Copy(T,6,30)));
                SubString := sRateDescription;
                if (Length(SubString) > 0) then begin
                  sListFields := sListFields + 'RateDescription,';
                  sListValues := sListValues +''''+ SubString +''',';
                end;
                FieldStringList.Add(sListFields);
                ValueStringList.Add(sListValues);
                sListFields := '';
                sListValues := '';

                // Do not save to RevenueFlag - left NULL allows separation of
                // the data for creating revenue transactions. RevenueFlag is
                // set at the end of the importing process.
                if (ConvertString.Count < 15) or                  // AmountExGST.
                  (Length(Trim(ConvertString.Strings[DESC12_C]+
                  ConvertString.Strings[DESC14_C]+
                  ConvertString.Strings[DESC15_C])) = 0) then begin
                  SubString := Trim(Right(T,8));
                  ProveReal(SubString,lSaveToTable);        // True=Real.
                end
                else
                  SubString := Trim(ConvertString.Strings[DESC12_C]);// AmountExGST.
                  // This is empty for Rounding, so set to zero.
                  if (Length(SubString) = 0) then SubString := '0.00';
                  if (curFormat(StrToFloat(SubString))=0.00) and
                   ((Length(ConvertString.Strings[DESC14_C])=0) or
                    (curFormat(StrToFloat(ConvertString.Strings[DESC14_C]))=0.00)) and
                   ((Length(ConvertString.Strings[DESC15_C])=0) or
                    (curFormat(StrToFloat(ConvertString.Strings[DESC15_C]))=0.00))
                    then begin
                    lSaveToTable := False;               // No discount - ignore.
                  end
                else begin
                  sListFields := sListFields + 'AmountExGST,';
                  sListValues := sListValues + SubString +',';
                  SubString := Trim(ConvertString.Strings[DESC14_C]);// GSTAmount.
                  if (Length(SubString) = 0) then SubString := '0';
                  curValue := curFormat(StrToFloat(SubString));
                  sListFields := sListFields + 'GSTAmount,GSTFlag,';
                  if (curValue = 0.00) then begin
                    sListValues := sListValues + SubString + ',''N'',';
                  end
                  else begin
                    sListValues := sListValues + SubString + ',''Y'',';
                  end;
                end;  // Discount amounts.
                SubString := Trim(ConvertString.Strings[DESC15_C]);// AmountIncGST.
                if (Length(SubString) = 0) then SubString := '0';
                sListFields := sListFields + 'AmountIncGST,';
                sListValues := sListValues + SubString +',';
                if (Length(DBfromDate) = 0) or (Length(DBtoDate) = 0) then begin
                  SubString := '20'+ ConvertString.Strings[DESC05_C]; // ToDate.
                  DBfromDate :=
                    FormattedDateString(LastMonth(StrDMYtoDate(YMD2DMY(SubString))));
                  DBtoDate :=
                    FormattedDateString(StrDMYtoDate(YMD2DMY(SubString))-1);
                end;
                sListFields := sListFields + 'FromDate,';
                sListValues := sListValues +''''+ DBfromDate +''',';
                sListFields := sListFields + 'ToDate,';
                sListValues := sListValues +''''+ DBtoDate +''',';
              end  // 2so2d or 2so2v.

              else begin
                lSaveToTable := False;
              end;  // ..else begin.
            end  // ..else if (sInputLineCode='2S') then begin.

            else begin
              lSaveToTable := False;
            end;  // ..else begin.
            if lSaveToTable then begin
              // Now make the SQL statement to save to the database table.
              sListFields := sListFields + 'BatchID)';
              sListValues := sListValues + sBatch +')';
              FieldStringList.Add(sListFields);
              ValueStringList.Add(sListValues);
              qryStringList.Clear;
              qryStringList.Add('INSERT INTO tDataCollection');
              for nListValues := 0 to FieldStringList.Count-1 do begin
                qryStringList.Add(FieldStringList[nListValues]);
              end;
              for nListValues := 0 to ValueStringList.Count-1 do begin
                qryStringList.Add(ValueStringList[nListValues]);
              end;
              // Update and execute the query.
              qryDC.Close;
              qryDC.SQL := qryStringList;
              qryDC.ExecSQL;
              lDataSaved := True;
            end;  // if lSaveToTable then begin.
            Inc(nCount);
            // Report progress to the user.
            nSoFar := Round(nCount * nFactor);
            // Do not know how big the file is - make sure don't exceed 100%.
            if (nSoFar > PROGRESS_MAX_C) then begin
              frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
            end
          else begin
            frmProgress.ProgressBar1.Position := nSoFar;
          end;
          // Read the next line of text file data.
          Readln( TelstraFile, ReadString);
          if (Length(ReadString) > 0) then begin
            // Separate into strings before adding to ConvertString.
            ReadString := StringReplace(ReadString, ',', #13#10, [rfReplaceAll]) +'''';
            ConvertString.Text := ReadString;
            lContinue := False;
            if (ConvertString.strings[ACCT_C] = sAccountNumber) then
              lContinue := True

              else while not lContinue and
                (not ((Length(Trim(ReadString))=0) and Eof( TelstraFile ))) do begin
                lContinue := ((ConvertString.Strings[CODE_C]+
                               ConvertString.Strings[ACCT_C]) = SubString);
                if not lContinue then begin
                  Readln( TelstraFile, ReadString );
                  if (not ((Length(Trim(ReadString))=0) and Eof( TelstraFile )))
                  then begin
                    // Separate into strings before adding to ConvertString.
                    ReadString := StringReplace(ReadString, ',', #13#10,
                      [rfReplaceAll]) +'''';
                    ConvertString.Text := ReadString;
                  end;  // not empty or eof.
                  // Report progress to the user.
                  Inc(nCount);
                  nDummy := nFactor * nCount;
                  nSoFar := Round( nDummy );
                  // Do not know how big the file is - do not exceed 100%.
                  if (nSoFar > PROGRESS_MAX_C) then begin
                    frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
                  end
                  else begin
                    frmProgress.ProgressBar1.Position := nSoFar;
                  end;  // PROGRESS_MAX_C.
                end;  // not lContinue.


{            else while (not ((ReadString='') and Eof( TelstraFile ))) and
              ((ConvertString.Strings[CODE_C]+ConvertString.Strings[ACCT_C])
                    <> SubString) do begin
              Readln( TelstraFile, ReadString );
              // Separate into strings before adding to ConvertString.
              ReadString := StringReplace(ReadString, ',', #13#10, [rfReplaceAll]) +'''';
              ConvertString.Text := ReadString;
              lContinue := ((ConvertString.Strings[CODE_C]+
                             ConvertString.Strings[ACCT_C]) = SubString);
              // Report progress to the user.
              Inc(nCount);
              nDummy := nFactor * nCount;
              nSoFar := Round( nDummy );
              // Do not know how big the file is - do not exceed 100%.
              if (nSoFar > PROGRESS_MAX_C) then begin
                frmProgress.ProgressBar1.Position := PROGRESS_MAX_C;
              end
              else begin
                frmProgress.ProgressBar1.Position := nSoFar;
              end;
}
            end;  // AccountNumber.
          end;  // not empty string read from the input file.
        until not lContinue or ((ReadString='') and Eof( TelstraFile ));
        lReturnCode := lDataSaved;;
      except    // D.StartTransaction.
        lReturnCode := False;
//        D.Rollback;       // On failure, undo the changes - not useful anyway.
        raise;            // Raise the exception to avoid call to CommitUpdates.
      end;    // D.StartTransaction.
      end;  // not ((ReadString='') and Eof(TelstraFile).
      // Now need to process the last line if CR-LF not at end.
      //  BUT - nothing done !
//      D.Commit;
      //****************************************************************
      //*********** END READING BILLS **********************************
      //****************************************************************
      end;  // not lContinue.
    finally    // TelstraFile (try).
      // Done with the input file; close it.
      CloseFile( TelstraFile );
    end;    // TelstraFile (try).
    // Select the next file if exists - extension 001, 002, etc.
    Inc(FileIteration);
    CurrentFile := Copy(CurrentFile,1,Pos('.',CurrentFile));
    CurrentFile := CurrentFile + Right(('00'+IntToStr(FileIteration)),3);
  end;  // if FileExists(CurrentFile) then begin.
end;  //  GetMobileData.

{
procedure ImportDataFile
          (const FName: String;
           D: TDatabase;
           nBatch: LongInt;
           var lReturnCode: Boolean
           );
// This function determines the type of the input file (FName) and calls
// the appropriate procedure to read the information.
var
  TelstraFile: TextFile;               // Used for accessing the data.
  T: string;                           // Line of data read from TextFile.
  sSupplierAccount: string;            // Telstra's account number.
  sAccountType: String;                // Unique string for each type of account
  sSupplierInvoice: String;            // Telstra Bill Number.
  nBatchID : LongInt;
begin
  lReturnCode := False;
  // Open the input (text) billing file for reading.
  AssignFile( TelstraFile, Fname );
  Reset( TelstraFile );
  nCount := 0;
  // Assume that the results table has already been emptied.

  // Get the first occurence of the search string.
  Readln( TelstraFile, T );
  if Eof( TelstraFile ) then begin  // To be sure the file isn't empty too.
    CloseFile( TelstraFile );
  end
  else begin
    // Determine the Sequence Number first ********** Start TEMPORARY ********
    nBatchID := nBatch;
    // The required data is in the first line, so close the input file.
    // First, determine the account number.
    sSupplierAccount := Trim(Copy(T,10,17));
    sAccountType := UpperCase(Copy(T,1,8));
    // Next, ensure the Bill is for mobile accounts only.
    if (sAccountType='HDR:RACE') then begin
      // This is a Mobile phone account as required for data - continue.
      Readln( TelstraFile, T );
      // Search for the first line of the "printed" Bill - always at start of file.
      while not Eof( TelstraFile ) and (UpperCase(Copy(T,1,3))<>'RBM') do begin
        Readln( TelstraFile, T );
      end;  // while not do.
      if Eof( TelstraFile ) then begin
        CloseFile( TelstraFile );
      end
      else begin
        sSupplierInvoice := Trim(Copy(T,37,17));
        CloseFile( TelstraFile );
        GetMobileData( Fname, sSupplierAccount, sSupplierInvoice,
          D, nBatchID, lReturnCode );
      end;
    end;  // if (sAccountType='HDR:FCAB') then..else.
  end;   //  if Eof( TelstraFile ) then..else.
end;  // ImportDataFile().
}

procedure ApplyDataRates(const nBatch: integer;
  LumpedCosts, IncludedRate: boolean);
var
  MessageForm : TMessagesForm;
  sBatchID : string;
  sServiceTypeID : string;
  sSystemID : string;
  sFromDate : string;
  sToDate : string;
  curAmountExGST : currency;
  curGSTAmount : currency;
  curExcessAmtExGST : currency;
  curTotalGSTAmount : currency;
  sNewServiceID : string;
  nIncludedUse : integer;
  sIncludedUse : string;
  sTransactionTypeID : string;
  nTransactionTypeID : integer;
  slDC : TStringList;
  qryDC : TQuery;
  sVRTID : string;
  nkByte : integer;
  skByte : string;
  nExcessKByte : integer;
  nGByte : real;
  sGByte : string;
  nUsage : real;
  sGSTRate : string;
  sRateCost : string;
  sServiceType : string;
  fPartCost : real;
  nServiceCount : integer;
begin
  sBatchID := IntToStr(nBatch);
  MessageForm := TMessagesForm.Create(nil);
  MessageForm.Caption := 'Importing Batch:  '+sBatchID;
  MessageForm.Msg.Caption := 'Please wait. Creating revenue transactions...';
  slDC := TStringList.Create;
  qryDC := TQuery.Create(nil);
  qryDC.DatabaseName := 'dbPPdata';
  Str(MAINFORM.GST_RATE:6:4, sGSTRate);

  sVRTID := '0';                                   // Define VRT CustomerID.
  slDC.Add('SELECT ID');
  slDC.Add('FROM tCustomer');
  slDC.Add('WHERE ShipTo LIKE ''VRT''');
  slDC.Add('AND Active = 1');
  qryDC.SQL := slDC;
  qryDC.Open;
  if not qryDC.Eof then
    sVRTID := IntToStr(qryDC['id']);
  qryDC.Close;
  qryDC.SQL.Clear;

  // Read invoice data usage.                      // Determine total data used.
  slDC.Clear;
  slDC.Add('SELECT Sum(CAST(Var01 AS integer)) AS kByte');
  slDC.Add('FROM tDataCollection');
  slDC.Add('WHERE BatchID = '+sBatchID);
  slDC.Add('AND RevenueFlag = 0');
  qryDC.SQL := slDC;
  qryDC.Open;
  nkByte := qryDC.FieldByName('kByte').AsInteger;
  qryDC.Close;
  Str(nkByte:11,skByte);
  nGByte := nkByte / 1024.0 / 1024.0;
  Str(nGByte:11:4,sGByte);

  // Get the ServiceTypeID linked to this invoice.
  slDC.Clear;
  slDC.Add('SELECT tServiceType.ID AS "ServiceTypeID", '+
    'tSystem.ID AS "SystemID"');
  slDC.Add('FROM tServiceType');
  slDC.Add('INNER JOIN tSystem ON tServiceType.SystemID = tSystem.ID');
  slDC.Add('INNER JOIN tBatchType ON tSystem.BatchTypeID = tBatchType.ID');
  slDC.Add('INNER JOIN tBatch ON tBatchType.ID = tBatch.BatchTypeID');
  slDC.Add('WHERE tBatch.ID = '+sBatchID);
  qryDC.SQL := slDC;
  qryDC.Open;
  sServiceTypeID := qryDC.FieldByName('ServiceTypeID').AsString;
  sSystemID := qryDC.FieldByName('SystemID').AsString;
  qryDC.Close;
  if (Length(sServiceTypeID) = 0) then sServiceTypeID := 'NULL';
  if (Length(sSystemID) = 0) then sSystemID := 'NULL';

  // Now read the transaction type to be used.
  slDC.Clear;
  slDC.Add('SELECT tTransactionType.ID');
  slDC.Add('FROM tTransactionType');
  slDC.Add('WHERE tTransactionType.Description LIKE ''%Data Usage%''');
  slDC.Add('AND tTransactionType.Description LIKE ''%Aggregate%''');
  slDC.Add('AND tTransactionType.SystemID = '+sSystemID);
  slDC.Add('AND tTransactionType.Active = 1');
  qryDC.SQL := slDC;
  qryDC.Open;
  sTransactionTypeID:= qryDC.FieldByName('ID').AsString;
  qryDC.Close;
  if (Length(sTransactionTypeID) = 0) then sTransactionTypeID := 'NULL';

  // Now get the data usage included in the fixed cost.
  slDC.Clear;
  slDC.Add('SELECT CAST(Min(tRate.Length) AS int) AS "Length"');
  slDC.Add('FROM tRate');
  slDC.Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
  slDC.Add('AND tRate.Active = 1');
  qryDC.SQL := slDC;
  qryDC.Open;
  nIncludedUse := qryDC.FieldByName('Length').AsInteger;
  sIncludedUse := IntToStr(nIncludedUse);
  qryDC.Close;

  // Look up the appropriate rate from tRate and convert to monthly rates.
  slDC.Clear;
  slDC.Add('SELECT tRate.Cost');
  slDC.Add('FROM tRate');
  slDC.Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
  slDC.Add('AND tRate.Length = '+sIncludedUse);
  slDC.Add('AND tRate.Active = 1');
  qryDC.SQL := slDC;
  qryDC.Open;
//  curAmountExGST := qryDC.FieldByName('Cost').AsCurrency / 12;
  curAmountExGST := qryDC.FieldByName('Cost').AsCurrency / 12;
  Str(qryDC.FieldByName('Cost').AsCurrency:11:4,sRateCost);
  qryDC.Close;
  curGSTAmount := curAmountExGST * MAINFORM.GST_RATE;
  curTotalGSTAmount := curGSTAmount;

  // Try to get the dates from the tDataCollection table.
  slDC.Clear;
  slDC.Add('SELECT DISTINCT tDataCollection.FromDate, tDataCollection.ToDate');
  slDC.Add('FROM tDataCollection');
  slDC.Add('WHERE tDataCollection.BatchID = '+sBatchID);
  slDC.Add('AND tDataCollection.FromDate IS NOT NULL');
  slDC.Add('AND tDataCollection.ToDate IS NOT NULL');
  qryDC.SQL := slDC;
  qryDC.Open;
  if not (qryDC.Bof and qryDC.Eof) then begin
    sFromDate := ''''+ qryDC.FieldByName('FromDate').AsString +''', ';
    sToDate := ''''+ qryDC.FieldByName('ToDate').AsString +''', ';
  end;
  qryDC.Close;
  // If unsuccessful, use the dates in the tBatch table.
  if (Length(sFromDate) = 0) or (Length(sToDate) = 0) then begin
    // Read the dates from the tBatch table.
    slDC.Clear;
    slDC.Add('SELECT tBatch.RentStartDate, tBatch.RentEndDate');
    slDC.Add('FROM tBatch');
    slDC.Add('WHERE tBatch.ID = '+sBatchID);
    qryDC.SQL := slDC;
    qryDC.Open;
    if VarIsNull(qryDC['RentStartDate']) then begin
      sFromDate := 'NULL';
    end
    else begin
      sFromDate := ''''+
      FormattedDateString(qryDC.FieldByName('RentStartDate').AsDateTime)+''', ';
    end;
    if VarIsNull(qryDC['RentEndDate']) then begin
      sToDate := 'NULL';
    end
    else begin
      sToDate := ''''+
        FormattedDateString(qryDC.FieldByName('RentEndDate').AsDateTime)+''', ';
    end;
    qryDC.Close;
  end;  // FromDate or EndDate unknown.

  // Create the Revenue transactions in the tDataCollection table.
  if LumpedCosts then begin                   // add test for "IncludedRate".
    // For lumped sums, all transactions have AmtExGST=0.
    // Charges are lumped into 1 or 2 new transactions. Also,
    // any ServiceID not on the invoice is to be added from the tServiceID
    // table.
    // This applies to SmartBus invoices.

    // First, read the ServiceID to hold the base charges.
    slDC.Clear;
    slDC.Add('SELECT tParameter.ParmText');
    slDC.Add('FROM tParameter');
    slDC.Add('WHERE tParameter.Parm LIKE ''BaseDataServiceID''');
    qryDC.SQL := slDC;
    qryDC.Open;
    sNewServiceID := qryDC.FieldByName('ParmText').AsString;
    qryDC.Close;
    if (Length(sNewServiceID) = 0) then begin
      MessageDlg('No entry found for Parm=BaseDataServiceID in'+#13#10+
      'the tParameter table.  Data is incomplete.',
        mtError, [mbOk], 0);
    end
    else begin
      // Ready to add the basic (minimum) charge.
      slDC.Clear;
      slDC.Add('INSERT INTO tDataCollection');
      slDC.Add('(BatchID, ServiceID, FromDate, ToDate, TransactionTypeID, '+
        'AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag)');
      slDC.Add('VALUES('+ sBatchID+', ');
      slDC.Add(''''+sNewServiceID+''', ');
      slDC.Add(sFromDate);
      slDC.Add(sToDate);
      slDC.Add(sTransactionTypeID+', ');
      slDC.Add(CurrToStr(curAmountExGST)+', ');
      slDC.Add(CurrToStr(curGSTAmount)+', ');
      slDC.Add(CurrToStr(curAmountExGST + curGSTAmount)+', ');
      slDC.Add('''Y'', ');
      slDC.Add('1)');
      qryDC.SQL := slDC;
      qryDC.ExecSQL;

      // Now test to see if the total usage exceeds the basic allowance.
      // First, see if the data usage exceeds the included data. Values are in
      // kiloBytes (kB).
      if (nkByte > nIncludedUse) then begin
        nExcessKByte := nkByte - nIncludedUse;
        // Read the transaction type to be used.
        slDC.Clear;
        slDC.Add('SELECT tTransactionType.ID');
        slDC.Add('FROM tTransactionType');
        slDC.Add('WHERE tTransactionType.Description LIKE ''%Data Usage%''');
        slDC.Add('AND tTransactionType.Description LIKE ''%Excess%''');
        slDC.Add('AND tTransactionType.SystemID = '+sSystemID);
        slDC.Add('AND tTransactionType.Active = 1');
        
        qryDC.SQL := slDC;
        qryDC.Open;
        sTransactionTypeID:= qryDC.FieldByName('ID').AsString;
        qryDC.Close;
        if (Length(sTransactionTypeID) = 0) then sTransactionTypeID := 'NULL';
        // And determine the appropriate rate (PartCost) for excess usage.
        slDC.Clear;
        slDC.Add('SELECT tRate.PartCost');
        slDC.Add('FROM tRate');
        slDC.Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
        slDC.Add('AND tRate.Length =');
        slDC.Add(' (SELECT Max(tRate.Length)');
        slDC.Add(' FROM tRate');
        slDC.Add(' WHERE tRate.ServiceTypeID = '+sServiceTypeID);
        slDC.Add(' AND tRate.Length <= '+IntToStr(nkByte));
        slDC.Add(' AND tRate.Active = 1)');
        slDC.Add('AND tRate.Active = 1');
        qryDC.SQL := slDC;
        qryDC.Open;
        if (qryDC.Bof and qryDC.Eof) then begin
          curExcessAmtExGST := 0;
        end
        else begin
          // Round up the usage to the nearest MegaByte of data (rounded up).
          nUsage := Trunc((nkByte - nIncludedUse) / 1024.0) + 1;
          curExcessAmtExGST := nUsage * qryDC.FieldByName('PartCost').Value;
        end;
        qryDC.Close;
        if (curExcessAmtExGST > 0) then begin
          curGSTAmount := curExcessAmtExGST * MAINFORM.GST_RATE;
          curTotalGSTAmount := curTotalGSTAmount + curGSTAmount;
          // First, read the ServiceID to hold the Excess charges.
          slDC.Clear;
          slDC.Add('SELECT ParmText');
          slDC.Add('FROM tParameter');
          slDC.Add('WHERE tParameter.Parm LIKE ''ExcessDataServiceID''');
          qryDC.SQL := slDC;
          qryDC.Open;
          sNewServiceID := qryDC.FieldByName('ParmText').AsString;
          qryDC.Close;
          if (Length(sNewServiceID) = 0) then begin
            MessageDlg('No entry found for Parm=ExcessDataServiceID in'+#13#10+
              'the tParameter table.  Data is incomplete.',
               mtError, [mbOk], 0);
          end
          else begin
            // Ready to add the Excess charges above the included allowance.
            slDC.Clear;
            slDC.Add('INSERT INTO tDataCollection');
            slDC.Add('(BatchID, ServiceID, FromDate, ToDate, TransactionTypeID,'+
              ' AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag)');
            slDC.Add('VALUES('+ sBatchID+', ');
            slDC.Add(''''+sNewServiceID+''', ');
            slDC.Add(sFromDate);
            slDC.Add(sToDate);
            slDC.Add(sTransactionTypeID+', ');
            slDC.Add(CurrToStr(curExcessAmtExGST)+', ');
            slDC.Add(CurrToStr(curGSTAmount)+', ');
            slDC.Add(CurrToStr(curExcessAmtExGST + curGSTAmount)+', ');
            slDC.Add('''Y'', ');
            slDC.Add('1)');
            qryDC.SQL := slDC;
            qryDC.ExecSQL;
          end;  // empty NewServiceID.
        end;  // curAmountExGST > 0.
      end
      else begin
        curExcessAmtExGST := 0;
        nExcessKByte := 0;
      end;  // nkByte > nIncludedUse.
      // Set RevenueFlag=1 for all of the data transactions (with AmountExGST=0).
      // Any other RevenueFlag=NULL will be reset to "0" later.
      slDC.Clear;
      slDC.Add('UPDATE tDataCollection');
      slDC.Add('SET RevenueFlag = 1');
      slDC.Add('WHERE BatchID = '+ sBatchID);
      slDC.Add('AND AmountExGST = 0');
      slDC.Add('AND AmountIncGST = 0');
      qryDC.SQL := slDC;
      qryDC.ExecSQL;
      // Get the TransactionTypeID from the tDataCollection table where
      // no charges have been calculated.
      slDC.Clear;
      slDC.Add('SELECT DISTINCT tDataCollection.TransactionTypeID');
      slDC.Add('FROM tDataCollection');
      slDC.Add('WHERE tDataCollection.BatchID = '+ sBatchID);
      slDC.Add('AND tDataCollection.AmountExGST = 0');
      slDC.Add('AND tDataCollection.AmountIncGST = 0');
      qryDC.SQL := slDC;
      qryDC.Open;
      nTransactionTypeID := qryDC.FieldByName('TransactionTypeID').AsInteger;
      qryDC.Close;
      // Add those services that did not register any usage on the network.
      AddMissingServiceID(nBatch, nTransactionTypeID);

      // Make sure that all entries for this BatchID do not have
      // RevenueFlag=Null so that revenue Transactions will not be created when
      // transferring to the tTransaction table later using SaveToTransaction().
      slDC.Clear;
      slDC.Add('UPDATE tDataCollection');
      slDC.Add('SET RevenueFlag = 0');
      slDC.Add('WHERE RevenueFlag IS NULL');
      slDC.Add('AND BatchID = '+sBatchID);
      qryDC.SQL := slDC;
      qryDC.ExecSQL;

      // Update the values stored in the tBatch table.
      slDC.Clear;
      slDC.Add('UPDATE tBatch');
      slDC.Add('SET TotalRevenue = '+CurrToStr(curAmountExGST)+',');
      slDC.Add(' InvoiceNo = '''+ sServiceType +''',');
      slDC.Add(' OrderNumber = '''+ IntToStr(nExcesskByte) +''',');
      slDC.Add(' RequisitionNumber = '''+ IntToStr(nIncludedUse) +''',');
      slDC.Add(' OpeningBalance = '+ CurrToStr(curExcessAmtExGST) +',');
      slDC.Add(' TotalCredits = '+ sRateCost +',');
      slDC.Add(' GSTAmount = '+ CurrToStr(curTotalGSTAmount));
      slDC.Add('WHERE tBatch.ID = '+ sBatchID);
      qryDC.SQL := slDC;
      qryDC.ExecSQL;
    end;  // empty NewServiceID.
  end  // LumpedSum.
  else if IncludedRate then begin
    // A fixed fee for each ServiceID plus excess data for the total account.
    // To start, create the revenue transactions with the fixed monthly fee
    // for each service. Set All item charges to zero - included in rental.
    slDC.Clear;
    slDC.Add('INSERT INTO tDataCollection');
    slDC.Add('(SequenceNo, ServiceID, BatchID,');
    slDC.Add(' TxnDate, TxnTime, Origin,');
    slDC.Add(' Duration, CallCode, RateDescription,');
    slDC.Add(' FromDate, ToDate, TransactionTypeID,');
    slDC.Add(' AmountExGST, GSTAmount, GSTFlag, AmountIncGST,');
    slDC.Add(' RevenueFlag, Var01, Var04)');
    slDC.Add('SELECT tDataCollection.SequenceNo, tDataCollection.ServiceID, '+
      'tDataCollection.BatchID,');
    slDC.Add(' tDataCollection.TxnDate, tDataCollection.TxnTime, '+
      'tDataCollection.Origin,');
    slDC.Add(' tDataCollection.Duration, tDataCollection.CallCode, '+
      'tDataCollection.RateDescription,');
    slDC.Add(' tDataCollection.FromDate, tDataCollection.ToDate, '+
      'tDataCollection.TransactionTypeID,');
    // Set all recorded amounts to zero.
    slDC.Add(' (0) AS "AmountExGST", (0) AS "GSTAmount", '+
      'tDataCollection.GSTFlag,');
    slDC.Add(' (0) AS "AmountIncGST", (1) AS "RevenueFlag",'+
      ' tDataCollection.Var01, ('+skByte+') AS "Var04"');
    slDC.Add('FROM tDataCollection');
    slDC.Add('WHERE tDataCollection.BatchID = '+sBatchID);
    slDC.Add('AND tDataCollection.RevenueFlag = 0');
    qryDC.SQL := slDC;
    qryDC.ExecSQL;

    // Fix the date fields first.
    sFromDate := Copy(sFromDate, 1, Length(sFromDate) - 2);
    sToDate := Copy(sToDate, 1, Length(sToDate) - 2);
    // Read the transaction type to be used.
    slDC.Clear;
    slDC.Add('SELECT tTransactionType.ID');
    slDC.Add('FROM tTransactionType');
    slDC.Add('AND tTransactionType.Description LIKE ''%Excess%''');
    slDC.Add('AND tTransactionType.SystemID = '+sSystemID);
    slDC.Add('AND tTransactionType.Active = 1');
    // Count how many mobile services are being invoiced.
    slDC.Clear;
    slDC.Add('SELECT COUNT(DISTINCT tDataCollection.ServiceID) AS "TotalCount"');
    slDC.Add('FROM tDataCollection');
    slDC.Add('LEFT JOIN tServiceID '+
      'ON tDataCollection.ServiceID = tServiceID.ServiceID');
    slDC.Add('WHERE tDataCollection.BatchID = '+ sBatchID);
    slDC.Add('AND tServiceID.ServiceTypeID = '+sServiceTypeID);
    qryDC.SQL := slDC;
    qryDC.Open;
    nServiceCount := qryDC.FieldByName('TotalCount').AsInteger;
    qryDC.Close;

    // Get the rental & data usage included in the fixed cost for each mobile.
    slDC.Clear;
    slDC.Add('SELECT CAST(tRate.Length AS int) AS "Length", tRate.PartCost,'+
      ' tRate.Cost');
    slDC.Add('FROM tRate');
    slDC.Add('WHERE tRate.ServiceTypeID = '+sServiceTypeID);
    slDC.Add('AND tRate.Active = 1');
    qryDC.SQL := slDC;
    qryDC.Open;
    nIncludedUse := qryDC.FieldByName('Length').AsInteger * nServiceCount;
    sIncludedUse := IntToStr(nIncludedUse);
    fPartCost    := qryDC.FieldByName('PartCost').AsFloat;
    curExcessAmtExGST  := qryDC.FieldByName('Cost').AsCurrency / 12;
    qryDC.Close;
    curGSTAmount := curExcessAmtExGST * MAINFORM.GST_RATE;

    // Save the Rentals.
    slDC.Clear;
    slDC.Add('INSERT INTO tDataCollection');
    slDC.Add('(ServiceID, BatchID, FromDate, ToDate, TransactionTypeID,'+
      ' AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag,'+
      ' Var02, Var03, Var04)');
    slDC.Add('SELECT DC.ServiceID,');
    slDC.Add('DC.BatchID,');
    slDC.Add(sFromDate+' AS "FromDate",');
    slDC.Add(sToDate+' AS "ToDate",');
    slDC.Add(sTransactionTypeID+' AS "TransactionTypeID", ');
    slDC.Add(CurrToStr(curExcessAmtExGST)+' AS "AmountExGST", ');
    slDC.Add(CurrToStr(curGSTAmount)+' AS "GSTAmount",');
    slDC.Add(CurrToStr(curExcessAmtExGST + curGSTAmount)+' AS "AmountIncGST",');
    slDC.Add('''Y'' AS "GSTflag",');
    slDC.Add('(1) AS "RevenueFlag",');
    slDC.Add('('+ CurrToStr(curExcessAmtExGST * 12) +') AS "Var02",');
    slDC.Add('('+ CurrToStr(curExcessAmtExGST) +') AS "Var03",');
    slDC.Add('('+ skByte +') AS "Var04"');
    slDC.Add('FROM (SELECT DISTINCT tDataCollection.ServiceID, tDataCollection.BatchID');
    slDC.Add('FROM tDataCollection');
    slDC.Add('LEFT JOIN tServiceID ON tDataCollection.ServiceID = tServiceID.ServiceID');
    slDC.Add('WHERE tDataCollection.BatchID = '+sBatchID);
    slDC.Add('AND tDataCollection.Ratedescription NOT LIKE ''%ROUND%''');
    slDC.Add('AND tServiceID.ServiceTypeID = '+sServiceTypeID+') AS DC');
    slDC.Add('ORDER BY DC.ServiceID');
    qryDC.SQL := slDC;
    qryDC.ExecSQL;

    if (Length(sServiceTypeID) = 0) then sServiceTypeID := 'NULL';

    // Test to see if the total usage exceeds the nett included amount.
    if (nkByte < nIncludedUse) then begin
      // No excess fees are to be applied.
      curExcessAmtExGST := 0;
      nkByte := 0;
    end
    else begin
      // Included data bandwidth exceeded - charge for the excess amount.
      curExcessAmtExGST := (nkByte - nIncludedUse) * fPartCost;
      if (curExcessAmtExGST > 0) then begin
        curGSTAmount := curExcessAmtExGST * MAINFORM.GST_RATE;
        // First, read the ServiceID to hold the Excess charges.
        slDC.Clear;
        slDC.Add('SELECT ParmText');
        slDC.Add('FROM tParameter');
        slDC.Add('WHERE tParameter.Parm LIKE ''ExcessDataServiceID''');
        qryDC.SQL := slDC;
        qryDC.Open;
        sNewServiceID := qryDC.FieldByName('ParmText').AsString;
        qryDC.Close;
        // Now read the transaction type to be used.
        slDC.Clear;
        slDC.Add('SELECT tTransactionType.ID');
        slDC.Add('FROM tTransactionType');
        slDC.Add('WHERE tTransactionType.Description LIKE ''%Data Usage%''');
        slDC.Add('AND tTransactionType.Description LIKE ''%Excess%''');
        slDC.Add('AND tTransactionType.Active = 1');
        qryDC.SQL := slDC;
        qryDC.Open;
        sTransactionTypeID:= qryDC.FieldByName('ID').AsString;
        qryDC.Close;
        if (Length(sTransactionTypeID) = 0) then sTransactionTypeID := 'NULL';
        // Ready to add the Excess charges above the included allowance.
        slDC.Clear;
        slDC.Add('INSERT INTO tDataCollection');
        slDC.Add('(ServiceID, BatchID, FromDate, ToDate, TransactionTypeID,'+
          ' AmountExGST, GSTAmount, AmountIncGST, GSTFlag, RevenueFlag, Var02)');
        slDC.Add('VALUES('''+sNewServiceID+''',');
        slDC.Add(sBatchID+',');
        slDC.Add(sFromDate+',');
        slDC.Add(sToDate+',');
        slDC.Add(sTransactionTypeID+',');
        slDC.Add(CurrToStr(curExcessAmtExGST)+',');
        slDC.Add(CurrToStr(curGSTAmount)+',');
        slDC.Add(CurrToStr(curExcessAmtExGST + curGSTAmount)+',');
        slDC.Add('''Y'',');
        slDC.Add('1, '+CurrToStr(curExcessAmtExGST)+')');
        qryDC.SQL := slDC;
        qryDC.ExecSQL;

      end;  // curExcessAmtExGST.
    end;  // Excess data fees.
    // Make sure that all entries for this BatchID do not have
    // RevenueFlag=Null so that revenue Transactions will not be created when
    // transferring to the tTransaction table later using SaveToTransaction().
    slDC.Clear;
    slDC.Add('UPDATE tDataCollection');
    slDC.Add('SET RevenueFlag = 0');
    slDC.Add('WHERE RevenueFlag IS NULL');
    slDC.Add('AND BatchID = '+sBatchID);
    qryDC.SQL := slDC;
    qryDC.ExecSQL;

    // Update the values stored in the tBatch table.
    slDC.Clear;
    slDC.Clear;
    slDC.Add('UPDATE tBatch');
    slDC.Add('SET TotalRevenue = BatchRate.RevenueAmt,');
    slDC.Add(' TotalCredits = '+ FloatToStrF(fPartCost,ffFixed,10,5) +',');
    slDC.Add(' OpeningBalance = '+ CurrToStr(curExcessAmtExGST) +',');
    slDC.Add(' OrderNumber = '''+ IntToStr(nkByte) +''',');
    slDC.Add(' TenderNo = '''+ IntToStr(nServiceCount) +''',');
    slDC.Add(' InvoiceNo = '''+ sServiceTypeID +''',');
    slDC.Add(' RequisitionNumber = '''+ sIncludedUse +'''');
    slDC.Add('FROM (SELECT Sum(tDataCollection.AmountExGST) AS "RevenueAmt"');
    slDC.Add('FROM tDataCollection');
    slDC.Add('WHERE tDataCollection.BatchID = '+ sBatchID);
    slDC.Add('AND tDataCollection.RevenueFlag = 1) AS BatchRate');
    slDC.Add('WHERE tBatch.ID = '+ sBatchID);
    qryDC.SQL := slDC;
    qryDC.ExecSQL;
  end  // IncludedRate.
  else begin
    // The usage for each ServiceID is calculated and the transactions updated.
    slDC.Clear;
    slDC.Add('INSERT INTO tDataCollection');
    slDC.Add('(SequenceNo, ServiceID, BatchID,');
    slDC.Add(' TxnDate, TxnTime, Origin,');
    slDC.Add(' Duration, CallCode, RateDescription,');
    slDC.Add(' FromDate, ToDate, TransactionTypeID,');
    slDC.Add(' AmountExGST, GSTAmount, GSTFlag, AmountIncGST,');
    slDC.Add(' RevenueFlag, Var01, Var02, Var03, Var04)');
    slDC.Add('SELECT tDataCollection.SequenceNo, tDataCollection.ServiceID, '+
      'tDataCollection.BatchID,');
    slDC.Add(' tDataCollection.TxnDate, tDataCollection.TxnTime, '+
      'tDataCollection.Origin,');
    slDC.Add(' tDataCollection.Duration, tDataCollection.CallCode, '+
      'tDataCollection.RateDescription,');
    slDC.Add(' tDataCollection.FromDate, tDataCollection.ToDate, '+
      'tDataCollection.TransactionTypeID,');
    slDC.Add(' DC3.AmountExGST, DC3.GSTAmount, DC3.GSTFlag,');
    slDC.Add(' CAST((DC3.AmountExGST + DC3.GSTAmount) AS money) AS '+
      '"AmountIncGST", (1) AS "RevenueFlag",');
    slDC.Add(' tDataCollection.Var01, CONVERT(varchar,DC3.Cost,2) AS "Var02",');
    slDC.Add(' CONVERT(varchar,DC3.AmountExGST,2) AS "Var03", ('+
      sGByte+') AS "Var04"');
    slDC.Add('FROM (SELECT DC2.*,');
    slDC.Add(' CAST((CASE WHEN DC2.GSTFlag LIKE ''Y'' THEN (DC2.AmountExGST * '+
      sGSTRate+') ELSE 0 END) AS money) AS "GSTAmount"');
    slDC.Add('FROM (SELECT DC1.DataCollectionID, DC1.Cost,');
    slDC.Add(' (CAST(tDataCollection.Var01 AS Integer)*DC1.Cost) AS "AmountExGST",');
    slDC.Add(' (CASE WHEN DC1.CustomerID = '+sVRTID+
      ' THEN ''N'' ELSE ''Y'' END) AS "GSTFlag"');
    // Include Incremental values in determining the final charge rate (cost).
    slDC.Add('FROM (SELECT DCFullRate.DataCollectionID, '+
      'DCFullRate.ServiceTypeID, DCFullRate.CustomerID,');
    slDC.Add(' (DCFullRate.Cost + (CASE WHEN '+sGByte+' > DCFullRate.Usage THEN');
    slDC.Add(' (DCFullRate.PartCost * ('+sGByte+' - DCFullRate.Usage)) '+
      'ELSE 0 END)) AS "Cost"');
    // Use Min() rate a/c price is reduced for more usage.
    slDC.Add('FROM (SELECT DCRate.*, tRate.Length AS "Usage",');
    slDC.Add(' (CASE WHEN tRate.PartCost IS NULL THEN 0 ELSE tRate.PartCost END'+
      ') AS "PartCost"');
    slDC.Add('FROM (SELECT DC0.*, Min(tRate.Cost) AS "Cost"');
    slDC.Add('FROM (SELECT tDataCollection.ID AS "DataCollectionID",');
    slDC.Add(' tServiceID.ServiceTypeID, tGroup.CustomerID');
    slDC.Add('FROM tDataCollection');
    slDC.Add('LEFT JOIN tServiceID '+
      'ON tDataCollection.ServiceID = tServiceID.ServiceID');
    slDC.Add('LEFT JOIN tCentre ON tServiceID.CustomerCentreID = tCentre.ID');
    slDC.Add('LEFT JOIN tDepartment ON tCentre.DepartmentID = tDepartment.ID');
    slDC.Add('LEFT JOIN tGroup ON tDepartment.GroupID = tGroup.ID');
    slDC.Add('WHERE tDataCollection.BatchID = '+sBatchID);
    slDC.Add('AND tDataCollection.RevenueFlag = 0) AS DC0');
    slDC.Add('LEFT JOIN tRate '+
      'ON ((DC0.ServiceTypeID = tRate.ServiceTypeID) AND (tRate.Length <= '+
      sGByte+') AND (tRate.Active = 1))');
    slDC.Add('GROUP BY DC0.DataCollectionID, DC0.ServiceTypeID, DC0.CustomerID'+
      ') AS DCRate');
    slDC.Add('LEFT JOIN tRate ON ((DCRate.ServiceTypeID = tRate.ServiceTypeID)');
    slDC.Add(' AND (tRate.Cost = DCRate.Cost) AND (tRate.Active = 1))');
    slDC.Add('WHERE tRate.Cost = DCRate.Cost) AS DCFullRate) AS DC1');
    slDC.Add('LEFT JOIN tDataCollection '+
      'ON DC1.DataCollectionID = tDataCollection.ID) AS DC2) AS DC3');
    slDC.Add('LEFT JOIN tDataCollection '+
      'ON DC3.DataCollectionID = tDataCollection.ID');
    qryDC.SQL := slDC;
    qryDC.ExecSQL;

    // Read a value to include in the update of the tBatch table.
    slDC.Clear;
    slDC.Add('SELECT CAST(CAST(DC1.Cost AS real) AS varchar) AS "RateCost",');
    slDC.Add(' CAST(ServiceTypeID AS varchar) AS "ServiceType"');
    slDC.Add('FROM (SELECT Min(tRate.Cost) AS "Cost", DC0.ServiceTypeID');
    slDC.Add('FROM (SELECT tServiceID.ServiceTypeID');
    slDC.Add('FROM tDataCollection');
    slDC.Add('LEFT JOIN tServiceID ON tDataCollection.ServiceID = tServiceID.ServiceID');
    slDC.Add('WHERE tDataCollection.BatchID = '+ sBatchID);
    slDC.Add('AND tDataCollection.RevenueFlag = 0) AS DC0');
    slDC.Add('LEFT JOIN tRate ON ((DC0.ServiceTypeID = tRate.ServiceTypeID)');
    slDC.Add(' AND (tRate.Active = 1)');
    slDC.Add(' AND (tRate.Length <= '+ sGByte+'))');
    slDC.Add('GROUP BY DC0.ServiceTypeID) AS DC1');
    slDC.Add('WHERE DC1.Cost <> 0');
    qryDC.SQL := slDC;
    qryDC.Open;
    sRateCost := qryDC.FieldByName('RateCost').AsString;
    sServiceType := qryDC.FieldByName('ServiceType').AsString;
    if (Length(sRateCost) = 0) then sRateCost := '0';  // Avoids invalid "Set =".
    qryDC.Close;

    // Make sure that all entries for this BatchID do not have
    // RevenueFlag=Null so that extra revenue Transactions will not be created
    // when transferring to tTransaction later using SaveToTransaction().
    slDC.Clear;
    slDC.Add('UPDATE tDataCollection');
    slDC.Add('SET RevenueFlag = 0');
    slDC.Add('WHERE RevenueFlag IS NULL');
    slDC.Add('AND BatchID = '+sBatchID);
    qryDC.SQL := slDC;
    qryDC.ExecSQL;

    // Update some more of the values stored in the tBatch table.
    slDC.Clear;
    slDC.Add('UPDATE tBatch');
    slDC.Add('SET TotalRevenue = BatchRate.RevenueAmt,');
    slDC.Add(' TotalCredits = '+ sRateCost +',');
    slDC.Add(' OrderNumber = '+ sGByte +',');
    slDC.Add(' InvoiceNo = '''+ sServiceType +''',');
    slDC.Add(' RequisitionNumber = '+ IntToStr(nkByte));
    slDC.Add('FROM (SELECT Sum(tDataCollection.AmountExGST) AS "RevenueAmt"');
    slDC.Add('FROM tDataCollection');
    slDC.Add('WHERE tDataCollection.BatchID = '+ sBatchID);
    slDC.Add('AND tDataCollection.RevenueFlag = 1) AS BatchRate');
    slDC.Add('WHERE tBatch.ID = '+ sBatchID);
    qryDC.SQL := slDC;
    qryDC.ExecSQL;
  end;  // LumpedCosts.

  // Done. Do some housework.
  MessageForm.Close;
  MessageForm.Free;
  slDC.Free;
  qryDC.Free;
end;  // ApplyDataRates.



procedure BillingInputData( const LumpedCosts, IncludedRate: boolean;
  const stInFiles: Tstrings; nBatch: LongInt );
// This program processes the Telstra billing reports supplied on disk
// and places the data into the tDataCollection table for verification.
//
// The parameter passed contains the names of the input files, which are
// passed on to the GetMobileData() procedure.
//
var
  lContinue : Boolean;
  sPartFileName : String;
  nBackSlashAt: Integer;
  EmptyList : TStringList;          // Used only when replacing DC transactions.
  nPos : integer;
begin
  // initialise the Static table elements defined as Private for this form.
  lContinue := True;
  qryTelstra := TQuery.Create(MainForm);
  with qryTelstra do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('SELECT id');
    SQL.Add('FROM tSupplier');
    SQL.Add('WHERE SupplierName LIKE ''TELSTRA%''');
    Open;
  end;
  if qryTelstra.Eof then begin
    MessageDlg('TELSTRA not found in the tSupplier table. Processing stopped.',
      mtWarning, [mbOk], 0);
    lContinue := False;
  end
  else begin
    nSupplierID := qryTelstra['id'];
    sSupplierID := IntToStr(nSupplierID);
    with qryTelstra do begin
      Close;
      SQL.Clear;
      SQL.Add('SELECT tValidAccountNumber.AccountNo');
      SQL.Add('FROM tValidAccountNumber');
      SQL.Add('INNER JOIN tBatch ON tValidAccountNumber.BatchTypeID = tBatch.BatchTypeID');
      SQL.Add('WHERE tBatch.ID = '+IntToStr(nBatch));
      Open;
      if Eof then begin
        MessageDlg('BatchType not found in the tValidAccountNumber table. '+
          'Processing stopped.',
          mtWarning, [mbOk], 0);
        lContinue := False;
      end
      else begin
        sAccountNumber := Trim(FieldByName('AccountNo').AsString);
        nPos := Pos(' ',sAccountNumber);
        while (nPos > 0) do begin
          sAccountNumber := Copy(sAccountNumber,1,nPos-1) +
                            Copy(sAccountNumber,nPos+1,Length(sAccountNumber)-nPos);
          nPos := Pos(' ',sAccountNumber);
        end;  // nPos.
      end;  // Eof.
      Close;
    end;  // qryTelstra.
  end;
  if lContinue then begin
    with qryTelstra do begin
      Close;
      SQL.Clear;
      SQL.Add('SELECT Code, CAST(TransactionTypeID AS VARCHAR(20)) '+
        'AS TransactionType');
      SQL.Add('FROM tSupplierServiceCode');
      SQL.Add('WHERE SupplierID = '+sSupplierID);
      SQL.Add('AND Description LIKE ''RENT''');
      Open;
      if Eof then sRentCodeID := 'NULL'
      else sRentCodeID := Trim(FieldByName('Code').AsString);
      Close;
      SQL.Strings[qryTelstra.SQL.Count-1] := 'AND Description LIKE ''CALLS''';
      Open;
      if Eof then sCallCodeID := 'NULL'
      else sCallCodeID := Trim(FieldByName('Code').AsString);
      Close;
      SQL.Strings[qryTelstra.SQL.Count-1] := 'AND Description LIKE ''Unknown%''';
      Open;
      if Eof then sOtherCodeID := 'NULL'
      else sOtherCodeID := FieldByName('Code').AsString;
      Close;
      SQL.Strings[qryTelstra.SQL.Count-1] :='AND Description LIKE ''MISCELLANEOUS''';
      Open;
      if qryTelstra.Eof then sDiscountCodeID := 'NULL'
      else sDiscountCodeID := Trim(FieldByName('Code').AsString);
      Close;
      SQL.Delete(SQL.Count-1);
      Open;
    end;  // with qryTelstra.

    // Initialise the progress bar to display to the operator.
    frmProgress := TProgressForm.Create(MainForm);
    frmProgress.btnCancel.Visible := False;
    frmProgress.Show;

    sPartFileName := stInFiles.Strings[0];
    while Pos('\',sPartFileName)>0 do begin
      nBackSlashAt  := Pos('\',sPartFileName);
      sPartFileName := Copy(sPartFileName,
        nBackSlashAt+1, Length(sPartFileName)-nBackSlashAt);
    end;  // while Pos('\',sPartFileName)>0 do begin.
    frmProgress.Msg.Caption := 'Reading ' + sPartFileName + ' ...';
    frmProgress.ProgressBar1.Position := 0;
    frmProgress.Msg.Refresh;

    // Flush out any data in case this is a reload.
    EmptyList := TStringList.Create;       // Removes all entries for the batch.
    lContinue := True;
    DeleteFromDC( nBatch, EmptyList, lContinue);
    EmptyList.Free;

    // Now move the text data into the Table.
    if lContinue then begin
      GetMobileData( stInFiles.Strings[0], DM.dbPPdata, nBatch, lContinue);
    end;

    frmProgress.Hide;
    frmProgress.Free;
    if lContinue then begin
      ApplyDataRates(nBatch, LumpedCosts, IncludedRate);
      if not LumpedCosts then UpdateBatchTotals(nBatch);
      UpdateBatchStatus(nBatch, MainForm.BATCH_STATUS_IMPORT);

      // **********************************************************************
      // **********************************************************************
      // Test for critical errors that prevent progress to the next stage.
      lContinue := False;
      CriticalErrorTest( nBatch, lContinue );
      if lContinue then begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Passed all tests for Critical Errors.', mtInformation,
          [mbOk], 0);
      end
      else begin
        MessageDlg('Batch '+ IntToStr(nBatch) +
          ': Critical errors found.'#13#10#13#10'These must be resolved ' +
          'before this invoice can be progressed.',
          mtWarning, [mbOk], 0);
      end;
    end;  // if lContinue then begin.
  end;  // if qryTelstra.Eof then begin..else.
end;    // BillingInputData.



procedure TelstraData( const LumpedCosts, IncludedRate: boolean;
  const nBatch: LongInt; sDirectory: string );
// This function prompts the user for the name of the Telstra supplied
// text file and then calls the BillingInput function (which reads the
// file into a table for further processing).
var
  lContinue : Boolean;
  stFiles : TStringList;
  OpenDlg: TOpenDialog;
  EmptyList : TStringList;
begin
  // Create the stringlist which contains the names of selected filenames.
  stFiles := TStringList.Create;
  try
    // Now, we must establish which files are to be processed.
    // Select the source directory.
    OpenDlg := TOpenDialog.Create(MainForm);
    OpenDlg.Filter := 'Telstra Billing file (*.csv)|*.CSV|All files (*.*)|*.*';
    OpenDlg.FilterIndex := 1;
    // Use the program directory as the default directory.
    if (Length(sDirectory) > 0) then OpenDlg.InitialDir := sDirectory
    else OpenDlg.InitialDir := ExtractFilePath(ParamStr(0));
    OpenDlg.Title := 'Select The Telstra Data Account File';
    OpenDlg.Options := [ofFileMustExist, ofNoChangeDir];
    // Read and save the user's choices before continuing.
    lContinue := OpenDlg.Execute;
    stFiles.Text := OpenDlg.Files.Text;
    OpenDlg.Free;
    if lContinue then begin
      lContinue := stFiles.Count > 0;
      // Now call for processing of the input files.
      if lContinue then begin
        // Empty stringlist => Remove all entries for BatchID.
        EmptyList := TStringList.Create;
        DeleteFromDC( nBatch, EmptyList, lContinue);
        EmptyList.Free;
        BillingInputData(LumpedCosts, IncludedRate, stFiles, nBatch);
      end;  // if lContinue then.
    end;  // if lContinue then.
  finally
    stFiles.Free;
  end;
end;    // procedure TelstraData.


end.
