unit AccessRightsSys;

interface

uses
  Classes, dbtables, Dialogs, SysUtils, inifiles;

function SetAccessRights(const AskUser: boolean;
  var aAccess : array of boolean): boolean;

implementation

uses
  ConstantValuesSys, DataMod, LoginFrm, PPSysMain;

const
  IMPORTMANUAL  = 0;          // PP-view Manual Imports.
  IMPORTKEYED   = 1;          // PP-view Keyed Imports.
  IMPORTAUTO    = 2;          // PP-view Automtic Imports.
  BATCHREVIEW   = 3;          // PP and TBIMS.
  TBSREVENUE    = 4;          // TBIMS only, NOT PP.
  TBSEXPENSE    = 5;          // TBIMS only, NOT PP.
  REFDATA       = 6;          // PP and TBIMS.
  REFDATAVIEW   = 7;          // PP and TBIMS.
  AUDIT         = 8;          // PP and TBIMS.
  PROCESSIMPORT = 9;          // PP-Import buttons enabled on Initiate tab.
  VIEWDATA      = 10;         // TBIMS only, NOT PP.
  INVOICE       = 11;         // TBIMS only, NOT PP.
  CREDITNOTES   = 12;         // TBIMS only, NOT PP.
  USERMAINT     = 13;         // PP and TBIMS.
  DEVELOPER     = 14;         // PP and TBIMS.
  PAYABLES      = 15;         // TBIMS only, NOT PP.

{
function SetAccessRights(const AskJeeves: boolean;
  var aAccess : array of boolean): boolean;
var
  LoginParams : TStringList;
  NewPPIniFile : TIniFile;
  IniFileName : string;
  sUserName : string;
  sPassword : string;
  sDummy : string;
  q : TQuery;
  lReturn, lSuccess : boolean;
  nUserID : integer;
  sUserID : string;
  ShowLogForm : boolean;
  nRite : integer;
begin
  IniFileName := LowerCase(ParamStr(0));
  IniFileName := Copy(IniFileName,1,Length(IniFileName)-3) + 'ini';
  NewPPIniFile := TIniFile.Create(IniFileName);
  // dbPPdata - SQLPP (most common) database.
  with NewPPIniFile do begin
    ShowLogForm := (ReadString('General','Log', '') = '0');
  end;
  NewPPIniFile.Free;
  if ShowLogForm then begin
    LoginParams := TStringList.Create;
    // "User Access" not used as next parameter is False - not Startup condition.
    lReturn := GetLoginParams('User Access',False,sDummy,LoginParams);
  end
  else begin
    lReturn := True;
  end;
  if lReturn then begin
    if ShowLogForm then begin
      sUserName := LoginParams.Values['USER NAME'];
      sPassword := LoginParams.Values['PASSWORD'];
      LoginParams.Free;
    end
    else begin
      sUserName := MainForm.OperatorName;
    end;
    q := TQuery.Create(nil);
    q.DatabaseName := 'dbPPdata';              // Contained in "DataMod" module.
    q.SQL.Add('SELECT *');
    q.SQL.Add('FROM tUser');
    q.SQL.Add('WHERE UserLogin LIKE '''+sUserName+'''');
    q.SQL.Add('AND Active = 1');
    if ShowLogForm and AskJeeves then
      q.SQL.Add('AND Password LIKE '''+sPassword+'''');
    q.Open;
    lSuccess := not (q.Bof and q.Eof);
    if lSuccess then begin
      nUserID := q.FieldByName('id').AsInteger;
      sUserID := IntToStr(nUserID);
      MainForm.OPERATORNUMBER := nUserID;
      MainForm.OPERATORNAME := sUserName;
      MainForm.OPERATORFULLNAME := Trim(q.FieldByName('FirstName').AsString +
        ' '+ q.FieldByName('Surname').AsString);
    end;
    q.Close;
    if not lSuccess then begin
      lReturn := False;
      MessageDlg('User and Password not found. Access is denied.',
        mtWarning, [mbOk], 0);
    end
    else begin
      if ShowLogForm then begin
        // Reset all privileges.
        for nRite := 0 to Length(aAccess) do
          aAccess[nRite] := False;
        q.SQL.Clear;
        q.SQL.Add('SELECT *');
        q.SQL.Add('FROM tUserFunction');
        q.SQL.Add('WHERE UserID = '+sUserID);
        q.SQL.Add('ORDER BY FunctionID');
        q.Open;
        while not q.Eof do begin
          aAccess[q.FieldByName('FunctionID').AsInteger-1] := True;
          q.Next;
        end;  // while.
        q.Close;
      end
      else begin
        // Set all privileges - full access for everyone.
        for nRite := 0 to Length(aAccess) do
          aAccess[nRite] := True;
      end;
    end;  // not lSuccess.
    q.Free;
  end;  // lReturn.
  Result := lReturn;
end;  // SetAccessRights.
}

function SetAccessRights(const AskUser: boolean;
  var aAccess : array of boolean): boolean;
var
  LoginParams : TStringList;
  NewPPIniFile : TIniFile;
  IniFileName : string;
  ShowLogForm : boolean;
  AskJeeves : boolean;
  sUserName : string;
  sPassword : string;
  sDummy : string;
  q : TQuery;
  lReturn, lSuccess : boolean;
  sUserID : string;
  nUserID : integer;
  nRite : integer;
begin
  lReturn := False;
  AskJeeves := AskUser;
  IniFileName := INIFILEDIRECTORY+ChangeFileExt(ExtractFileName(ParamStr(0)),'.ini');
  NewPPIniFile := TIniFile.Create(IniFileName);
  ShowLogForm := (NewPPIniFile.ReadString('General','GetLog', '1') = '1');
  LoginParams := TStringList.Create;
  q := TQuery.Create(nil);
  q.DatabaseName := 'dbPPdata';              // Contained in "DataMod" module.
  if not AskJeeves then begin
    // Check that the parameter passed is a valid UserID.
    nUserID := MainForm.OPERATORNUMBER;
    if nUserID > 0 then begin
      sUserID := IntToStr(nUserID);
      q.SQL.Add('SELECT *');
      q.SQL.Add('FROM tUser');
      q.SQL.Add('WHERE ID = '+sUserID);
      q.SQL.Add('AND Active = 1');
      q.Open;
      if not (q.Bof and q.Eof) then begin    // True if any record returned.
        sUserName := q.FieldByName('UserLogin').AsString;
        MainForm.OPERATORNAME := sUserName;
        lReturn := True;
      end
      else begin
        AskJeeves := True;
      end;
      q.Close;
      q.SQL.Clear;
    end;
  end;
  if AskJeeves then begin
    IniFileName := INIFILEDIRECTORY+ChangeFileExt(ExtractFileName(ParamStr(0)),'.ini');
    NewPPIniFile := TIniFile.Create(IniFileName);
    ShowLogForm := (NewPPIniFile.ReadString('General','GetLog', '1') = '1');
    NewPPIniFile.Free;
    if ShowLogForm then begin
      // "User Access" not used as 2nd parameter is False - not a startup condition.
      lReturn := GetLoginParams('User Access',False,sDummy,LoginParams);
      sUserName := LoginParams.Values['USER NAME'];
      sPassword := LoginParams.Values['PASSWORD'];
    end
    else begin
      lReturn := True;
      sUserName := MainForm.OperatorName;
    end;
  end;
  if lReturn then begin
    q.SQL.Add('SELECT *');
    q.SQL.Add('FROM tUser');
    q.SQL.Add('WHERE UserLogin LIKE '''+sUserName+'''');
    q.SQL.Add('AND Active = 1');
    if ShowLogForm and AskJeeves then
//      q.SQL.Add('AND Password LIKE '''+EncodeString(sPassword)+'''');
      q.SQL.Add('AND Password LIKE '''+sPassword+'''');
    q.Open;
    lSuccess := not (q.Bof and q.Eof);
    if lSuccess then begin
      nUserID := q.FieldByName('id').AsInteger;
      sUserID := IntToStr(nUserID);
      MainForm.OPERATORNUMBER := nUserID;
      MainForm.OPERATORNAME := sUserName;
      MainForm.OPERATORFULLNAME := Trim(q.FieldByName('FirstName').AsString +
        ' '+ q.FieldByName('Surname').AsString);
    end;  // bof & eof.
    q.Close;
    if not lSuccess then begin
      lReturn := False;
      MessageDlg('User and Password not found. Access is denied.',
        mtWarning, [mbOk], 0);
    end
    else begin
      if ShowLogForm then begin
        // Reset all privileges.
        for nRite := 0 to Length(aAccess) do
          aAccess[nRite] := False;
        q.SQL.Clear;
        q.SQL.Add('SELECT *');
        q.SQL.Add('FROM tUserFunction');
        q.SQL.Add('WHERE UserID = '+sUserID);
        q.SQL.Add('ORDER BY FunctionID');
        q.Open;
        while not q.Eof do begin
          aAccess[q.FieldByName('FunctionID').AsInteger-1] := True;
          q.Next;
        end;
        q.Close;
        lReturn := aAccess[REFDATA];              // Only if permitted.
      end
      else begin
        // Set all privileges - full access for everyone.
        for nRite := 0 to Length(aAccess) do
          aAccess[nRite] := True;
      end;  // ShowLogForm.
    end;  // not lSuccess.
  end;  // lReturn.
  q.Free;
  LoginParams.Free;
  Result := lReturn;
end;  // SetAccessRights.

end.
