unit GenFns;
// ProveAnyKey() added - RJC 110522.
interface

uses
  SysUtils, dbtables, bde, classes, DBGrids, Dialogs, FileCtrl, inifiles;

  procedure SaveText(const s: string);
  procedure UpdateBatchStatus(const nThisBatch, nStatus: LongInt);
  procedure LogOffUser(const sID:string);
  procedure DeleteTransactions(const sTable: string; const nThisBatch: LongInt;
    const sFieldName,sFieldType,sOperation,sFieldValue: string;
    var lSuccess:boolean);
  procedure XCopyToAdjustment(const nThisBatch:LongInt;
    const FieldValues: TStringList; var lSuccess: boolean);
  procedure DeleteFromDC(const nThisBatch:LongInt;
    const FieldValues: TStringList; var lSuccess: boolean);
  function Left( const S: string; const N: integer): string;
  function Right( const S: string; const N: integer): string;
  procedure QuickCopyTable(T: TTable; DestTblName: string;
                           Overwrite: Boolean);
  function AccountFormat( const S: string ): string;
  function curSum(curValue1, curValue2: Currency) : Currency;
  function curFormat(curValue: Currency) : Currency;
  function CleanString( const S:string): string;
  function SingleSpaces( const S:string): string;
  function Curr2Str(const curValue: Currency): string;
  procedure GetOperatorReference(const Operator:string;var sID:string);
  function SecsToHrMinSec(const sTotalSecs: String): string;
  function YMD2DMY( const sDate: String ): string;
  procedure AddToTable(const sTableInsert: string;
    const sFields, sValues: TStringList);
  procedure GridTitleSort(Column: TColumn; slQuery: TStrings);
  procedure ProveInteger(var edtValue : string; var lValueAltered : boolean);
  procedure ProveWholeInteger(var edtValue : string; var lValueAltered : boolean);
  procedure ProveReal(var edtValue : string; var lRealValue : boolean);
  procedure ProveForeignKey(const sTable: string;
    const nID: integer; var lSuccess: boolean);
  procedure ProveAnyKey(const sTable: string; const sField: string;
    const nValue: integer;  var lSuccess: boolean);
  function TestForAudit(const TableName: string): boolean;
  procedure SetParameter(const sParm, sParmValue, sParmField: string);
  procedure GetParameter(const sParm:string; var sParmValue: string);
  procedure FileQuery(const qryData:TQuery;const FileName: string );
{
  procedure SetAuditData(const FieldName, OldValue, NewValue: string;
    var AuditData: string);
  procedure SetAuditSelect(const UserName, sAction, TableName: string;
    var AuditData: TStringList;var AuditField, AuditValue: string);
}
  procedure SetAuditStart(const UserName, sAction, TableName: string;
    var AuditInsert, AuditField, AuditValue: string);
  procedure SetAuditData(const FieldName, OldValue, NewValue: string;
    var AuditData: TStringList);
  procedure SetAuditID(const ID: string; var AuditValue: string);
  procedure SaveAudit(const AuditInsert, AuditField, AuditValue: string;
    const AuditData: TStringList);
  function TestForTestDB: boolean;

implementation

uses
  ConstantValuesSys, DataMod, DateFunctions, PPsysMain, MyMessage, Progress;

//

procedure ProveForeignKey(const sTable: string; const nID: integer;
  var lSuccess: boolean);
// Prove a ForeignKey (named "ID") - usually links two tables.
var
  qryFK : TQuery;
begin
  qryFK := TQuery.Create(MainForm);
  qryFK.DatabaseName := 'dbPPdata';
  qryFK.SQL.Add('SELECT id');
  qryFK.SQL.Add('FROM '+sTable);
  qryFK.SQL.Add('WHERE id = '+IntToStr(nID));
  qryFK.Open;
  lSuccess := not (qryFK.Bof and qryFK.Eof);
  qryFK.Close;
  qryFK.Free;
end;  // ProveForeignKey.

procedure ProveAnyKey(const sTable: string; const sField: string;
  const nValue: integer;  var lSuccess: boolean);
// Prove a ForeignKey (not called "ID") - that is used to link two tables.
var
  qryAK : TQuery;
begin
  qryAK := TQuery.Create(MainForm);
  qryAK.DatabaseName := 'dbPPdata';
  qryAK.SQL.Add('SELECT '+sField);
  qryAK.SQL.Add('FROM '+sTable);
  qryAK.SQL.Add('WHERE id = '+IntToStr(nValue));
  qryAK.Open;
  lSuccess := not (qryAK.Bof and qryAK.Eof);
  qryAK.Close;
  qryAK.Free;
end;  // ProveAnyKey.

procedure ProveInteger(var edtValue : string; var lValueAltered : boolean);
var
  sValue : string;
  nLenValue : integer;
  sChar : string;
  lValueChanged : boolean;
begin
  lValueChanged := False;
  sValue := Trim(edtValue);
  nLenValue := Length(sValue);
  if (nLenValue > 0) then begin
    sChar := Copy(sValue,nLenValue,1);
    if (Pos(sChar,'0123456789')=0) then begin
      if (nLenValue = 1) then edtValue := ''
      else edtValue := Copy(sValue,1,nLenValue-1);
      lValueChanged := True;
    end;
  end;  // not ProveInteger().
  lValueAltered := lValueChanged;
end;  // ProveInteger().

procedure SaveText(const s: string);
var
  lContinue : boolean;
  sFile : string;
  SaveDialog : TSaveDialog;
  SundryTextFile : TextFile;
  q : TQuery;
  sTBSdirectory : string;
begin
  q := TQuery.Create(nil);
  q.DatabaseName := 'dbPPdata';
  q.SQL.Add('SELECT ParmText');
  q.SQL.Add('FROM tParameter');
  q.SQL.Add('WHERE Parm LIKE ''TBSdirectory''');
  q.Open;
  if q.Eof then sTBSdirectory := '' else sTBSdirectory := q['ParmText'];
  q.Close;
  q.Free;
  lContinue := (Length(sTBSdirectory)>0) and DirectoryExists(sTBSdirectory);
  if lContinue then begin
    sFile := sTBSdirectory + '\Out' + StandardDateString(Date) + '.txt';
  end
  else begin
    // Ask the operator if the file is to be saved to file.
    SaveDialog := TSaveDialog.Create(nil);
    SaveDialog.Title := 'Save ID for Duplicate faults';
    SaveDialog.FileName := 'Out' + StandardDateString(Date) + '.txt';
    SaveDialog.Filter := 'Text Files (*.txt)|*.txt|All Files (*.*)|*.*';
    SaveDialog.FilterIndex := 1;
    SaveDialog.DefaultExt := 'txt';
    // Use the program directory as the default directory.
    SaveDialog.InitialDir := ExtractFilePath(ParamStr(0));
    SaveDialog.Options := [ofHideReadOnly,ofNoReadOnlyReturn];
    lContinue := SaveDialog.Execute;
    sFile := SaveDialog.FileName;
    SaveDialog.Free;
    sTBSdirectory := ExtractFileDir(sFile);
  end;
  if lContinue and DirectoryExists(sTBSdirectory) then begin
    // Open the required file.
    AssignFile( SundryTextFile, sFile );
    try
      Rewrite( SundryTextFile );
      // OK, now output the data to the file.
      Writeln( SundryTextFile, s );
    finally
      // Done, now close the output file.
      CloseFile( SundryTextFile );
    end;
  end;
end;  // SaveText.

procedure ProveWholeInteger(var edtValue : string; var lValueAltered : boolean);
var
  sValue : string;
  nLenValue : integer;
  nInt : integer;
  lValueChanged : boolean;
begin
  lValueChanged := False;
  sValue := Trim(edtValue);
  nLenValue := Length(sValue);
  if (nLenValue > 0) then begin
    for nInt := 1 to nLenValue do
      if (Pos(Copy(sValue,nInt,1),'0123456789+-')=0) then begin
        if (nInt = 1) then edtValue := ''
        else edtValue := Copy(sValue,1,nInt-1);
        lValueChanged := True;
        break;
      end;
  end;  // if (nLenValue > 0) then begin.
  lValueAltered := lValueChanged;
end;  // ProveWholeInteger().

procedure ProveReal(var edtValue : string; var lRealValue : boolean);
var
  sValue : string;
  nLenValue : integer;
  nInt : integer;
  lValueChanged : boolean;
begin
  lValueChanged := False;
  sValue := Trim(edtValue);
  nLenValue := Length(sValue);
  if (nLenValue > 0) then begin
    for nInt := 1 to nLenValue do
      if (Pos(Copy(sValue,nInt,1),'0123456789.+-')=0) then begin
        if (nInt = 1) then edtValue := ''
        else edtValue := Copy(sValue,1,nInt-1);
        lValueChanged := True;
        break;
      end;
  end;  // not ProveInteger().
  lRealValue := not lValueChanged;
end;  // ProveReal().

procedure GridTitleSort(Column: TColumn; slQuery: TStrings);
// Generic sorting routine. Called indirectly when the user clicks on a
// column title of a database grid (eg a TRxDBGrid object).
//   Column contains the field to apply the sort,
//   slQuery is the previous SQL statement to be altered.
// This routine will only act on a data source that is already sorted.
// (Requires ORDER BY to be in the last line.)
var
  nSpace : integer;
  SortString : string;
  SortFieldName : string;
begin
  // Change sort order - eg:"ORDER BY BatchID, fieldname DESC".
  SortString := Trim(slQuery[slQuery.Count-1]);
  if (UpperCase(Copy(SortString,1,8)) = 'ORDER BY') and
    (Length(Column.FieldName) > 0) then begin
    nSpace := Pos(Column.FieldName,SortString);
    if (nSpace > 0) and
      ((Length(SortString)-nSpace+1) = Length(Column.FieldName)) then begin
      if (Right(SortString,5) = ' DESC') then
        SortString := Copy(SortString,1,Length(SortString) - 5)
      else
        SortString := SortString + ' DESC';
    end
    else begin
      SortFieldName := '';
      SortString := Copy(SortString,10,Length(SortString)-9);
      nSpace := Pos(',',SortString);
      while (nSpace > 0) do begin  // Multiple Fields, retain first.
        SortFieldName := Trim(Copy(SortString,1,nSpace));
        nSpace := Pos(',',Copy(SortString,nSpace+1,Length(SortString)-nSpace-1));
      end;  // while (nSpace > 0 do begin.
      if (Pos(Column.FieldName,SortFieldName) > 0) then begin
        if (Right(SortString,5) = ' DESC') then
          SortString := Copy(SortString,1,Length(SortString) - 5)
        else
          SortString := SortString + ' DESC';
      end
      else begin
        SortString := SortFieldName +' '+ Column.FieldName;
      end;  // if (Pos(Column.FieldName,SortFieldName) > 0) then begin.
      SortString := 'ORDER BY '+ SortString;
    end;  // if (Pos(Column.FieldName,SortString) > 0) then begin.
    slQuery[slQuery.Count-1] := SortString;
  end;  // if (UpperCase(Copy(SortString,1,8)) = 'ORDER BY') then begin.
end;  // procedure GridTitleSort().

procedure AddToTable(const sTableInsert: string;
  const sFields, sValues: TStringList);
// Generic procedure to add the values to the named table.
// All parameters must be strings.
var
  qryAdd : TQuery;
  slStrings : TStringList;
  nItem, nTotal : integer;
  sResults : string;
  sThisTable : string;
begin
  sThisTable := sTableInsert;
  nTotal := sFields.Count;
  if (sValues.Count = nTotal) then begin
    slStrings := TStringList.Create;
    slStrings.Add('INSERT INTO '+ sThisTable);
    sResults := '(';
    for nItem := 0 to nTotal-1 do begin
      sResults := sResults+ sFields[nItem] + ',';
      if (Length(sResults) > 60) and (nItem < nTotal-1) then begin
        slStrings.Add(sResults);
        sResults := '';
      end;  // if line > 60 chars.
    end;  // for nItem := 0 to nTotal do begin.
    sResults := Copy(sResults, 1, Length(sResults)- 1);    // Remove last comma.
    sResults := sResults + ')';
    slStrings.Add(sResults);
    sResults := 'VALUES (';
    for nItem := 0 to nTotal-1 do begin
      sResults := sResults+ sValues[nItem] + ',';
      if (Length(sResults) > 60) and (nItem < nTotal-1) then begin
        slStrings.Add(sResults);
        sResults := '';
      end;  // if line > 60 chars.
    end;  // for nItem := 0 to nTotal do begin.
    sResults := Copy(sResults, 1, Length(sResults)- 1);    // Remove last comma.
    sResults := sResults + ')';
    slStrings.Add(sResults);
    qryAdd := TQuery.Create(MainForm);
    qryAdd.DatabaseName := 'dbPPdata';
    qryAdd.SQL := slStrings;
    qryAdd.ExecSQL;
    qryAdd.Free;
    slStrings.Free;
  end
  else begin
    MessageDlg('Number of fields and data differ when'#13#10'appending to '''+
      sThisTable+'''',mtWarning,[mbOk],0);
  end;  // same number of strings.
end;  // procedure AddToTable().

procedure UpdateBatchStatus(const nThisBatch, nStatus: LongInt);
var
  qryBA : TQuery;
  qryStringList : TStringList;
  nBatch : LongInt;
  nOutcome : integer;
begin
  nBatch := nThisBatch;
  nOutcome := nStatus;
  qryStringList := TStringList.Create;
  qryBA := TQuery.Create(MainForm);
  qryBA.DatabaseName := 'dbPPdata';
  qryStringList.Add('SELECT BatchStatusID FROM tBatch');
  qryStringList.Add('WHERE (ID = '+ IntToStr(nBatch)+')');
  qryBA.SQL := qryStringList;
  qryBA.Open;
  if not qryBA.Eof then begin
    // (Test avoids a possible error - but can't get here unless it exists).
    if (qryBA['BatchStatusID'] <> nOutcome) then begin
      // Update the tBatch table.
      qryStringList.Clear;
      qryStringList.Add('UPDATE tBatch');
      qryStringList.Add('SET BatchStatusID = '+IntToStr(nOutcome));
      qryStringList.Add('FROM tBatch');
      qryStringList.Add('WHERE (ID = '+ IntToStr(nBatch)+')');
      qryBA.Close;
      qryBA.SQL := qryStringList;
      qryBA.ExecSQL;
    end;  // if (qryDC['BatchStatusID'] <> nOutcome) then.

    // Update the tBatchStatusHistory table.
    qryStringList.Clear;
    qryStringList.Add('INSERT INTO tBatchStatusHistory');
    qryStringList.Add('  (BatchID, BatchStatusID, StatusChangeDate,UserID)');
    qryStringList.Add('  VALUES ('+IntToStr(nBatch)+','+IntToStr(nOutcome)+
      ','''+StandardDateString(Now)+''','+IntToStr(MainForm.OPERATORNUMBER)+')');
    qryBA.Close;
    qryBA.SQL := qryStringList;
    qryBA.ExecSQL;
  end;  // if qryDC.Eof and qryDC.Bof then.
  qryBA.Close;
  qryBA.Free;
end;  // UpdateBatchStatus().


function YMD2DMY( const sDate: String ): string;
var
  SubString : string;
begin
  SubString := Copy(sdate,7,2) + '/' + Copy(sdate,5,2) + '/' + Copy(sdate,1,4);
  Result := SubString;
end;  // YMD2DMY().


function SecsToHrMinSec
          (
          const sTotalSecs: String
          ): string;
var
  sHour, sMin, sSec : string;
  nHour, nMin, nSec : integer;
  nTotalSecs : integer;
  SubString : string;
begin
  // Initialise variables.
  nHour := 0;
  nMin := 0;
  // Convert the parameter to an integer.
  nTotalSecs := StrToInt(sTotalSecs);
  // Define the number of hours, minutes, seconds.
  if (nTotalSecs < 60) then
    begin
    nSec := nTotalSecs;
    end
  else
    begin
    nSec := nTotalSecs;
    nMin := Trunc(nTotalSecs / 60);
    nSec := nSec - (nMin * 60);
    if (59 < nMin) then
      begin
      nHour := Trunc(nMin / 60);
      nMin := nMin - (nHour * 60);
      end;
    end;

  // Now format the result string.
  sMin := Trim(IntToStr(nMin));
  if (0 < nHour) then
    begin
    sHour := Trim(IntToStr(nHour));
    if (nMin < 10) then
      begin
      sMin := '0' + sMin;
      end;
    end;
  sSec := Trim(IntToStr(nSec));
  if (nSec < 10) then
    begin
    sSec := '0' + sSec;
    end;
  SubString := '';
  if (nHour > 0) then
    begin
    SubString := sHour + ':';
    end;
  SubString := SubString + sMin + ':' + sSec;
  Result := SubString;
end;  // SecsToHrMinSec().


procedure GetOperatorReference(const Operator:string;var sID:string);
var
  sOp : string;
  qry : TQuery;
  sQueryString : TStringList;
begin
  sOp := LowerCase(Trim(Operator));
  if (Length(sOp)=0) or (sOp='unknown') then
    begin
    sID := '0';              // Default.
    end
  else
    begin
    sQueryString := TStringList.Create;
    qry := TQuery.Create(nil);
    qry.DatabaseName := 'dbPPdata';
    sQueryString.Add('SELECT id FROM tUser');
    sQueryString.Add('WHERE (UserLogin LIKE '''+sOp+''')');
    qry.SQL := sQueryString;
    qry.Open;

    // *************************************************************************
    // ************  START DEVELOPER MODE  *************************************
    // *************************************************************************
{
    if (qry.Bof and qry.Eof) then begin
      // Not in table (must be logging on), so add it.
      qry.Close;
      qry.SQL.Clear;
      qry.SQL.Add('INSERT INTO tUser (UserLogin,LastLoggedOn)');
      qry.SQL.Add('VALUES ('''+sOp+''','''+
        StandardDateString(Now) +''')');
      qry.ExecSQL;
      qry.Close;
      qry.SQL.Clear;
      qry.SQL.Add('SELECT ID FROM tUser WHERE UserLogin = '''+sOp+'''');
      qry.Open;
      sUserID := IntToStr(qry.FieldByName('ID').AsInteger);
      // Not in the table - add it:
      // - grant full access to the program.
      qryFunction := TQuery.Create(nil);
      qryFunction.DatabaseName := 'dbPPdata';
      qryFunction.SQL.Add('SELECT ID FROM tFunction');
      qryFunction.Open;
      while not qryFunction.Eof do begin
        qry.Close;
        qry.SQL.Clear;
        qry.SQL.Add('INSERT INTO tUserFunction (UserID,FunctionID)');
        qry.SQL.Add('VALUES ('+sUserID+','+IntToStr(qryFunction['id'])+')');
        qry.ExecSQL;
        qryFunction.Next;
      end;  // while not qryFunction.Eof do begin.
      qryFunction.Close;
      qryFunction.Free;
    end;
}
    // *************************************************************************
    // **************  END DEVELOPER MODE  *************************************
    // *************************************************************************
    if (qry.Bof and qry.Eof) then begin
      sID := '0';              // Default.
    end
    else begin
      sID := IntToStr(qry.FieldByName('id').AsInteger);
      qry.Close;
      qry.SQL.Clear;
      qry.SQL.Add('UPDATE tUser');
      qry.SQL.Add('SET LastLoggedOn =''' + StandardDateTimeString(Now) + '''');
      qry.SQL.Add('WHERE (ID = '+sID+')');
      qry.ExecSQL;
    end;
    qry.Close;
    qry.Free;
    sQueryString.Free;
  end;  // no operator name given.
end;  // procedure GetOperatorReference().


procedure LogOffUser(const sID:string);
var
  sUser : string;
  qry : TQuery;
begin
  sUser := LowerCase(Trim(sID));
  if (Length(sUser)>0) and not (sUser='0') then begin
    qry := TQuery.Create(nil);
    qry.DatabaseName := 'dbPPdata';
    qry.SQL.Add('UPDATE tUser');
    qry.SQL.Add('SET LastLoggedOff =''' + StandardDateTimeString(Now) + '''');
    qry.SQL.Add('WHERE (ID = '+sID+')');
    qry.ExecSQL;
    qry.Close;
    qry.Free;
  end;  // if (Length(sUser)>0) and not (sUser='0') then begin.
end;  // procedure LogOffUser().


procedure DeleteTransactions(const sTable: string; const nThisBatch: LongInt;
  const sFieldName,sFieldType,sOperation,sFieldValue: string;
  var lSuccess:boolean);
var
  MessageForm : TMessagesForm;
  qry : TQuery;
  qryStringList : TStringList;
  sField : string;
  sFType : string;
  sValue : string;
  sAction : string;
  lSuccessful : boolean;
begin
  MessageForm := TMessagesForm.Create(Nil);
  MessageForm.Msg.Caption := 'Updating the invoice data ...';
  MessageForm.Show;
  MessageForm.Refresh;
  lSuccessful := True;
  sField := sFieldName;
  sFType := UpperCase(sFieldType);
//  if Length(sFType) = 0 then sFType := 'S';               // Default = string.
  sValue := sFieldValue;
  sAction := sOperation;
  qryStringList := TStringList.Create;
  qryStringList.Add('DELETE');
  qryStringList.Add('FROM '+sTable);
  qryStringList.Add('WHERE (BatchID = '+IntToStr(nThisBatch)+')');
  if (Length(sField) > 0) and (Length(sFType) > 0) then begin
    // Test full data is available.
    if (Length(sValue) > 0) and (Length(sAction) > 0) then begin
      if (Pos(sFType,'DS')>0) then begin
        sValue := ''''+sValue+'''';                 // Add quote marks.
      end;  // if (Pos(sFType,'DS')>0) then begin.
      qryStringList.Add('AND ('+sField+' '+sAction+' '+sValue+')');
    end
    else lSuccessful := False;
  end
  else if (Length(sValue) = 0) then lSuccessful := True
  else lSuccessful := False;
  if lSuccessful then begin
    qry := TQuery.Create(nil);
    qry.DatabaseName := 'dbPPdata';
    qry.SQL := qryStringList;
    qry.ExecSQL;
    qry.Close;
    qry.Free;
  end;  // if lSuccessful then begin.
  qryStringList.Free;
  MessageForm.Close;
  MessageForm.Free;
  lSuccess := lSuccessful;
end;  // procedure DeleteTransactions().


procedure XCopyToAdjustment(const nThisBatch:LongInt;
  const FieldValues: TStringList; var lSuccess: boolean);
// Two field values are in each string in FieldValues, separated by 2x'_' chars.
// The values are for ServiceID and StatusID in the tDataCollection table.
var
  MessageForm : TMessagesForm;
  qry : TQuery;
//  tblAdjust : TTable;
  qryStringList : TStringList;
  sServiceID : string;
  sStatusID : string;
  nComma : integer;
  nCount : integer;
  nSelect : integer;
  lSuccessful : boolean;
begin
  MessageForm := TMessagesForm.Create(Nil);
  MessageForm.Msg.Caption := 'Transferring selected data ...';
  MessageForm.Show;
  MessageForm.Refresh;
  lSuccessful := True;
  nCount := FieldValues.Count;
  qryStringList := TStringList.Create;
  qryStringList.Add('INSERT INTO tAdjustment');
  qryStringList.Add('SELECT *');
  qryStringList.Add('FROM tDataCollection');
  qryStringList.Add('WHERE (BatchID = '+IntToStr(nThisBatch)+')');
  qryStringList.Add('AND (');
  sServiceID := FieldValues.Strings[0];
  nComma := Pos('__',sServiceID);
  sStatusID := Copy(sServiceID,nComma+2,Length(sServiceID)-nComma-1) +'))';
  if nComma > 1 then begin
    sServiceID := ''''+Copy(sServiceID,1,nComma-1) +''')';
    qryStringList.Add('((ServiceID LIKE '+sServiceID);
  end
  else begin
    qryStringList.Add('((ServiceID IS NULL)');
  end;
  qryStringList.Add('AND (StatusID = '+sStatusID);
  for nSelect := 1 to nCount - 1 do begin
    sServiceID := FieldValues.Strings[nSelect];
    nComma := Pos('__',sServiceID);
    sStatusID := Copy(sServiceID,nComma+2,Length(sServiceID)-nComma-1) +'))';
    if nComma > 1 then begin
      sServiceID := ''''+Copy(sServiceID,1,nComma-1) +''')';
      qryStringList.Add('OR ((ServiceID LIKE '+sServiceID);
    end
    else begin
      qryStringList.Add('OR ((ServiceID IS NULL)');
    end;
    qryStringList.Add('AND (StatusID = '+sStatusID);
  end;
  // Now add a closing bracket to the WHERE statement.
  nSelect := qryStringList.Count - 1;
  qryStringList[nSelect] := qryStringList[nSelect]+')';
  qry := TQuery.Create(nil);
  qry.DatabaseName := 'dbPPdata';
  qry.SQL := qryStringList;
  try
    qry.ExecSQL;
  except
    lSuccess := False;
    qry.Free;
  end;
  qry.Free;
  qryStringList.Free;
  MessageForm.Close;
  MessageForm.Free;
  lSuccess := lSuccessful;
end;  // procedure XCopyToAdjustment().


procedure DeleteFromDC(const nThisBatch:LongInt;
  const FieldValues: TStringList; var lSuccess: boolean);
// Two field values are in each string in FieldValues, separated by 2x'_' chars.
// The values are for ServiceID and StatusID in the tDataCollection table.
var
  MessageForm : TMessagesForm;
  qry : TQuery;
  qryStringList : TStringList;
  sServiceID : string;
  sStatusID : string;
  nComma : integer;
  nCount : integer;
  nSelect : integer;
  lSuccessful : boolean;
begin
  MessageForm := TMessagesForm.Create(Nil);
  MessageForm.Msg.Caption := 'Correcting billing data ...';
  MessageForm.Show;
  MessageForm.Refresh;
  lSuccessful := True;
  nCount := FieldValues.Count;
  qryStringList := TStringList.Create;
  qryStringList.Add('DELETE');
  qryStringList.Add('FROM tDataCollection');
  qryStringList.Add('WHERE (BatchID = '+IntToStr(nThisBatch)+')');
  qryStringList.Add('AND (');
  sServiceID := FieldValues.Strings[0];
  nComma := Pos('__',sServiceID);
  sStatusID := Copy(sServiceID,nComma+2,Length(sServiceID)-nComma-1) +'))';
  if nComma > 1 then begin
    sServiceID := ''''+Copy(sServiceID,1,nComma-1) +''')';
    qryStringList.Add('((ServiceID LIKE '+sServiceID);
  end
  else begin
    qryStringList.Add('((ServiceID IS NULL)');
  end;
  qryStringList.Add('AND (StatusID = '+sStatusID);
  for nSelect := 1 to nCount - 1 do begin
    sServiceID := FieldValues.Strings[nSelect];
    nComma := Pos('__',sServiceID);
    sStatusID := Copy(sServiceID,nComma+2,Length(sServiceID)-nComma-1) +'))';
    if nComma > 1 then begin
      sServiceID := ''''+Copy(sServiceID,1,nComma-1) +''')';
      qryStringList.Add('OR ((ServiceID LIKE '+sServiceID);
    end
    else begin
      qryStringList.Add('OR ((ServiceID IS NULL)');
    end;
    qryStringList.Add('AND (StatusID = '+sStatusID);
  end;
  // Now add a closing bracket to the WHERE statement.
  nSelect := qryStringList.Count - 1;
  qryStringList[nSelect] := qryStringList[nSelect]+')';
  if lSuccessful then begin
    qry := TQuery.Create(nil);
    qry.DatabaseName := 'dbPPdata';
    qry.SQL := qryStringList;
    qry.ExecSQL;
    qry.Close;
    qry.Free;
  end;  // if lSuccessful then begin.
  qryStringList.Free;
  MessageForm.Close;
  MessageForm.Free;
  lSuccess := lSuccessful;
end;  // procedure XCopyToAdjustment().

// A function to remove any apostrophe from the passed string.
// required to avoid confusing SQLServer that uses this char for string limits.
function CleanString( const S: string) : string;
var
  ans : string;
  nCharAt : integer;
begin
  ans := S;
  nCharAt := Pos('''', ans);
  while (nCharAt > 0) do
    begin
    if (nCharAt = 1) then
      ans := Copy( ans,2,Length(ans))
    else if (nCharAt = Length(ans)) then
      ans := Copy( ans,1,Length(ans)-1)
    else
      begin
      ans := Copy(ans,1,nCharAt-1) + Copy(ans,nCharAt+1,Length(ans));
      end;
    nCharAt := Pos('''', ans);
    end;
  Result := ans;
end;    // function CleanString().


function SingleSpaces( const S:string): string;
var
  sPad: string;
  nSpace: integer;
begin
  sPad := S;
  nSpace := Pos('  ',sPad);
  while (nSpace > 0) do begin
    sPad := Copy(sPad,1,nSpace) + Copy(sPad,nSpace+2,Length(sPad)-nSpace+1);
    nSpace := Pos('  ',sPad);
  end;  // while..do.
  Result := sPad;
end;  // function SingleSpaces().


// Over-rides the default "ffGeneral" format, used to insert/update with SQL.
function Curr2Str(const curValue: Currency): string;
begin
  Result := CurrToStrF(curValue, ffFixed, 2);
end;  // function Curr2Str().


// General functions to correct rounding errors with the Currency field.

function curSum(curValue1, curValue2: Currency) : Currency;
begin
  Result := (Round(curValue1*100.0) + Round(curValue2*100.0))/100.00;
end;    // function curSum().

function curFormat(curValue: Currency) : Currency;
begin
  Result := (Round(curValue*100.0)/100.00);
end;    // function curSum().


// Some general functions to simplify the transition from dBase to Delphi4.

function Left( const S: string; const N: integer): string;
// Returns the first N characters of string S.
var
  ans : string;
  iOk : integer;
begin
  ans := S;
  if N > Length(ans) then iOk := Length(ans) else iOk := N;
  SetLength( ans, iOk);
  Result := ans;
end;    // function Left().


function Right( const S: string; const N: integer): string;
// Returns the last N characters of string S.
var
  ans : string;
  i : word;
  iOk, iStart : integer;
begin
  if N > Length(S) then iOk := Length(S) else iOk := N;
  SetLength( ans, iOk);
  iStart := Length(S) - iOk;
  for i := 1 to iOk do ans[i] := S[i+iStart];
  Result := ans;
end;    // function Right().


procedure QuickCopyTable(T: TTable; DestTblName: string;
                         Overwrite: Boolean);
// Copies TTable T to an identical table with name DestTblName.
// Will overwrite an existing table with name DestTblName if
// Overwrite is True.
var
  DBType: DBINAME;
  WasOpen: Boolean;
  NumCopied: Word;
begin
  WasOpen := T.Active;                 // Save the table state;
  if (not Wasopen) then T.Open;        // Ensure table is open.
  // Get the driver type string.
  Check(DbiGetProp(hDBIObj(T.Handle), drvDRIVERTYPE, @DBType,
    Sizeof(DBINAME), NumCopied));
  // Copy the table.
  Check(DbiCopyTable(T.DBHandle, Overwrite, PChar(T.TableName), DBType,
    PChar(DestTblName)));
  T.Active := WasOpen;
end;    // procedure QuickCopyTable().


function AccountFormat( const S: string ): string;
var
  SubString : String;
  nSubString : integer;
begin
  SubString := S;
  nSubString := Length(SubString);
  if (nSubString<12) then begin
    if (Pos(Copy(SubString,1,2),'_03_02_07_08_')>0) then begin
      // Business (standard) phone.
      SubString := Trim(Copy(SubString,1,2)+'-'+Copy(SubString,3,nSubString-2));
    end
    else if Copy(SubString,1,2)='04' then begin
      // Digital mobile phone.
      SubString := Copy(SubString,1,4)+'-'+Copy(SubString,5,nSubString-4);
    end
    else if (Pos(Copy(SubString,1,4),'_1300_1800_2DIR_1902_')>0) then begin
      // Customer service calls, etc.
      SubString := Copy(SubString,1,4)+'-'+Copy(SubString,5,nSubString-4);
    end
    else if (Copy(SubString,1,3)='130') then begin
      SubString := Copy(SubString,1,3)+'-'+Copy(SubString,4,nSubString-3);
    end
    else if (Copy(SubString,1,2)='01') then begin
      // Analogue mobile phone.
      SubString := Copy(SubString,1,3)+'-'+Copy(SubString,4,nSubString-3);
    end
    else if (Length(SubString) = 8) and
      (Pos(Copy(SubString,5,1),'0123456789') > 0) then begin
      // Not an SACC account number, assume '03' prefix is missing.
      SubString := '03-' + SubString;
    end
    else if (Length(SubString) = 9) and
      (Pos(Copy(SubString,1,1),'2378') > 0) then begin
      // Not an SACC account number, assume '0x' prefix is missing.
      SubString := '0' + Copy(SubString,1,1)+ '-' +Copy(SubString,2,nSubString-1);
    end;
  end;  // if (nSubString<12) then begin.
  Result := SubString;
end;    // function AccountFormat().

function TestForAudit(const TableName: string): boolean;
// Tests to see if changes to "TableName" are to be audited.
var
  qry : TQuery;
  sTableName : string;
begin
  sTableName := Trim(TableName);
  if (Length(sTableName) > 0) then begin
    qry := TQuery.Create(nil);
    qry.DatabaseName := 'dbPPdata';
    qry.SQL.Add('SELECT tAudit.id');
    qry.SQL.Add('FROM tAudit');
    qry.SQL.Add('WHERE tAudit.[Table] LIKE '''+sTableName+'''');
    qry.Open;
    Result := not (qry.Bof and qry.Eof);  // Audit only if listed in tAudit.
    qry.Close;
    qry.Free;
  end
  else begin
    Result := False;
  end;
end;  // TestForAudit.

procedure SetAuditStart(const UserName, sAction, TableName: string;
  var AuditInsert, AuditField, AuditValue: string);
var
  qryAudit : TQuery;
begin
  qryAudit := TQuery.Create(nil);
  qryAudit.DatabaseName := 'dbPPdata';
  AuditInsert := 'INSERT INTO tAuditTrail';
  AuditField := '(UserID,ActionTime,Action,TableName,TableID,' +
    'FieldName,OldValue,NewValue)';
  AuditValue := 'VALUES ('''+UserName+''','''+StandardDateTimeString(Now)+
    ''','+ sAction +','''+ TableName +''',';
end;  // SetAuditStart.

procedure SetAuditData(const FieldName, OldValue, NewValue: string;
  var AuditData: TStringList);
// Test for a change is data, & if so adds the change to "AuditData".
var
  sStarted : string;
begin
  sStarted := NewValue;
  if (sStarted <> OldValue) then begin
    if (Length(sStarted)=0) then begin
      AuditData.Add(','''+ FieldName +''','''+ OldValue +''',NULL)');
    end
    else if (Length(OldValue)=0) then begin
      AuditData.Add(','''+ FieldName +''',NULL,'''+ sStarted +''')');
    end
    else begin
      AuditData.Add(','''+ FieldName +''','''+ OldValue +''','''+sStarted +''')');
    end;
  end;  // changed.
end;  // SetAuditData.

procedure SetAuditID(const ID: string; var AuditValue: string);
begin
  AuditValue := AuditValue + ID;
end;  // SetAuditID.

procedure SaveAudit(const AuditInsert, AuditField, AuditValue: string;
  const AuditData: TStringList);
// Save all changed data.
var
  iData : integer;
  AuditValues : string;
  qryAudit : TQuery;
begin
  // Save the data change.
  AuditValues := AuditValue;
  qryAudit := TQuery.Create(nil);
  with qryAudit do begin
    DatabaseName := 'dbPPdata';
    SQL.Clear;
    SQL.Add(AuditInsert);
    SQL.Add(AuditField);
    SQL.Add(AuditValues);
    for iData := 0 to AuditData.Count - 1 do begin
      SQL.Strings[2] := AuditValues + AuditData[iData];
{
// Start Test.
SaveText(SQL.Text);
// End Test.
}
      ExecSQL;
    end;  // for iData...
  end;  // qryAudit.
  qryAudit.Free;
end;  // SaveAudit.

function TestForTestDB: boolean;
var
  NewPPIniFile : TIniFile;
  IniFileName : string;
  sDSN : string;
  sDSNtest : string;
begin
  IniFileName := INIFILEDIRECTORY+ChangeFileExt(ExtractFileName(ParamStr(0)),'.ini');
  NewPPIniFile := TIniFile.Create(IniFileName);
  // dbPPdata - SQLPP (most common) database.
  with NewPPIniFile do begin
    sDSN := ReadString('DBconnection','PPDSN', '');
    sDSNtest := DM.DecodeString(ReadString('TestCfg','id1', ''));
  end;
  NewPPIniFile.Free;
  // True in the following = Startup call only.
  Result := (sDSN = sDSNtest) or (Pos('TEST', UpperCase(sDSN)) > 0);
end;  // TestForTestDB.

procedure SetParameter(const sParm, sParmValue, sParmField: string);
var
  qParam : TQuery;
  sValue : string;
begin
  qParam := TQuery.Create(MainForm);
  with qParam do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('SELECT ParmField');
    SQL.Add('FROM tParameter');
    SQL.Add('WHERE Parm LIKE '''+sParm+'''');
    Open;
    sValue := UpperCase(FieldByName('Parmfield').AsString);
    Close;
    SQL.Clear;
    if (Length(sValue) = 0) then begin
      SQL.Add('INSERT INTO tParameter');
      SQL.Add('(Parm, ParmField, '+sParmField+', ParmUserDefined)');
      if (sParmField = 'PARMTEXT') or (sParmField = 'PARMDATE') then begin
        SQL.Add('VALUES ('''+sParm+''', '''+sParmField+''', '''+sParmValue+''',1)');
      end
      else begin
        SQL.Add('VALUES ('''+sParm+''', '''+sParmField+''', '+sParmValue+',1)');
      end;
    end
    else begin
      SQL.Add('UPDATE tParameter');
      if (sParmField = 'PARMTEXT') or (sParmField = 'PARMDATE') then begin
        SQL.Add('SET '+sParmField+' = '''+sParmValue+'''');
      end
      else begin
        SQL.Add('SET '+sParmField+' = '+sParmValue);
      end;
      SQL.Add('WHERE Parm LIKE '''+sParm+'''');
    end;
    ExecSQL;
  end;
end;  // SetParameter.

procedure GetParameter(const sParm:string; var sParmValue: string);
var
  qParam : TQuery;
  sValue : string;
begin
  qParam := TQuery.Create(MainForm);
  with qParam do begin
    DatabaseName := 'dbPPdata';
    SQL.Add('SELECT ParmField');
    SQL.Add('FROM tParameter');
    SQL.Add('WHERE Parm LIKE '''+sParm+'''');
    Open;
    sValue := FieldByName('Parmfield').AsString;
    Close;
    if (Length(sValue) = 0) then begin
      sParmValue := sValue;
    end
    else begin
      SQL[0] := 'SELECT '+sValue;
      Open;
      sParmValue := FieldByName(sValue).AsString;
      Close;
    end;
  end;
end;  // GetParameter.

// *****************************************************************
//                SAVE THE QUERY DATA PASSED TO A CSV FILE
// *****************************************************************
procedure FileQuery(const qryData:TQuery;const FileName: string );
// This procedure creates a CSV file from the query parameter and saves it
// in the directory selected below asnd the filename chosen below.
var
  SaveDialog : TSaveDialog;
  lContinue : Boolean;
  CSVTextFile : TextFile;
  Progress : TProgressForm;
  nFactor : Real;
  sFile : String;
  sline : Integer;
  nFieldNumber : integer;
  sOriginalDirectory : string;
  sUserDirectory : string;
  sCSVString : string;
  FieldList : TStringList;
begin
  FieldList := TStringList.Create;
  // The table required for printing must already be open.
  // Determine the parameters for the progress bar.
  sline := qryData.RecordCount;
  if sLine = 0 then begin
    MessageDlg('No entries found in table '+ FileName,
      mtInformation,[mbOk],0);
  end
  else begin
    nFactor := 100.0 / sline;        // Data passed twice:Verify,CostPart.
    // Reset the record counter.
    sline := 0;

    GetParameter('UserDirectory',sUserDirectory);
    lContinue := (Length(sUserDirectory)>0) and DirectoryExists(sUserDirectory);
    if lContinue then sOriginalDirectory := sUserDirectory;
    // Confirm with the operator where the file is to be saved.
    SaveDialog := TSaveDialog.Create(Nil);
    SaveDialog.Title := 'Save the selected table values in CSV format';
    SaveDialog.FileName := FileName;
    SaveDialog.Filter := 'All Files (*.*)|*.*';
    SaveDialog.FilterIndex := 1;
    SaveDialog.DefaultExt := 'csv';
    if lContinue then begin
      // Use the default directory obtained from tParameters.
      SaveDialog.InitialDir := sUserDirectory;
    end
    else begin
      // Use the program directory as the default directory.
      SaveDialog.InitialDir := ExtractFilePath(ParamStr(0));
    end;
    SaveDialog.Options := [ofHideReadOnly,ofNoReadOnlyReturn];
    lContinue := SaveDialog.Execute;
    sFile := SaveDialog.FileName;
    SaveDialog.Free;
    sUserDirectory := ExtractFileDir(sFile);
    if lContinue and DirectoryExists(sUserDirectory) and
      (sUserDirectory <> sOriginalDirectory) then begin
      // Save the directory name in tParameter.
      SetParameter('UserDirectory',sUserDirectory,'PARMTEXT');
    end;  // DirectoryExists.

    if lContinue then begin
      // Open the progress form.
      Progress := TProgressForm.Create(NIL);
      // Disable the Abort button - Printer not active.
      Progress.btnCancel.Enabled := False;
      Progress.btnCancel.Visible := False;
      // Display a message to the user.
      Progress.Msg.Caption := 'Saving account data to file ...';
      Progress.Show;

      // Open the required file.
      AssignFile( CSVTextFile, sFile );
      try
        Rewrite( CSVTextFile );
        // OK, now output the data to the file.
        // Print the field names first.
        sCSVString := '';
        qryData.GetFieldNames(FieldList);
        for nFieldNumber := 0 to FieldList.Count -1 do begin
          sCSVString := sCSVString + '"' +
            FieldList.Strings[nFieldNumber]+'"';
          if (nFieldNumber < (FieldList.Count -1)) then begin
            sCSVString := sCSVString + ',';
          end;
        end;
        // Output the information to the file.
        Writeln( CSVTextFile, sCSVString );
        // Start at the beginning.
        qryData.First;
        while (not qryData.Eof) do begin
          Progress.ProcessWinMessages(MainForm); // Do not lockup the computer.
          // Develop the line of information.
          sCSVString := '';
          for nFieldNumber := 0 to qryData.FieldCount -1 do begin
            sCSVString := sCSVString +'"' +
              qryData.Fields[nFieldNumber].AsString +'"';
            if (nFieldNumber < (qryData.FieldCount -1)) then begin
              sCSVString := sCSVString + ',';
            end;
          end;

          // Output the information to the file.
          Writeln( CSVTextFile, sCSVString );
          // Report progress to the user.
          Inc(sLine);
          Progress.ProgressBar1.Position := Trunc(sline * nFactor);

          // Go to the next line in the result set.
          qryData.Next;
        end;    // while (not qryData.Eof) and.
      finally
        // Done, now close the output file.
        CloseFile( CSVTextFile );
        Progress.Close;
      end;    // try.
    end;    // if sLine > 0  (some data to write).
  end;    // if lContinue then.
end;  // procedure FileQuery().

end.
