unit SysEditAgreement;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, dbtables, Mask, ToolEdit, CurrEdit, Db;

type
  TSysEditAgreementForm = class(TForm)
    edtID: TEdit;
    lblID: TLabel;
    edtAgreementNo: TEdit;
    lblAgreementNo: TLabel;
    edtDescription: TEdit;
    lblDescription: TLabel;
    lblSupplier: TLabel;
    lblAgreementType: TLabel;
    lblBatchType: TLabel;
    lblValidAccountNumber: TLabel;
    lblStartDate: TLabel;
    lblEndDate: TLabel;
    btnOk: TButton;
    btnCancel: TButton;
    edtStartDate: TDateEdit;
    edtEndDate: TDateEdit;
    cbBatchType: TComboBox;
    cbValidAccountNumber: TComboBox;
    cbAgreementType: TComboBox;
    cbSupplier: TComboBox;
    qBatch: TQuery;
    qValidAcct: TQuery;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnOkClick(Sender: TObject);
    procedure cbSupplierChange(Sender: TObject);
    procedure cbBatchTypeChange(Sender: TObject);
  private
    { Private declarations }
    AmendString : TStringList;
    AuditString : TStringlist;
    slAgreementIndex : TStringList;
    slSupplierIndex  : TStringList;
    slValidAcctIndex : TStringList;
    slBatchTypeIndex : TStringList;
    sUserName : string;
    lAppend : boolean;
    lAudit : boolean;
    AuditActionUpdate : string;
    AuditActionDelete : string;
    AuditActionInsert : string;
    Original_ID              : integer;
    Original_AgreementNo     : string;
    Original_Description     : string;
    Original_AgreementTypeID : integer;
    Original_SupplierID      : integer;
    Original_ValidAcctID     : integer;
    Original_BatchTypeID     : integer;
    Original_StartDate       : string;
    Original_EndDate         : string;
  protected
    procedure SetAppend(const Value: boolean);
    procedure SetID(const Value: integer);
    function GetID: integer;
    procedure SetAgreementNo(const Value: string);
    function GetAgreementNo: string;
    procedure SetDescription(const Value: string);
    function GetDescription: string;
    procedure SetAgreementTypeID(const Value: integer);
    function GetAgreementTypeID: integer;
    procedure SetSupplierID(const Value: integer);
    function GetSupplierID: integer;
    procedure SetValidAcctID(const Value: integer);
    function GetValidAcctID: integer;
    procedure SetBatchTypeID(const Value: integer);
    function GetBatchTypeID: integer;
    procedure SetStartDate(const Value: TDateTime);
    function GetStartDate: TDateTime;
    procedure SetEndDate(const Value: TDateTime);
    function GetEndDate: TDateTime;
  published
    property AppendMode: boolean read lAppend write SetAppend;
    property ID: integer read GetID write SetID;
    property AgreementNo: string read GetAgreementNo write SetAgreementNo;
    property Description: string read GetDescription write SetDescription;
    property AgreementTypeID: integer read GetAgreementTypeID write SetAgreementTypeID;
    property SupplierID: integer read GetSupplierID write SetSupplierID;
    property BatchTypeID: integer read GetBatchTypeID write SetBatchTypeID;
    property ValidAccountNumberID: integer read GetValidAcctID write SetValidAcctID;
    property StartDate: TDateTime read GetStartDate write SetStartDate;
    property EndDate: TDateTime read GetEndDate write SetEndDate;
  public
    { Public declarations }
  end;

var
  SysEditAgreementForm: TSysEditAgreementForm;

implementation

uses
  DataMod, DateFunctions, GenFns, PPSysMain;

{$R *.DFM}

procedure TSysEditAgreementForm.FormCreate(Sender: TObject);
var
  qFormCreate : TQuery;
  ActionDescription : string;
begin
  lAppend := False;
  AmendString := TStringList.Create;
  AuditString := TStringList.Create;
  slAgreementIndex := TStringList.Create;
  slSupplierIndex  := TStringList.Create;
  slValidAcctIndex := TStringList.Create;
  slBatchTypeIndex := TStringList.Create;
  Original_ID              := 0;
  Original_AgreementNo     := '';
  Original_Description     := '';
  Original_AgreementTypeID := 0;
  Original_SupplierID      := 0;
  Original_ValidAcctID     := 0;
  Original_BatchTypeID     := 0;
  Original_StartDate       := '';
  Original_EndDate         := '';
  sUserName := Trim(MainForm.OPERATORNAME);
  qFormCreate := TQuery.Create(SELF);
  qFormCreate.DatabaseName := 'dbPPdata';
  // Read the audit event IDs.
  with qFormCreate do begin
    SQL.Add('SELECT *');
    SQL.Add('FROM tEvent');
    SQL.Add('WHERE Description LIKE ''Reference%''');
    Open;
    while not Eof do begin
      ActionDescription := UpperCase(FieldByName('Description').AsString);
      if (Pos('UPDATE', ActionDescription) > 0) then
        AuditActionUpdate := FieldByName('id').AsString
      else if (Pos('DELETE', ActionDescription) > 0) then
        AuditActionDelete := FieldByName('id').AsString
      else if (Pos('INSERT', ActionDescription) > 0) then
        AuditActionInsert := FieldByName('id').AsString;
      Next;
    end;
    Close;
    SQL.Clear;                                         // AgreementTypeID..
    SQL.Add('SELECT tAgreementType.ID, tAgreementType.Description');
    SQL.Add('FROM tAgreementType');
    SQL.Add('WHERE tAgreementType.Active = 1');
    SQL.Add('ORDER BY tAgreementType.Description');
    Open;
    while not Eof do begin
      cbAgreementType.Items.Add(FieldByName('Description').AsString);
      slAgreementIndex.Add(IntToStr(FieldByName('ID').AsInteger));
      Next;
    end;
    Close;
    SQL.Clear;                                         // SupplierID..
    SQL.Add('SELECT tSupplier.ID, tSupplier.SupplierName');
    SQL.Add('FROM tSupplier');
    SQL.Add('WHERE tSupplier.Active = 1');
    SQL.Add('ORDER BY tSupplier.SupplierName');
    Open;
    while not Eof do begin
      cbSupplier.Items.Add(FieldByName('SupplierName').AsString);
      slSupplierIndex.Add(IntToStr(FieldByName('ID').AsInteger));
      Next;
    end;
    Close;
  end;  // with qformCreate.
  with qBatch do begin
    SQL.Clear;                                         // BatchTypeID..
    SQL.Add('SELECT tBatchType.ID, tBatchType.Description');
    SQL.Add('FROM tBatchType');
    SQL.Add('WHERE tBatchType.Active = 1');
    SQL.Add('AND tBatchType.ID > 0');
    SQL.Add('ORDER BY tBatchType.Description');
    Open;
    while not Eof do begin
      cbBatchType.Items.Add(FieldByName('Description').AsString);
      slBatchTypeIndex.Add(IntToStr(FieldByName('ID').AsInteger));
      Next;
    end;
    Close;
  end;  // with qBatch.
  with qValidAcct do begin
    SQL.Clear;                                         // ValidAccountNumberID..
    SQL.Add('SELECT tValidAccountNumber.ID, tValidAccountNumber.AccountNo');
    SQL.Add('FROM tValidAccountNumber');
    SQL.Add('WHERE tValidAccountNumber.Active = 1');
    SQL.Add('AND tValidAccountNumber.ID > 0');
    SQL.Add('ORDER BY tValidAccountNumber.AccountNo');
    Open;
    while not Eof do begin
      cbValidAccountNumber.Items.Add(FieldByName('AccountNo').AsString);
      slValidAcctIndex.Add(IntToStr(FieldByName('ID').AsInteger));
      Next;
    end;
    Close;
  end;  // with qValidAcct.
  qFormCreate.Free;
  lAudit := TestForAudit('tAgreement');
end;  // FormCreate.

procedure TSysEditAgreementForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  qBatch.Close;
  qValidAcct.Close;
  AmendString.Free;
  slAgreementIndex.Free;
  slSupplierIndex.Free;
  slValidAcctIndex.Free;
  slBatchTypeIndex.Free;
end;

procedure TSysEditAgreementForm.SetAppend(const Value: boolean);
begin
  lAppend := Value;
  if lAppend then begin
    Caption := Caption + ' (ADD)';
  end  // if lAppend then begin.
  else begin
    Caption := Caption + ' (EDIT)';
  end;  // if lAppend then begin..else.
end;  // SetAppend.

procedure TSysEditAgreementForm.SetID(const Value: integer);
begin
  if (Value = 0) then Original_ID := 0
  else                Original_ID := Value;
  edtID.Text := IntToStr(Original_ID);
end;  // SetID.

function TSysEditAgreementForm.GetID: integer;
begin
  Result := StrToInt(Trim(edtID.Text));
end;  // GetID.

procedure TSysEditAgreementForm.SetAgreementNo(const Value: string);
begin
  Original_AgreementNo := Trim(Value);
  edtAgreementNo.Text := Original_AgreementNo;
end;  // SetAgreementNo.

function TSysEditAgreementForm.GetAgreementNo: string;
begin
  Result := Trim(edtAgreementNo.Text);
end;  // GetAgreementNo.

procedure TSysEditAgreementForm.SetDescription(const Value: string);
begin
  Original_Description := Trim(Value);
  edtDescription.Text := Original_Description;
end;  // SetDescription.

function TSysEditAgreementForm.GetDescription: string;
begin
  Result := Trim(edtDescription.Text);
end;  // GetDescription.

procedure TSysEditAgreementForm.SetAgreementTypeID(const Value: integer);
var
  sSelected : string;
  iCount : integer;
begin
  if (Value > 0) then begin
    sSelected := IntToStr(Value);
    for iCount := 0 to slAgreementIndex.Count - 1 do begin
      if (slAgreementIndex[iCount] = sSelected) then begin
        cbAgreementType.ItemIndex := iCount;
        Original_AgreementTypeID := Value;
        break;
      end;
    end;
  end;
end;  // SetAgreementTypeID.

function TSysEditAgreementForm.GetAgreementTypeID: integer;
var
  nSelected : integer;
begin
  nSelected := cbAgreementType.ItemIndex;
  if (nSelected < 0) or (Length(Trim(cbAgreementType.Text)) = 0) then
    Result := 0
  else
    Result := StrToInt(slAgreementIndex[nSelected]);
end;  // GetAgreementTypeID.

procedure TSysEditAgreementForm.SetSupplierID(const Value: integer);
var
  sSelected : string;
  iCount : integer;
begin
  if (Value > 0) then begin
    sSelected := IntToStr(Value);
    for iCount := 0 to slSupplierIndex.Count - 1 do begin
      if (slSupplierIndex[iCount] = sSelected) then begin
        cbSupplier.ItemIndex := iCount;
        Original_SupplierID := Value;
        break;
      end;
    end;
  end;
end;  // SetSupplierID.

function TSysEditAgreementForm.GetSupplierID: integer;
var
  nSelected : integer;
begin
  nSelected := cbSupplier.ItemIndex;
  if (nSelected < 0) or (Length(Trim(cbSupplier.Text)) = 0) then
    Result := 0
  else
    Result := StrToInt(slSupplierIndex[nSelected]);
end;  // GetSupplierID.

procedure TSysEditAgreementForm.SetValidAcctID(const Value: integer);
var
  sSelected : string;
  iCount : integer;
begin
  if (Value > 0) then begin
    sSelected := IntToStr(Value);
    for iCount := 0 to slValidAcctIndex.Count - 1 do begin
      if (slValidAcctIndex[iCount] = sSelected) then begin
        cbValidAccountNumber.ItemIndex := iCount;
        Original_ValidAcctID := Value;
        break;
      end;
    end;
  end;
end;  // SetValidAcctID.

function TSysEditAgreementForm.GetValidAcctID: integer;
var
  nSelected : integer;
begin
  nSelected := cbValidAccountNumber.ItemIndex;
  if (nSelected < 0) or (Length(Trim(cbValidAccountNumber.Text)) = 0) then
    Result := 0
  else
    Result := StrToInt(slValidAcctIndex[nSelected]);
end;  // GetValidAcctID.

procedure TSysEditAgreementForm.SetBatchTypeID(const Value: integer);
var
  sSelected : string;
  iCount : integer;
begin
  if (Value > 0) then begin
    sSelected := IntToStr(Value);
    for iCount := 0 to slBatchTypeIndex.Count - 1 do begin
      if (slBatchTypeIndex[iCount] = sSelected) then begin
        cbBatchType.ItemIndex := iCount;
        Original_BatchTypeID := Value;
        break;
      end;
    end;
  end;
end;  // SetBatchTypeID.

function TSysEditAgreementForm.GetBatchTypeID: integer;
var
  nSelected : integer;
begin
  nSelected := cbBatchType.ItemIndex;
  if (nSelected < 0) or (Length(Trim(cbBatchType.Text)) = 0) then
    Result := 0
  else
    Result := StrToInt(slBatchTypeIndex[nSelected]);
end;  // GetBatchTypeID.

procedure TSysEditAgreementForm.SetStartDate(const Value: TDateTime);
begin
  if not VarIsNull(Value) then begin
    Original_StartDate := StandardDateString(Value);
    edtStartDate.Date := Value;
  end;
end;  // SetStartDate.

function TSysEditAgreementForm.GetStartDate: TDateTime;
begin
  if (Length(Original_StartDate)=8) then
       Result := edtStartDate.Date
  else Result := NULL;
end;  // GetStartDate.

procedure TSysEditAgreementForm.SetEndDate(const Value: TDateTime);
begin
  if not VarIsNull(Value) then begin
    Original_EndDate := StandardDateString(Value);
    edtEndDate.Date := Value;
  end;
end;  // SetEndDate.

function TSysEditAgreementForm.GetEndDate: TDateTime;
begin
  if (Length(Original_EndDate)=8) then
       Result := edtEndDate.Date
  else Result := NULL;
end;  // GetEndDate.

procedure TSysEditAgreementForm.cbSupplierChange(Sender: TObject);
var
  nSelected : integer;
begin
  // Changes here alter the available selections in cbBatchType.
  cbBatchType.Items.Clear;
  nSelected := cbSupplier.ItemIndex;
  if (nSelected > 0) then begin
    qBatch.Close;
    qBatch.SQL.Strings[3] := 'AND SupplierID = '+ slSupplierIndex[nSelected];
    qBatch.Open;
    if not qBatch.Eof then begin
      while not qBatch.Eof do begin
        cbBatchType.Items.Add(qBatch['Description']);
        qBatch.Next;
      end;  // while.
      cbBatchType.ItemIndex := 0;
    end;  // qBatch.Eof.
    qBatch.Close;
  end;  // ItemIndex.
end;  // cbSupplierChange.

procedure TSysEditAgreementForm.cbBatchTypeChange(Sender: TObject);
var
  nSelected : integer;
begin
  // Changes here alter the available selections in cbValidAcct.
  cbValidAccountNumber.Items.Clear;
  nSelected := cbBatchType.ItemIndex;
  if (nSelected > 0) then begin
    qValidAcct.Close;
    qValidAcct.SQL.Strings[3] := 'AND BatchTypeID = '+ slBatchTypeIndex[nSelected];
    qValidAcct.Open;
    if not qValidAcct.Eof then begin
      while not qValidAcct.Eof do begin
        cbValidAccountNumber.Items.Add(qValidAcct['AccountNo']);
        qValidAcct.Next;
      end;  // while.
      cbValidAccountNumber.ItemIndex := 0;
    end;  // qValidAcct.Eof.
    qValidAcct.Close;
  end;  // ItemIndex.
end;  // cbBatchTypeChange.

procedure TSysEditAgreementForm.btnOkClick(Sender: TObject);
var
  q: TQuery;
  sField : string;
  sValue : string;
  sAuditInsert : string;
  sAuditField : string;
  sAuditValue : string;
//  AuditTrailDesc : string;
  AuditAction : string;
  lReturn : boolean;
  sStarted : string;
  nStarted : integer;
  sOriginal : string;          // Stores original value for when an ID=0;
  sError : string;
begin
  if lAppend then AuditAction := AuditActionInsert
  else            AuditAction := AuditActionUpdate;  // Delete is not an option.
  q := TQuery.Create(SELF);
  q.DatabaseName := 'dbPPdata';
  lReturn := True;
  if (Length(Trim(edtAgreementNo.Text)) = 0) then begin
    sError := 'AgreementNo MUST be entered.';
    lReturn := False;
  end
  else if (Length(Trim(edtDescription.Text)) = 0) then begin
    sError := 'Description MUST be entered.';
    lReturn := False;
  end
  else begin
    if lAppend then begin       ///////////////  APPEND  ////////////////////
      sStarted := CleanString(AgreementNo);
      AmendString.Add('INSERT INTO tAgreement');
      sField := '(AgreementNo,';
      sValue := 'VALUES ('''+ sStarted +''',';
      if lAudit then begin
        SetAuditStart(sUserName, AuditAction, 'tAgreement',
          sAuditInsert, sAuditField, sAuditValue);
        SetAuditData('AGREEMENTNO', '', sStarted, AuditString);
      end;  // lAudit.
      sStarted := CleanString(Description);
      if (Length(sStarted) > 0) then begin
        sField := sField + 'Description,';
        sValue := sValue +''''+sStarted+''',';
        if lAudit then begin
          SetAuditData('DESCRIPTION', '', sStarted, AuditString);
        end;  // lAudit.
      end;
      nStarted := AgreementTypeID;
      if (nStarted > 0) then begin
        sStarted := IntToStr(nStarted);
        sField := sField + 'AgreementTypeID,';
        sValue := sValue + sStarted +',';
        if lAudit and (Length(sStarted) > 0) then begin
          SetAuditData('AGREEMENTTYPEID', '', sStarted, AuditString);
        end;  // lAudit.
      end;
      nStarted := SupplierID;
      if (nStarted > 0) then begin
        sStarted := IntToStr(nStarted);
        sField := sField + 'SupplierID,';
        sValue := sValue + sStarted +',';
        if lAudit and (Length(sStarted) > 0) then begin
          SetAuditData('SUPPLIERID', '', sStarted, AuditString);
        end;  // lAudit.
      end;
      nStarted := BatchTypeID;
      sStarted := IntToStr(nStarted);
      if (nStarted > 0) then begin
        sField := sField + 'BatchTypeID,';
        sValue := sValue + IntToStr(nStarted) +',';
        if lAudit then begin
          SetAuditData('BATCHTYPEID', '', sStarted, AuditString);
        end;  // lAudit.
      end;
      nStarted := ValidAccountNumberID;
      sStarted := IntToStr(nStarted);
      if (nStarted > 0) then begin
        sField := sField + 'ValidAccountNumberID,';
        sValue := sValue + sStarted +',';
        if lAudit then begin
          SetAuditData('VALIDACCOUNTNUMBERID', '', sStarted, AuditString);
        end;  // lAudit.
      end;
      if (Length(Trim(edtStartDate.Text)) > 0) and
        (Copy(Trim(edtStartDate.Text),1,1) <> '/') then begin
        sStarted := StandardDateString(edtStartDate.Date);
        sField := sField + 'StartDate,';
        sValue := sValue + ''''+ sStarted +''',';
        if lAudit then begin
          SetAuditData('STARTDATE', '', sStarted, AuditString);
        end;
      end;
      if (Length(Trim(edtEndDate.Text)) > 0) and
        (Copy(Trim(edtEndDate.Text),1,1) <> '/') then begin
        sStarted := StandardDateString(edtEndDate.Date);
        sField := sField + 'EndDate,Active)';
        sValue := sValue + '''' + sStarted +'''0)';
        if lAudit then begin
          SetAuditData('ENDDATE', '', sStarted, AuditString);
          SetAuditData('ACTIVE', '', '0', AuditString);
        end;
      end
      else begin
        sField := sField + 'Active)';
        sValue := sValue + '1)';
        if lAudit then begin
          SetAuditData('ACTIVE', '', '1', AuditString);
        end;
      end;
      AmendString.Add(sField);
      AmendString.Add(sValue);
    end
    else begin                  ///////////////  UPDATE  ////////////////////
      sStarted := CleanString(AgreementNo);
      AmendString.Add('UPDATE tAgreement');
      AmendString.Add('SET AgreementNo = '''+ sStarted +''',');
      if lAudit then begin
        SetAuditStart(sUserName, AuditAction, 'tAgreement',
          sAuditInsert, sAuditField, sAuditValue);
        SetAuditID(Trim(edtID.Text), sAuditValue);
        if (sStarted <> Original_AgreementNo) then begin
          SetAuditData('AGREEMENTNO', Original_AgreementNo, sStarted, AuditString);
        end;
      end;  // lAudit.
      sStarted := CleanString(Description);
      if (sStarted <> Original_Description) then begin
        sValue := 'Description = ';
        if (Length(sStarted) > 0) then sValue := sValue +''''+sStarted+''','
        else                           sValue := sValue + 'NULL,';
        AmendString.Add(sValue);
        if lAudit then begin
          SetAuditData('DESCRIPTION', Original_Description, sStarted,
            AuditString);
        end;
      end;  // changed.
      nStarted := AgreementTypeID;
      if (nStarted <> Original_AgreementTypeID) then begin
        sValue := 'AgreementTypeID = ';
        if (nStarted = 0) then begin
          sValue := sValue + 'NULL,';
          sStarted := '';                               // empty forces NULL.
        end
        else begin
          sStarted := IntToStr(nStarted);
          sValue := sValue + sStarted +',';
          if (Original_AgreementTypeID = 0) then begin
            sOriginal := '';                            // empty forces NULL.
          end
          else begin
            sOriginal := IntToStr(Original_AgreementTypeID); // empty forces NULL.
          end;
        end;
        AmendString.Add(sValue);
        if lAudit then begin
          SetAuditData('AGREEMENTTYPEID', sOriginal, sStarted, AuditString);
        end;  // lAudit.
      end;  // AgreementTypeID changed.
      nStarted := SupplierID;
      if (nStarted <> Original_SupplierID) then begin
        sValue := 'SupplierID = ';
        if (nStarted = 0) then begin
          sValue := sValue + 'NULL,';
          sStarted := '';                               // empty forces NULL.
        end
        else begin
          sStarted := IntToStr(nStarted);
          sValue := sValue + sStarted +',';
          if (Original_SupplierID = 0) then begin
            sOriginal := '';                            // empty forces NULL.
          end
          else begin
            sOriginal := IntToStr(Original_SupplierID); // empty forces NULL.
          end;
        end;
        AmendString.Add(sValue);
        if lAudit then begin
          SetAuditData('SUPPLIERID', sOriginal, sStarted, AuditString);
        end;  // lAudit.
      end;  // SupplierID changed.
      nStarted := BatchTypeID;
      if (nStarted <> Original_BatchTypeID) then begin
        sValue := 'BatchTypeID = ';
        if (nStarted = 0) then begin
          sValue := sValue + 'NULL,';
          sStarted := '';                               // empty forces NULL.
        end
        else begin
          sStarted := IntToStr(nStarted);
          sValue := sValue + sStarted +',';
          if (Original_BatchTypeID = 0) then begin
            sOriginal := '';                            // empty forces NULL.
          end
          else begin
            sOriginal := IntToStr(Original_BatchTypeID);// empty forces NULL.
          end;
        end;
        AmendString.Add(sValue);
        if lAudit then begin
          SetAuditData('BATCHTYPEID', sOriginal, sStarted, AuditString);
        end;  // lAudit.
      end;  // BatchTypeID changed.
      nStarted := ValidAccountNumberID;
      if (nStarted <> Original_ValidAcctID) then begin
        sValue := 'ValidAccountNumberID = ';
        if (nStarted = 0) then begin
          sValue := sValue + 'NULL,';
          sStarted := '';                               // empty forces NULL.
        end
        else begin
          sStarted := IntToStr(nStarted);
          sValue := sValue + sStarted +',';
          if (Original_ValidAcctID = 0) then begin
            sOriginal := '';                            // empty forces NULL.
          end
          else begin
            sOriginal := IntToStr(Original_ValidAcctID);// empty forces NULL.
          end;
        end;
        AmendString.Add(sValue);
        if lAudit then begin
          SetAuditData('VALIDACCOUNTNUMBERID', sOriginal, sStarted, AuditString);
        end;  // lAudit.
      end;  // ValidAccountNumberID changed.
      if (Length(Trim(edtStartDate.Text)) > 0) and
        (Copy(Trim(edtStartDate.Text),1,1) <> '/') then begin
        sStarted := StandardDateString(edtStartDate.Date);
        if (sStarted <> Original_StartDate) then begin
          sValue := ' StartDate = '''+sStarted+''',';
          AmendString.Add(sValue);
          if lAudit then begin
            SetAuditData('STARTDATE', Original_StartDate, sStarted,
              AuditString);
          end;
        end;
      end;
      if (Length(Trim(edtEndDate.Text)) = 0) or
        (Copy(Trim(edtEndDate.Text),1,1) = '/') then begin
        if (Length(Original_EndDate) = 8) then begin
          sValue := 'EndDate = NULL, Active = 1)';
          AmendString.Add(sValue);
          if lAudit then begin
            SetAuditData('ENDDATE', Original_EndDate, '', AuditString);
            SetAuditData('ACTIVE', '0', '1', AuditString);
          end;
        end
        else begin
          sValue := 'Active = 1)';
          AmendString.Add(sValue);
        end;
      end
      else begin
        sStarted := StandardDateString(edtEndDate.Date);
        if (sStarted <> Original_EndDate) then begin
          sValue := 'EndDate = '''+sStarted+''', Active = 0)';
          AmendString.Add(sValue);
          if lAudit then begin
            SetAuditData('ENDDATE', Original_EndDate, sStarted, AuditString);
            if (Length(Original_EndDate) <> 8) then begin
              SetAuditData('ACTIVE', '1', '0', AuditString);
            end;
          end;
        end;
      end;
      // Done - remove the last trailing comma.
      with AmendString do begin
        sStarted := Strings[Count-1];
        Strings[Count-1] := Copy(sStarted,1,Length(sStarted)-1);
      end;
      sStarted := Trim(edtID.Text);
      AmendString.Add('WHERE id = '+ sStarted);
    end;
    q.SQL := AmendString;
    q.ExecSQL;
    if lAudit then begin
      if lAppend then begin
        // Get and store the new ID for this table.
        with q do begin
          SQL.Clear;
          SQL.Add('SELECT ID');
          SQL.ADD('FROM tAgreement');
          SQL.Add('WHERE AgreementNo LIKE '''+ AgreementNo +'''');
          SQL.Add('AND Active = 1');
          Open;
          SetAuditID(FieldByName('ID').AsString, sAuditValue);
          Close;
        end;
      end;  // lAppend.
      SaveAudit(sAuditInsert, sAuditField, sAuditValue, AuditString);
    end;  // lAudit.
  end;
  AmendString.Clear;
  q.Free;
  if lReturn then begin
    ModalResult := mrOk;
  end
  else begin
    MessageDlg(sError, mtWarning, [mbOk], 0);
  end;  // lReturn.
end;  // btnOkClick().

end.
